(* ::Package:: *)

(* ::Title:: *)
(*AnalysisPackage*)


(* ::Subtitle:: *)
(*This package contains the plot definitions & calculations shared between the Analysis and Combined Analysis. It consists mainly of functions in the Global` context.*)


BeginPackage["AnalysisPackage`"];


(* ::Chapter:: *)
(*Usage Definitions*)


(* ::Chapter:: *)
(*Function Definitions*)


Begin["`Private`"];


(* ::Chapter:: *)
(*End of Package*)


End[];


EndPackage[];


(* ::Chapter:: *)
(*Global*)


(* ::Text:: *)
(*The functions in this section need to be in "Global`" context in order to avoid problems with variable references. For example, the values loaded with the Import & Export functions are otherwise not properly defined.*)


(* ::Section:: *)
(*Initialisation*)


(* ::Subsection::Closed:: *)
(*Box Vector*)


boxVec = Range@2;


(* ::Subsection::Closed:: *)
(*Image Size*)


lImageSize = Medium;


(* ::Subsection::Closed:: *)
(*Colors & Markers*)


colors = ColorData[97,"ColorList"];

Graphics[
	{colors[[#]],Disk[]},
	ImageSize->{70,70},
	PlotLabel->#
] &/@ Range[1,Length@colors]


markers = Charting`CommonDump`GraphicsOpenPlotMarkers[];
ListPlot[Table[Range[4] *iMarkers, {iMarkers, Length@markers}], 
 PlotMarkers -> markers, PlotLegends->Range[Length@markers]]


(* ::Section:: *)
(*Calculations*)


(* ::Subsection::Closed:: *)
(*Helper: Blocking Functions*)


(* accumulate over the last n items instead over all items that lay behind *)
BlockAccumulate[list_,blockSize_] := Module[{resultList},
		
	resultList = Table[0.,Length@list];
		
	Do[
		If[i <= blockSize,
			resultList[[i]] = Total[list[[1;;i]]];
			,(* ELSE *)
			resultList[[i]] = Total[list[[i-blockSize+1;;i]]];
		];
	
	,{i,Length@list}];
		
	(* RETURN *)
	resultList		
];


(* accumulate over the last n items instead over all items that lay behind *)
BlockStdDeviation[list_,blockSize_] := Module[{resultList},
		
	resultList = Table[0.,Length@list -1];
		
	Do[
		If[i <= blockSize,
			resultList[[i]] = StandardDeviation[list[[1;;i +1]]];
			,(* ELSE *)
			resultList[[i]] = StandardDeviation[list[[i +1 -blockSize+1;;i +1]]];
		];
	
	,{i,Length@list -1}];
		
	(* RETURN *)
	resultList		
];


(* ::Subsection::Closed:: *)
(*Pressure*)


PressureCalculations[] := Module[{},
	(*
	10^30 \[Rule] convert volume \[CapitalARing]^3 to m^3
	4184 \[Rule] convert kcal to Joule
	nAvogadro \[Rule] convert number of molecules to mol
	10^5 \[Rule] convert Pa to bar
	*)

	pressIdeal1 = nvecList[[All,1]] * R * T / VvecList[[All,1]] * 10^30 * 4184 / nAvogadro / 10^5; 
	pressIdeal2 = nvecList[[All,2]] * R * T / VvecList[[All,2]] * 10^30 * 4184 / nAvogadro / 10^5;

	pressViral1 = interVirialList[[All,1]] / VvecList[[All,1]] * 10^30 * 4184 / nAvogadro / 10^5; 
	pressViral2 = interVirialList[[All,2]] / VvecList[[All,2]] * 10^30 * 4184 / nAvogadro / 10^5;

	press1 = pressIdeal1 + pressViral1;
	press2 = pressIdeal2 + pressViral2;

	tailPress1 = tailCorrPressureList[[All,1]] / VvecList[[All,1]] * 10^30 * 4184 / nAvogadro / 10^5;
	tailPress2 = tailCorrPressureList[[All,2]] / VvecList[[All,2]] * 10^30 * 4184 (*J/mol*) / nAvogadro (*J/atom*) / 10^5;

	(*
	tailPress1 = If[nvecList[[#,1]] != 0, tailCorrPressureList[[#,1]] / nvecList[[#,1]], 0.] &/@ Range[Length@nvecList] / VvecList[[All,1]] * 10^30 * 4184 / nAvogadro / 10^5;
	tailPress2 = If[nvecList[[#,2]] != 0, tailCorrPressureList[[#,2]] / nvecList[[#,2]], 0.] &/@ Range[Length@nvecList] / VvecList[[All,2]] * 10^30 * 4184 / nAvogadro / 10^5;
	*)
];


(* ::Subsection::Closed:: *)
(*Potential Energy*)


PotentialEnergyCalculations[] := Module[{},
	interEnergy1 = If[nvecList[[#,1]] != 0, interEnergyList[[#,1]] / nvecList[[#,1]], 0.] &/@ Range[Length@nvecList];
	interEnergy2 = If[nvecList[[#,2]] != 0, interEnergyList[[#,2]] / nvecList[[#,2]], 0.] &/@ Range[Length@nvecList];

	intraEnergy1 = If[nvecList[[#,1]] != 0, intraEnergyList[[#,1]] / nvecList[[#,1]], 0.] &/@ Range[Length@nvecList];
	intraEnergy2 = If[nvecList[[#,2]] != 0, intraEnergyList[[#,2]] / nvecList[[#,2]], 0.] &/@ Range[Length@nvecList];

	tailEnergy1 = If[nvecList[[#,1]] != 0, tailCorrEnergyList[[#,1]] / nvecList[[#,1]], 0.] &/@ Range[Length@nvecList];
	tailEnergy2 = If[nvecList[[#,2]] != 0, tailCorrEnergyList[[#,2]] / nvecList[[#,2]], 0.] &/@ Range[Length@nvecList];
];


(* ::Subsection::Closed:: *)
(*Mole Fractions*)


MoleFractionsCalculations[] := Module[{},
	moleFracBox1 = If[Total[#] != 0, # / Total[#], Table[0., Length@#]] &/@ nCompVecList[[All,1]];
	moleFracBox2 = If[Total[#] != 0, # / Total[#], Table[0., Length@#]] &/@ nCompVecList[[All,2]];
];



(* ::Subsection::Closed:: *)
(*Number Density*)


NumberDensityCalculations[] := Module[{},

	(*since the total volume changes at const. pressure ensembles we normalize with the
	total volume at each cycle*)
	VvecTotal = Total@Transpose@VvecList;
	nvecListTotal = Total@Transpose@nvecList;

	(*workaround: replace zero values with the median value*)
	VvecTotal = VvecTotal /. 0 -> Median@VvecTotal;
	nvecListTotal = nvecListTotal /. 0 -> Median@nvecListTotal;
	
	(*ratio of values per box*)
	shareNvec = nvecList[[All,#]]/nvecListTotal &/@ boxVec;
	shareVvec = VvecList[[All,#]]/VvecTotal &/@ boxVec;
	
	numberDensity1 = nvecList[[All,1]] / VvecList[[All,1]];
	numberDensity2 = nvecList[[All,2]] / VvecList[[All,2]];
];


(* ::Subsection::Closed:: *)
(*Molar Volume*)


MolarVolumeCalculations[] := Module[{meanMolarVolume1, meanMolarVolume2},
	(*
	/ 10^30 \[Rule] convert volume \[CapitalARing]^3 to m^3
	* 1000 \[Rule] convert m^3 to dm^3
	* nAvogadro \[Rule] convert number of molecules to mol
	*)

	molarVolume1 = If[nvecList[[#,1]] != 0, VvecList[[#,1]] / nvecList[[#,1]], 0.] &/@ Range[Length@nvecList] / 10^30 * nAvogadro * 1000;
	molarVolume2 = If[nvecList[[#,2]] != 0, VvecList[[#,2]] / nvecList[[#,2]], 0.] &/@ Range[Length@nvecList] / 10^30 * nAvogadro * 1000;
	
	(*workaround to prevent DIV/0: replace zero values with the median value*)
	meanMolarVolume1 = Mean@molarVolume1;
	meanMolarVolume2 = Mean@molarVolume2;
	molarVolume1 = If[# > 10^(-15), #, meanMolarVolume1] &/@ molarVolume1;
	molarVolume2 = If[# > 10^(-15), #, meanMolarVolume2] &/@ molarVolume2;
	
	density1 = 1 /molarVolume1;
	density2 = 1 /molarVolume2;
];


(* ::Subsection::Closed:: *)
(*Chemical Potential*)


ChemicalPotentialCalculations[] := Module[{},
	(*
	defines how many of the last cycles should be accumulated and averaged for the current cycle result
	EXAMPLE: 100 cycles were simulated, blockSize=10 \[Rule] chemical potential for cycle 75 is the average of the cycles 66,67,68,69,70,71,72,73,74,75
	*)
	blockSize = Min[200, Round[nDocuCycles/2]];

	massList = uAtomic * molMass[[#]] &/@ system; (* [kg] *)
	thWavelengthList = hPlanck / Sqrt[2 * Pi * massList * kB * T]; (* [m] *)

	(*
	* 10^30 \[Rule] convert 1/Angstrom^3 to 1/m^3
	*)
	numDensitiesBox1 = nCompVecList[[All,1,#]] / VvecList[[All,1]] * 10^30 &/@ Range@Length@system; (* 1 / m^3 *)
	numDensitiesBox2 = nCompVecList[[All,2,#]] / VvecList[[All,2]] * 10^30 &/@ Range@Length@system; (* 1 / m^3 *)

	(*
	Excess Chemical Potential includes the number density here
	(which is not exactly the excess chemical potential but in the gibbs ensemble the number density
	must be averaged over the trial insertions as well, which would otherwise be included in
	the ideal chemical potential)
	*)

	(* ideal chemical potential *)
	idChPotList = (kB * T * Log[{
		numDensitiesBox1[[#]] * thWavelengthList[[#]]^3, (* Box 1 *)
		numDensitiesBox2[[#]] * thWavelengthList[[#]]^3  (* Box 2 *)
	}] &/@ Range@Length@system) * nAvogadro / 1000; (* [kJ / mol] *)

	(* excess chemical potential *)
	(*
	exChPotList = (- kB * T * Log[{
		Accumulate[muPartialList\[LeftDoubleBracket]All,1,#\[RightDoubleBracket]] / Accumulate[muCounterList\[LeftDoubleBracket]All,1,#\[RightDoubleBracket]], (* Box 1 *)
		Accumulate[muPartialList\[LeftDoubleBracket]All,2,#\[RightDoubleBracket]] / Accumulate[muCounterList\[LeftDoubleBracket]All,2,#\[RightDoubleBracket]]  (* Box 2 *)
	}] &/@ Range@Length@system) * nAvogadro / 1000; (* [kJ / mol] *)
	*)
	exChPotList = (- kB * T * Log[{
		Quiet[BlockAccumulate[muPartialList[[All,1,#]],blockSize] / BlockAccumulate[muCounterList[[All,1,#]],blockSize]], (* Box 1 *)
		Quiet[BlockAccumulate[muPartialList[[All,2,#]],blockSize] / BlockAccumulate[muCounterList[[All,2,#]],blockSize]]  (* Box 2 *)
	}] &/@ Range@Length@system) (* [J * Angstrom^3] *) * nAvogadro / 1000; (* [kJ / mol] *)
	(*Quiet because in case the counter is 0 (no accepted Widom Insertions) -> Potential will also be 0 and result is Indetermined*)

	(* total chemical potential *)
	totChPotList = (
		- kB * T * Log[
			(1 / thWavelengthList[[#]]^3) (* [1/m^3] *) / 10^30 (* [1/Angstrom^3] *) *
			{
				Quiet[BlockAccumulate[muPartialList[[All,1,#]],blockSize] / BlockAccumulate[muCounterList[[All,1,#]],blockSize]], (* Box 1 *)
				Quiet[BlockAccumulate[muPartialList[[All,2,#]],blockSize] / BlockAccumulate[muCounterList[[All,2,#]],blockSize]]  (* Box 2 *)
			}
		]
	) &/@ Range@Length@system (* [J * Angstrom^3] *) * nAvogadro / 1000; (* [kJ / mol] *)
	(*Quiet because in case the counter is 0 (no accepted Widom Insertions) -> Potential will also be 0 and result is Indetermined*)
];


(* ::Subsection::Closed:: *)
(*Execute Calculations*)


ExecuteCalculations[] := Module[{},

	(*ensure number of average cycles for the moving average is reasonable (maximum one quater of the cycles)*)
	nAvgCycles = 1000;
	If[nDocuCycles/4 < nAvgCycles

		,(*too few cycles \[Rule] reduce number of average cycles*)
		nAvgCycles = Round[nDocuCycles/4];
	];
	
	(*zero vector to initilaise variables which are not present since a previous version has been used*)
	zeroVec = Table[0,2];
	
	(*Shading for displaying the evaluated area in the plots*)
	meanRangeCycles = Range[nDocuCycles][[meanRange]];
	prologRatio = {Min@meanRangeCycles,Max@meanRangeCycles}/nDocuCycles;
	If[meanRangeCycles == {}, prologRatio = {1,1}];
	prologDef =  {
		GrayLevel[0.85],
		Rectangle[Scaled[{prologRatio[[1]], 0}], Scaled[{prologRatio[[2]], 1}]]};
	
	(*check if the old system of counters was used for the documentation*)
	If[!ValueQ@counterTransRejeDuDocu,
	
		counterTransRejeDuDocu = counterTransRejectDocu;
		counterTransRejeOvDocu = zeroVec;
	
		counterRotaRejeDuDocu = counterRotaRejectDocu;
		counterRotaRejeOvDocu = zeroVec;
	
		counterInsertRejeDuDocu = counterInsertRejectDocu;
		counterInsertRejeOvDocu = zeroVec;
	
		(*unfortunetely no infomrations on the widom insertions were collected at all*)
	];

	(*check if the new counter for mixtures is present*)
	If[!ValueQ@counterInsertRejeNpDocu,
	
		counterInsertRejeNpDocu = zeroVec;
	];
	
	SetupSpeciesCounters[];
	
	PressureCalculations[];
	PotentialEnergyCalculations[];
	MoleFractionsCalculations[];
	MolarVolumeCalculations[];
	ChemicalPotentialCalculations[];
	NumberDensityCalculations[];

	(*the vapor box is always the first box, thus we establish a vector containing the respective indices*)
	expInd = If[Mean@molarVolume1[[meanRange]] > Mean@molarVolume2[[meanRange]], {1, 2}, {2, 1}];
	
	(*for back compatibility: check if counters for widom insertion are present*)
	If[ValueQ[counterWidomAcceptDocu] && ValueQ[counterWidomRejectDocu],
		(*PRESENT \[Rule] show bar chart*)
		AppendTo[barCharts, {barChartWidomInsertions}];
	];
];


(* ::Section:: *)
(*Plot Definitions*)


(* ::Subsection::Closed:: *)
(*Visual Check*)


plotBoxes[plotSize_] := visualizeBoxesMC[moleculeAtomsCoordVec,moleculeAtomsNamesVec,VvecList[[-1]],plotSize];


plotProbabilityCheck[] := ListPlot[
	Transpose@{shareNvec[[#]], shareVvec[[#]]} &/@ {1,2},
	Frame->True,
	FrameLabel->{"\!\(\*SubscriptBox[\(x\), \(i\)]\) = \!\(\*SubscriptBox[\(n\), \(i\)]\)/\!\(\*SubscriptBox[\(n\), \(Box\)]\)"," \!\(\*SubscriptBox[\(V\), \(i\)]\)/\!\(\*SubscriptBox[\(V\), \(Tot\)]\)" },
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	ImageSize->lImageSize,
	PlotRange->{{0,1},{0,1}}
];
(*Ep[{"plotProbabilityCheckOld",plotProbabilityCheck[]}, True];*)


plotProbabilityCheckV2[] := Show[DensityHistogram[
	Transpose[{shareNvec[[#]], shareVvec[[#]]}],
	Automatic, "Probability",
	PlotRange->{{0,1},{0,1}},
	Frame->True,
	FrameLabel-> {
		Style["\!\(\*SubscriptBox[\(x\), \(i\)]\) = \!\(\*SubscriptBox[\(n\), \(i\)]\) / \!\(\*SubscriptBox[\(n\), \(Box\)]\)", 16, Bold],
		Style["\!\(\*SubscriptBox[\(V\), \(i\)]\) / \!\(\*SubscriptBox[\(V\), \(Tot\)]\)", 16, Bold]},
	ChartLegends->Automatic,
	ImageSize->lImageSize,
	ColorFunction->If[#==1, "BlueGreenYellow", "FuchsiaTones"]] &/@ boxVec];
(*Ep[{"plotProbabilityCheckNew",plotProbabilityCheckV2[]}, True];*)


(* ::Subsection::Closed:: *)
(*Plot - Pressures*)


plotPressure1 := ListLinePlot[
	{press1 + tailPress1, MovingAverage[press1 + tailPress1, nAvgCycles]},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","P [bar]" },
	PlotLabel->Style["Pressure in Box 1 with moving average of 1000 cycles",Bold],
	ImageSize->lImageSize,
	PlotStyle->{Automatic, AbsoluteThickness@5}
];

plotPressure2 := ListLinePlot[
	{press2 + tailPress2, MovingAverage[press2 + tailPress2, nAvgCycles]},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","P [bar]" },
	PlotLabel->Style["Pressure in Box 2 with moving average of 1000 cycles",Bold],
	ImageSize->lImageSize,
	PlotStyle->{Automatic, AbsoluteThickness@5}
];

plotTailCorrPressure := ListLinePlot[
	{
		tailPress1,
		tailPress2
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","Tail Correction [bar]" },
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Tail Correction Pressure",Bold],
	ImageSize->lImageSize
];

plotTailComparisonPressure1 := ListLinePlot[
	{
		press1,
		press1 + tailPress1
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","Pressure [bar]" },
	PlotLegends->{Placed[{"without","with"},Below]},
	PlotLabel->Style["Pressure Tail Correction Comparison Box 1",Bold],
	ImageSize->lImageSize,
	PlotStyle->({Darker[colors[[#]],0.5],colors[[#]]} &@ 1)
];

plotTailComparisonPressure2 := ListLinePlot[
	{
		press2,
		press2 + tailPress2
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","Pressure [bar]" },
	PlotLegends->{Placed[{"without","with"},Below]},
	PlotLabel->Style["Pressure Tail Correction Comparison Box 2",Bold],
	ImageSize->lImageSize,
	PlotStyle->({Darker[colors[[#]],0.5],colors[[#]]} &@ 2)
];


plotPressureHistogram[] := Module[{},
	
	SmoothHistogram[{
			press1[[meanRange]] + tailPress1[[meanRange]],
			press2[[meanRange]] + tailPress2[[meanRange]]
		},
		PlotLabel->"Pressure",
		PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
		ImageSize->lImageSize,
		Frame->True,
		FrameLabel->(Style[#, 14]&/@{"P [bar]","PDF [1]"}),
		PlotRange->Automatic
	]
];


(* ::Subsection::Closed:: *)
(*Plot - Potential Energy*)


plotInterEnergy := ListLinePlot[
	{
		interEnergy1 + tailEnergy1,
		interEnergy2 + tailEnergy2
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","U [kcal/mol]"},
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Intermolecular Potential Energy",Bold],
	ImageSize->lImageSize
];

plotIntraEnergy := ListLinePlot[
	{
		intraEnergy1,
		intraEnergy2
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","U [kcal/mol]"},
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Intramolecular Potential Energy",Bold],
	ImageSize->lImageSize
];

plotTotalEnergy := ListLinePlot[
	{
		interEnergy1 + tailEnergy1 + intraEnergy1,
		interEnergy2 + tailEnergy2 + intraEnergy2
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","U [kcal/mol]"},
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Total Potential Energy",Bold],
	ImageSize->lImageSize
];

plotTailCorrEnergy := ListLinePlot[
	{
		tailEnergy1,
		tailEnergy2
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","Tail Correction [kcal/mol]"},
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Tail Correction Potential Energy",Bold],
	ImageSize->lImageSize
];

plotTailComparisonEnergy1 := ListLinePlot[
	{
		interEnergy1,
		interEnergy1 + tailEnergy1
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","U [kcal/mol]"},
	PlotLegends->{Placed[{"without","with"},Below]},
	PlotLabel->Style["Inter. Pot. Energy Tail Correction Comparison Box 1",Bold],
	ImageSize->lImageSize,
	PlotStyle->({Darker[colors[[#]],0.5],colors[[#]]} &@ 1)
];

plotTailComparisonEnergy2 := ListLinePlot[
	{
		interEnergy2,
		interEnergy2 + tailEnergy2
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","U [kcal/mol]"},
	PlotLegends->{Placed[{"without","with"},Below]},
	PlotLabel->Style["Inter. Pot. Energy Tail Correction Comparison Box 2",Bold],
	ImageSize->lImageSize,
	PlotStyle->({Darker[colors[[#]],0.5],colors[[#]]} &@ 2)
];


(* ::Subsection::Closed:: *)
(*Plot - Mole Fractions*)


plotMoleFractions1 := ListLinePlot[
	Transpose@moleFracBox1,
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","x [mol/mol]"},
	PlotLegends->{Placed[system,Below]},
	PlotLabel->Style["Mole Fractions in Box 1",Bold],
	ImageSize->lImageSize
]


plotMoleFractions2 := ListLinePlot[
	Transpose@moleFracBox2,
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","x [mol/mol]"},
	PlotLegends->{Placed[system,Below]},
	PlotLabel->Style["Mole Fractions in Box 2",Bold],
	ImageSize->lImageSize
]


plotMolarFractions[] := Module[{moleFracBox1, moleFracBox2},

	(*calculation*)
	moleFracBox1 = If[Total[#] != 0, # / Total[#], Table[0., Length@#]] &/@ nCompVecList[[All,1]];
	moleFracBox2 = If[Total[#] != 0, # / Total[#], Table[0., Length@#]] &/@ nCompVecList[[All,2]];
	
	SmoothHistogram[
		Flatten[{
			moleFracBox1[[meanRange,#]],
			moleFracBox2[[meanRange,#]]
		} &/@ Range@Length@system, 1],
		PlotRange->{{0,1},All},
		PlotLabel->"Molar Fractions",
		PlotLegends->{Placed[
			Flatten[{"Box 1, " <> system[[#]],"Box 2, "<> system[[#]]} &/@ Range@Length@system, 1]
			,Below]},
		Frame->True,
		FrameLabel->{"x [mol/mol]","PDF"},
		ImageSize->lImageSize
	]
];


(* ::Subsection::Closed:: *)
(*Plot - Number of Molecules*)


plotNumOfMolecules := ListLinePlot[
	{
		nvecList[[All,1]],
		nvecList[[All,2]]
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","Molecules [1]" },
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Number of Molecules",Bold],
	ImageSize->lImageSize
];


(* ::Subsection::Closed:: *)
(*Plot - Box Volume*)


plotVolume := ListLinePlot[
	{
		VvecList[[All,1]],
		VvecList[[All,2]]
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","Volume [\[CapitalARing]^3]" },
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Box Volume",Bold],
	ImageSize->lImageSize
];


(* ::Subsection::Closed:: *)
(*Plot - Molar Volume*)


plotMolarVolume := ListLinePlot[
	{
		molarVolume1,
		molarVolume2
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","molar volume [dm\.b3/mol]" },
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Molar Volume",Bold],
	ImageSize->lImageSize
];


plotMolarVolumeStd[avgCycles_, list_, name_] := ListLinePlot[
	{BlockStdDeviation[list, avgCycles]},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","std(molar volume) [dm\.b3/mol]" },
	PlotLegends->{Placed[{name},Below]},
	PlotLabel->Style["Molar Volume",Bold],
	ImageSize->lImageSize
]


plotAllMolarVolumeStd[avgCycles_] := ExportGridGraphic[{
	{"transMax - Box 1", plotMolarVolumeStd[avgCycles, molarVolume1, "Box 1"]},
	{"transMax - Box 1", plotMolarVolumeStd[avgCycles, molarVolume2, "Box 2"]}
	}, False]


(* ::Subsection::Closed:: *)
(*Plot - Density*)


plotMolarDensity[] := Module[{}, 

	ListLinePlot[{
			density1,
			density2
		},
		Prolog -> prologDef,
		Frame->True,
		FrameLabel->{"Iterations","molar density [mol/dm\.b3]" },
		PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
		PlotLabel->Style["Molar Density",Bold],
		ImageSize->(*Scaled[0.45]*)Medium,
		PlotRange->{Automatic, {0, 40}}
	]
];


plotDensityHistogram[] := Module[{},
	
	SmoothHistogram[{
			density1,
			density2
		},
		PlotLabel->"Box Density",
		PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
		ImageSize->lImageSize,
		Frame->True,
		FrameLabel->{"Density [mol/\!\(\*SuperscriptBox[\(dm\), \(3\)]\)]","PDF"},
		PlotRange->Full
	]
];


(* ::Subsection::Closed:: *)
(*Plot - Number Density*)


plotDensityTotal := ListLinePlot[
	{
		numberDensity1, numberDensity2
	},
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","Number Density [1/\!\(\*SuperscriptBox[\(\[CapitalARing]\), \(3\)]\)]" },
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Number Density Total (both Components)",Bold],
	ImageSize->lImageSize
];


(* ::Subsection::Closed:: *)
(*Plot - Chemical Potentials*)


plotIdealChemPotential := ListLinePlot[
	idChPotList[[#]],
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","\!\(\*SuperscriptBox[SubscriptBox[\(\[Mu]\), \(1\)], \(id\)]\) [kJ/mol]"},
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Ideal Chemical Potential of "<>system[[#]],Bold],
	ImageSize->lImageSize
] &/@ Range@Length@system;


plotExcessChemPotential := ListLinePlot[
	exChPotList[[#]],
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","\!\(\*SuperscriptBox[SubscriptBox[\(\[Mu]\), \(1\)], \(E\)]\) [kJ/mol]"},
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["(not really) Excess Chemical Potential of "<>system[[#]],Bold],
	ImageSize->lImageSize
] &/@ Range@Length@system;


plotTotalChemPotential := ListLinePlot[
	totChPotList[[#]],
	Prolog -> prologDef,
	PlotRange->{plotRange,vertRange},
	Frame->True,
	FrameLabel->{"Iterations","\!\(\*SubscriptBox[\(\[Mu]\), \(1\)]\) [kJ/mol]"},
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	PlotLabel->Style["Total Chemical Potential of "<>system[[#]],Bold],
	ImageSize->lImageSize
] &/@ Range@Length@system;


(* ::Subsection::Closed:: *)
(*Trial Move Counters*)


(* ::Subsubsection::Closed:: *)
(*Total Move Counters*)


barChartTranslations := BarChart[
	Map[Total,{
		{counterTransAcceptDocu[[1]],counterTransRejeOvDocu[[1]],counterTransRejeDuDocu[[1]]},
		{counterTransAcceptDocu[[2]],counterTransRejeOvDocu[[2]],counterTransRejeDuDocu[[2]]}
	},{2}],
	PlotLabel->Style["Translations",Bold],
	ChartLegends->Placed[{"Successful","Rej. OV","Rej. \[CapitalDelta]U"},Below],
	ChartStyle->{colors[[3]],colors[[2]],colors[[4]]},
	ChartLabels->{{"Box 1","Box 2"},None},
	LabelingFunction->Above,
	ImageSize->lImageSize
];


barChartRotations := BarChart[
	Map[Total,{
		{counterRotaAcceptDocu[[1]],counterRotaRejeOvDocu[[1]],counterRotaRejeDuDocu[[1]]},
		{counterRotaAcceptDocu[[2]],counterRotaRejeOvDocu[[2]],counterRotaRejeDuDocu[[2]]}
	},{2}],
	PlotLabel->Style["Rotations",Bold],
	ChartLegends->Placed[{"Successful","Rej. OV","Rej. \[CapitalDelta]U"},Below],
	ChartStyle->{colors[[3]],colors[[2]],colors[[4]]},
	ChartLabels->{{"Box 1","Box 2"},None},
	LabelingFunction->Above,
	ImageSize->lImageSize
];


barChartVolumeChanges := BarChart[
	{
		{counterVolChangeAcceptDocu[[1]],counterVolChangeRejectDocu[[1]]},
		{counterVolChangeAcceptDocu[[2]],counterVolChangeRejectDocu[[2]]}
	},
	PlotLabel->Style["Volume Changes",Bold],
	ChartLegends->Placed[{"Successful","Rej. \[CapitalDelta]U"},Below],
	ChartStyle->{colors[[3]],colors[[4]]},
	ChartLabels->{
		If[ensembleType == 1,
			(* Gibbs Ensemble with constant total Volume \[Rule] both boxes change volume in one trial move *)
			{"Box 1 Reduction","Box 2 Reduction"},
			(* Gibbs Ensemble with constant Pressure \[Rule] only one box changes its volume in one trial move *)
			{"Box 1 VolChange","Box 2 VolChange"}
		],
		None
	},
	LabelingFunction->Above,
	ImageSize->lImageSize
];


barChartInsertions := BarChart[
	Map[Total,{
		{counterInsertAcceptDocu[[1]],counterInsertRejeOvDocu[[1]],counterInsertRejeDuDocu[[1]], counterInsertRejeNpDocu[[1]]},
		{counterInsertAcceptDocu[[2]],counterInsertRejeOvDocu[[2]],counterInsertRejeDuDocu[[2]], counterInsertRejeNpDocu[[1]]}
	},{2}],
	PlotLabel->Style["Particle Insertions",Bold],
	ChartLegends->Placed[{"Successful","Rej. OV","Rej. \[CapitalDelta]U","Rej. Not Present"},Below],
	ChartStyle->{colors[[3]],colors[[2]],colors[[4]],colors[[5]]},
	ChartLabels->{{"Insertion in Box 1","Insertion in Box 2"},None},
	LabelingFunction->Above,
	ImageSize->lImageSize
];


barChartWidomInsertions := BarChart[
	{
		{counterWidomAcceptDocu[[1]],counterWidomRejectDocu[[1]]},
		{counterWidomAcceptDocu[[2]],counterWidomRejectDocu[[2]]}
	},
	PlotLabel->Style["Widom Insertions",Bold],
	ChartLegends->Placed[{"Successful","Rej. \[CapitalDelta]U"},Below],
	ChartStyle->{colors[[3]],colors[[4]]},
	ChartLabels->{{"Insertion in Box 1","Insertion in Box 2"},None},
	LabelingFunction->Above,
	ImageSize->lImageSize
];


barCharts := {
	{barChartTranslations,barChartRotations},
	{barChartVolumeChanges,barChartInsertions}
};


plotCountersGrid := Grid[
	barCharts,
	ItemSize->Scaled[0.5],
	Alignment->Center
];


(* ::Subsubsection::Closed:: *)
(*Species Specific Move Counters*)


SetupSpeciesCounters[] := If[Depth@counterTransAcceptDocu == 3 && Depth@counterRotaAcceptDocu == 3 && Depth@counterInsertAcceptDocu == 3 && Length@system > 1,
	(*species specific data has been collected*)
	barChartSpeciesTranslations := BarChart[
		{
			{counterTransAcceptDocu[[1,#]],counterTransRejeOvDocu[[1,#]],counterTransRejeDuDocu[[1,#]]},
			{counterTransAcceptDocu[[2,#]],counterTransRejeOvDocu[[2,#]],counterTransRejeDuDocu[[2,#]]}
		},
		PlotLabel->Style[StringForm["Tranlations of component ``",system[[#]]],Bold],
		ChartLegends->Placed[{"Successful","Rej. OV","Rej. \[CapitalDelta]U"},Below],
		ChartStyle->{colors[[3]],colors[[2]],colors[[4]]},
		ChartLabels->{{"Box 1","Box 2"},None},
		LabelingFunction->Above,
		ImageSize->lImageSize
	] &/@ Range[Length@system];
	
	barChartSpeciesRotations := BarChart[
		{
			{counterRotaAcceptDocu[[1,#]],counterRotaRejeOvDocu[[1,#]],counterRotaRejeDuDocu[[1,#]]},
			{counterRotaAcceptDocu[[2,#]],counterRotaRejeOvDocu[[2,#]],counterRotaRejeDuDocu[[2,#]]}
		},
		PlotLabel->Style[StringForm["Rotations of component ``",system[[#]]],Bold],
		ChartLegends->Placed[{"Successful","Rej. OV","Rej. \[CapitalDelta]U"},Below],
		ChartStyle->{colors[[3]],colors[[2]],colors[[4]]},
		ChartLabels->{{"Box 1","Box 2"},None},
		LabelingFunction->Above,
		ImageSize->lImageSize
	] &/@ Range[Length@system];
	
	barChartSpeciesInsertions := BarChart[
		{
			{counterInsertAcceptDocu[[1,#]],counterInsertRejeOvDocu[[1,#]],counterInsertRejeDuDocu[[1,#]], counterInsertRejeNpDocu[[1,#]]},
			{counterInsertAcceptDocu[[2,#]],counterInsertRejeOvDocu[[2,#]],counterInsertRejeDuDocu[[2,#]], counterInsertRejeNpDocu[[1,#]]}
		},
		PlotLabel->Style[StringForm["Particle Insertions of component ``",system[[#]]],Bold],
		ChartLegends->Placed[{"Successful","Rej. OV","Rej. \[CapitalDelta]U","Rej. Not Present"},Below],
		ChartStyle->{colors[[3]],colors[[2]],colors[[4]],colors[[5]]},
		ChartLabels->{{"Insertion in Box 1","Insertion in Box 2"},None},
		LabelingFunction->Above,
		ImageSize->lImageSize
	] &/@ Range[Length@system];
	
	plotSpeciesCountersGrid := Grid[
		{barChartSpeciesTranslations,
		barChartSpeciesRotations,
		barChartSpeciesInsertions},
		ItemSize->Scaled[Min[{1/Length@system, 0.5}]],
		Alignment->Center
	];
	
	, (* else: can/should not plot species specific counters*)
	If[Length@system == 1, (*Only one component in system*)
		plotSpeciesCountersGrid = "No speciese specific plots if only one species is simulated";
		,(* else: no species specific data has been collected*)
		plotSpeciesCountersGrid = "No species specific data has been collected";
	];
]


(* ::Subsubsection::Closed:: *)
(*Intramolecular Trial Move Counters*)


barChartBondStretching := BarChart[
	Map[Total,{
		{counterBondStretchAcceptDocu[[1]],counterBondStretchRejeOvDocu[[1]],counterBondStretchRejeDuDocu[[1]]},
		{counterBondStretchAcceptDocu[[2]],counterBondStretchRejeOvDocu[[2]],counterBondStretchRejeDuDocu[[2]]}
	},{2}],
	PlotLabel->Style["Bond Stretches",Bold],
	ChartLegends->Placed[{"Successful","Rej. OV","Rej. \[CapitalDelta]U"},Below],
	ChartStyle->{colors[[3]],colors[[2]],colors[[4]]},
	ChartLabels->{{"Box 1","Box 2"},None},
	LabelingFunction->Above,
	ImageSize->lImageSize
];


barChartAngleBending := BarChart[
	Map[Total,{
		{counterAngleBendAcceptDocu[[1]],counterAngleBendRejeOvDocu[[1]],counterAngleBendRejeDuDocu[[1]]},
		{counterAngleBendAcceptDocu[[2]],counterAngleBendRejeOvDocu[[2]],counterAngleBendRejeDuDocu[[2]]}
	},{2}],
	PlotLabel->Style["Angle Bends",Bold],
	ChartLegends->Placed[{"Successful","Rej. OV","Rej. \[CapitalDelta]U"},Below],
	ChartStyle->{colors[[3]],colors[[2]],colors[[4]]},
	ChartLabels->{{"Box 1","Box 2"},None},
	LabelingFunction->Above,
	ImageSize->lImageSize
];


barChartTorsion := BarChart[
	Map[Total,{
		{counterTorsionAcceptDocu[[1]],counterTorsionRejeOvDocu[[1]],counterTorsionRejeDuDocu[[1]]},
		{counterTorsionAcceptDocu[[2]],counterTorsionRejeOvDocu[[2]],counterTorsionRejeDuDocu[[2]]}
	},{2}],
	PlotLabel->Style["Torsion",Bold],
	ChartLegends->Placed[{"Successful","Rej. OV","Rej. \[CapitalDelta]U"},Below],
	ChartStyle->{colors[[3]],colors[[2]],colors[[4]]},
	ChartLabels->{{"Box 1","Box 2"},None},
	LabelingFunction->Above,
	ImageSize->lImageSize
];


plotIntraCountersGrid := Grid[
	{
		{barChartBondStretching,barChartAngleBending},
		{barChartTorsion}
	},
	ItemSize->Scaled[0.5],
	Alignment->Center
];


(* ::Subsubsection::Closed:: *)
(*Trial Move Limit Course*)


courseTransMax[iBox_] := ListLinePlot[
	Transpose@transMaxVecDoc[[All, iBox]],
	Prolog -> If[!SameQ[checkGetAssocParameter[custParam, "eqAdjAvrCycles"],False] && custParam["eqAdjAvrCycles"] <= nEquiCycles,
		equAvrCycles = custParam["eqAdjAvrCycles"];
		avrTransMaxVec = Mean@transMaxVecDoc[[nEquiCycles-equAvrCycles+1;;nEquiCycles, iBox]];
		{Line[{{nEquiCycles-equAvrCycles+1,#},{nEquiCycles,#}}] &/@ avrTransMaxVec},{}
	],
	PlotLabel->Style["\!\(\*SubscriptBox[\(dr\), \(Max\)]\) translations, box " <> ToString@iBox,Bold],
	Frame->True,
	PlotRange->{{0,nEquiCycles}, Full},
	FrameLabel->{"Iterations","\!\(\*SubscriptBox[\(dr\), \(Max\)]\)"},
	PlotLegends->{Placed[system,Below]},
	ImageSize->lImageSize
	];
courseRotaMax[iBox_] := ListLinePlot[
	Transpose@rotaMaxVecDoc[[All, iBox]],
	Prolog -> If[!SameQ[checkGetAssocParameter[custParam, "eqAdjAvrCycles"],False] && custParam["eqAdjAvrCycles"] <= nEquiCycles,
		avrRotaMaxVec = Mean@rotaMaxVecDoc[[nEquiCycles-equAvrCycles+1;;nEquiCycles, iBox]];
		{Line[{{nEquiCycles-equAvrCycles+1,#},{nEquiCycles,#}}] &/@ avrRotaMaxVec},{}
	],
	PlotLabel->Style["\!\(\*SubscriptBox[\(d\[Phi]\), \(Max\)]\) rotations, box " <> ToString@iBox,Bold],
	Frame->True,
	PlotRange->{{0,nEquiCycles}, Full},
	FrameLabel->{"Iterations","\!\(\*SubscriptBox[\(d\[Phi]\), \(Max\)]\)"},
	PlotLegends->{Placed[system,Below]},
	ImageSize->lImageSize
	];
courseVolMax[] := ListLinePlot[
	Transpose@volMaxDoc,
	Prolog -> If[!SameQ[checkGetAssocParameter[custParam, "eqAdjAvrCycles"],False] && custParam["eqAdjAvrCycles"] <= nEquiCycles,
		avrVolMaxVec = Mean@volMaxDoc[[nEquiCycles-equAvrCycles+1;;nEquiCycles]];
		{Line[{{nEquiCycles-equAvrCycles+1,#},{nEquiCycles,#}}] &/@ avrVolMaxVec},{}
	],
	PlotLabel->Style["\!\(\*SubscriptBox[\(dV\), \(Max\)]\) volume change",Bold],
	Frame->True,
	PlotRange->{{0,nEquiCycles}, Full},
	FrameLabel->{"Iterations","\!\(\*SubscriptBox[\(dV\), \(Max\)]\)"},
	PlotLegends->{Placed[{"Box 1","Box 2"},Below]},
	ImageSize->lImageSize
	];


(*prepare the grid function*)
trialMoveLimitsPlots := {
	{courseTransMax, courseRotaMax},
	{courseVolMax}
};


histoTransMax[box_, cp_] := Module[{},
	returns = transMaxVecDoc[[1;;nEquiCycles, box, cp]];
	{bins,heights}=HistogramList[returns, Automatic,"Probability"];
	maxFreq=Max[heights];
	
	Histogram[
		returns,
		Automatic, Probability,
		PlotLabel->Style["\!\(\*SubscriptBox[\(dr\), \(Max\)]\) translations - Box " <> ToString@box <> ", " <> system[[cp]],Bold],
		Frame->True,
		FrameLabel->{"\[CapitalDelta]max / -","Probability / -"},
		Epilog->{Thick,Darker[Green], Line[{{returns[[-1]],0},{returns[[-1]],maxFreq}}]},
		ImageSize->lImageSize
	]
];(*histoTransMax[1]*)


histoRotaMax[box_, cp_] := Module[{},
	returns = rotaMaxVecDoc[[1;;nEquiCycles, box, cp]];
	{bins,heights}=HistogramList[returns, Automatic, "Probability"];
	maxFreq=Max[heights];
	
	Histogram[
		returns,
		Automatic, Probability,
		PlotLabel->Style["\!\(\*SubscriptBox[\(d\[Phi]\), \(Max\)]\) rotations - Box " <> ToString@box <> ", " <> system[[cp]],Bold],
		Frame->True,
		FrameLabel->{"\[CapitalDelta]max / -","Probability / -"},
		Epilog->{Thick,Darker[Green], Line[{{returns[[-1]],0},{returns[[-1]],maxFreq}}]},
		ImageSize->lImageSize
	]
];(*histoRotaMax[1]*)


histoVolMax[box_] := Module[{},
	returns = volMaxDoc[[1;;nEquiCycles, box]];
	{bins,heights}=HistogramList[returns, Automatic,"Probability"];
	maxFreq=Max[heights];
	
	Histogram[
		returns,
		Automatic, Probability,
		PlotLabel->Style["\!\(\*SubscriptBox[\(dV\), \(Max\)]\) volume change - Box " <> ToString@box,Bold],
		Frame->True,
		FrameLabel->{"\[CapitalDelta]max / -","Probability / %"},
		Epilog->{Thick,Darker[Green], Line[{{returns[[-1]],0},{returns[[-1]],maxFreq}}]},
		ImageSize->lImageSize
	]
];

(*histoVolMax[1]*)


histoTrialMovesSwaps[] := Module[{},
	returns = nMovesDocu[[1;;nEquiCycles, 4]];
	If[returns == {},(*no equilibration cycles*)
		returns = {nMovesPerCycles[[4]]};
	];
	{bins,heights}=HistogramList[returns, Automatic,"Probability"];
	maxFreq=Max[heights];
	
	Histogram[
		returns,
		Automatic, Probability,
		PlotLabel->Style["Number of attempted Swaps",Bold],
		Frame->True,
		FrameLabel->{"\!\(\*SubscriptBox[\(N\), \(Swaps\)]\) / -","Probability / -"},
		Epilog->If[Length@returns == 0, {},
			{Thick,Darker[Green], Line[{{returns[[-1]],0},{returns[[-1]],maxFreq}}]}],
		ImageSize->lImageSize
	]
];

pltCourseTrialMoves[] :=ListLinePlot[
	Transpose@nMovesDocu[[All,1;;5]],
	Frame->True,FrameLabel->{"cycles / 1", "n / 1"},
	PlotLegends->Placed[{"Trans.","Rot.", "dV", "Swaps", "Widom"}, Below],
	ImageSize->lImageSize,
	PlotRange->Full];
	
(*histoTransMax[1]*)
(*pltCourseTrialMoves[]*)


(* ::Subsection::Closed:: *)
(*CPU Time Usage*)


pltTimingCPUTotal[] := ListLinePlot[
	(timingCPUList-timingCPUList[[1]]) (*s*) /60 (*min*) /60 (*h*),
	Frame->True,FrameLabel->{"cycles / 1", "total CPU time / h"},
	ImageSize->lImageSize,
	PlotRange->Full]


pltTimingCPUperCycle[] := ListLinePlot[
	(timingCPUList[[2;;-1]]-timingCPUList[[1;;-2]]) (*s*),
	Frame->True,FrameLabel->{"cycles / 1", "CPU time per cycle / seconds"},
	ImageSize->lImageSize,
	PlotRange->Full]


(* ::Section:: *)
(*Ensemble Averages & Comparison*)


(* ::Subsection::Closed:: *)
(*Grid Function*)


showAveragedValues := SetPrecision[Grid[
	{
		{Style["AVERAGED VALUES",Bold,FontSize->16],SpanFromLeft,SpanFromLeft},
		{"averaged from "<>ToString[meanRange[[1]]]<>" to "<>ToString[meanRange[[2]]],SpanFromLeft,SpanFromLeft},
		{"Box V corresponds to Box " <> ToString@expInd[[1]] <> ", Box L corresponds to Box " <> ToString@expInd[[2]],SpanFromLeft,SpanFromLeft},
		Style[#,Bold] &/@ {"","Box V","","Box L"},
		{Style["System", Bold], "mean", "std", "mean", "std"},
		
		{"T [K]",NumToStr@T,"-", NumToStr@T, "-"},
		MeanStdStr["v [dm^3 / mol]", meanRange, {molarVolume1, molarVolume2}],
		MeanStdStr["rho [mol / dm^3]", meanRange, {1/molarVolume1, 1/molarVolume2}],
		MeanStdStr["n [1]", meanRange, Transpose@nvecList],
		
		{Style["Pressure", Bold]},
		MeanStdStr["P ideal [bar]", meanRange, {pressIdeal1, pressIdeal2}],
		MeanStdStr["P viral [bar]", meanRange, {pressViral1, pressViral2}],
		MeanStdStr["P tail [bar]", meanRange, {tailPress1, tailPress2}],
		MeanStdStr["P [bar]", meanRange, {press1+tailPress1, press2+tailPress2}],
		
		{Style["Internal Energy", Bold]},
		MeanStdStr["U Inter [kcal/mol]", meanRange, 
			{interEnergy1 + tailEnergy1, interEnergy2 + tailEnergy2}],
		MeanStdStr["U Intra [kcal/mol]", meanRange,
			{intraEnergy1, intraEnergy2}],
		MeanStdStr["U Total [kcal/mol]", meanRange, 
			{interEnergy1 + tailEnergy1 + intraEnergy1, interEnergy2 + tailEnergy2 + intraEnergy2}],
		
		{Style["Mole Fractions", Bold]},
		MeanStdSys["Mole Fractions", system, meanRange, 
			{moleFracBox1, moleFracBox2}],
		
		{Style["Ideal \[Mu] [kJ/mol]", Bold]},
		MeanStdSys["Ideal \[Mu]", system, meanRange, 
			{Transpose@idChPotList[[All,1,All]], Transpose@idChPotList[[All,2,All]]}],
		
		{Style["Excess \[Mu] [kJ/mol]", Bold]},
		MeanStdSys["Excess \[Mu]", system, meanRange, -
			{Transpose@exChPotList[[All,1,All]], Transpose@exChPotList[[All,2,All]]}],
		
		{Style["Total \[Mu] [kJ/mol]", Bold]},
		MeanStdSys["Total \[Mu]", system, meanRange,
			{Transpose@totChPotList[[All,1,All]], Transpose@totChPotList[[All,2,All]]}]
	},
	Alignment->{{Left,Center,Center}},
	Dividers->{
		{True, True, False, True, False, True},
		{
			True,False, False,
			True, False,
			True, False, False, False,
			True, False, False, False, False,
			True, False, False, False, True, False, 
			True, False, 
			True, False, 
			True, False, True}},
	Spacings->{3,1.2}
],5];


(* ::Subsection::Closed:: *)
(*Reduced Units*)


(* ::Subsubsection::Closed:: *)
(*Setup*)


AssociateToBoxes[key_, boxValues_] := Module[{},

	AssociateTo[normalizedUnits, {key <> " V" -> boxValues[[1]], key <> " L" -> boxValues[[2]]}];
];


getNormalizedData[component_] := Module[{\[Sigma], \[Epsilon], tCrit, vCrit, pCrit},

	(*check if physical property data exists for this component*)
	If[KeyExistsQ[physicalPropertyData, component],
		(*True: take critical data from physicalPropertyData*)
		tCrit = physicalPropertyData[component,"tCrit"]; (* K *)
		vCrit = physicalPropertyData[component,"vCrit"] (*dm^3/mol*);
		pCrit = physicalPropertyData[component,"pCrit"] (*bar*);
	
		, (*FALSE \[Rule] calculate critical data from reduced values Thol2016*)
		{\[Sigma](*[\[Angstrom]]*),\[Epsilon](*[kcal mol^-1]*)} = oplsNonBonded[ffLabelsNonBonded[[component]][[1]]][[{2,3}]];
		tCrit = 1.32*\[Epsilon]/kB (**)/nAvogadro*4184(* K *);
		vCrit = ((0.31/\[Sigma]^3)(* N/A^3 *))^-1 *nAvogadro*10^-27 (* dm^3/mol *);
		pCrit =  0.13006 (*Thol, 2016*)
			/(\[Sigma] (*A*) *10^(-10) (*m*))^3 (*m3)*)
			*\[Epsilon] (*kcal/mol*) *kCalToJ (*J /mol*) /nAvogadro (*J /1 \[Rule] Pa*) /10 ^5 (*bar / Pa*);
	];
	
	(*return*)
	{tCrit, vCrit, pCrit}
];
(*getNormalizedData[system[[1]]]*)


NormalizedUnits[] := Module[{
		\[Sigma], \[Epsilon],
		\[Sigma]ij, \[Sigma]m, xVec, yVec,
		tCrit, vCrit, pCrit,
		rho, p, mu,
		tRed, rhoRed, pRed, muRed,
		tStar, rhoStar, pStar, muStar,
		gridRange
	},
	
	normalizedUnits = <||>;

	{tCrit, vCrit, pCrit} = getNormalizedData[system[[1]]];
	\[Sigma](*[\[Angstrom]]*) = oplsNonBonded[ffLabelsNonBonded[[#]][[1]]][[2]] &/@ system;
	\[Epsilon](*[kcal mol^-1]*) = oplsNonBonded[ffLabelsNonBonded[[#]][[1]]][[3]] &/@ system;
	
	Switch[checkGetAssocParameter[custParam,"ffParamCombiningRule"]
		,"Lorentz-Berthelot",
			\[Sigma]ij = Table[(\[Sigma][[i]]+\[Sigma][[j]])/2,{i,Length@system},{j,Length@system}];
		,_, (*Default*)
			\[Sigma]ij = Table[(\[Sigma][[i]]*\[Sigma][[j]])^0.5,{i,Length@system},{j,Length@system}];
	];

	\[Sigma]m[xVec_] := (Sum[Sum[xVec[[i]]*xVec[[j]]*\[Sigma]ij[[i,j]]^3,{i,Length@system}],{j,Length@system}])^(1/3);

	xVec = {assocResults[["Mole Fractions Box L "<>ToString@system[[1]]<>" mean"]],
			1-assocResults[["Mole Fractions Box L "<>ToString@system[[1]]<>" mean"]]};
	yVec = {assocResults[["Mole Fractions Box V "<>ToString@system[[1]]<>" mean"]],
			1-assocResults[["Mole Fractions Box V "<>ToString@system[[1]]<>" mean"]]};

	rho = {assocResults[["rho [mol / dm^3] Box V mean"]], assocResults[["rho [mol / dm^3] Box L mean"]]};
	p = {assocResults[["P [bar] Box V mean"]], assocResults[["P [bar] Box L mean"]]};
	mu = {
		assocResults[["Total \[Mu] Box V " <> system[[1]] <> " mean"]],
		assocResults[["Total \[Mu] Box L " <> system[[1]] <> " mean"]]};
	
	tRed = T /tCrit;
	rhoRed = rho *vCrit;
	pRed = p /pCrit;
	
	If[Max[Length@# &/@ Values@moleculeData] == 1,
		(*Only single atom components are in the system*)
		tStar = T /(\[Epsilon][[1]]/kB (**)/nAvogadro*4184);(* K *)
		rhoStar = rho (*mol/dm3*) *nAvogadro (*1/dm3*) *10^(-27) (*dm3/A3)*) *{\[Sigma]m[yVec],\[Sigma]m[xVec]}^3;
		pStar =  p (*bar*) *10 ^5 (*Pa / bar*)
				*(\[Sigma][[1]] (*A*) *10^(-10) (*m*))^3 (*m3)*)
				/(\[Epsilon][[1]] (*kcal/mol*) *kCalToJ (*J /mol*) /nAvogadro (*J /1*));
		muStar = mu (*kJ / mol*) *1000 (*J/mol*)
				/(\[Epsilon][[1]] (*kcal/mol*) *kCalToJ (*J /mol*));
		gridRange = All;
		
		,(*else*)
		tStar = "-";
		rhoStar = pStar=  muStar = {"-","-"};
		gridRange = 1;;-3;
	];

	AssociateTo[normalizedUnits, {"tRed" -> tRed}];
	AssociateToBoxes["rhoRed", rhoRed];
	AssociateToBoxes["pRed", pRed];
	
	AssociateTo[normalizedUnits, {"tStar" -> tStar}];
	AssociateToBoxes["rhoStar", rhoStar];
	AssociateToBoxes["pStar", pStar];
	AssociateToBoxes["muStar", muStar];

	Print@SetPrecision[Grid[
		{
			Style[#,Bold] &/@ {"name", "dimensional", "unit", "critical norm", "", "LJ norm", ""},
			{"T", T, "K", "\!\(\*SubscriptBox[\(T\), \(red\)]\)=T/\!\(\*SubscriptBox[\(T\), \(C\)]\)", tRed, "\!\(\*SuperscriptBox[\(T\), \(*\)]\) = \!\(\*SubscriptBox[\(k\), \(B\)]\)T/\[Epsilon]", tStar},
			{"\!\(\*SubscriptBox[\(p\), \(V\)]\)", p[[1]], "bar", "\!\(\*SubscriptBox[\(p\), \(red\)]\)=p/\!\(\*SubscriptBox[\(p\), \(C\)]\)", pRed[[1]], "\!\(\*SuperscriptBox[\(p\), \(*\)]\)=p * \!\(\*SuperscriptBox[\(\[Sigma]\), \(3\)]\)/\[Epsilon]", pStar[[1]]},
			{"\!\(\*SubscriptBox[\(p\), \(L\)]\)", p[[2]], "bar", "", pRed[[2]], "", pStar[[2]]},
			{"\!\(\*SubscriptBox[\(\[Rho]\), \(V\)]\)", rho[[1]], "mol / dm^3",  "\!\(\*SubscriptBox[\(\[Rho]\), \(red\)]\)=\[Rho]/\!\(\*SubscriptBox[\(\[Rho]\), \(C\)]\)",  rhoRed[[1]], "\!\(\*SuperscriptBox[\(\[Rho]\), \(*\)]\)=\[Rho] * \!\(\*SuperscriptBox[\(\[Sigma]\), \(3\)]\)", rhoStar[[1]]},
			{"\!\(\*SubscriptBox[\(\[Rho]\), \(L\)]\)", rho[[2]], "mol / dm^3",  "", rhoRed[[2]], "", rhoStar[[2]]},
			{"\!\(\*SubscriptBox[\(\[Mu]\), \(V\)]\)", mu[[1]], "kJ / mol",  "\!\(\*SubscriptBox[\(\[Mu]\), \(red\)]\)=\[Mu] /\!\(\*SubscriptBox[\(\[Mu]\), \(C\)]\)",  "-", "\!\(\*SuperscriptBox[\(\[Mu]\), \(*\)]\)=\[Mu] /\[Epsilon]", muStar[[1]]},
			{"\!\(\*SubscriptBox[\(\[Mu]\), \(L\)]\)", mu[[2]], "kJ / mol",  "",  "-", "", muStar[[2]]}
		}[[All,gridRange]],
		Dividers->{False,  {True, True, False, False, False, False, False, False, True}},
		Alignment->{{Left}},
		Spacings->{2, 1},
		ItemSize->Full], 3]
];


(* ::Subsection::Closed:: *)
(*Equilibrate State*)


Showcheckfinalvalues := Module[{},
	
	checkfinalvalues := CheckingSimulationResults[];
	simulationChecksAssoc = <|
		"min 10 particles gas"->checkfinalvalues[[1]], 
			"min 200 particles liquid"->checkfinalvalues[[2]], 
		"gas partilces makes 20% of the total particles"->checkfinalvalues[[3]], 
		"Accepted insertions 10 times or more over equilibration period"->checkfinalvalues[[4]], 
		"Accepted insertions 10 times or more over production period"->checkfinalvalues[[5]]
	|>;

	Print@Grid[
	{		
		{Style["Checking minimum particle and length of the simulation",Bold],SpanFromLeft},
		{"min 10 particles in gas phase", Style[visualizeCheck[checkfinalvalues[[1]]],FontSize->36]},
		{"min 200 particles in liquid phase", Style[visualizeCheck[checkfinalvalues[[2]]],FontSize->36]},
		{"gas partilces make up more than 20% of the total particles", Style[visualizeCheck[checkfinalvalues[[3]]],FontSize->36]},
		{"Accepted insertions 10 times or more over equilibration period",Style[visualizeCheck[checkfinalvalues[[4]]],FontSize->36]},
		{"Accepted insertions 10 times or more over production period", Style[visualizeCheck[checkfinalvalues[[5]]],FontSize->36]}
	},
	Dividers->{All, All},
	Alignment->{{Left,Left}},
	Spacings->{3, 2},
	ItemSize->Full
	];
];


(* ::Subsection::Closed:: *)
(*Checking Functions*)


CheckingSimulationResults::usage = "This function checks if there are a minimum amount of partilces in the phases,
as well as the overall length of the the simulation according to the literature. ";
CheckingSimulationResults[] := Module[{c1, c2, c3, c4, c5, AcceptInsertPerCycle},

	AcceptInsertPerCycle = Total[counterInsertAcceptDocu,2]/nDocuCycles;

	(*Check if enough particles are in the gas phase*)
	c1 = Mean@nvecList[[meanRange,expInd[[1]]]] >= 10;

	(*Check if enough particles are in the liquid phase*)
	c2 = Mean@nvecList[[meanRange,expInd[[2]]]] >= 200;

	(*Check if enough particles are in the gas phase*)
	c3 = Mean@nvecList[[meanRange,expInd[[1]]]] / (Mean@nvecList[[meanRange,expInd[[1]]]] + Mean@nvecList[[meanRange,expInd[[2]]]]) >= 0.2;

	(*Check if enough accepted Insertions are made during the equilibrium phase*)
	c4 = AcceptInsertPerCycle*nEquiCycles >= 10*Total@nvec;

	(*Check if enough accepted Insertions are made during the equilibrium phase*)
	c5 = AcceptInsertPerCycle*nProdCycles >= 10*Total@nvec;

	(*Return*)
	{c1 ,c2 ,c3 ,c4 ,c5}
];


(* ::Section:: *)
(*Test specific evaluations*)


(* ::Subsection::Closed:: *)
(*CFC Vlugt*)


(* ::Subsubsection::Closed:: *)
(*Setup - retrieve Parameters*)


setupCfcParameters[] := Module[{},
	(*check the custom paramter settings and initialise the default values*)
	If[KeyExistsQ[custParam, "considerHistogramBiasing"]

		,(*TRUE \[Rule] parameter is set*)
		considerHistogramBiasing = custParam[["considerHistogramBiasing"]];

		,(*FALSE \[Rule] parameter is not set, use biasing by default*)
		considerHistogramBiasing = True;
	];
	If[KeyExistsQ[custParam, "numberBins"],
	
		(*TRUE \[Rule] parameter is set*)
		numberBins = custParam[["numberBins"]];
		
		, (*FALSE \[Rule] parameter is not set, set to 10 as suggested by (Shi, 2008)*)
		numberBins = 10;
	];
	If[KeyExistsQ[custParam, "interpolationOrder"],
	
		(*TRUE \[Rule] parameter is set*)
		interpolationOrder = custParam[["interpolationOrder"]];
		
		, (*FALSE \[Rule] parameter is not set, set to 0.01 (1%) as suggested by (Shi, 2008)*)
		interpolationOrder = 3;
	];
	If[KeyExistsQ[custParam, "useBoundaryBins"],
	
		(*TRUE \[Rule] parameter is set*)
		useBoundaryBins = custParam[["useBoundaryBins"]];
		
		, (*FALSE \[Rule] parameter is not set, set to True to avoid divergence of the interpolation*)
		useBoundaryBins = True;
	];
];


(* ::Subsubsection::Closed:: *)
(*Timing*)


timingTableCfcVlugt[] := Module[{totalLambdaChangeTime,totalExchangeTime,totalSwapTime,totalCfcTime},
	
	totalLambdaChangeTime = Total@Values[evalTime[[{"cfcLambdaChange-Accept","cfcLambdaChange-Reject"}]]];
	totalExchangeTime = Total@Values[evalTime[[{"cfcExchange-Accept","cfcExchange-Reject"}]]];
	totalSwapTime = Total@Values[evalTime[[{"cfcSwap-Accept","cfcSwap-RejeDu","cfcSwap-RejeOv"}]]];
	If[totalLambdaChangeTime + totalExchangeTime + totalSwapTime != 0, (*to prevent division by zero*)
		totalCfcTime = totalLambdaChangeTime + totalExchangeTime + totalSwapTime;
		,totalCfcTime = 1
	];
	
	Grid[
		{
			{Style["CFC Vlugt Trial Move Timing",Bold],SpanFromLeft},
			{"Part", "Time / total", "\!\(\*SubscriptBox[\(t\), \(total\)]\) / %", "\!\(\*SubscriptBox[\(t\), \(avr\)]\) / ms","\!\(\*SubscriptBox[\(t\), \(acc\)]\) / ms","\!\(\*SubscriptBox[\(t\), \(rejDu\)]\) / ms", "\!\(\*SubscriptBox[\(t\), \(rejOv\)]\) / ms"},
			{"Lambda Change",
				SecondsToDurationString[totalLambdaChangeTime],
				Round[totalLambdaChangeTime / totalCfcTime * 100, 0.01],
				totalLambdaChangeTime / Total[{
					counterCfcAcceptLambdaChangeDocu, counterCfcRejectLambdaChangeDocu
				},-1] * 1000,
				evalTime[["cfcLambdaChange-Accept"]] / Total[counterCfcAcceptLambdaChangeDocu,-1] *1000,
				evalTime[["cfcLambdaChange-Reject"]] / Total[counterCfcRejectLambdaChangeDocu,-1] *1000,
				"-"
			},
			{"Exchange",
				SecondsToDurationString[totalExchangeTime],
				Round[totalExchangeTime / totalCfcTime *100, 0.01],
				totalExchangeTime / Total[{
					counterCfcAcceptExchangeDocu, counterCfcRejectExchangeDocu
				},-1] * 1000,
				evalTime[["cfcExchange-Accept"]] / Total[counterCfcAcceptExchangeDocu,-1] *1000,
				evalTime[["cfcExchange-Reject"]] / Total[counterCfcRejectExchangeDocu,-1] *1000,
				"-"
			},
			{"Swap",
				SecondsToDurationString[totalSwapTime],
				Round[totalSwapTime / totalCfcTime *100, 0.01],
				totalSwapTime / Total[{
					counterCfcAcceptSwapDocu, counterCfcRejectSwapEnergyDocu, counterCfcRejectSwapOverlapDocu
				},-1] * 1000,
				evalTime[["cfcSwap-Accept"]] / Total[counterCfcAcceptSwapDocu,-1] *1000,
				evalTime[["cfcSwap-RejeDu"]] / Total[counterCfcRejectSwapEnergyDocu,-1] *1000,
				evalTime[["cfcSwap-RejeOv"]] / Total[counterCfcRejectSwapOverlapDocu,-1] *1000
			}
		}
		,Dividers->{All,{True,True,True,{False},True}}
		,Spacings->{1.5, 2},ItemSize->Full
	]
];


(* ::Subsubsection::Closed:: *)
(*Counts*)


barChartCfcVlugtLamdaChange[component_] := BarChart[
	{counterCfcAcceptLambdaChangeDocu[[#, component]], counterCfcRejectLambdaChangeDocu[[#, component]]} &/@ boxVec,
	PlotLabel->Style["\[Lambda] Change, " <> system[[component]],Bold],
	ChartLegends->Placed[{
		"Accepted",
		"Rejected"
		},Below],
	ChartStyle->{colors[[3]],colors[[4]]},
	ChartLabels->{{"Box 1","Box 2"},None},
	LabelingFunction->Above,
	ImageSize->Scaled[1]
];
(*barChartCfcVlugtLamdaChange[1]*)


barChartCfcVlugtSwap[component_] := BarChart[
	{counterCfcAcceptExchangeDocu[[#, component]], counterCfcRejectExchangeDocu[[#, component]]} &/@ boxVec,
	PlotLabel->Style["CFC-Particle Change, " <> system[[component]],Bold],
	ChartLegends->Placed[{
		"Accepted",
		"Rejected"
		},Below],
	ChartStyle->{colors[[3]],colors[[4]]},
	ChartLabels->{{"Box 1","Box 2"},None},
	LabelingFunction->Above,
	ImageSize->Scaled[1]
];
(*barChartCfcVlugtSwap[1]*)


barChartCfcExchange[component_] := BarChart[
	{
		counterCfcAcceptSwapDocu[[#, component]],
		counterCfcRejectSwapOverlapDocu[[#, component]],
		counterCfcRejectSwapEnergyDocu[[#, component]]
	} &/@ boxVec,
	PlotLabel->Style["CFC Swap, " <> system[[component]],Bold],
	ChartLegends->Placed[{
		"Accepted", (*accepted \[Rule] sucessful moleucle exchages*)
		"Rej. Ov",  (*overlap*)
		"Rej. \[CapitalDelta]U"  (*metropolis check \[Rule] energy*)
		},Below],
	ChartStyle->{colors[[3]],colors[[2]],colors[[4]],colors[[1]]},
	ChartLabels->{{"Box 1","Box 2"},None},
	LabelingFunction->Above,
	ImageSize->Scaled[1]
];
(*barChartCfcExchange[1]*)


courseLambdaMaxVlugt[component_] := ListLinePlot[
	{cfcMaxLambdaChangeList[[1;;nEquiCycles, 1, component]], cfcMaxLambdaChangeList[[1;;nEquiCycles, 2, component]]},
	PlotLabel->Style["d\[Lambda] max, " <> system[[component]],Bold],
	Frame->True,
	FrameLabel->{"Iterations","d\[Lambda] max"},
	ImageSize->Scaled[1],
	PlotLegends->Placed[{
		"Box 1",
		"Box 2"
		},Below]
	];
(*courseLambdaMaxVlugt[1]*)

histoLambdaMaxVlugt[component_] := Histogram[
	{cfcMaxLambdaChangeList[[1;;nEquiCycles, 1, component]], cfcMaxLambdaChangeList[[1;;nEquiCycles, 2, component]]},
	100, Probability,
	PlotLabel->Style["d\[Lambda] max, " <> system[[component]],Bold],
	Frame->True,
	FrameLabel->{"d\[Lambda] max / -","Probability / %"},
	ImageSize->Scaled[1],
	ChartLegends->Placed[{
		"Box 1",
		"Box 2"
		},Below],
		ChartStyle->{colors[[1]],colors[[2]]}
	];
(*histoLambdaMaxVlugt[1]*)


(* ::Subsubsection::Closed:: *)
(*Lambda States Distribution*)


barChartCfcVlugtBiasing[component_] := Module[{dat},

	dat = # - Min@# &/@ cfcEta[[All, component, All, 1]];	
	dat = Transpose@dat;
	
	BarChart[
	dat,
	PlotLabel->Style["Final Values of the biasing function, " <> system[[component]],Bold],
	ChartLegends->Placed[{
		"Box 1",
		"Box 2"
		},Below],
	ChartStyle->{colors[[3]],colors[[4]]},
	ImageSize->Scaled[1]
]];


histoLambdaVlugt[component_] := BarChart[
	Transpose@cfcMy[[All, component]],
	PlotLabel->Style["\[Lambda] bin share - 100 bins/box, " <> system[[component]],Bold],
	Frame->True,
	FrameLabel->{"box","Counts"},
	ImageSize->Scaled[1],
	ChartLegends->Placed[{
		"Box 1",
		"Box 2"
		},Below],
	ChartStyle->{colors[[1]],colors[[2]]}
	];
histoLambdaVlugtProp[component_] := BarChart[
	Transpose[# / Total@# &/@ cfcMy[[All, component]]],
	PlotLabel->Style["\[Lambda] bin share - 100 bins/box, " <> system[[component]],Bold],
	Frame->True,
	FrameLabel->{"box","Propability"},
	ImageSize->Scaled[1],
	ChartLegends->Placed[{
		"Box 1",
		"Box 2"
		},Below],
	ChartStyle->{colors[[1]],colors[[2]]}
	];
(*histoLambdaVlugt[1]
histoLambdaVlugtProp[1]*)


histoLnvVlugt[component_] := BarChart[
	Transpose@{#[[2]] &/@ cfcEta[[component, 1]], #[[2]] &/@ cfcEta[[component, 2]]},
	PlotLabel->Style["\[Lambda] bin share - 100 bins/box, " <> system[[component]],Bold],
	Frame->True,
	FrameLabel->{"box","Propability"},
	ImageSize->Scaled[1],
	ChartLegends->Placed[{
		"Box 1",
		"Box 2"
		},Below],
	ChartStyle->{colors[[1]],colors[[2]]}
	];
(*histoLnvVlugt[1]*)


courseLnvVlugt[component_] := ListLogPlot[
	cfcLnVDocu[[All,#, component]] &/@ boxVec,
	PlotLabel->Style["Ln[v], " <> system[[component]],Bold],
	Frame->True,
	FrameLabel->{"Iterations","Ln[v]"},
	ImageSize->Scaled[1],
	Joined->True,
	PlotLegends->Placed[{
		"Box 1",
		"Box 2"
		},Below]
	];


courseMinBin[component_] := ListPlot[
	Transpose@cfcMinBinDocu[[1;;nDocuCycles, All, component]] *100,
	PlotLabel->Style["Minimal bin share, " <> system[[component]],Bold],
	Frame->True,
	FrameLabel->{"cycles","Probability / %"},
	ImageSize->Scaled[1],
	PlotLegends->Placed[{
		"Box 1",
		"Box 2"
		},Below]
	];
(*courseLnvVlugt[1]*)
(*courseMinBin[1]*)

histoMinBin[component_] := Histogram[
	Transpose@cfcMinBinDocu[[1;;nEquiCycles, All, component]] *100,
	100, Probability,
	PlotLabel->Style["Minimal bin share, -" <> system[[component]],Bold],
	Frame->True,
	FrameLabel->{"minimal share / %","Share / -"},
	ImageSize->Scaled[1],
	ChartLegends->Placed[{
		"Box 1",
		"Box 2"
		},Below],
		ChartStyle->{colors[[1]],colors[[2]]}
	];
(*histoMinBin[1]*)


(* ::Subsubsection::Closed:: *)
(*Chemical Potential Calculation*)


SetupBiasingFunction[box_, component_] := Module[{lambdaBins, xBins, f0},

	(*check if the biasing is active*)
	If[!considerHistogramBiasing
		, (*not active \[Rule] return zero function*)
		f0[x_] := If[Dimensions@x == {},0, Table[0, Length@x]];
		f[[box]] = f0;
		Return[f0];
	];

	(*\[Lambda] values of the bins*)
	xBins = Range@numberBins/numberBins -1 /numberBins /2;

	lambdaBins = cfcEta[[box, component, All, 1]];
	lambdaBins = lambdaBins - Min@lambdaBins;
	If[useBoundaryBins,
		xBins = Flatten@{0, xBins, 1};
		lambdaBins = Flatten@{lambdaBins[[1]], lambdaBins, lambdaBins[[-1]]};
	];
	dat = Transpose@{xBins,lambdaBins};
	f[[box]] = Interpolation[dat, InterpolationOrder->interpolationOrder];
	
	(*return the function handle*)
	f[[box]]
];


LinearFitChemPotential[data_, points_, box_] := Module[{
		xPoints0, xPoints1,
		data0, data1,
		lm0, lm1
	},
	
	(*extrapolation for \[Lambda] \[Rule] 0*)
	xPoints0 = Range@points/100 - 0.005;
	data0 = Transpose@{xPoints0[[1;;points]], data[[1;;points]]};
	lm0 = LinearModelFit[data0,x,x];
	
	(*extrapolation for \[Lambda] \[Rule] 1*)
	xPoints1 = 1 - Reverse@Range@points/100 + 0.005;
	data1 = Transpose@{xPoints1, data[[-points;;-1]]};
	lm1 = LinearModelFit[data1,x,x];

	(*return the ratio of (\[Lambda]\[Rule]1)/(\[Lambda]\[Rule]0)*)
	lm1[1] *Exp[-f[[box]][1]] / (lm0[0] *Exp[-f[[box]][0]])
];


ChemicalPotentialVlugt[cp_] := Module[{

	xPoints, boxS,
	iV, iL,
	plt1, plt2,
	header, rows
	},

	(*get the biasing function*)
	f = Table[{}, 2 (*boxes*)];
	SetupBiasingFunction[#, cp] &/@ boxVec;
	
	(*setup vector of \[Lambda] values*)
	points = 100;
	xPoints = Range@points/100 - 0.005;
	
	(*calculate the bins*)
	states = cfcMy[[All, cp, All]];
	statesS = Total@states[[#]] &/@ boxVec;
	statesP = states[[#]] /statesS[[#]] &/@ boxVec;
	statesPair = Transpose@{xPoints, statesP[[#]]} &/@ boxVec;
	
	biased = states[[#]] *Exp[-f[[#]][xPoints]] &/@ boxVec;
	biasedS = Total@biased[[#]] &/@ boxVec;
	biased = biased[[#]] /biasedS[[#]] &/@ boxVec;
	biasedPair = Transpose@{xPoints, biased[[#]]} &/@ boxVec;
	
	(*visualize the \[Lambda] distribution*)
	Do[
		boxS = If[expInd[[1]] == iBox, "V", "L"];
		plt1 = Plot[
			{f[[iBox]][x]}, {x, 0, 1},
			ImageSize->Scaled[1],
			PlotLegends->Placed[{"biasing"},Below],
			Frame->True,
			FrameLabel->{"\[Lambda] / -", "\[Eta](\[Lambda]) / -"},
			PlotLabel->Style["Box " <> ToString@iBox <> ", Phase: " <> boxS <> ", " <> system[[cp]],Bold]];
		plt2 = ListPlot[
			{statesPair[[iBox]], biasedPair[[iBox]]},
			ImageSize->Scaled[1],
			PlotLegends->Placed[{"states", "biased"},Below],
			Frame->True,
			FrameLabel->{"\[Lambda]","p(\[Lambda])"},
			PlotLabel->Style["Box " <> ToString@iBox <> ", Phase: " <> boxS <> ", " <> system[[cp]],Bold]];
		
		ExportGridGraphic[{{"biasing", plt1}, {"lambdaStates", plt2}}, False];
	, {iBox, 2}];
	
	(*calculate the excess potential*)
	ratios = {
		biased[[#, -1]] /biased[[#, 1]] ,
		LinearFitChemPotential[states[[#]], 5, #],
		LinearFitChemPotential[states[[#]], 10, #],
		LinearFitChemPotential[states[[#]], 15, #],
		LinearFitChemPotential[states[[#]], 20, #],
		LinearFitChemPotential[states[[#]], 25, #]
	} &/@ boxVec;
	myVlugtExcess = -kB*T*nAvogadro*Log[ratios]/1000;
	
	(*reference: conventional vlugt method*)
	myWidExcess = Mean[totChPotList[[cp, #]] - idChPotList[[cp, #]]] &/@ boxVec;
	
	iV = expInd[[1]];
	iL = expInd[[2]];
	
	header = {
		"type calculation",
		"\!\(\*SubscriptBox[\(\[Mu]\), \(v, Ex\)]\)", 
		"\!\(\*SubscriptBox[\(\[Mu]\), \(L, Ex\)]\)"};
	rows = {
		{"widom", myWidExcess[[iV]], myWidExcess[[iL]]},
		{"last recorded bin",myVlugtExcess[[iV,1]], myVlugtExcess[[iL,1]]},
		{"linear fit, 5 bins",myVlugtExcess[[iV,2]], myVlugtExcess[[iL,2]]},
		{"linear fit, 10 bins",myVlugtExcess[[iV,3]], myVlugtExcess[[iL,3]]},
		{"linear fit, 15 bins",myVlugtExcess[[iV,4]], myVlugtExcess[[iL,4]]},
		{"linear fit, 20 bins",myVlugtExcess[[iV,5]], myVlugtExcess[[iL,5]]},
		{"linear fit, 25 bins",myVlugtExcess[[iV,6]], myVlugtExcess[[iL,6]]}
	};
	ExportTableLaTeX["WidomCalculationVlugt", header, rows, 3, True];
	
	(*add the value of the 10-bin interpolation to the result association*)
	testSpecificResults[["muExcessWidom_" <> system[[cp]] <> "_V"]] = myWidExcess[[iV]];
	testSpecificResults[["muExcessWidom_" <> system[[cp]] <> "_L"]] = myWidExcess[[iL]];
	testSpecificResults[["muExcessVlugt_" <> system[[cp]] <> "_V"]] = myVlugtExcess[[iV,3]];
	testSpecificResults[["muExcessVlugt_" <> system[[cp]] <> "_L"]] = myVlugtExcess[[iL,3]];
];
(*ChemicalPotentialVlugt[1]*)


(* ::Subsubsection::Closed:: *)
(*Acceptance Rate as a Function of \[Lambda]*)


visualizeAccProp[typeMove_, component_] := Module[{moveTypeS, acc, sum, dat},

	moveTypeS = Switch[typeMove, 1, "\[Lambda] change", 2, "cfc exchange", 3, "cfc swap"];
	Table[

		(*calculation*)
		acc = recordAcceptanceRateDocu[[typeMove, 2, box, component]];
		sum = recordAcceptanceRateDocu[[typeMove, 1, box, component]] +recordAcceptanceRateDocu[[typeMove, 2, box, component]];
		dat = If[sum[[#]] == 0, 0, acc[[#]]/sum[[#]]] &/@ Range@Length@sum;

		(*visualization*)
		ListPlot[		
			dat,
			PlotLabel->Style[moveTypeS <> ", " <> system[[component]],Bold],
			Frame->True,
			FrameLabel->{"\[Lambda]","p(\[Lambda])"},
			PlotLegends->Placed[{"Box " <> ToString@box},Below],
			ImageSize->Scaled[1],
			Joined->True]
	, {box, 2}]
];


(* ::Subsubsection::Closed:: *)
(*Course of cfcEta and cfcMy*)


pltLamda[datMy_, rangeBins_, rangeCycles_, label_] := ListPlot[
	datMy[[rangeBins, rangeCycles]],
	Frame->True,
	FrameLabel-> {"cycle", "p(\[Lambda])"},
	PlotLabel->label,
	ImageSize->Scaled[1],
	PlotLegends->Placed[N[#/100 -0.005] &/@ rangeBins, Below]
];

VisualizeCourseMyCycles[cp_] := Module[{datMy, label},
	label = "\[Lambda] States, Box " <> ToString@# <> " , " <> system[[cp]] &/@ boxVec;
	datMy = Transpose[# /Total@# &/@ cfcMyDocu[[nEquiCycles+1;;nDocuCycles,#,cp,All]]] &/@ boxVec;
	Print@Grid[{
		pltLamda[datMy[[#]], Range[5] *20, All, label[[#]]] &/@ boxVec,
		pltLamda[datMy[[#]], Range[5] *20, -Min[2000, nProdCycles];;-1, label[[#]]] &/@ boxVec,
		VisualizeCourseMyHelper[Range[1, 100], Ceiling[nProdCycles/5], #, cp, False] &/@ boxVec
		},
		ItemSize->Scaled[0.5]];
];
(*VisualizeCourseMyCycles[1, 1]*)


VisualizeCourseEtaCycles[cp_, box_] := Module[{},

	datEta = cfcEtaDocu[[1;;nEquiCycles, box, cp, All, 1]];
	dataPltEta = Transpose[# -Min@# &/@ datEta];
		
	ListPlot[
		dataPltEta,
		Frame->True,
		FrameLabel-> {"cycle / 1", "\[Eta] / 1"},
		PlotLabel->"Biasing Function, Box " <> ToString@box <> ", " <> system[[cp]],
		ImageSize->Scaled[1]]			
]
(*VisualizeCourseEtaCycles[1]*)


VisualizeCourseMyHelper[range_, step_, box_, cp_, print_] := Module[{indices, size},

	size = If[print, 0.5, 1];
	indices = nEquiCycles + step*(Range[Ceiling[nProdCycles/step]-1]);
	ListPlot[
		Transpose@{
			range/100,
			cfcMyDocu[[#,box,cp,range]]/Total@cfcMyDocu[[#,box,cp,1;;100]]} &/@ indices,
		PlotLegends->Placed[ToString[# -nEquiCycles] &/@indices, Below],
		Joined->True,
		Frame->True,
		FrameLabel->{"\[Lambda]", "p(\[Lambda])"},
		ImageSize->Scaled[size],
		PlotLabel->"Box " <> ToString@box <> ", " <> system[[cp]]]
];


VisaulizeCourseMy[box_, cp_, step_, print_] := Module[{pltAll, pltTo0, pltTo1},

	pltAll = VisualizeCourseMyHelper[Range[1, 100], step, box, cp, print];
	pltTo0 = VisualizeCourseMyHelper[Range[1, 20], step, box, cp, print];
	pltTo1 = VisualizeCourseMyHelper[Range[80, 100], step, box, cp, print];
	
	If[print
		, (*TRUE \[Rule] print individually for export*)
		Print@#; &/@ {pltAll, pltTo0, pltTo1};
		
		, (*FALSE \[Rule] print in grid for more conceise output*)
		Print@Grid[{{pltAll},{pltTo0, pltTo1}},
			ItemSize->Scaled[0.5],
			Alignment->Center];
	];
];


VisualizeCourseEtaHelper[range_, step_, box_, cp_, print_] := Module[{},

	indices = step*(Range[Ceiling[nEquiCycles/step]-1]);
	ListPlot[
		Transpose@{
			range/10-0.05,
			values = cfcEtaDocu[[#, box, cp, range, 1]];
			values -Min@values} &/@ indices,
		PlotLegends->Placed[ToString[#] &/@indices, Below],
		Joined->True,
		Frame->True,
		FrameLabel->{"\[Lambda]", "\[Eta](\[Lambda])"},
		ImageSize->Scaled[1],
		PlotLabel->"Biasing Function, Box " <> ToString@box <> ", " <> system[[cp]],
		PlotRange->{{0,1},Automatic}]
];


VisaulizeCourseEta[cp_, step_, print_] := Module[{pltAll, pltTo0, pltTo1, pltBox1, pltBox2},
	
	ExportGridGraphic[{
		{"etaCyclesBox1", VisualizeCourseEtaCycles[cp, 1]},
		{"etaCyclesBox2", VisualizeCourseEtaCycles[cp, 2]}}
		, False];
	ExportGridGraphic[{
		{"etaCourseBox1", VisualizeCourseEtaHelper[Range[1, 10], step, 1, cp, False]},
		{"etaCourseBox2", VisualizeCourseEtaHelper[Range[1, 10], step, 2, cp, False]}}
		, False];
];
(*VisaulizeCourseEta[1, 500, False]*)


(* ::Subsubsection::Closed:: *)
(*Box Biasing*)


boxBiasing[] := If[checkGetAssocParameter[custParam, "cfcBiasBoxes"], 
	Print@Grid[{{
		ListPlot[
			cfcBoxBiasValueDocu[[All, 1]],
			Frame->True,
			FrameLabel->{"cycle / -", "\!\(\*SubscriptBox[\(\[Eta]\), \(Box, \\\ 1\)]\)"},
			ImageSize->Scaled[1]
		],
		ListPlot[
			cfcBoxBiasValueDocu[[All, 2]],
			Frame->True,
			FrameLabel->{"cycle / -", "share box 1"},
			ImageSize->Scaled[1]
		]
	}},
	ItemSize->Scaled[0.5]];];


(* ::Subsubsection::Closed:: *)
(*Visualization*)


(*for back compatibility: check if counters for widom insertion are present*)
visualizeCfcVlugtBasic[] := Switch[checkGetAssocParameter[custParam, "particleInsertionMode"]
	
	, "CfcVlugt",
	setupCfcParameters[];
	Print@timingTableCfcVlugt[];
	Do[
		Print@Grid[{
			{barChartCfcExchange[cp], barChartCfcVlugtLamdaChange[cp]},
			{barChartCfcVlugtSwap[cp], courseLnvVlugt[cp]},
			{courseMinBin[cp], histoMinBin[cp]},
			{courseLambdaMaxVlugt[cp], histoLambdaMaxVlugt[cp]}			
		},
		ItemSize->Scaled[0.5],
		Alignment->Center];
		
		ChemicalPotentialVlugt[cp];
		
	, {cp, Length@system}];
	markShowCell[EvaluationCell[]];
];


(*for back compatibility: check if counters for widom insertion are present*)
visualizeCfcVlugtExtensive[] := Switch[checkGetAssocParameter[custParam, "particleInsertionMode"]
	
	, "CfcVlugt",
	setupCfcParameters[];
	Do[		
		If[checkGetAssocParameter[custParam, "recordAcceptanceProbabilities"],
			Print@Grid[{
				visualizeAccProp[1, cp],
				visualizeAccProp[2, cp],
				visualizeAccProp[3, cp]
			},
			ItemSize->Scaled[0.5],
			Alignment->Center]];
			
		If[checkGetAssocParameter[custParam, "cfcStateDocu"],
		
			VisaulizeCourseEta[cp, Ceiling[nEquiCycles/5], False];
			VisualizeCourseMyCycles[cp];
			VisualizeCourseMyCycles[cp];
		];
		
	, {cp, Length@system}];
	boxBiasing[];
];


(* ::Subsection::Closed:: *)
(*SwapMove Species Choice Probabilities*)


VisualizeSMPR[] := Module[{},
	Switch[checkGetAssocParameter[custParam, "speciesSwapProbabilityMode"]
	, "auto" | "autoSwap",
		Module[{avrCycles, nCycles},
			avrCycles = custParam[["eqAdjAvrCycles"]];
			nCycles = custParam[["speciesSwapCyclesAdjustment"]];
			If[!SameQ[checkGetAssocParameter[custParam, "eqAdjAvrCycles"],False] && nEquiCycles >= custParam[["eqAdjAvrCycles"]]
				,(*TRUE: An average was taken from the equilibration phase*)
				(* Calculate the average over the specified cycles in the custom parameter *)
				SPfinal = N@Mean@SPdoc[[(nEquiCycles-avrCycles+1);;-1, 2]];
				SMPR = SPfinal[[1]]/SPfinal[[2]];
				,(*ELSE: Take the last documented value*)
				If[Floor[nEquiCycles/custParam[["speciesSwapCyclesAdjustment"]]] == 0,
					(*Not a single adjustment was done/documented*)
					SMPR = "No Adjustments performed!";
					,(*ELSE*)
					SPfinal = SPdoc[[Floor[nEquiCycles/custParam[["speciesSwapCyclesAdjustment"]]],2]];
					SMPR = SPfinal[[1]]/SPfinal[[2]];
				];
			];
		];
		
		(*visualize the documentation*)
		speciesSwapProbabilityPlotGrid ={{
			ListLinePlot[
				Transpose@SPdoc[[1;;nEquiCycles,1]],
				(*PlotRange\[Rule]{Automatic, {0,1}},*)
				PlotLegends->{Placed[system,Below]},
				Frame->True,
				FrameLabel->{"cycle [1]","share / -" },
				ImageSize->Scaled[0.55]],SpanFromLeft},
			ListLinePlot[
				SPdoc[[1;;nEquiCycles,2,#]],
				Prolog -> If[!SameQ[checkGetAssocParameter[custParam, "eqAdjAvrCycles"],False] && custParam[["speciesSwapCyclesAdjustment"]] <= nEquiCycles,
					{Line[{{(nEquiCycles-custParam[["eqAdjAvrCycles"]]+1),SPfinal[[#]]},{nEquiCycles,SPfinal[[#]]}}]}
					,{}
				],
				PlotLegends->{Placed[system[[#]],Below]},
				Frame->True,
				FrameLabel->{"cycle [1]","weights / -" },
				ImageSize->Scaled[0.45]
			] &/@ Range@Length@system
		};

	,"fixed",
		SPfinal = checkGetAssocParameter[custParam, "speciesSwapProbability"];
		SMPR = SPfinal[[1]]/SPfinal[[2]];

	,"_", (* default = equal *)
		SMPR = 1;
	];
	
	(*show absolute results *)
	Echo[N@SMPR, "Swap Move Probability Rate (P1/P2): "];
	SPacceptedTot = Total@Flatten@counterInsertAcceptDocu;
	Echo[N[Total[counterInsertAcceptDocu[[All, #]]] /SPacceptedTot], "share " <> system[[#]]] &/@ Range@Length@system;
];


VisualizeSwapProbailities[] := Module[{
		avgCycles, sum, ratio,
		pltP1P2, pltShare,
		pltNatt, pltNsucc,
		pltP1P2Bin, pltShareBin},
	
	avgCycles = Min@{custParam[["speciesSwapCyclesAdjustment"]], Floor[nEquiCycles/4]};

	(*swap species probability P1/P2*)
	pltP1P2 = ListLinePlot[
		Transpose@SPdoc[[All, 2, All]],
		PlotLegends->{Placed[system,Below]},
		Frame->True,
		FrameLabel->{"cycle [1]","pseudo-probability \!\(\*SubscriptBox[\(p\), \(i\)]\) [1]" },
		ImageSize->Scaled[1]
		];
	sum = Total[Transpose@SPdoc[[All, 1, All]], 1];
	ratio = If[sum[[#]] != 0,SPdoc[[#, 1, All]]/sum[[#]], Table[0, Length@system]] &/@ Range@Length@sum;
	pltShare = ListLinePlot[
		MovingAverage[ratio[[All, #]], avgCycles] &/@ Range@Length@system,
		PlotLegends->{Placed[system,Below]},
		Frame->True, PlotRange->All,
		FrameLabel->{"cycle [1]","\!\(\*SubscriptBox[\(n\), \(Swap, Acc, i\)]\) / \!\(\*SubscriptBox[\(n\), \(Swap, Acc, total\)]\) [1]" },
		ImageSize->Scaled[1]
		];
	ExportGridGraphic[{
		{"Attempted Probability", pltP1P2}, {"Successful Proability", pltShare}
	}, False];
		
	(*number of swap moves*)
	pltNatt = ListLinePlot[
		# *nMovesDocu[[All, 4]] &/@ Transpose@SPdoc[[All, 2, All]],
		PlotLegends->{Placed[system,Below]},
		Frame->True,
		FrameLabel->{"cycle [1]","Attempted \!\(\*SubscriptBox[\(n\), \(Swaps\)]\) [1]" },
		ImageSize->Scaled[1]
		];
	pltNsucc = ListLinePlot[
		MovingAverage[SPdoc[[All, 1, #]], avgCycles] &/@ Range@Length@system,
		PlotLegends->{Placed[system,Below]},
		Frame->True, PlotRange->All,
		FrameLabel->{"cycle [1]","Successful \!\(\*SubscriptBox[\(n\), \(Swaps\)]\) [1]" },
		ImageSize->Scaled[1]
		];
	ExportGridGraphic[{
		{"Attempted Swaps", pltNatt}, {"Successful Swaps", pltNsucc}
	}, False];

	(*simplified visualization for binary mixtures*)
	If[Length@system == 2,
		pltP1P2Bin = ListLinePlot[
			SPdoc[[All, 2, 1]] /SPdoc[[All, 2, 2]],
			Frame->True,
			FrameLabel->{"cycle [1]","pseudo-probability rate \!\(\*SubscriptBox[\(P\), \(1\)]\)/ \!\(\*SubscriptBox[\(P\), \(2\)]\) [1]" },
			ImageSize->Scaled[1]
			];
		pltShareBin = ListLinePlot[
			MovingAverage[ratio[[All, 1]] ,avgCycles] /MovingAverage [ratio[[All, 2]], avgCycles],
			Frame->True,
			FrameLabel->{"cycle [1]","\!\(\*SubscriptBox[\(n\), \(Swap, Acc, 1\)]\) / \!\(\*SubscriptBox[\(n\), \(Swap, Acc, 2\)]\) [1]" },
			ImageSize->Scaled[1]
			];
		ExportGridGraphic[{
			{"Attempted Probability Ratio (binary)", pltP1P2Bin}, {"Successful Proability Ratio (binary)", pltShareBin}
		}, False];
	];
];
(*VisualizeSwapProbailities[]*)


(* ::Subsection::Closed:: *)
(*PAN Binary Swaps*)


panBinarySwaps[export_] := Module[{plt, panExchangeSpecies},

	(*get the sepcies that is exchanged*)
	panExchangeSpecies = custParam[["panExchangeSpecies"]];
	Echo["For component " <> system[[panExchangeSpecies]] <> " the pan-binary exchange is used."];

	plt = BarChart[
		{
			{counterPanAcceptDocu[[1,#]], counterPanRejDuDocu[[1,#]], counterPanRejeNpDocu[[1,#]]},
			{counterPanAcceptDocu[[2,#]], counterPanRejDuDocu[[2,#]], counterPanRejeNpDocu[[2,#]]}
		},
		PlotLabel->Style[StringForm["Particle Insertions of component ``",system[[#]]],Bold],
		ChartLegends->Placed[{"Successful", "Rej. \[CapitalDelta]U","Rej. Not Present"},Below],
		ChartStyle->{colors[[3]],colors[[4]],colors[[5]]},
		ChartLabels->{{"Insertion in Box 1","Insertion in Box 2"},None},
		LabelingFunction->Above,
		ImageSize->lImageSize
	] &/@ {panExchangeSpecies};
	
	timingTablePanBinarySwap[];
	
	ExportGridGraphic[{"panBinarySwaps " <> system[[#]], plt[[#]]} &/@ {panExchangeSpecies}, export];
];


timingTablePanBinarySwap[] := Module[{totalPanInsertionTime,totalGEMCInsertionTime, totalInsertionTime},
	
	totalInsertionTime = Total@Values[evalTime[[{"insertion-Accept","insertion-RejeDu","insertion-RejeOv","insertion-RejeNp"}]]];
	totalPanInsertionTime = Total@Values[evalTime[[{"panInsertion-Accept","panInsertion-RejeDu","panInsertion-RejeNp"}]]];
	totalGEMCInsertionTime = totalInsertionTime - totalPanInsertionTime;
	
	Print@Grid[
		{
			{Style["PAN Binary Swap Move Timing",Bold],SpanFromLeft},
			{"Part", "Time / total", "\!\(\*SubscriptBox[\(t\), \(total\)]\) / %", "\!\(\*SubscriptBox[\(t\), \(avr\)]\) / ms","\!\(\*SubscriptBox[\(t\), \(acc\)]\) / ms","\!\(\*SubscriptBox[\(t\), \(rejDu\)]\) / ms", "\!\(\*SubscriptBox[\(t\), \(rejOv\)]\) / ms"},
			{"Insertion",
				SecondsToDurationString[totalGEMCInsertionTime],
				Round[totalGEMCInsertionTime / totalInsertionTime * 100, 0.01],
				totalGEMCInsertionTime / Total[{
					counterInsertAcceptDocu-counterPanAcceptDocu, counterInsertRejeDuDocu-counterPanRejDuDocu, counterInsertRejeNpDocu-counterPanRejeNpDocu
				},-1] * 1000,
				(evalTime[["insertion-Accept"]]-evalTime[["panInsertion-Accept"]]) / Total[counterInsertAcceptDocu-counterPanAcceptDocu,-1] *1000,
				(evalTime[["insertion-RejeDu"]]-evalTime[["panInsertion-RejeDu"]]) / Total[counterInsertRejeDuDocu-counterPanRejDuDocu,-1] *1000,
				If[Total[counterInsertRejeOvDocu,-1] == 0, "-", (evalTime[["insertion-RejeOv"]]) / Total[counterInsertRejeOvDocu,-1] *1000]
			},
			{"PAN Binary Swap",
				SecondsToDurationString[totalPanInsertionTime],
				Round[totalPanInsertionTime / totalInsertionTime * 100, 0.01],
				totalPanInsertionTime / Total[{
					counterPanAcceptDocu, counterPanRejDuDocu, counterPanRejeNpDocu
				},-1] * 1000,
				evalTime[["panInsertion-Accept"]] / Total[counterPanAcceptDocu,-1] *1000,
				evalTime[["panInsertion-RejeDu"]] / Total[counterPanRejDuDocu,-1] *1000,
				"-"
			}
		}
		,Dividers->{All,{True,True,True,{False},True}}
		,Spacings->{1.5, 2},ItemSize->Full
	];
]


(* ::Subsection::Closed:: *)
(*Electrostatic Long Range Correction*)


(* ::Subsubsection::Closed:: *)
(*Ewald*)


VisualizeELRCEwald[export_] := Module[{eLRCkappa, eLRCkRange},
	eLRCkappa = custParam["eLRCkappa"];
	eLRCkRange = custParam["eLRCkRange"];
	eLRCewaldKappaRangePlots = ListLinePlot[
			eLRCkappaRangeplotdata[[#]],
			Epilog->InfiniteLine[{{eLRCkappa[[#]], 0},{eLRCkappa[[#]], 1}}],
			PlotLabel->Style["Total electrostatic Energy Box "<>ToString@#<>
				"\nk-Range = "<>ToString@eLRCkRange[[#]]<>"\tkappa = "<>ToString@eLRCkappa[[#]]<>"/L",Bold],
			Frame->True,
			FrameLabel->{"kappa \[CenterDot] L [-]","\!\(\*SubscriptBox[\(U\), \(Coulomb, "<>ToString@#<>"\)]\) [kcal/mol]"},
			ImageSize->lImageSize,
			Axes->False
		] &/@Range@2;
	ExportGridGraphic[{"ewaldKappaRange Box " <> ToString@#, eLRCewaldKappaRangePlots[[#]]} &/@ Range@2, export];
];


(* ::Subsubsection::Closed:: *)
(*Wolf*)


VisualizeELRCWolf[] := Module[{eLRCkappa, RcList},
	eLRCkappa = custParam["eLRCkappa"];
	RcList = Table[ Floor[Lvec[[#]]/2 - n*Lvec[[#]]/16],{n,0,3}] &/@ Range@2;
	
	eLRCwolfKappaRangePlots = ListLinePlot[
			eLRCkappaRangeplotdata[[#]]
			,Epilog->InfiniteLine[{{eLRCkappa[[#]], 0},{eLRCkappa[[#]], 1}}]
			,PlotLabel->Style["Relative Error to Ewald Calculation Box " <> ToString@# <>
				"\nChosen parameter \[Kappa] = " <> ToString@eLRCkappa[[#]] <> " \!\(\*SuperscriptBox[\(A\), \(-1\)]\)"]
			,PlotLegends->Table[ "\!\(\*SubscriptBox[\(R\), \(C\)]\) = "<> ToString@RcList[[#,i]] <> " A",{i,Length@RcList[[#]]}]
			,FrameLabel->{"\[Kappa] [\!\(\*SuperscriptBox[\(A\), \(-1\)]\)]","\!\(\*FractionBox[\(\*SubsuperscriptBox[\(U\), \(Wolf\), \(C\)] - \*SubsuperscriptBox[\(U\), \(Ewald\), \(C\)]\), SubsuperscriptBox[\(U\), \(Ewald\), \(C\)]]\)*100 [%]"}
			,ImageSize->Scaled[0.8],Frame->True,Axes->True
			,PlotRange->{
				{{0,0.15},{-5,10}},
				{{0,0.3},{-5,20}}
			}[[expInd[[#]]]]
		] &/@Range@2;
	
	Print@Grid[{#} &/@ eLRCwolfKappaRangePlots,ItemSize->Scaled[1],Alignment->Center];
]


(* ::Subsection::Closed:: *)
(*Radial Distribution Function*)


VisualizeRDFAtom[] := Module[{dr,nBins,rVec},
	dr = rdfDocudr;
	nBins = Length@rdfDocuAtom[[#,1,1]] &/@ Range[2];
	Print@Grid[
		Flatten[Table[
			ListPlot[
				rVec = Range[0.5,nBins[[boxIndex]],1]*dr;
				Transpose@{
					rVec,
					rdfDocuAtom[[boxIndex,ind1,ind2,;;Length@rVec]]
				}
				,PlotLabel->"RDF "<>ToString@atomTypes[[ind1]]<>"-"<>ToString@atomTypes[[ind2]] <> " Box "<>ToString@boxIndex
				,Filling->Axis ,PlotRange->All ,PlotRangePadding->{{None,None},{None,Automatic}}
				,Frame->True ,FrameLabel->{"r [A]","g(r) [-]"}
				,FrameLabel->{"r [A]","g(r) [-]"}
				,ImageSize->lImageSize
			]
			,{ind1,Length@atomTypes}
			,{ind2,ind1,Length@atomTypes}
			,{boxIndex,2}
		],1]
		,ItemSize->Scaled[0.5]
	]
];


VisualizeRDFMolec[] := Module[{dr,nBins,rVec},
	dr = rdfDocudr;
	nBins = Length@rdfDocuMolec[[#,1,1]] &/@ Range[2];
	Print@Grid[
		Flatten[Table[
			ListPlot[
				rVec = Range[0.5,nBins[[boxIndex]],1]*dr;
				Transpose@{
					rVec,
					rdfDocuMolec[[boxIndex,ind1,ind2,;;Length@rVec]]
				}
				,PlotLabel->"RDF "<>ToString@system[[ind1]]<>"-"<>ToString@system[[ind2]] <> " Box "<>ToString@boxIndex
				,Filling->Axis ,PlotRange->All ,PlotRangePadding->{{None,None},{None,Automatic}}
				,Frame->True ,FrameLabel->{"r [A]","g(r) [-]"}
				,ImageSize->lImageSize
			]
			,{ind1,Length@system}
			,{ind2,ind1,Length@system}
			,{boxIndex,2}
		],1]
		,ItemSize->Scaled[0.5]
	]
];


(* ::Subsection::Closed:: *)
(*Intramolecular Configuration Distribution*)


VisualizeIntraConfigDocu[] := Module[{plotList},
	Do[
		plotList = {};
		
		If[nMovesPerCycles[[6]] > 0
			,(*True: at least one bond change is executed*)
			AppendTo[plotList,plotIntraConfigBonds[speciesIndex]];
		];
		If[nMovesPerCycles[[7]] > 0
			,(*True: at least one bond angle change is executed*)
			AppendTo[plotList,plotIntraConfigAngles[speciesIndex]];
		];
		If[nMovesPerCycles[[8]] > 0
			,(*True: at least one torsion angle change is executed*)
			AppendTo[plotList,plotIntraConfigTorsion[speciesIndex]];
		];
		
		plotList = Flatten@plotList;
		
		Print@Grid[ArrayReshape[plotList,{Ceiling[Length@plotList/2],2},""],ItemSize->{{{Scaled[0.5]}}}]
		
		,{speciesIndex,Length@system}
	];
]


plotIntraConfigBonds[speciesIndex_] := Module[
	{
		bondLableVec,bondIndex,bondPlots,
		dataBox1,dataBox2,lRange,
		lEqu,dataList,lMean,lStd
	},
	
	bondLableVec = DeleteDuplicates@Values@ffLabelsBondStretching[[speciesIndex,All,1]];
	
	bondPlots = Table[
		bondIndex = Flatten@Position[Values@ffLabelsBondStretching[[speciesIndex,All,1]],bondLableVec[[bondLableIndex]]];
		dataBox1 = Total[intraConfigBondBins[[1,speciesIndex,bondIndex]],1];
		dataBox2 = Total[intraConfigBondBins[[2,speciesIndex,bondIndex]],1];
		lRange = Range[0.5,Length@dataBox1]*intraConfigDocudr;
		
		lEqu = oplsBondStretching[bondLableVec[[bondLableIndex]]][[1]];
		dataList = Flatten@Table[ConstantArray[lRange[[i]],dataBox1[[i]]+dataBox2[[i]]],{i,1,Length@lRange}];
		lMean = Mean@dataList;
		lStd = StandardDeviation@dataList;
		
		Show[{
			ListLinePlot[{
					Transpose@{lRange,dataBox1/Total@dataBox1},
					Transpose@{lRange,dataBox2/Total@dataBox2}
				}
				,PlotLabel->"Bond Length - "<>ToString@system[[speciesIndex]]<>" - "<>"\""<>ToString@bondLableVec[[bondLableIndex]]<>"\""
				,Frame->True ,FrameLabel->(Style[#,14,SingleLetterItalics->False]&/@{"\!\(\*SubscriptBox[\(l\), \(Bond\)]\) [\[CapitalARing]]","PDF [1]"})
				,PlotRange->{{lMean-4*lStd,lMean+4*lStd},All} ,PlotRangePadding->{None,{None,Automatic}}
				,Filling->Axis
				,PlotLegends->Placed[{"Box 1","Box 2"},Below]
				,ImageSize->lImageSize
			],
			Graphics[{Dashed,InfiniteLine[{{lEqu,0},{lEqu,1}}]}]
		}]
		,{bondLableIndex,Length@bondLableVec}
	];
	
	(*RETURN*)
	bondPlots
];


plotIntraConfigAngles[speciesIndex_] := Module[
	{
		angleLableVec,angleIndex,anglePlots,
		dataBox1,dataBox2,phiRange,
		phiEq,dataList,phiMean,phiStd
	},
	
	angleLableVec = DeleteDuplicates@Values@ffLabelsAngleBending[[speciesIndex,All,1]];
	
	anglePlots = Table[
		angleIndex = Flatten@Position[Values@ffLabelsAngleBending[[speciesIndex,All,1]],angleLableVec[[angleLableIndex]]];
		dataBox1 = Total[intraConfigAngleBins[[1,speciesIndex,angleIndex]],1];
		dataBox2 = Total[intraConfigAngleBins[[2,speciesIndex,angleIndex]],1];
		phiRange = Range[0.5,Length@dataBox1]*intraConfigDocudphi; (*Deg*)
		
		phiEq = oplsAngleBending[angleLableVec[[angleLableIndex]]][[1]];
		dataList = Flatten@Table[ConstantArray[phiRange[[i]],dataBox1[[i]]+dataBox2[[i]]],{i,1,Length@phiRange}];
		phiMean = Mean@dataList;
		phiStd = StandardDeviation@dataList;
		
		Show[{
			ListLinePlot[{
					Transpose@{phiRange,dataBox1/Total@dataBox1},
					Transpose@{phiRange,dataBox2/Total@dataBox2}
				}
				,PlotLabel->"Bond Angle - "<>ToString@system[[speciesIndex]]<>" - "<>"\""<>ToString@angleLableVec[[angleLableIndex]]<>"\""
				,Frame->True ,FrameLabel->(Style[#,14,SingleLetterItalics->False]&/@{"\!\(\*SubscriptBox[\(\[Phi]\), \(Angle\)]\) [Deg]","PDF [1]"})
				,PlotRange->{{phiMean-4*phiStd,phiMean+4*phiStd},All} ,PlotRangePadding->{None,{None,Automatic}}
				,Filling->Axis
				,PlotLegends->Placed[{"Box 1","Box 2"},Below]
				,ImageSize->lImageSize
			],
			Graphics[{Dashed,InfiniteLine[{{phiEq,0},{phiEq,1}}]}]
		}]
		,{angleLableIndex,Length@angleLableVec}
	];
	
	(*RETURN*)
	anglePlots
];


plotIntraConfigTorsion[speciesIndex_] := Module[
	{
		torsionLableVec,torsionIndex,torsionPlots,
		dataBox1,dataBox2,thetaRange
	},
	
	torsionLableVec = DeleteDuplicates@Values@ffLabelsTorsion[[speciesIndex,All,1]];
	
	torsionPlots = Table[
		torsionIndex = Flatten@Position[Values@ffLabelsTorsion[[speciesIndex,All,1]],torsionLableVec[[torsionLableIndex]]];
		dataBox1 = Total[intraConfigTorsionBins[[1,speciesIndex,torsionIndex]],1];
		dataBox2 = Total[intraConfigTorsionBins[[2,speciesIndex,torsionIndex]],1];
		thetaRange = Range[0.5,Length@dataBox1]*intraConfigDocudtheta; (*Deg*)
		
		Show[{
			ListLinePlot[{
					Transpose@{thetaRange,dataBox1/Total@dataBox1},
					Transpose@{thetaRange,dataBox2/Total@dataBox2}
				}
				,PlotLabel->"Torsion Angle - "<>ToString@system[[speciesIndex]]<>" - "
					<>"\""<>ToString@torsionLableVec[[torsionLableIndex]]<>"\""
				,Frame->True ,FrameLabel->(Style[#,14,SingleLetterItalics->False]&/@{"\!\(\*SubscriptBox[\(\[Phi]\), \(Torsion\)]\) [Deg]","PDF [1]"})
				,PlotRange->{All,{0,All}} ,PlotRangePadding->{{0.6,1.0},{None,Automatic}}
				,FrameTicks->{{Automatic,None},{Range[0,360,30],None}}
				,Filling->Axis
				,PlotLegends->Placed[{"Box 1","Box 2"},Below]
				,ImageSize->lImageSize
			]
		}]
		,{torsionLableIndex,Length@torsionLableVec}
	];
	
	(*RETURN*)
	torsionPlots
];


(* ::Section:: *)
(*Component specific EoS*)


(* ::Subsection::Closed:: *)
(*Friend1991 EoS (Ethane)*)


Module[
	{
		tCrit,pCrit,\[Rho]Crit,zCrit,
		H,G,J,
		TAstFriend1991
	},

	tCrit = 305.33; (*K*)
	pCrit = 48.718; (*bar*)
	\[Rho]Crit = 6.87;   (*mol/dm^3*)
	zCrit = 0.27934;(**)
	H = {-7.955315, 1.532027, 14.78068, -13.43179, 4.704891};
	G = {1.930740, -0.6539856, 0.8141362, -0.3397430, -0.3838141};
	J = {-0.7483719, -1.372895, -1.192597, 1.861505, 1.313649};

	TAstFriend1991[t_]:=((tCrit-t)/tCrit);

	pSatRedLJFriend1991[t_]:=Evaluate[pCrit*Exp[
		H[[1]]*TAstFriend1991[t]/(1-TAstFriend1991[t])+
		H[[2]]*TAstFriend1991[t]+
		H[[3]]*TAstFriend1991[t]^1.90+
		H[[4]]*TAstFriend1991[t]^2+
		H[[5]]*TAstFriend1991[t]^3
	]];

	\[Rho]SatLiqRedLJFriend1991[t_]:= Evaluate[\[Rho]Crit*(1+(
		G[[1]]*TAstFriend1991[t]^0.355+
		G[[2]]*TAstFriend1991[t]^2+
		G[[3]]*TAstFriend1991[t]^3+
		G[[5]]*TAstFriend1991[t]^4)
		/(1 + G[[4]]*TAstFriend1991[t]^(1-0.355))
	)];

	\[Rho]SatVapRedLJFriend1991[t_]:= Evaluate[pSatRedLJFriend1991[t]/(rJoule*t)*100*(
		1+pSatRedLJFriend1991[t]*(tCrit/t)^8*((zCrit-1)/pCrit)*(1+(
			J[[1]]*TAstFriend1991[t]^0.355+
			J[[2]]*TAstFriend1991[t]^2*0.355+
			J[[3]]*(TAstFriend1991[t]+TAstFriend1991[t]^4)+
			J[[4]]*TAstFriend1991[t]^2)
			/(1+J[[5]]*TAstFriend1991[t]))	
		)^-1
	];
];


(* ::Chapter:: *)
(*Literature*)


(* ::Text:: *)
(*There is no package specific literature.*)
