(* ::Package:: *)

(* ::Title:: *)
(*ChemicalThermoPackage*)


(* ::Subtitle:: *)
(*A repository of functions frequently used in chemical thermo*)


(* ::Text:: *)
(*2022-10-20 Maintained by Thomas Wallek*)


BeginPackage["ChemicalThermoPackage`",{"PropertyPackage`"}];


(* ::Chapter::Closed:: *)
(*Usage Definitions*)


(* ::Section::Closed:: *)
(*Lennard-Jones (LJ) Fluid*)


tCritLJ::usage="tCritLJ:
 Critical temperature \!\(\*SuperscriptBox[SubscriptBox[\(t\), \(c\)], \(*\)]\) of the LJ-fluid according to Thol 2016 and the Allen-Tildesley textbook";


pCritLJ::usage="pCritLJ:
 Critical pressure \!\(\*SuperscriptBox[SubscriptBox[\(P\), \(c\)], \(*\)]\) of the LJ-fluid according to Thol 2016 and the Allen-Tildesley textbook";


\[Rho]CritLJ::usage="\[Rho]CritLJ:
 Critical density \!\(\*SuperscriptBox[SubscriptBox[\(\[Rho]\), \(c\)], \(*\)]\) of the LJ-fluid according to Thol 2016 and the Allen-Tildesley textbook";


tRedLJ::usage="tRedLJ[t [K], \[Epsilon] [kcal \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Convert temperature t[K] to the reduced Lennard-Jones temperature [1]";


tLJ::usage="tLJ[tRedLennardJones [1], \[Epsilon] [kcal \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]:
 Convert the reduced Lennard-Jones temperature [1] to temperature [K]";


pVdWLJ::usage="pVdWLJ[t [K], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], \[Sigma] [\[Angstrom]], \[Epsilon] [kcal \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]:
 Van der Waals pressure [bar] of a Lennard-Jones fluid";


vVdWLJ::usage="vVdWLJ[t [K], p [bar], \[Sigma] [\[Angstrom]], \[Epsilon] [kcal \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]:
 Van der Waals molar volume [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] of a Lennard-Jones fluid";


tCritVTPRLJ::usage="tCritVTPRLJ:
 Critical temperature \!\(\*SuperscriptBox[SubscriptBox[\(t\), \(c\)], \(*\)]\) of the LJ-fluid according to Harismiadis 1994";


pCritVTPRLJ::usage="pCritVTPRLJ:
 Critical pressure \!\(\*SuperscriptBox[SubscriptBox[\(P\), \(c\)], \(*\)]\) of the LJ-fluid according to Harismiadis 1994";


\[Rho]CritVTPRLJ::usage="\[Rho]CritVTPRLJ:
 Critical density \!\(\*SuperscriptBox[SubscriptBox[\(\[Rho]\), \(c\)], \(*\)]\) of the LJ-fluid according to Harismiadis 1994";


tRedVTPRLJ::usage="tRedVTPRLJ[t [K], \[Epsilon] [kcal/mol]]:
 Reduced temperature [1] using the critical LJ values used in Harismiadis 1994";


aVTPRLJ::usage="aVTPRLJ[tRed [1], \[Sigma] [\[Angstrom]], \[Epsilon] [kcal/mol]]:
 Parameter a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)] of the VTPRLJ equation of state";


bVTPRLJ::usage="bVTPRLJ[\[Sigma] [\[Angstrom]], \[Epsilon] [kcal/mol]]:
 Parameter b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\)/mol] of the VTPRLJ equation of state";


volumeTranslationVTPRLJ::usage="volumeTranslationVTPRLJ[tRed [1], \[Sigma] [\[Angstrom]]]:
 Parameter t [\!\(\*SuperscriptBox[\(dm\), \(3\)]\)/mol] of the VTPRLJ equation of state";


pVTPRLJ::usage="pVTPRLJ[T [K], v [dm^3/mol], a [(dm^3)^2 bar mol^-2], b [dm^3/mol], volumeTranslation [dm^3/mol]]:
 Pressure [bar] of the VTPRLJ equation of state";


vVTPRLJ::usage="vVTPRLJ[T [K], p [bar], a [(dm^3)^2 bar mol^-2], b [dm^3/mol], volumeTranslation [dm^3/mol]]:
 Molar volume [\!\(\*SuperscriptBox[\(dm\), \(3\)]\)/mol] of the VTPRLJ equation of state
 returns all three solutions, including complex ones";


\[CurlyPhi]PureVTPRLJ::usage="\[CurlyPhi]PureVTPRLJ[a [(dm^3)^2 bar mol^-2], b [dm^3/mol], volumeTranslation [dm^3/mol], p [bar], v [dm^3/mol], t [K]]:
 Pure component fugacity coefficient [1] of the VTPRLJ equation of state";


\[CurlyPhi]MixtureVTPRLJ::usage="\[CurlyPhi]MixtureVTPRLJ[aij [(dm^3)^2 bar mol^-2], bij [dm^3/mol], volumeTranslationPure [dm^3/mol], x [mol/mol], p [bar], v [dm^3/mol], t [K]]:
 Mixture fugacity coefficient [1] of the VTPRLJ equation of state
 Parameter a and b to be supplied as symmetric matrices:
 {
  {a11, a12},
  {a21, a22}
 };
 {
  {b11, b12},
  {b21, b22}
 };
Parameter volumeTranslationPure and molar fractions to be supplied as lists:
{volumeTranslation1, volumeTranslation2}   {x1, x2}";


b2LJ::usage="b2LJ[t [K], \[Sigma] [\[Angstrom]], \[Epsilon] [kcal \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Second virial coefficient [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] for LJ fluids";


pSatRedLJ::usage="pSatRedLJ[tRed =\!\(\*SuperscriptBox[\(t\), \(*\)]\)/tCritLJ]:
 Reduced saturation pressure [1] of the LJ-fluid according to Thol 2016";


\[Rho]SatLiqRedLJ::usage="\[Rho]SatLiqLJ[tRed =\!\(\*SuperscriptBox[\(t\), \(*\)]\)/tCritLJ]:
 Reduced saturated liquid density of the LJ-fluid according to Thol 2016";


\[Rho]SatVapRedLJ::usage="\[Rho]SatVapLJ[tRed =\!\(\*SuperscriptBox[\(t\), \(*\)]\)/tCritLJ]:
 Reduced saturated vapor density of the LJ-fluid according to Thol 2016";


uLJidealThol2016::usage="uLJidealThol2016[tAsterisk [1], \[Rho]Asterisk [1]]:
 Ideal gas dimensionless internal energy of the LJ-fluid according to Thol 2016";


uLJconfigThol2016::usage="uLJconfigThol2016[tAsterisk [1], \[Rho]Asterisk [1]]:
 Configurational dimensionless internal energy of the LJ-fluid according to Thol 2016";


uLJtotalThol2016::usage="uLJtotalThol2016[tAsterisk [1], \[Rho]Asterisk [1]]:
 Total dimensionless internal energy of the LJ-fluid (ideal gas + configurational) according to Thol 2016";


zLJThol2016::usage="zLJtotalThol2016[tAsterisk [1], \[Rho]Asterisk [1]]:
 Compressibility factor [1] of the LJ-fluid according to Thol 2016";


uLJconfigMecke1996::usage="uLJconfigMecke1996[tAsterisk [1], \[Rho]Asterisk [1]]:
 Configurational dimensionless internal energy of the LJ-fluid according to Mecke 1996";


zLJKolafa1994::usage="zLJconfigKolafa1994[t =\!\(\*SuperscriptBox[\(t\), \(*\)]\) [1], \[Rho] =\!\(\*SuperscriptBox[\(\[Rho]\), \(*\)]\) [1]]:
 Compressibility factor [1] of the LJ-fluid with \[Epsilon] = \[Sigma] = \!\(\*SubscriptBox[\(k\), \(B\)]\) = 1, according to Kolafa 1994";


uLJconfigKolafa1994::usage="uLJconfigKolafa1994[t [1], \[Rho] [1]]:
 Configurational dimensionless internal energy of the LJ-fluid with \[Epsilon] = \[Sigma] = \!\(\*SubscriptBox[\(k\), \(B\)]\) = 1, according to Kolafa 1994";


zLJNicolas1979::usage="zLJNicolas1979[tAsterisk [1], \[Rho]Asterisk [1]]:
 Compressibility factor of the LJ-fluid according to Nicolas 1979";


(* ::Section::Closed:: *)
(*Saturation Pressure*)


pAntoine::usage="pAntoine[t [?], par [list of parameters], log [string 'ln' or 'log10']]:
 Antoine saturation pressure
 the temperature and pressure units depend on the set of parameters";


tAntoine::usage="tAntoine[p [?], par [list of parameters], log [string 'ln' or 'log10']]:
 Antoine saturation temperature
 the temperature and pressure units depend on the set of parameters";


pExtendedAntoine::usage="pExtendedAntoine[t [K], plxant [list of parameters]]:
 Extended Antoine saturation pressure, as used by ASPEN
 From up to 9 parameters supplied, 7 are actually used";


pHarlacher::usage="pHarlacher[t [?], pGuess [starting value for solver], par [list of 4 parameters], log [string 'ln' or 'log10']]:
 Harlacher (Frost-Kalkwarf-Thodos) saturation pressure
 the temperature and pressure units depend on the set of parameters";


tHarlacher::usage="pHarlacher[p [?], tGuess [starting value for solver], par [list of 4 parameters], log [string 'ln' or 'log10']]:
 Harlacher (Frost-Kalkwarf-Thodos) saturation temperature
 the temperature and pressure units depend on the set of parameters";


(* ::Section::Closed:: *)
(*Thermal Equations of State (EoS)*)


pIdealGas::usage="pIdealGas[t [K], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Pressure [bar] from ideal gas equation of state";


vIdealGas::usage="vIdealGas[t [K], p [bar]]:
 Molar volume [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] from ideal gas equation of state";


tIdealGas::usage="tIdealGas[p [bar],v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Temperature [K] from ideal gas equation of state";


vVirialLeiden::usage="vVirialLeiden[virialCoefficient2 [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K], p [bar]]:
 Molar volume [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] from Virial (Leiden form) equation of state";


\[CurlyPhi]PureVirialLeiden::usage="\[CurlyPhi]PureVirialLeiden[virialCoefficient2 [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], p [bar], t [K]]:
 Pure-component fugacity coefficient of the Virial equation of state (Leiden form)";


\[CurlyPhi]MixtureVirialBerlin::usage="\[CurlyPhi]MixtureVirialBerlin[virialCoefficient2 [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\), a symmetric matrix], p [bar], t [K], y [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Mixture fugacity coefficient of the Virial equation of state (Berlin form)
 virialCoefficient2 to be supplied as symmetric matrix:
 {
  {B11, B12},
  {B21, B22}
 };";


\[CurlyPhi]MixtureVirialLeiden::usage="\[CurlyPhi]MixtureVirialLeiden[virialCoefficient2 [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\), a symmetric matrix], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], y [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], z [compressibility factor [1]]:
 Mixture fugacity coefficient of the Virial equation of state (Leiden form)
 virialCoefficient2 to be supplied as symmetric matrix:
 {
  {B11, B12},
  {B21, B22}
 };";


aCritVdW::usage="aCritVdW[tCrit [K], pCrit [bar]]:
 Parameter a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)] of the Van der Waals equation of state from critical data";


bCritVdW::usage="bCritVdW[tCrit [K], pCrit [bar]]:
 Parameter b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] of the Van der Waals equation of state from critical data";


pVdW::usage="pVdW[t [K], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Pressure [bar] of the Van der Waals equation of state";


vVdW::usage="vVdW[t [K], p [bar], a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Molar volume [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] of the Van der Waals equation of state
 returns all three solutions, including complex ones";


\[CurlyPhi]PureVdW::usage="\[CurlyPhi]PureVdW[a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], p [bar], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K]]:
 Pure component fugacity coefficient [1] of the Van der Waals equation of state";


\[CurlyPhi]MixtureVdW::usage="\[CurlyPhi]MixtureVdW[aij [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], x [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], p [bar], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K]]:
 Mixture fugacity coefficient [1] of the Van der Waals equation of state
 Parameter a to be supplied as symmetric matrix:
 {
  {a11, a12},
  {a21, a22}
 };

Parameter bPure and molar fractions to be supplied as lists:
{b1, b2}   {x1, x2}

Parameter b represents the mixture and results from a mixing rule";


aCritRK::usage="aCritRK[tCrit [K], pCrit [bar]]:
 Parameter a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)] of the Redlich-Kwong equation of state from critical data";


bCritRK::usage="bCritRK[tCrit [K], pCrit [bar]]:
 Parameter b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] of the Redlich-Kwong equation of state from critical data";


pRK::usage="pRK[t [K], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Pressure [bar] of the Redlich-Kwong equation of state";


vRK::usage="vRK[t [K], p [bar], a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Molar volume [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] of the Redlich-Kwong equation of state
 returns all three solutions, including complex ones";


\[CurlyPhi]PureRK::usage="\[CurlyPhi]PureRK[a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], p [bar], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K]]:
 Pure component fugacity coefficient [1] of the Redlich-Kwong equation of state";


\[CurlyPhi]MixtureRK::usage="\[CurlyPhi]MixtureRK[a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], aij [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], x [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], p [bar], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K]]:
 Mixture fugacity coefficient [1] of the Redlich-Kwong equation of state
 Parameter aij to be supplied as symmetric matrix:
 {
  {a11, a12},
  {a21, a22}
 };

Parameter bPure and molar fractions to be supplied as lists:
{b1, b2}   {x1, x2}

Parameters a,b represent the mixture and result from a mixing rule";


aCritSRK::usage="aCritSRK[tCrit [K], pCrit [bar]]:
 Parameter a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)] of the SRK equation of state from critical data";


bCritSRK::usage="bCritSRK[tCrit [K], pCrit [bar]]:
 Parameter b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] of the SRK equation of state from critical data";


\[Alpha]SRK::usage="\[Alpha]SRK[tRed [1], \[Omega] [1]]:
 Original \[Alpha]-function of the SRK equation of state";


pSRK::usage="pSRK[t [K], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Pressure [bar] of the SRK equation of state";


vSRK::usage="vSRK[t [K], p [bar], a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Molar volume [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] of the SRK equation of state
 returns all three solutions, including complex ones";


\[CurlyPhi]PureSRK::usage="\[CurlyPhi]PureSRK[a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], p [bar], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K]]:
 Pure component fugacity coefficient [1] of the SRK equation of state";


\[CurlyPhi]MixtureSRK::usage="\[CurlyPhi]MixtureSRK[a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], aij [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], x [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], p [bar], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K]]:
 Mixture fugacity coefficient [1] of the SRK equation of state
 Parameter aij to be supplied as symmetric matrix:
 {
  {a11, a12},
  {a21, a22}
 };

Parameter bPure and molar fractions to be supplied as lists:
{b1, b2}   {x1, x2}

Parameters a,b represent the mixture and result from a mixing rule";


aCritPR::usage="aCritPR[tCrit [K], pCrit [bar]]:
 Parameter a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)] of the Peng-Robinson equation of state from critical data";


bCritPR::usage="bCritPR[tCrit [K], pCrit [bar]]:
 Parameter b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] of the Peng-Robinson equation of state from critical data";


\[Alpha]PR::usage="\[Alpha]PR[tRed [1], \[Omega] [1]]:
 Original \[Alpha]-function of the Peng-Robinson equation of state";


pPR::usage="pPR[t [K], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Pressure [bar] of the Peng-Robinson equation of state";


vPR::usage="vPR[t [K], p [bar], a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]]:
 Molar volume [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] of the Peng-Robinson equation of state
 returns all three solutions, including complex ones";


\[CurlyPhi]PurePR::usage="\[CurlyPhi]PurePR[a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], p [bar], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K]]:
 Pure component fugacity coefficient [1] of the Peng-Robinson equation of state";


\[CurlyPhi]MixturePR::usage="\[CurlyPhi]MixturePR[a [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], aij [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], b [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], x [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], p [bar], v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K]]:
 Mixture fugacity coefficient [1] of the Peng-Robinson equation of state
 Parameter aij to be supplied as symmetric matrix:
 {
  {a11, a12},
  {a21, a22}
 };

Parameter bPure and molar fractions to be supplied as lists:
{b1, b2}   {x1, x2}

Parameters a,b represent the mixture and result from a mixing rule";


(* ::Section::Closed:: *)
(*Mixing Rules for EoS*)


aij1PVdW::usage="aij1PVdW[aPure [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], kij [1]]:
 Van der Waals mixing rule for one pairwise binary interaction parameter kij
 kij to be supplied as symmetric matrix:
 {
  {  0, k12},
  {k21,  0 }
 };";


(* ::Section::Closed:: *)
(*Saturation Pressure from EoS*)


saturationPressureVTPRLJ::usage="saturationPressureVTPRLJ[aPure [(dm^3)^2 bar mol^-2], bPure [dm^3/mol], volumeTranslationPure [dm^3/mol], t [K], pGuess [bar], \[Epsilon] [1], verbose [True or False]]
 Calculates the saturation pressure [bar] with the t-PR-LJ equation of state
 Parameter a,b,volumeTranslation to be supplied as lists {a1, a2} {b1, b2} {volumeTranslation1, volumeTranslation2}
 pGuess [bar] is an initial estimation of the result
 \[Epsilon] is an accuracy threshold, e.g., 1. 10^-5
 the logical verbose enables (True) or suppresses (False) intermediate output";


saturationPressureVdW::usage="saturationPressureVdW[aPure [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K], pGuess [bar], \[CapitalDelta]\[Phi]Abs [1], verbose [True or False]]:
 Calculates the saturation pressure [bar] with the van der Waals equation of state
 EoS parameters a,b to be supplied as lists {a1, a2} {b1, b2}
 pGuess [bar] is an initial estimation of the result
 \[CapitalDelta]\[Phi]Abs is an accuracy threshold, e.g., 1. \!\(\*SuperscriptBox[\(10\), \(-7\)]\)
 the logical verbose enables (True) or suppresses (False) intermediate output";


saturationPressureRK::usage="saturationPressureRK[aPure [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K], pGuess [bar], \[CapitalDelta]\[Phi]Abs [1], verbose [True or False]]:
 Calculates the saturation pressure [bar] with the RK equation of state
 EoS parameters a,b to be supplied as lists {a1, a2} {b1, b2}
 pGuess [bar] is an initial estimation of the result
 \[CapitalDelta]\[Phi]Abs is an accuracy threshold, e.g., 1. \!\(\*SuperscriptBox[\(10\), \(-7\)]\)
 the logical verbose enables (True) or suppresses (False) intermediate output";


saturationPressureSRK::usage="saturationPressureSRK[aPure [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K], pGuess [bar], \[CapitalDelta]\[Phi]Abs [1], verbose [True or False]]:
 Calculates the saturation pressure [bar] with the SRK equation of state
 EoS parameters a,b to be supplied as lists {a1, a2} {b1, b2}
 pGuess [bar] is an initial estimation of the result
 \[CapitalDelta]\[Phi]Abs is an accuracy threshold, e.g., 1. \!\(\*SuperscriptBox[\(10\), \(-7\)]\)
 the logical verbose enables (True) or suppresses (False) intermediate output";


saturationPressurePR::usage="saturationPressurePR[aPure [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], t [K], pGuess [bar], \[CapitalDelta]\[Phi]Abs [1], verbose [True or False]]:
 Calculates the saturation pressure [bar] with the Peng-Robinson equation of state
 EoS parameters a,b to be supplied as lists {a1, a2} {b1, b2}
 pGuess [bar] is an initial estimation of the result
 \[CapitalDelta]\[Phi]Abs is an accuracy threshold, e.g., 1. \!\(\*SuperscriptBox[\(10\), \(-7\)]\)
 the logical verbose enables (True) or suppresses (False) intermediate output";


(* ::Section::Closed:: *)
(*Bubble point and dew point calculations with EoS*)


bubblePressureVTPRLJ::usage="bubblePressureVTPRLJ[aij [(dm^3)^2 bar mol^-2], bij [dm^3/mol], volumeTranslationPure [dm^3/mol], t [K], x [mol/mol], pGuess [bar], yGuess [mol/mol], \[Epsilon] [1], verbose [True or False]]
 Returns {p,x,y,vLiq,vVap} resulting from a bubble pressure calculation for given {\!\(\*SubscriptBox[\(x\), \(i\)]\),T} with the t-PR-LJ equation of state
 Parameter aij and bij to be supplied as symmetric matrices:
 {
  {a11, a12},
  {a21, a22}
 };
 {
  {b11, b12},
  {b21, b22}
 };
 Parameter volumeTranslationPure and molar fractions to be supplied as lists:
 {volumeTranslation1, volumeTranslation2}   {x1, x2}
 pGuess [bar] is an initial estimation of the bubble pressure
 yGuess [mol mol^-1] is an initial estimation of the vapor phase composition
 \[Epsilon] is an accuracy threshold, e.g., 1. 10^-5
 the logical verbose enables (True) or suppresses (False) intermediate output";


bubblePressureSRK::usage="bubblePressureSRK[aPure [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], kij [1], t [K], x [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], pGuess [bar], yGuess [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], \[Epsilon] [1], verbose [True or False]]:
 Returns {p,x,y,vLiq,vVap} resulting from a bubble pressure calculation for given {\!\(\*SubscriptBox[\(x\), \(i\)]\),T} with the SRK equation of state
 EoS parameters a,b to be supplied as lists {a1, a2} {b1, b2}
 kij to be supplied as symmetric matrix: {{ 0, k12},{k21, 0 }}
 pGuess [bar] is an initial estimation of the bubble pressure
 yGuess [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] is an initial estimation of the vapor phase composition
 \[Epsilon] is an accuracy threshold, e.g., 1. \!\(\*SuperscriptBox[\(10\), \(-5\)]\)
 the logical verbose enables (True) or suppresses (False) intermediate output";


dewPressureSRK::usage="dewPressureSRK[aPure [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], kij [1], t [K], y [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], pGuess [bar], xGuess [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], \[Epsilon] [1], verbose [True or False]]:
 Returns {p,x,y,vLiq,vVap} resulting from a bubble pressure calculation for given {\!\(\*SubscriptBox[\(y\), \(i\)]\),T} with the SRK equation of state
 EoS parameters a,b to be supplied as lists {a1, a2} {b1, b2}
 kij to be supplied as symmetric matrix: {{ 0, k12},{k21, 0 }}
 pGuess [bar] is an initial estimation of the bubble pressure
 xGuess [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] is an initial estimation of the liquid phase composition
 \[Epsilon] is an accuracy threshold, e.g., 1. \!\(\*SuperscriptBox[\(10\), \(-5\)]\)
 the logical verbose enables (True) or suppresses (False) intermediate output";


bubblePressurePR::usage="bubblePressurePR[aPure [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], kij [1], t [K], x [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], pGuess [bar], yGuess [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], \[Epsilon] [1], verbose [True or False]]:
 Returns {p,x,y,vLiq,vVap} resulting from a bubble pressure calculation for given {\!\(\*SubscriptBox[\(x\), \(i\)]\),T} with the Peng-Robinson equation of state
 EoS parameters a,b to be supplied as lists {a1, a2} {b1, b2}
 kij to be supplied as symmetric matrix: {{ 0, k12},{k21, 0 }}
 pGuess [bar] is an initial estimation of the bubble pressure
 yGuess [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] is an initial estimation of the vapor phase composition
 \[Epsilon] is an accuracy threshold, e.g., 1. \!\(\*SuperscriptBox[\(10\), \(-5\)]\)
 the logical verbose enables (True) or suppresses (False) intermediate output";


dewPressurePR::usage="dewPressurePR[aPure [(\!\(\*SuperscriptBox[\(dm\), \(3\)]\)\!\(\*SuperscriptBox[\()\), \(2\)]\) bar \!\(\*SuperscriptBox[\(mol\), \(-2\)]\)], bPure [\!\(\*SuperscriptBox[\(dm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], kij [1], t [K], y [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], pGuess [bar], xGuess [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], \[Epsilon] [1], verbose [True or False]]:
 Returns {p,x,y,vLiq,vVap} resulting from a bubble pressure calculation for given {\!\(\*SubscriptBox[\(y\), \(i\)]\),T} with the Peng-Robinson equation of state
 EoS parameters a,b to be supplied as lists {a1, a2} {b1, b2}
 kij to be supplied as symmetric matrix: {{ 0, k12},{k21, 0 }}
 pGuess [bar] is an initial estimation of the bubble pressure
 xGuess [mol \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] is an initial estimation of the liquid phase composition
 \[Epsilon] is an accuracy threshold, e.g., 1. \!\(\*SuperscriptBox[\(10\), \(-5\)]\)
 the logical verbose enables (True) or suppresses (False) intermediate output";


(* ::Chapter:: *)
(*Function Definitions*)


Begin["`Private`"];


(* ::Section::Closed:: *)
(*Conversion of concentration measures*)


(* ::Text:: *)
(*Convert mole fractions, Subscript[x, i], to volume fractions, Subscript[\[CurlyPhi], i], using mMolar [ g mol^-1], \[Rho]Molar [mol cm^-3]*)


\[CurlyPhi]FromX=Function[{x,mMolar,\[Rho]Molar},
	((x mMolar)/\[Rho]Molar)/Total[(x mMolar)/\[Rho]Molar]
];


(* ::Text:: *)
(*Convert mass fractions, Subscript[w, i], to volume fractions, Subscript[\[CurlyPhi], i], using  mMolar [ g mol^-1], \[Rho]Molar [mol cm^-3]*)


\[CurlyPhi]FromW=Function[{w,mMolar,\[Rho]Molar},
	(w /(\[Rho]Molar mMolar))/Total[w /(\[Rho]Molar mMolar)]
];


(* ::Text:: *)
(*Convert mass fractions, Subscript[w, i], to mole fractions, Subscript[x, i], using mMolar [ g mol^-1]*)


xFromW=Function[{w,mMolar},
	(w/mMolar)/Total[w/mMolar]
];


(* ::Text:: *)
(*Convert volume fractions, Subscript[\[CurlyPhi], i], to mole fractions, Subscript[x, i], using \[Rho]Molar [mol cm^-3]*)


xFrom\[CurlyPhi]=Function[{\[CurlyPhi],\[Rho]Molar},
	\[CurlyPhi] \[Rho]Molar/Total[\[CurlyPhi] \[Rho]Molar]
];


(* ::Text:: *)
(*Convert volume fractions, Subscript[\[CurlyPhi], i], to mass fractions, Subscript[w, i], using mMolar [ g mol^-1], \[Rho]Molar [mol cm^-3]*)


wFrom\[CurlyPhi]=Function[{\[CurlyPhi],mMolar,\[Rho]Molar},
	\[CurlyPhi] mMolar \[Rho]Molar/Total[\[CurlyPhi] mMolar \[Rho]Molar]
];


(* ::Text:: *)
(*Convert mole fractions, Subscript[x, i], to mass fractions, Subscript[w, i], using mMolar [ g mol^-1]*)


wFromX=Function[{x,mMolar},
	(x mMolar)/Total[x mMolar]
];


(* ::Section::Closed:: *)
(*Lennard-Jones (LJ) Fluid*)


(* ::Text:: *)
(*Many quantities in this section use so-called 'reduced LJ units', usually designated by a superscripted asterisk: *)
(*T* = kB T / \[Epsilon]*)
(*P* = P \[Sigma]\.b3 / \[Epsilon]*)
(*\[Rho]* = \[Rho] \[Sigma]\.b3*)


(* ::Subsection::Closed:: *)
(*Critical data according to Thol 2016 and the Allen-Tildesley textbook*)


(* ::Text:: *)
(*Temperature*)


tCritLJ=1.32;(* temperature unit *)


(* ::Text:: *)
(*Pressure*)


pCritLJ=0.13;(* pressure unit *)


(* ::Text:: *)
(*Density*)


\[Rho]CritLJ=0.31;(* density unit *)


(* ::Subsection::Closed:: *)
(*Conversions*)


(* ::Text:: *)
(*Lennard-Jones parameters are given in Angstrom and kcal/mol, respectively*)


(* ::Text:: *)
(*Convert temperature t[K] to the reduced Lennard-Jones temperature T*[1]*)


With[{
	kB=kB,
	nAvogadro=nAvogadro},
	tRedLJ=Function[{t,\[Epsilon]},
		t kB nAvogadro/(4184. \[Epsilon])(* 1 *)
	]
];


(* ::Text:: *)
(*Convert the reduced Lennard-Jones temperature [1] to temperature [K]*)


With[{
	kB=kB,
	nAvogadro=nAvogadro},
	tLJ=Function[{tRedLennardJones,\[Epsilon]},
		tRedLennardJones \[Epsilon]/kB 4184./nAvogadro(* K *)
	]
];


(* ::Subsection::Closed:: *)
(*Van der Waals EoS for LJ fluids*)


(* ::Text:: *)
(*Nordholm S, Forsman J, Woodward C, Freasier B, Abbas Z: Generalized van der Waals Theory of Molecular Fluids in bulk and at Surface. Elsevier, 2019. ISBN: 978-0-12-811136-9*)


(* ::Text:: *)
(*Units:	Pressure [bar], temperature [K], molar volume [dm\.b3/mol]*)
(*		For consistency with these units, use rJoule/100. = [dm\.b3 bar / (mol K)]*)
(*		LJ-parameters \[Sigma][\[Angstrom]], \[Epsilon][kcal/mol]*)


(* ::Text:: *)
(*Pressure*)


With[{
	rJoule=rJoule},
	pVdWLJ=Function[{t,v,\[Sigma],\[Epsilon]},
		Module[{a,b,pHardSphere,pAttraction},
			a=6.02214076 16 Pi 4.184 10^-3/9 \[Epsilon] \[Sigma]^3;
			b=6.02214076 10^-4 \[Sigma]^3;
			pHardSphere=rJoule 10^-2 t/(v-b);(* bar *)
			pAttraction=-a/v^2;(* bar *)
			{pHardSphere,pAttraction,pHardSphere+pAttraction}
		]
	]
];


(* ::Text:: *)
(*Molar volume*)


With[{
	rJoule=rJoule},
	vVdWLJ=Function[{t,p,\[Sigma],\[Epsilon]},
		Module[{a,b,pHardSphere,pAttraction},
			a=6.02214076 16 Pi 4.184 10^-3/9 \[Epsilon] \[Sigma]^3;
			b=6.02214076 10^-4 \[Sigma]^3;
			pHardSphere=rJoule 10^-2 t/(v-b);(* bar *)
			pAttraction=-a/v^2;(* bar *)
			NSolve[pHardSphere+pAttraction-p==0.,v,Reals]
		]
	]
];


(* ::Subsection::Closed:: *)
(*Volume-translated Peng-Robinson EoS for LJ fluids (VTPRLJ)*)


(* ::Text:: *)
(*Harismiadis V I, Panagiotopoulos A Z, Tassios D P: Phase equilibria of binary Lennard-Jones mixtures with cubic equations of state. Fluid Phase Equilibria 1994, 94, 1-18.*)


(* ::Text:: *)
(*Combining rules used for mixtures:*)
(*a =\[Sum]\[Sum]Subscript[x, i] Subscript[x, j] Subscript[a, ij]*)
(*b =\[Sum]\[Sum]Subscript[x, i] Subscript[x, j] Subscript[b, ij]*)
(*t =\[Sum]Subscript[x, i] Subscript[t, i]*)
(**)
(*Subscript[a, ij]=(Subscript[a, i] Subscript[a, j])^(1/2) Subscript[b, ij]/((Subscript[b, i] Subscript[b, j])^(1/2)) *)
(*Subscript[b, ij]^(1/3)=1/2 (Subscript[b, i]^(1/3)+Subscript[b, j]^(1/3))*)


(* ::Text:: *)
(*Critical data for the VTPRLJ*)


tCritVTPRLJ=1.316;(* Smith  1.316 *)


pCritVTPRLJ=0.13;(* Lotfi 0.13 *)


\[Rho]CritVTPRLJ=0.304;(* Smith 0.304 *)


(* ::Text:: *)
(*Reduced Temperature for the t-PR-LJ EoS	Subscript[T, r] = SuperStar[T]/SuperStar[Subscript[T, C]]*)


With[{
	tCritVTPRLJ=tCritVTPRLJ},
	tRedVTPRLJ=Function[{t,\[Epsilon]},
		Evaluate[tRedLJ[t,\[Epsilon]]/tCritVTPRLJ]
	]
];


(* ::Text:: *)
(*Parameters a, b and volume translation. \[Sigma] [\[Angstrom]] , \[Epsilon] [kcal/mol]*)


With[{
	kB=kB,
	nAvogadro=nAvogadro,
	rJoule=rJoule,
	tCritVTPRLJ=tCritVTPRLJ,
	pCritVTPRLJ=pCritVTPRLJ},
	aVTPRLJ=Function[{tRed,\[Sigma],\[Epsilon]},
		Module[{tC,pC},
			tC=tCritVTPRLJ*\[Epsilon]/kB (* 1 *)/nAvogadro*4184.;(* K *)
			pC=pCritVTPRLJ*\[Epsilon]/\[Sigma]^3 (* 1 *)/nAvogadro*4184. *10^30(*Pa*) *10^-5 (* bar *);
			0.45724*(rJoule/100.)^2*tC^2/pC* (1.+0.384401(1.-tRed^0.5))^2 (* (dm^3)^2 bar mol^-2 *)
		]
	]
];


With[{
	kB=kB,
	nAvogadro=nAvogadro,
	rJoule=rJoule,
	tCritVTPRLJ=tCritVTPRLJ,
	pCritVTPRLJ=pCritVTPRLJ},
	bVTPRLJ=Function[{\[Sigma],\[Epsilon]},
		Module[{tC,pC},
			tC=tCritVTPRLJ*\[Epsilon]/kB (* 1 *)/nAvogadro*4184.;(* K *)
			pC=pCritVTPRLJ*\[Epsilon]/\[Sigma]^3 (* 1 *)/nAvogadro*4184. *10^30(* Pa *) *10^-5 (* bar *);
			0.07780*(rJoule/100.)*tC/pC(* dm^3 mol^-1 *)
		]
	]
];


With[{
	nAvogadro=nAvogadro,
	tCritVTPRLJ=tCritVTPRLJ,
	pCritVTPRLJ=pCritVTPRLJ,
	\[Rho]CritVTPRLJ=\[Rho]CritVTPRLJ},
	volumeTranslationVTPRLJ=Function[{tRed,\[Sigma]},
		Module[{\[Alpha],\[Beta],zcPR,\[CapitalDelta]V},
			\[Alpha]=0.245029;(* Parameter *)
			\[Beta]=-25.6632;(* Parameter *)
			zcPR=0.3074;(* Critical compressability factor from PR-EoS *)
			\[CapitalDelta]V=\[Alpha]+(zcPR*tCritVTPRLJ/pCritVTPRLJ-1./\[Rho]CritVTPRLJ-\[Alpha])*Exp[\[Beta]*Abs[1.-tRed]];(* 1 *)
			(-1)*nAvogadro*(\[Sigma]^3)*\[CapitalDelta]V (* A^3 mol^-1 *) *10^-27 (* dm^3 mol^-1 *) 
		]
	]
];


(* ::Text:: *)
(*Pressure*)


With[{
	rJoule=rJoule},
	pVTPRLJ=Function[{t,v,a,b,volumeTranslation},
		(rJoule/100.) t/(v+volumeTranslation-b)-a/((v+volumeTranslation)*(v+volumeTranslation+b)+b*(v+volumeTranslation-b))
	]
];


(* ::Text:: *)
(*Molar volume*)


With[{
	rJoule=rJoule},
	vVTPRLJ=Function[{t,p,a,b,volumeTranslation},
		NSolve[((rJoule/100.) t/(v+volumeTranslation-b)-a/((v+volumeTranslation)*(v+volumeTranslation+b)+b*(v+volumeTranslation-b)))==p,v]
	]
];


(* ::Text:: *)
(*Pure component fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]PureVTPRLJ=Function[{a,b,volumeTranslation,p,v,t},
		Exp[Log[\[CurlyPhi]PurePR[a,b,p,v,t]]+volumeTranslation p/((rJoule/100.)t)]
	]
];


(* ::Text:: *)
(*Mixture fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]MixtureVTPRLJ=Function[{aij,bij,volumeTranslationPure,x,p,v,t},
		Module[{nComp,z,a,b,\[CurlyPhi]MixturePRadvanced},
			nComp=Length[x];
			z=p v/((rJoule/100.) t);
			a = Sum[Sum[x[[i]]x[[j]]aij[[i,j]],{i,1,Length@x}],{j,1,Length@x}];
			b = Sum[Sum[x[[i]]x[[j]]bij[[i,j]],{i,1,Length@x}],{j,1,Length@x}];
			\[CurlyPhi]MixturePRadvanced = Table[
				Exp[(2*Sum[x[[j]]bij[[j,i]],{j,1,nComp}]-b)/b(z-1)-Log[p (v-b)/(rJoule/100. t)]-
				2.*a/(2. Sqrt[2.] b (rJoule/100.) t)(Sum[x[[j]] aij[[j,i]],{j,nComp}]/a-Sum[x[[j]]bij[[j,i]],{j,1,nComp}]/b+0.5)
				*Log[(v+(1.+Sqrt[2.])b)/(v+(1.-Sqrt[2.])b)]],
				{i,nComp}];
			Exp[Log[\[CurlyPhi]MixturePRadvanced]+p/((rJoule/100.)*t)*volumeTranslationPure]
		]
	]
];


(* ::Subsection::Closed:: *)
(*Second virial coefficient for LJ fluids*)


(* ::Text:: *)
(*[dm\.b3/mol]*)
(*Nordholm S, Forsman J, Woodward C, Freasier B, Abbas Z: Generalized van der Waals Theory of Molecular Fluids in bulk and at Surface. Elsevier, 2019. ISBN: 978-0-12-811136-9*)
(*Chapter 6.1.2.1*)


b2LJ=Function[{t,\[Sigma],\[Epsilon]},
	(2. Pi 6.02214076)/3. 10^-1 \[Sigma]^3-((16. Pi 4.184)/(9 1.380649)) (10^2) (\[Epsilon]/t) (\[Sigma]^3)/1000.(* dm^3 mol^-1 *)
];


(* ::Subsection::Closed:: *)
(*Thol 2016 fundamental equation and derivatives*)


(* ::Text:: *)
(*Thol 2016 Equation of State for the Lennard-Jones Fluid. J. Phys. Chem. Ref. Data 2016, 45(2), 023101-1 - 023101-36.*)
(*Valid for 0.5 < T/Subscript[T, c] < 7, p/Subscript[p, c] < 500; \[Tau] = Subscript[T, c]/T; \[Delta] = \[Rho] / Subscript[\[Rho], c]*)


(* ::Subsubsection::Closed:: *)
(*Reduced saturation pressure*)


(* ::Item:: *)
(*tRed = t* / tCritLJ*)


pSatRedLJ=Function[{tRed},
	Module[{n,t},
		n={-5.4000,0.44704,-1.8530,0.19890,-1.1250};
		t={1.0,1.5,4.7,2.5,21.4};
		Exp[Total[n (1.-tRed)^t]/tRed]
	]
];


(* ::Subsubsection::Closed:: *)
(*Reduced saturation densities*)


(* ::Item:: *)
(*tRed = t* / tCritLJ*)


(* ::Text:: *)
(*Reduced saturated liquid density*)


\[Rho]SatLiqRedLJ=Function[{tRed},
	Module[{n,t},
		n={1.362,2.093,-2.110,0.329,1.410};
		t={0.313,0.940,1.630,17.,2.4};
		1.+Total[n (1.-tRed)^t]
	]
];


(* ::Text:: *)
(*Reduced saturated vapor density*)


\[Rho]SatVapRedLJ=Function[{tRed},
	Module[{n,t},
		n={-6.9655,-103.31,-2.0325,-44.481,-18.463,-260.70};
		t={1.320,19.240,0.360,8.780,4.040,41.60};
		Exp[Total[n (1.-tRed)^t]]
	]
];


(* ::Subsubsection::Closed:: *)
(*Fundamental equation (Helmholtz free energy)*)


(* ::Text:: *)
(*Parameters for the ideal gas contribution*)


c1=-1.515151515;
c2=6.262265814;


(* ::Text:: *)
(*Parameters for the residual contribution (Table 2)*)


n={0.0052080730,2.1862520,-2.1610160,1.4527000,-2.0417920,0.18695286,-0.090988445,-0.49745610,0.10901431,-0.80055922,-0.56883900,-0.62086250,-1.4667177,1.8914690,-0.13837010,-0.38696450,0.12657020,0.60578100,1.1791890,-0.47732679,-9.9218575,-0.57479320,0.0037729230};
t={1.000,0.320,0.505,0.672,0.843,0.898,1.294,2.590,1.786,2.770,1.786,1.205,2.830,2.548,4.650,1.385,1.460,1.351,0.660,1.496,1.830,1.616,4.970};
d={4.,1.,1.,2.,2.,3.,5.,2.,2.,3.,1.,1.,1.,1.,2.,3.,3.,2.,1.,2.,3.,1.,1.};
l={0.,0.,0.,0.,0.,0.,1.,2.,1.,2.,2.,1.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
\[Eta]={0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,2.067,1.522,8.820,1.722,0.679,1.883,3.925,2.461,28.20,0.753,0.820};
\[Beta]={0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.625,0.638,3.910,0.156,0.157,0.153,1.160,1.730,383.0,0.112,0.119};
\[Gamma]={0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.710,0.860,1.940,1.480,1.490,1.945,3.020,1.110,1.170,1.330,0.240};
\[Epsilon]={0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.2053,0.4090,0.6000,1.2030,1.8290,1.3970,1.3900,0.5390,0.9340,2.3690,2.4300};


(* ::Text:: *)
(*The term \[Alpha]^0(\[Tau],\[Delta]) relates to a hypothetical ideal gas, while \[Alpha]^r(\[Tau],\[Delta]) represents the residual Helmholtz energy under full intermolecular interactions in the fluid.*)


\[Alpha]0=Log[\[Delta]]+1.5 Log[\[Tau]]+c1 \[Tau]+c2;
\[Alpha]r=Sum[n[[i]] \[Delta]^d[[i]] \[Tau]^t[[i]],{i,1,6}]+Sum[n[[i]] \[Delta]^d[[i]] \[Tau]^t[[i]] Exp[-\[Delta]^l[[i]]],{i,7,12}]+Sum[n[[i]] \[Delta]^d[[i]] \[Tau]^t[[i]] Exp[-\[Eta][[i]] (\[Delta]-\[Epsilon][[i]])^2-\[Beta][[i]](\[Tau]-\[Gamma][[i]])^2],{i,13,23}];


(* ::Subsubsection::Closed:: *)
(*Quantities derived from the fundamental equation*)


(* ::Text:: *)
(*Ideal gas dimensionless internal energy of the LJ-fluid*)


uLJidealThol2016=Function[{tAsterisk,\[Rho]Asterisk},
	Evaluate[
		ReplaceAll[{\[Tau]->1.32/tAsterisk,\[Delta]->\[Rho]Asterisk/0.31}][\[Tau] D[\[Alpha]0,\[Tau]]] tAsterisk
	](* u^* *)
];


(* ::Text:: *)
(*Configurational dimensionless internal energy of the LJ-fluid*)


uLJconfigThol2016=Function[{tAsterisk,\[Rho]Asterisk},
	Evaluate[
		ReplaceAll[{\[Tau]->1.32/tAsterisk,\[Delta]->\[Rho]Asterisk/0.31}][\[Tau] D[\[Alpha]r,\[Tau]]] tAsterisk
	](* u^* *)
];


(* ::Text:: *)
(*Total dimensionless internal energy of the LJ-fluid (ideal gas + configurational)*)


uLJtotalThol2016=Function[{tAsterisk,\[Rho]Asterisk},
	Evaluate[
		ReplaceAll[{\[Tau]->(1.32/tAsterisk),\[Delta]->\[Rho]Asterisk/0.31}][\[Tau] D[\[Alpha]0+\[Alpha]r,\[Tau]]] tAsterisk
	](* u^* *)
];


(* ::Text:: *)
(*Compressibility factor*)


zLJThol2016=Function[{tAsterisk,\[Rho]Asterisk},
	Evaluate[
		ReplaceAll[{\[Tau]->(1.32/tAsterisk),\[Delta]->\[Rho]Asterisk/0.31}][1.+\[Delta] D[\[Alpha]r,\[Delta]]]
	](* z = p V R^-1T^-1 *)
];


(* ::Text:: *)
(*Clean up*)


Clear[c1,c2,n,t,d,l,\[Eta],\[Beta],\[Gamma],\[Epsilon],\[Alpha]0,\[Alpha]r];


(* ::Subsection::Closed:: *)
(*Mecke 1996 fundamental equation and derivatives*)


(* ::Text:: *)
(*Mecke M, M\[UDoubleDot]ller A, Winkelmann J, Vrabec J, Fischer J, Span R, Wagner W. An accurate van der Waals-type equation of state for the Lennard-Jones fluid. Int J Thermophysics 1996, 17(2), 391-404; Erratum 1. Int J Thermophysics 1998, 19(5), 1493.*)


(* ::Text:: *)
(*Table III. Coefficients for the attractive part of the helmholtz energy, fART*)


c={0.33619760720 10^-5,-0.14707220591 10^+1,-0.11912121043 10^0,-0.11350363539 10^-4,-0.26778688896 10^-4,0.12755936511 10^-5,0.40088615477 10^-2,0.52305580273 10^-5,-0.10214454556 10^-7,-0.14526799362 10^-1,0.64975356409 10^-1,-0.60304755494 10^-1,-0.14925537332 10^0,-0.31664355868 10^-3,0.28312781935 10^-1,0.13039603845 10^-3,0.10121435381 10^-1,-0.15425936014 10^-4,-0.61568007279 10^-1,0.76001994423 10^-2,-0.18906040708 10^0,0.33141311846 10^0,-0.25229604842 10^0,0.13145401812 10^0,-0.48672350917 10^-1,0.14756043863 10^-2,-0.85996667747 10^-2,0.33880247915 10^-1,0.69427495094 10^-2,-0.22271531045 10^-7,-0.22656880018 10^-3,0.24056013779 10^-2};
m={-2.0,-1.0,-1.0,-1.0,-0.5,-0.5,0.5,0.5,1.0,-5.0,-4.0,-2.0,-2.0,-2.0,-1.0,-1.0,0.0,0.0,-5.0,-4.0,-3.0,-2.0,-2.0,-2.0,-1.0,-10.0,-6.0,-4.0,0.0,-24.0,-10.0,-2.0};
n={9.,1.,2.,9.,8.,10.,1.,7.,10.,1.,1.,1.,2.,8.,1.,10.,4.,9.,2.,5.,1.,2.,3.,4.,2.,3.,4.,2.,2.,5.,2.,10.};
p={0.,0.,0.,0.,0.,0.,0.,0.,0.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.,-1.};
q={0.,0.,0.,0.,0.,0.,0.,0.,0.,1.,1.,1.,1.,1.,1.,1.,1.,1.,2.,2.,2.,2.,2.,2.,2.,3.,3.,3.,3.,4.,4.,4.};


(* ::Text:: *)
(*Packing fraction, approximated by a function of temperature and density, eq (3)*)


\[Zeta]=0.1617(\[Rho]Asterisk/0.3107)(0.689+0.311(tAsterisk/1.328)^0.3674)^-1;


(* ::Text:: *)
(*Hard-sphere residual Helmholtz energy as a function of the packing fraction, eq (2)*)


fHRT=(4. \[Zeta]-3. \[Zeta]^2)/(1.-\[Zeta])^2;


(* ::Text:: *)
(*Attractive residual Helmholtz energy, eq (4)*)


fART=Total[c (tAsterisk/1.328)^m (\[Rho]Asterisk/0.3107)^n Exp[p (\[Rho]Asterisk/0.3107)^q]];


(* ::Text:: *)
(*Total residual Helmholtz energy, eq (1)*)


fRT=fHRT+fART;


(* ::Text:: *)
(*Configurational dimensionless internal energy of the LJ-fluid*)


uLJconfigMecke1996=Function[{tAsterisk,\[Rho]Asterisk},
	Evaluate[
		ReplaceAll[z->tAsterisk^-1][D[ReplaceAll[tAsterisk-> z^-1][fRT],z]]
	]
];


(* ::Text:: *)
(*Clean up*)


Clear[c,m,n,p,q,\[Zeta],fHRT,fART,fRT];


(* ::Subsection::Closed:: *)
(*Kolafa 1994 fundamental equation and derivatives*)


(* ::Text:: *)
(*Kolafa J, Nezbeda I: The Lennard-Jones fluid: An accurate analytic and theoretically-based equation of state. Fluid Phase Equilibria 1994, 100, 1-34*)


(* ::Text:: *)
(*Coefficients for the hard-sphere parameter approximations, Table 2*)


dhBH=0.011117524 t^(-2./2.)+(-0.076383859) t^(-1./2.)+1.080142248 t^(0./2.)+0.000693129 t^(1./2.)+(-0.063920968) Log[t];
\[CapitalDelta]B2hBH=(-0.58544978) t^(-7./2.)+0.43102052 t^(-6./2.)+0.87361369 t^(-5./2.)+(-4.13749995) t^(-4./2.)+2.90616279 t^(-3./2.)+(-7.02181962) t^(-2./2.)+0.02459877 t^(0./2.);


(* ::Text:: *)
(*Partial derivatives of above coefficients with respect to 1/T, eq (33)*)


dhBHDerivative=-t((-2./2.) 0.011117524 t^(-2./2.)+(-1./2.) (-0.076383859) t^(-1./2.)+(0./2.) (1.080142248) t^(0./2.)+(1./2.) 0.000693129 t^(1./2.)+(-0.063920968) );
\[CapitalDelta]B2hBHDerivative=-t((-7./2.) (-0.58544978)t^(-7./2.)+(-6./2.) 0.43102052 t^(-6./2.)+(-5./2.)0.87361369 t^(-5./2.)+(-4./2.) (-4.13749995) t^(-4./2.)+(-3./2.)2.90616279 t^(-3./2.)+(-2./2.)(-7.02181962) t^(-2./2.)+(0./2.)0.02459877 t^(0./2.));


(* ::Text:: *)
(*Approximation of the hard-sphere packing fraction \[Eta]*)


\[Eta]=Pi \[Rho] dhBH^3/6.;


(* ::Text:: *)
(*Hard-sphere compressibility factor, eq (4)*)


zHS=(1.+\[Eta]+\[Eta]^2-2. \[Eta]^3 (1.+\[Eta])/3.)/(1.-\[Eta])^3;


(* ::Text:: *)
(*Hard-sphere Helmholtz free energy, eq (5)*)


aHS=t(5. Log[1.-\[Eta]]/3.+\[Eta] (34.-33. \[Eta]+4. \[Eta]^2)/(6.(1.-\[Eta])^2));


(* ::Text:: *)
(*Table 3*)


i={0.,0.,0.,0.,-1.,-1.,-1.,-1.,-1.,-2.,-2.,-2.,-2.,-2.,-4.,-4.,-4.,-4.,-4.};
j={2.,3.,4.,5.,2.,3.,4.,5.,6.,2.,3.,4.,5.,6.,2.,3.,4.,5.,6.};
cij={2.01546797,-28.17881636,28.28313847,-10.42402873,-19.58371655,75.62340289,-120.70586598,93.92740328,-27.37737354,29.34470520,-112.35356937,170.64908980,-123.06669187,34.42288969,-13.37031968,65.38059570,-115.09233113,88.91973082,-25.62099890};
\[Gamma]=1.92907278;


(* ::Text:: *)
(*Resulting LJ-fluid compressibility factor, eq (31), a function of {t*, \[Rho]*}*)


z=zHS+\[Rho](1.-2. \[Gamma] \[Rho]^2) Exp[-\[Gamma] \[Rho]^2]\[CapitalDelta]B2hBH+Total[j cij t^(i/2.-1.) \[Rho]^j];


zLJKolafa1994=Function[{t,\[Rho]},
	Evaluate[z]
];


(* ::Text:: *)
(*Resulting LJ-fluid internal energy, eq (32), a function of {t*, \[Rho]*}*)


uLJconfigKolafa1994=Function[{t,\[Rho]},
	Evaluate[
		3.(zHS-1.)/dhBH dhBHDerivative+\[Rho] Exp[-\[Gamma] \[Rho]^2]\[CapitalDelta]B2hBHDerivative-Total[(i/2.-1.) cij t^(i/2.) \[Rho]^j]
	]
];


(* ::Text:: *)
(*Clean up*)


Clear[dhBH,\[CapitalDelta]B2hBH,dhBHDerivative,\[CapitalDelta]B2hBHDerivative,\[Eta],zHS,aHS,t,i,j,cij,\[Gamma],z,u];


(* ::Subsection::Closed:: *)
(*Nicolas 1979 EoS for LJ fluids*)


(* ::Text:: *)
(*Nicolas 1979 Equation of state for the Lennard-Jones fluid. Molecular Physics 1979, 37(5), 1429-1454.*)


(* ::Text:: *)
(*Compressibility factor*)


zLJNicolas1979=Function[{tAsterisk,\[Rho]Asterisk},
	Module[{x,\[Gamma]},
		x={-0.044480725,7.2738221,-14.343368,3.8397096,-2.0057745,1.9084472,-5.7441787,25.110073,-4523.2787,0.0089327162,9.8163358,-61.434572,14.161454,43.353841,1107.8327,-35.429519,10.591298,497.70046,-353.38542,4503.6093,7.7805296,13567.114,-8.5818023,16646.578,-14.092234,19386.911,38.585868,3380.0371,-185.67754,8487.4693,97.508689,-14.483060};
		\[Gamma]=3.;(* Table 4 *)
		(\[Rho]Asterisk tAsterisk+\[Rho]Asterisk^2 (x[[1]] tAsterisk+x[[2]] tAsterisk^0.5+x[[3]]+x[[4]] tAsterisk^-1+x[[5]] tAsterisk^-2)
		+\[Rho]Asterisk^3 (x[[6]] tAsterisk+x[[7]]+x[[8]] tAsterisk^-1+x[[9]] tAsterisk^-2)
		+\[Rho]Asterisk^4 (x[[10]] tAsterisk+x[[11]]+x[[12]] tAsterisk^-1)+\[Rho]Asterisk^5 x[[13]]+\[Rho]Asterisk^6 (x[[14]] tAsterisk^-1+x[[15]] tAsterisk^-2)
		+\[Rho]Asterisk^7 x[[16]] tAsterisk^-1+\[Rho]Asterisk^8 (x[[17]] tAsterisk^-1+x[[18]] tAsterisk^-2)+\[Rho]Asterisk^9 x[[19]] tAsterisk^-2
		+\[Rho]Asterisk^3 (x[[20]] tAsterisk^-2+x[[21]] tAsterisk^-3) Exp[-\[Gamma] \[Rho]Asterisk^2]
		+\[Rho]Asterisk^5 (x[[22]] tAsterisk^-2+x[[23]] tAsterisk^-4) Exp[-\[Gamma] \[Rho]Asterisk^2]
		+\[Rho]Asterisk^7 (x[[24]] tAsterisk^-2+x[[25]] tAsterisk^-3) Exp[-\[Gamma] \[Rho]Asterisk^2]
		+\[Rho]Asterisk^9 (x[[26]] tAsterisk^-2+x[[27]] tAsterisk^-4) Exp[-\[Gamma] \[Rho]Asterisk^2]
		+\[Rho]Asterisk^11 (x[[28]] tAsterisk^-2+x[[29]] tAsterisk^-3) Exp[-\[Gamma] \[Rho]Asterisk^2]
		+\[Rho]Asterisk^13 (x[[30]] tAsterisk^-2+x[[31]]tAsterisk^-3+x[[32]]tAsterisk^-4)Exp[-\[Gamma] \[Rho]Asterisk^2])/(\[Rho]Asterisk tAsterisk)(* Eq (3) *)
	]
];


(* ::Section::Closed:: *)
(*Saturation Pressure*)


(* ::Subsection::Closed:: *)
(*Antoine*)


(* ::Text:: *)
(*Pressure-explicit*)


pAntoine=Function[{t,par,log},
	If[log=="ln",
		Exp[par[[1]]-par[[2]]/(par[[3]]+t)],(* ln *)
		10^(par[[1]]-par[[2]]/(par[[3]]+t))(* log10 *)
	]
];


(* ::Text:: *)
(*Temperature-explicit*)


tAntoine=Function[{p,par,log},
	If[log=="ln",
		par[[2]]/(par[[1]]-Log[p])-par[[3]],(* ln *)
		par[[2]]/(par[[1]]-Log10[p])-par[[3]](* log10 *)
	]
];


(* ::Subsection::Closed:: *)
(*Extended Antoine used by ASPEN*)


(* ::Text:: *)
(*Source: ASPEN Physical Property Methods and Models 11.1*)


pExtendedAntoine=Function[{t,plxant},
	Exp[plxant[[1]]+plxant[[2]]/(t+plxant[[3]])+plxant[[4]] t+plxant[[5]] Log[t]+plxant[[6]] t^plxant[[7]]]
];


(* ::Subsection::Closed:: *)
(*Harlacher*)


(* ::Text:: *)
(*Also called: Frost-Kalkwarf-Thodos equation*)


(* ::Text:: *)
(*Pressure-explicit*)


pHarlacher=Function[{t,pGuess,par,log},
	If[log=="ln",
		FindRoot[par[[1]]+par[[2]]/t+par[[3]] Log[t]+(par[[4]] p)/t^2-Log[p],{p,pGuess}][[1,2]],(* ln *)
		FindRoot[par[[1]]+par[[2]]/t+par[[3]] Log[t]+(par[[4]] p)/t^2-Log10[p],{p,pGuess}][[1,2]](* log10 *)
	]
];


(* ::Text:: *)
(*Temperature-explicit*)


tHarlacher=Function[{p,tGuess,par,log},
	If[log=="ln",
		FindRoot[par[[1]]+par[[2]]/t+par[[3]] Log[t]+(par[[4]] p)/t^2-Log[p],{t,tGuess}][[1,2]],(* ln *)
		FindRoot[par[[1]]+par[[2]]/t+par[[3]] Log[t]+(par[[4]] p)/t^2-Log10[p],{t,tGuess}][[1,2]](* log10 *)
	]
];


(* ::Section::Closed:: *)
(*Thermal Equations of State (EoS)*)


(* ::Text:: *)
(*Units:	Pressure [bar], temperature [K], molar volume [dm\.b3/mol]*)
(*		For consistency with these units,  rJoule/100. = [dm\.b3 bar / (mol K)] is used as the gas constant.*)
(*		*)
(*Parameters and molar fractions are to be provided as matrix or list, respectively. For example:*)
(**)
(*		a11 a12 a13	{b1, b2, b3}	   0    k12   k13*)
(*		a21 a22 a23				k21     0     k23*)
(*		a31 a32 a33				k31  k32     0  *)
(**)
(*where a11, a22, a33 represent the pure-component parameters.*)


(* ::Subsection::Closed:: *)
(*Ideal gas*)


(* ::Text:: *)
(*Pressure-explicit*)


With[{
	rJoule=rJoule},
	pIdealGas=Function[{t,v},
		rJoule/100. t/v (* bar *)
	]
];


(* ::Text:: *)
(*Volume-explicit*)


With[{
	rJoule=rJoule},
	vIdealGas=Function[{t,p},
		rJoule/100. t/p (* dm^3 mol^-1 *)
	]
];


(* ::Text:: *)
(*Temperature-explicit*)


With[{
	rJoule=rJoule},
	tIdealGas=Function[{p,v},
		p v/(rJoule/100.) (* K *)
	]
];


(* ::Subsection::Closed:: *)
(*Virial equation*)


(* ::Text:: *)
(*Virial coefficients to be supplied in [dm\.b3/mol]*)


(* ::Text:: *)
(*Molar volume-explicit*)


With[{
	rJoule=rJoule},
	vVirialLeiden=Function[{virialCoefficient2,t,p},
		Module[{a,b,c},
			a=p/(rJoule/100. t);
			b=-1.;
			c=-virialCoefficient2;
			{(-b-Sqrt[b^2-4. a c])/(2. a),(-b+Sqrt[b^2-4. a c])/(2. a)}
		]
	]
];


(* ::Text:: *)
(*Pure component fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]PureVirialLeiden=Function[{virialCoefficient2,p,t},
		Exp[virialCoefficient2 p/(rJoule/100. t)]
	]
];


(* ::Text:: *)
(*Mixture fugacity coefficient (derived from Berlin-form)	*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]MixtureVirialBerlin=Function[{virialCoefficient2,p,t,y},
		Module[{virialBtot=y[[1]]^2 virialCoefficient2[[1,1]]+2 y[[1]] y[[2]] virialCoefficient2[[1,2]]+y[[2]]^2 virialCoefficient2[[2,2]]},
			{
			Exp[(2(y[[1]] virialCoefficient2[[1,1]]+y[[2]] virialCoefficient2[[2,1]])-virialBtot) (p 100.)/(rJoule t)],
			Exp[(2(y[[1]] virialCoefficient2[[1,2]]+y[[2]] virialCoefficient2[[2,2]])-virialBtot) (p 100.)/(rJoule t)]
			}
		]
	]
];


(* ::Text:: *)
(*Mixture fugacity coefficient (derived from Leiden-form)*)


\[CurlyPhi]MixtureVirialLeiden=Function[{virialCoefficient2,v,y,z},
	{
	Exp[-Log[z]+2./v (y[[1]] virialCoefficient2[[1,1]]+y[[2]] virialCoefficient2[[2,1]])],
	Exp[-Log[z]+2./v (y[[1]] virialCoefficient2[[1,2]]+y[[2]] virialCoefficient2[[2,2]])]
	}
];


(* ::Subsection::Closed:: *)
(*Van der Waals*)


(* ::Text:: *)
(*Parameters {a, b}*)


With[{
	rJoule=rJoule},
	aCritVdW=Function[{tCrit,pCrit},
		27.(rJoule/100.)^2 tCrit^2/(64. pCrit)
	]
];


With[{
	rJoule=rJoule},
	bCritVdW=Function[{tCrit,pCrit},
		(rJoule/100.)tCrit/(8. pCrit)
	]
];


(* ::Text:: *)
(*Pressure*)


With[{
	rJoule=rJoule},
	pVdW=Function[{t,v,a,b},
		(rJoule/100.) t/(v-b)-a/v^2
	]
];


(* ::Text:: *)
(*Molar volume*)


With[{
	rJoule=rJoule},
	vVdW=Function[{t,p,a,b},
		Values[
			Flatten[
				NSolve[v^3+(-b-((rJoule/100.) t)/p)v^2+a/p v+((-a b)/p)==0.,v]
			]
		]
	]
];


(* ::Text:: *)
(*Pure component fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]PureVdW=Function[{a,b,p,v,t},
		Module[{z=p v/(rJoule/100. t)},
			Exp[z-1.-Log[z(1.-b/v)]-a/(rJoule/100. t v)]
		]
	]
];


(* ::Text:: *)
(*Mixture fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]MixtureVdW=Function[{aij,b,bPure,x,p,v,t},
		Module[{i,j},
			Table[
				Exp[Log[v/(v-b)]+bPure[[i]]/(v-b)-Log[(p v)/(rJoule 10^-2 t)]-2 Sum[x[[j]] aij[[j,i]],{j,Length[bPure]}]/(rJoule 10^-2 t v)],{i,Length[bPure]}
			]
		](* 1 *)
	]
];


(* ::Subsection::Closed:: *)
(*Redlich-Kwong (RK)*)


(* ::Text:: *)
(*Parameters*)


With[{
	rJoule=rJoule},
	aCritRK=Function[{tCrit,pCrit},
		0.42748 (rJoule/100.)^2 tCrit^2.5/pCrit
	]
];


With[{
	rJoule=rJoule},
	bCritRK=Function[{tCrit,pCrit},
		0.08664 (rJoule/100.) tCrit/pCrit
	]
];


(* ::Text:: *)
(*Pressure*)


With[{
	rJoule=rJoule},
	pRK=Function[{t,v,a,b},
		(rJoule/100.) t/(v-b)-a/(Sqrt[t] v(v+b))
	]
];


(* ::Text:: *)
(*Molar volume*)


With[{
	rJoule=rJoule},
	vRK=Function[{t,p,a,b},
		Values[
			Flatten[
				NSolve[v^3+v^2(-rJoule/100. t/p)+v (a/Sqrt[t]-p b^2-rJoule/100. t b)/p+(-a b)/(Sqrt[t]p)==0.,v]
			]
		]
	]
];


(* ::Text:: *)
(*Pure component fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]PureRK=Function[{a,b,p,v,t},
		Module[{z=p v/(rJoule/100. t)},
			Exp[z-1.-Log[z(1.-b/v)]-a/(rJoule/100. t^(3/2) b) Log[1.+b/v]]
		]
	]
];


(* ::Text:: *)
(*Mixture fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]MixtureRK=Function[{a,aij,b,bPure,x,p,v,t},
		Module[{i,j,nComp},
			nComp=Length[bPure];
			Table[
				Exp[Log[v/(v-b)]+bPure[[i]]/(v-b)-Log[(p v)/(rJoule 10^-2 t)]-2 Sum[x[[j]] aij[[j,i]],{j,nComp}]/(rJoule 10^-2 t^(2/3) b) Log[(v+b)/v]+(a bPure[[i]])/(rJoule 10^-2 t^(2/3) b^2) (Log[(v+b)/v]-b/(v+b))],{i,nComp}
			]
		](* 1 *)
	]
];


(* ::Subsection::Closed:: *)
(*Soave-Redlich-Kwong (SRK)*)


(* ::Text:: *)
(*Parameters*)


With[{
	rJoule=rJoule},
	aCritSRK=Function[{tCrit,pCrit},
		0.42748 (rJoule/100.)^2 tCrit^2/pCrit (* (dm^3)^2 bar mol^-2 *)
	]
];


With[{
	rJoule=rJoule},
	bCritSRK=Function[{tCrit,pCrit},
		0.08664 (rJoule/100.) tCrit/pCrit (* dm^3 mol^-1 *)
	]
];


\[Alpha]SRK=Function[{tRed,\[Omega]},
	(1+(0.48+1.574 \[Omega]-0.176 \[Omega]^2) (1.-tRed^0.5))^2 (* 1 *)
];


(* ::Text:: *)
(*Pressure*)


With[{
	rJoule=rJoule},
	pSRK=Function[{t,v,a,b},
		(rJoule/100.) t/(v-b)-a/(v(v+b)) (* bar *)
	]
];


(* ::Text:: *)
(*Molar volume*)


With[{
	rJoule=rJoule},
	vSRK=Function[{t,p,a,b},
		Values[
			Flatten[
				NSolve[v^3-(rJoule/100. t)/p v^2+(a/p-(b rJoule/100. t)/p-b^2)v-(a b)/p==0.,v]
			]
		]
	]
];


(* ::Text:: *)
(*Pure component fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]PureSRK=Function[{a,b,p,v,t},
		Module[{z=p v/(rJoule/100. t)},
			Exp[z-1.-Log[z(1.-b/v)]-a/(rJoule/100. t b) Log[1.+b/v]]
		](* 1 *)
	]
];


(* ::Text:: *)
(*Mixture fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]MixtureSRK=Function[{a,aij,b,bPure,x,p,v,t},
		Module[{i,j,nComp},
			nComp=Length[bPure];
			Table[
				Exp[Log[v/(v-b)]+bPure[[i]]/(v-b)-Log[(p v)/(rJoule 10^-2 t)]-2 Sum[x[[j]] aij[[j,i]],{j,nComp}]/(rJoule 10^-2 t b) Log[(v+b)/v]+(a bPure[[i]])/(rJoule 10^-2 t b^2) (Log[(v+b)/v]-b/(v+b))],{i,nComp}
			]
		](* 1 *)
	]
];


(* ::Subsection::Closed:: *)
(*Peng-Robinson (PR)*)


(* ::Text:: *)
(*Parameters*)


With[{
	rJoule=rJoule},
	aCritPR=Function[{tCrit,pCrit},
		0.45724 (rJoule/100.)^2 tCrit^2/pCrit
	]
];


With[{
	rJoule=rJoule},
	bCritPR=Function[{tCrit,pCrit},
		0.0778 (rJoule/100.) tCrit/pCrit
	]
];


\[Alpha]PR=Function[{tRed,\[Omega]},
	(1.+(0.37464+1.54226 \[Omega]-0.26992 \[Omega]^2) (1.-tRed^0.5))^2
];


(* ::Text:: *)
(*Pressure*)


With[{
	rJoule=rJoule},
	pPR=Function[{t,v,a,b},
		(rJoule/100.) t/(v-b)-a/(v(v+b)+b(v-b))
	]
];


(* ::Text:: *)
(*Molar volume*)


With[{
	rJoule=rJoule},
	vPR=Function[{t,p,a,b},
		Values[
			Flatten[
				NSolve[v^3+v^2(p b-rJoule/100. t)/p+v (a-2 rJoule/100. t b-3 p b^2)/p+(p b^3+rJoule/100. t b^2-a b)/p,v]
			]
		]
	]
];


(* ::Text:: *)
(*Pure component fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]PurePR=Function[{a,b,p,v,t},
		Module[{z=p v/(rJoule/100. t)},
			Exp[z-1.-Log[z(1.-b/v)]-a/(2. Sqrt[2.] rJoule/100. t b) Log[(v+(1.+Sqrt[2.])b)/(v+(1.-Sqrt[2.])b)]]
		]
	]
];


(* ::Text:: *)
(*Mixture fugacity coefficient*)


With[{
	rJoule=rJoule},
	\[CurlyPhi]MixturePR=Function[{a,aij,b,bPure,x,p,v,t},
		Module[{i,j,nComp,z},
			nComp=Length[bPure];
			z=p v/(rJoule/100. t);
			Table[
				Exp[bPure[[i]]/b(z-1)-Log[p (v-b)/(rJoule/100. t)]-a/(2. Sqrt[2.] b rJoule/100. t)(2. Sum[x[[j]] aij[[j,i]],{j,nComp}]/a-bPure[[i]]/b) Log[(v+(1.+Sqrt[2.])b)/(v+(1.-Sqrt[2.])b)]],{i,nComp}
			]
		]
	]
];


(* ::Section::Closed:: *)
(*Mixing Rules for EoS*)


(* ::Subsection::Closed:: *)
(*Van der Waals, one pairwise binary interaction parameter (1PVdW)*)


aij1PVdW=Function[{aPure,kij},
	Module[{nComp,i,j},
		nComp=Length[aPure];
		Table[
			If[i!=j,Sqrt[aPure[[i]] aPure[[j]]](1.-kij[[i,j]]),aPure[[i]]],{i,nComp},{j,nComp}
		]
	]
];


(* ::Section::Closed:: *)
(*Saturation Pressure from EoS*)


(* ::Text:: *)
(*Based on the demand \[CurlyPhi]Liquid = \[CurlyPhi]Vapor for VLE*)


(* ::Subsection::Closed:: *)
(*Volume-translated Peng-Robinson EoS for LJ fluids (VTPRLJ)*)


saturationPressureVTPRLJ=Function[{aPure,bPure,volumeTranslationPure,t,pGuess,\[Epsilon],verbose},
	Module[
		(* Local variables *)
		{vSol,vLiquid,vVapor,\[Phi]Liquid,\[Phi]Vapor,p},
		p=pGuess;
		
		(* Iteration *)
		If[verbose,
		Print["Step \!\(\*SubscriptBox[\(v\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(v\), \(Vapor\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Vapor\)]\) \t p [bar]"];
			];
		Do[
			vSol=vVTPRLJ[t,p,aPure,bPure,volumeTranslationPure];
			{vLiquid,vVapor}=MinMax[Values[vSol]]; (* dm^3/mol *)
			\[Phi]Liquid=\[CurlyPhi]PureVTPRLJ[aPure,bPure,volumeTranslationPure,p,vLiquid,t];
			\[Phi]Vapor=\[CurlyPhi]PureVTPRLJ[aPure,bPure,volumeTranslationPure,p,vVapor,t];
			If[verbose,
				Print[i,"\t",vLiquid,"\t",vVapor,"\t",\[Phi]Liquid,"\t",\[Phi]Vapor,"\t",p]
			];
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]\[LessSlantEqual]\[Epsilon],Break[]];
			p=p \[Phi]Liquid/\[Phi]Vapor;
			,{i,500}
		];
		If[verbose,
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]>\[Epsilon],
				Print["\n No convergence after 100 iterations.\n"],
				Print["\[HappySmiley]\n"]
			]
		];
		(* Return saturation pressure [bar], liquid molar volume [dm^3/mol] and vapor molar volume [dm^3/mol] *)
		{p,vLiquid,vVapor}
		]
];


(* ::Subsection::Closed:: *)
(*Van der Waals equation of state*)


saturationPressureVdW=Function[{aPure,bPure,t,pGuess,\[CapitalDelta]\[Phi]Abs,verbose},
	Module[
		(* Local variables *)
		{vSol,vLiquid,vVapor,\[Phi]Liquid,\[Phi]Vapor,p,iMax},
		iMax=100;
		p=pGuess;
		
		(* Iteration *)
		If[verbose,
			Print["Step \!\(\*SubscriptBox[\(v\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(v\), \(Vapor\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Vapor\)]\) \t p [bar]"]
			];
		Do[
			vSol=vVdW[t,p,aPure,bPure];
			{vLiquid,vVapor}=MinMax[vSol];
			\[Phi]Liquid=\[CurlyPhi]PureVdW[aPure,bPure,p,vLiquid,t];
			\[Phi]Vapor=\[CurlyPhi]PureVdW[aPure,bPure,p,vVapor,t];
			p=p \[Phi]Liquid/\[Phi]Vapor;
			If[verbose,
				Print[i,"\t",vLiquid,"\t",vVapor,"\t",\[Phi]Liquid,"\t",\[Phi]Vapor,"\t",p]
				];
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]\[LessSlantEqual]\[CapitalDelta]\[Phi]Abs,Break[]];
			,{i,1,iMax}
		];
		If[verbose,
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]>\[CapitalDelta]\[Phi]Abs,
				Print["\n No convergence after 100 iterations.\n"],
				Print["\[HappySmiley]\n"]
			]
		];
		
		(* Return saturation pressure [bar] *)
		p
	]
];


(* ::Subsection::Closed:: *)
(*Redlich-Kwong (RK) equation of state*)


saturationPressureRK=Function[{aPure,bPure,t,pGuess,\[CapitalDelta]\[Phi]Abs,verbose},
	Module[
		(* Local variables *)
		{vSol,vLiquid,vVapor,\[Phi]Liquid,\[Phi]Vapor,p,iMax},
		iMax=100;
		p=pGuess;
		
		(* Iteration *)
		If[verbose,
			Print["Step \!\(\*SubscriptBox[\(v\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(v\), \(Vapor\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Vapor\)]\) \t p [bar]"]
			];
		Do[
			vSol=vRK[t,p,aPure,bPure];
			{vLiquid,vVapor}=MinMax[vSol];
			\[Phi]Liquid=\[CurlyPhi]PureRK[aPure,bPure,p,vLiquid,t];
			\[Phi]Vapor=\[CurlyPhi]PureRK[aPure,bPure,p,vVapor,t];
			p=p \[Phi]Liquid/\[Phi]Vapor;
			If[verbose,
				Print[i,"\t",vLiquid,"\t",vVapor,"\t",\[Phi]Liquid,"\t",\[Phi]Vapor,"\t",p]
				];
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]\[LessSlantEqual]\[CapitalDelta]\[Phi]Abs,Break[]];
			,{i,1,iMax}
		];
		If[verbose,
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]>\[CapitalDelta]\[Phi]Abs,
				Print["\n No convergence after 100 iterations.\n"],
				Print["\[HappySmiley]\n"]
			]
		];
		
		(* Return saturation pressure [bar] *)
		p
		]
];


(* ::Subsection::Closed:: *)
(*Soave-Redlich-Kwong (SRK) equation of state*)


saturationPressureSRK=Function[{aPure,bPure,t,pGuess,\[CapitalDelta]\[Phi]Abs,verbose},
	Module[
		(* Local variables *)
		{vSol,vLiquid,vVapor,\[Phi]Liquid,\[Phi]Vapor,p,iMax},
		iMax=100;
		p=pGuess;
		
		(* Iteration *)
		If[verbose,
			Print["Step \!\(\*SubscriptBox[\(v\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(v\), \(Vapor\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Vapor\)]\) \t p [bar]"]
			];
		Do[
			vSol=vSRK[t,p,aPure,bPure];
			{vLiquid,vVapor}=MinMax[vSol];
			\[Phi]Liquid=\[CurlyPhi]PureSRK[aPure,bPure,p,vLiquid,t];
			\[Phi]Vapor=\[CurlyPhi]PureSRK[aPure,bPure,p,vVapor,t];
			p=p \[Phi]Liquid/\[Phi]Vapor;
			If[verbose,
				Print[i,"\t",vLiquid,"\t",vVapor,"\t",\[Phi]Liquid,"\t",\[Phi]Vapor,"\t",p]
				];
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]\[LessSlantEqual]\[CapitalDelta]\[Phi]Abs,Break[]];
			,{i,1,iMax}
		];
		If[verbose,
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]>\[CapitalDelta]\[Phi]Abs,
				Print["\n No convergence after 100 iterations.\n"],
				Print["\[HappySmiley]\n"]
			]
		];
		
		(* Return saturation pressure [bar] *)
		p
		]
];


(* ::Subsection::Closed:: *)
(*Peng-Robinson (PR) equation of state*)


saturationPressurePR=Function[{aPure,bPure,t,pGuess,\[CapitalDelta]\[Phi]Abs,verbose},
	Module[
		(* Local variables *)
		{vSol,vLiquid,vVapor,\[Phi]Liquid,\[Phi]Vapor,p,iMax},
		iMax=100;
		p=pGuess;
		
		(* Iteration *)
		If[verbose,
			Print["Step \!\(\*SubscriptBox[\(v\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(v\), \(Vapor\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Liquid\)]\) \t \!\(\*SubscriptBox[\(\[CurlyPhi]\), \(Vapor\)]\) \t p [bar]"];
			];
		Do[
			vSol=vPR[t,p,aPure,bPure];
			{vLiquid,vVapor}=MinMax[vSol];
			\[Phi]Liquid=\[CurlyPhi]PurePR[aPure,bPure,p,vLiquid,t];
			\[Phi]Vapor=\[CurlyPhi]PurePR[aPure,bPure,p,vVapor,t];
			p=p \[Phi]Liquid/\[Phi]Vapor;
			If[verbose,
				Print[i,"\t",vLiquid,"\t",vVapor,"\t",\[Phi]Liquid,"\t",\[Phi]Vapor,"\t",p]
				];
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]\[LessSlantEqual]\[CapitalDelta]\[Phi]Abs,Break[]];
			,{i,1,iMax}
		];
		If[verbose,
			If[Abs[\[Phi]Liquid-\[Phi]Vapor]>\[CapitalDelta]\[Phi]Abs,
				Print["\n No convergence after 100 iterations.\n"],
				Print["\[HappySmiley]\n"]
			]
		];
		
		(* Return saturation pressure [bar] *)
		p
		]
];


(* ::Section::Closed:: *)
(*Bubble point and dew point calculations with EoS*)


(* ::Subsection::Closed:: *)
(*Volume-translated Peng-Robinson EoS for LJ fluids (VTPRLJ)*)


bubblePressureVTPRLJ=Function[{aij,bij,volumeTranslationPure,t,x,pGuess,yGuess,\[Epsilon],verbose},
	Module[
		(* Local variables *)
		{nComp,p,y,v,aLiq,aVap,bLiq,bVap,tLiq,tVap,vLiq,\[CurlyPhi]Liq,vVap,\[CurlyPhi]Vap,kValue,s,trouble},
		trouble=False;(* flag *)
		nComp=Length[x];(* number of components in the mixture *)
		p=pGuess;(* bar *)
		y=yGuess;(* mol mol^-1 *)
		
		(* Start of calculations *)
		aLiq=Sum[Sum[x[[i]] x[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
		bLiq=Sum[Sum[x[[i]] x[[j]] bij[[i,j]],{i,nComp}],{j,nComp}];(* dm^3 mol^-1 *)
		tLiq=Total[x volumeTranslationPure];(* dm^3 mol^-1 *)
		
		(* Iteration *)
		If[verbose,
			Print["Step   \!\(\*SubscriptBox[\(\[CapitalSigma]\), \(i\)]\)\!\(\*SubscriptBox[\(y\), \(i\)]\)","\n------------"]
		];
		Do[
			vLiq=(Flatten@Values@vVTPRLJ[t,p,aLiq,bLiq,tLiq])[[3]];(* dm^3 mol^-1*)
			(* Check if negative volume or imaginary solution was found*)
			If[Im[vLiq]!=0 || vLiq<0.,trouble=True;Break[]];
			\[CurlyPhi]Liq=\[CurlyPhi]MixtureVTPRLJ[aij,bij,volumeTranslationPure,x,p,vLiq,t];(* 1 *)
			
			aVap=Sum[Sum[y[[i]] y[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
			bVap=Sum[Sum[y[[i]] y[[j]] bij[[i,j]],{i,nComp}],{j,nComp}];(* dm^3 mol^-1 *)
			tVap=Total[y volumeTranslationPure];(* dm^3 mol^-1 *)
			vVap=(Flatten@Values@vVTPRLJ[t,p,aVap,bVap,tVap])[[1]];(* dm^3 mol^-1*)
			\[CurlyPhi]Vap=\[CurlyPhi]MixtureVTPRLJ[aij,bij,volumeTranslationPure,y,p,vVap,t];(* 1 *)
		
			kValue=\[CurlyPhi]Liq/\[CurlyPhi]Vap;(* 1 *)
			s=Total[x kValue];(* 1 *)
			If[verbose,Print[n,"\t",s]];
			If[Abs[s-1.]/s 100.<\[Epsilon],Break[]];(* accuracy threshold *)
			p=p s;
			y=x kValue/s;
			,{n,100}
		];
		If[trouble&&verbose,Print["Negative or non-real liquid molar volume solution."]];
		If[verbose,
			If[Abs[s-1.]/s 100.>\[Epsilon],Print["\[SadSmiley] No convergence.\n"],
				Print["\[HappySmiley] Bubble condition fulfilled, converged.\n"]
			]
		];
		
		(* Return {Pressure, {liquid phase molar fractions}, {vapor phase molar fractions}, liquid molar volume, vapor molar volume} *)
		{p,x,y,vLiq,vVap}
	]
];


(* ::Subsection::Closed:: *)
(*Soave-Redlich-Kwong (SRK) equation of state*)


bubblePressureSRK=Function[{aPure,bPure,kij,t,x,pGuess,yGuess,\[Epsilon],verbose},
	Module[
		(* Local variables *)
		{nComp,p,y,v,aLiq,aij,aVap,bLiq,bVap,vLiq,\[CurlyPhi]Liq,vVap,\[CurlyPhi]Vap,kValue,s,trouble},
		trouble=False;(* flag *)
		nComp=Length[aPure];(* number of components in the mixture *)
		p=pGuess;(* bar *)
		y=yGuess;(* mol mol^-1 *)
		aij=aij1PVdW[aPure,kij];(* (dm^3)^2 bar mol^-2 *)
		aLiq=Sum[Sum[x[[i]] x[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
		bLiq=Total[x bPure];(* dm^3 mol^-1 *)
		
		(* Iteration *)
		If[verbose,
			Print["Step   \!\(\*SubscriptBox[\(\[CapitalSigma]\), \(i\)]\)\!\(\*SubscriptBox[\(y\), \(i\)]\)","\n------------"]
		];
		
		Do[
			vLiq=Min[NSolve[p==pSRK[t,v,aLiq,bLiq],v,Reals][[All,1,2]]];(* dm^3 mol^-1*)
			If[vLiq<0.,
				trouble=True;Break[]
			];
			\[CurlyPhi]Liq=\[CurlyPhi]MixtureSRK[aLiq,aij,bLiq,bPure,x,p,vLiq,t];(* 1 *)
			aVap=Sum[Sum[y[[i]] y[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
			bVap=Total[y bPure];(* dm^3 mol^-1 *)
			vVap=Max[NSolve[p ==pSRK[t,v,aVap,bVap],v,Reals][[All,1,2]]];(* dm^3 mol^-1*)
			\[CurlyPhi]Vap=\[CurlyPhi]MixtureSRK[aVap,aij,bVap,bPure,y,p,vVap,t];(* 1 *)
			kValue=\[CurlyPhi]Liq/\[CurlyPhi]Vap;(* 1 *)
			s=Total[x kValue];(* 1 *)
			If[verbose,Print[n,"\t",s]];
			If[Abs[s-1.]/s 100.<\[Epsilon],Break[]];(* accuracy threshold *)
			p=p s;
			y=x kValue/s,
			{n,100}
		];
		If[trouble&&verbose,Print["Negative or non-real liquid molar volume solution."]];
		If[verbose,
			If[Abs[s-1.]/s 100.>\[Epsilon],
				Print["\[SadSmiley] No convergence.\n"],
				Print["\[HappySmiley] Bubble condition fulfilled, converged.\n"]
			]
		];
		
		(* Return {Pressure, {liquid phase molar fractions}, {vapor phase molar fractions}, liquid molar volume, vapor molar volume} *)
		{p,x,y,vLiq,vVap}
	]
];


dewPressureSRK=Function[{aPure,bPure,kij,t,y,pGuess,xGuess,\[Epsilon],verbose},
	Module[
		(* Local variables *)
		{nComp,p,x,v,aLiq,aij,aVap,bLiq,bVap,vLiq,\[CurlyPhi]Liq,vVap,\[CurlyPhi]Vap,kValue,s,i,j,n,trouble},
		trouble=False;(* flag *)
		nComp=Length[aPure];(* number of components in the mixture *)
		p=pGuess;(* bar *)
		x=xGuess;(* mol mol^-1 *)
		aij=aij1PVdW[aPure,kij];(* (dm^3)^2 bar mol^-2 *)
		aVap=Sum[Sum[y[[i]] y[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
		bVap=Total[y bPure];(* dm^3 mol^-1 *)
		
		
		(* Iteration *)
		If[verbose,
			Print["Step   \!\(\*SubscriptBox[\(\[CapitalSigma]\), \(i\)]\)\!\(\*SubscriptBox[\(x\), \(i\)]\)","\n------------"]
		];
		Do[
			vVap=Max[NSolve[p==pSRK[t,v,aVap,bVap],v,Reals][[All,1,2]]];(* dm^3 mol^-1*)
			\[CurlyPhi]Vap=\[CurlyPhi]MixtureSRK[aVap,aij,bVap,bPure,y,p,vVap,t];(* 1 *)
			aLiq=Sum[Sum[x[[i]] x[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
			bLiq=Total[x bPure];(* dm^3 mol^-1 *)
			vLiq=Min[NSolve[p ==pSRK[t,v,aLiq,bLiq],v,Reals][[All,1,2]]];(* dm^3 mol^-1*)
		If[vLiq<0.,
				trouble=True;Break[]
			];
			\[CurlyPhi]Liq=\[CurlyPhi]MixtureSRK[aLiq,aij,bLiq,bPure,x,p,vLiq,t];(* 1 *)
			kValue=\[CurlyPhi]Liq/\[CurlyPhi]Vap;(* 1 *)
			s=Total[y/kValue];(* 1 *)
			If[verbose,Print[n,"\t",s]];
			If[Abs[s-1.]/s 100.<\[Epsilon],Break[]];(* accuracy threshold *)
			p=p / s;
			x=y /kValue /s,
			{n,100}
		];
		If[trouble&&verbose,Print["Negative or non-real liquid molar volume solution."]];
		If[verbose,
			If[Abs[s-1.]/s 100.>\[Epsilon],Print["\[SadSmiley] No convergence.\n"],
				Print["\[HappySmiley] Bubble condition fulfilled, converged.\n"]
			]
		];
		
		(* Return {Pressure, {liquid phase molar fractions}, {vapor phase molar fractions}, liquid molar volume, vapor molar volume} *)
		{p,x,y,vLiq,vVap}
	]
];


(* ::Subsection::Closed:: *)
(*Peng-Robinson (PR) equation of state*)


bubblePressurePR=Function[{aPure,bPure,kij,t,x,pGuess,yGuess,\[Epsilon],verbose},
	Module[
		(* Local variables *)
		{nComp,p,y,v,aLiq,aij,aVap,bLiq,bVap,vLiq,\[CurlyPhi]Liq,vVap,\[CurlyPhi]Vap,kValue,s,trouble},
		trouble=False;(* flag *)
		nComp=Length[aPure];(* number of components in the mixture *)
		p=pGuess;(* bar *)
		y=yGuess;(* mol mol^-1 *)
		aij=aij1PVdW[aPure,kij];(* (dm^3)^2 bar mol^-2 *)
		aLiq=Sum[Sum[x[[i]] x[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
		bLiq=Total[x bPure];(* dm^3 mol^-1 *)
		
		(* Iteration *)
		If[verbose,
			Print["Step   \!\(\*SubscriptBox[\(\[CapitalSigma]\), \(i\)]\)\!\(\*SubscriptBox[\(y\), \(i\)]\)","\n------------"]
		];
		Do[
			vLiq=Min[NSolve[p==pPR[t,v,aLiq,bLiq],v,Reals][[All,1,2]]];(* dm^3 mol^-1*)
			If[vLiq<0.,
				trouble=True;Break[]
			];
			\[CurlyPhi]Liq=\[CurlyPhi]MixturePR[aLiq,aij,bLiq,bPure,x,p,vLiq,t];(* 1 *)
			aVap=Sum[Sum[y[[i]] y[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
			bVap=Total[y bPure];(* dm^3 mol^-1 *)
			vVap=Max[NSolve[p ==pPR[t,v,aVap,bVap],v,Reals][[All,1,2]]];(* dm^3 mol^-1*)
			\[CurlyPhi]Vap=\[CurlyPhi]MixturePR[aVap,aij,bVap,bPure,y,p,vVap,t];(* 1 *)
			kValue=\[CurlyPhi]Liq/\[CurlyPhi]Vap;(* 1 *)
			s=Total[x kValue];(* 1 *)
			If[verbose,Print[n,"\t",s]];
			If[Abs[s-1.]/s 100.<\[Epsilon],Break[]];(* accuracy threshold *)
			p=p s;
			y=x kValue/s,
			{n,100}
		];
		If[trouble&&verbose,Print["Negative or non-real liquid molar volume solution."]];
		If[verbose,
			If[Abs[s-1.]/s 100.>\[Epsilon],Print["\[SadSmiley] No convergence.\n"],
				Print["\[HappySmiley] Bubble condition fulfilled, converged.\n"]
			]
		];
		
		(* Return {Pressure, {liquid phase molar fractions}, {vapor phase molar fractions}, liquid molar volume, vapor molar volume} *)
		{p,x,y,vLiq,vVap}
	]
];


dewPressurePR=Function[{aPure,bPure,kij,t,y,pGuess,xGuess,\[Epsilon],verbose},
	Module[
		(* Local variables *)
		{nComp,p,x,v,aLiq,aij,aVap,bLiq,bVap,vLiq,\[CurlyPhi]Liq,vVap,\[CurlyPhi]Vap,kValue,s,i,j,n,trouble},
		trouble=False;(* flag *)
		nComp=Length[aPure];(* number of components in the mixture *)
		p=pGuess;(* bar *)
		x=xGuess;(* mol mol^-1 *)
		aij=aij1PVdW[aPure,kij];(* (dm^3)^2 bar mol^-2 *)
		aVap=Sum[Sum[y[[i]] y[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
		bVap=Total[y bPure];(* dm^3 mol^-1 *)
		
		
		(* Iteration *)
		If[verbose,
			Print["Step   \!\(\*SubscriptBox[\(\[CapitalSigma]\), \(i\)]\)\!\(\*SubscriptBox[\(x\), \(i\)]\)","\n------------"]
		];
		Do[
			vVap=Max[NSolve[p==pPR[t,v,aVap,bVap],v,Reals][[All,1,2]]];(* dm^3 mol^-1*)
			\[CurlyPhi]Vap=\[CurlyPhi]MixturePR[aVap,aij,bVap,bPure,y,p,vVap,t];(* 1 *)
			aLiq=Sum[Sum[x[[i]] x[[j]] aij[[i,j]],{i,nComp}],{j,nComp}];(* (dm^3)^2bar mol^-2 *)
			bLiq=Total[x bPure];(* dm^3 mol^-1 *)
			vLiq=Min[NSolve[p ==pPR[t,v,aLiq,bLiq],v,Reals][[All,1,2]]];(* dm^3 mol^-1*)
		If[vLiq<0.,
				trouble=True;Break[]
			];
			\[CurlyPhi]Liq=\[CurlyPhi]MixturePR[aLiq,aij,bLiq,bPure,x,p,vLiq,t];(* 1 *)
			kValue=\[CurlyPhi]Liq/\[CurlyPhi]Vap;(* 1 *)
			s=Total[y/kValue];(* 1 *)
			If[verbose,Print[n,"\t",s]];
			If[Abs[s-1.]/s 100.<\[Epsilon],Break[]];(* accuracy threshold *)
			p=p / s;
			x=y /kValue /s,
			{n,100}
		];
		If[trouble&&verbose,Print["Negative or non-real liquid molar volume solution."]];
		If[verbose,
			If[Abs[s-1.]/s 100.>\[Epsilon],Print["\[SadSmiley] No convergence.\n"],
				Print["\[HappySmiley] Bubble condition fulfilled, converged.\n"]
			]
		];
		
		(* Return {Pressure, {liquid phase molar fractions}, {vapor phase molar fractions}, liquid molar volume, vapor molar volume} *)
		{p,x,y,vLiq,vVap}
	]
];


(* ::Section::Closed:: *)
(*End of Package*)


End[];


EndPackage[];
