(* ::Package:: *)

(* ::Title:: *)
(*HelperFunctions*)


(* ::Subtitle:: *)
(*This package contains helper functions which are used throughout the present simulator including functions for import and export of files.*)


BeginPackage["HelperFunctions`"];


(* ::Chapter::Closed:: *)
(*Usage Definitions*)


(* Notebook Handling *)
GetNbHandleFromWslFile::usage = "This function return a valid notebook handle of a given wsl file without touching
the wsl-file itself.";

(* Import & Export *)
BaseDirectorySelection::usage="Select and load a simulation either automatically if the 
Execution Handler is used or manually through a dialog window.";

ImportResultData::usage="This functions extracts the *.m files from a given archive and
loads them. Thereby this function is flexibile if the number of
result files is changed.";

GetAssotiationThread::usage="This function is used to import saved .m files as an AssotioationThread. The Keys are extracted with the Head given as a second argument or HoldForm by default.";
StringBlock::usage = "StringBlock[par_, body_] -> Evaluates 'body' in a Block environment with 'par' as a list of strings defining the local variables.";
ImportResultAssosiation::usage="This function is used to import the simulation results
of an extension-simulation as an AssosiationThread";
AddResultAssosiation::usage="This function is used to add the imported defnintions and results
to the local variables.";

CreateSimulationDefinition::usage="This function is used to create the simulation definition file in the directory defined by 'importExportDir'.";
CreateResultFile::usage="This function is used to create the main results file at the given path.";
simDefVariableStrings::usage = "Contains the names of the variables for the 'simulationDefinition.m'-file as a list of strings.";
simResVariableStrings::usage = "Contains the names of the variables for the 'simulationResults.m'-file as a list of strings.";

CreateTestSpecificResultFiles::usage="This is the overhead function to create the test specififc result files in the directory defined by 'resultsDir'.";
CreateCfcTrackFile::usage="This function to create the test specific export file for the cfc state documentation.";
CreateSpDocuFile::usage = "This function to create the test specific export file for the species choice probability documentation.";
CreateExpFilePanSwaps::usage = "This function is used to create the test specific export file for the pan swaps.";
CreateExpFileELRC::usage = "This function is used to create the test specific export file for the electrostatic long range correction.";
CreateExpFileRDF::usage = "This function is used to create the test specific export file for the radial distribution function sampling.";
CreateExpFileIntraConfig::usage = "This function is used to create the test specific export file for the intramolecular configuration distribution documentation.";

CompressResultData::usage="This functions creates +.m files for every given dataset and
compresses it to a single results archive. The data cluster has
the follwoing format: {name, {data1, data2, dataN}}";

(* Compatibility Check *)
CompatibilityCheckSimDef::usage = "This function checks if variables from the 'simulationDefinition'-file are set appropriately for the current version
and clears declarations of old variable names.";
CompatibilityCheckResults::usage = "This function checks if variables from the 'result'-files are set appropriately for the current version
and clears declarations of old variable names.";

(* Result Export *)
markShowCell::usage="Marks the given cell with the 'Show' Tag";

ExportToCsv::usage="This function exports a given dataset reliably to a given csvFilePath.";

ExportGraphic::usage="Function for simplified export of graphics";
ExportGridGraphic::usage="Function for simplified export of multiple graphics";
Ep::usage="Simplified handle for ExportGridGraphic";
ExporGraphicManual::usage="Function for simplified export of graphics by name";

ExportTableLaTeX::usage="Exports the specified table with headers in the LaTeX format";

NumToStr::usage="This function returns a string corresponding to the given number represented as a decimal.";
MeanStdStr::usage="This function returns the mean value and standard deviation of each entry of the given vector";
MeanStdCol::usage="";
MeanStdSys::usage="";

List2assoc::usage = "List2assoc[listAssoc_, nameAssoc_] generates an association for every entry of the given
list with the given name and the corresponding count.";
Boxes2assoc::usage = "Boxes2assoc[listAssoc_, nameAssoc_] generates an association for every entry of the given
list with the given name and the corresponding box phase.";

(* Custom Parameters *)
checkGetAssocParameter::usage="This function returns the value corresponding to a given key of a given association,
if it is set. Otherwise the function return false. Thereby it can be checked if the parameters required for a
specific section of code are defined.";

(* Debugging Helper *)
EchoP::usage="Echos the given value/notice and pauses";

(* Checking Functions *)
visualizeCheck::usage="Give you an Smiley back :)";
HelloWorld::usage="Test function for package";


(* ::Chapter:: *)
(*Function Definitions*)


Begin["`Private`"];


(* ::Section::Closed:: *)
(*Notebook Handling*)


GetNbHandleFromWslFile[wslPath_] := Module[
	{nbWsl, wslPathTmp, nbPathTmp, nbTmp},
	(*
	Create temporary analysis notebook for printing the result pdf.
	This approach is necessary since
	1) Mathematica ignores the "InsertResults"-option for *.wsl Files
	2) The style is lost in content is importetd with NotebookImport
	3) NotebookSave does not only save the open notebook at the given location
	but also trashes the neat *.wsl files with its cell configurations.
	4) Wait for a second after file operations.
	>> Final Note:
	Yes, this is a totally inconvenient approach but after having tested 8 different
	variations it has been the only one working reliably.
	*)

	(*
	Copy original *.wsl file so that is not proteccted from encroaching mathematica.
	*)
	wslPathTmp = FileNameJoin@{
		DirectoryName@wslPath,
		FileBaseName@wslPath <> "_tmp.wls"
		};
	CopyFile[wslPath, wslPathTmp];
	Pause@1;

	(*
	Interestingly mathematica needs the second argument to properly convert a given
	notebook to another format (e.g. *.wsl \[Rule] *.nb). At the same time it is incapable
	of saving the file with the given extension right away.

	Therefore the notebook is first safed (whereby the content of the *.wsl file is overwritten
	with the content of an equivalent *.nb file, then closed to be finally renamed with the
	needed extension so that mathematica allows the "InsertResults" Option.
	*)
	nbWsl = NotebookOpen@wslPathTmp;
	nbPathTmp = FileNameJoin@{
		DirectoryName@wslPath,
		FileBaseName@wslPath <> "_tmp.nb"};
	NotebookSave[nbWsl, nbPathTmp];

	(*
	Surprisingly mathematica does NOT wait until the saving operation is complete.
	Let's pause for a second to reflect on the working hours spent on this "feature".
	*)
	Pause@1;
	NotebookClose@nbWsl;

	(*
	Delete the temporary notebook if it was not properly removed.
	Rename the "saved" file for finally having the proper extension.
	*)
	If[FileExistsQ[nbPathTmp],
		DeleteFile[nbPathTmp];
	];
	RenameFile[wslPathTmp, nbPathTmp];

	(*
	Close and delete the temporary *.wls file. Now we finally have a *.nb file in which
	the output will be printed so that it can then be exported as a single pdf.
	*)
	If[FileExistsQ[wslPathTmp],
		DeleteFile[wslPathTmp];
	];

	(*Open and return the notebook handle*)
	{nbPathTmp, NotebookOpen@nbPathTmp}
];


(* ::Section::Closed:: *)
(*File Import & Export*)


(* ::Subsection::Closed:: *)
(*Base Directory Selection*)


(* ::Item:: *)
(*"Dialog" for a input-window in which the folder can be set*)


(* ::Item:: *)
(*"SkipIfSet" to use the same directory if it is already loaded in the kernel*)


(* ::Item:: *)
(* "Notebook" for manual execution*)


(* ::Item:: *)
(*"Automated" for automated execution using the execution handler*)


BaseDirectorySelection[] := Module[{executionType, parameterFile, customffparamFiles},

	If[!SameQ[Global`executionHandlerMode, True] && !SameQ[Global`cmdExecutionMode, True]
	, (*TRUE: automatic setting of directory not active \[Rule] set importExportDir manually*)
		executionType = "Dialog";
		Switch[executionType
			
			,"Dialog",
				Global`importExportDir = SystemDialogInput[
					"Directory",
					If[DirectoryQ@Global`importExportDir, Global`importExportDir, NotebookDirectory[]]];

			,"SkipIfSet",
				If[ValueQ@Global`importExportDir
	
					, (*already set \[Rule] use the same as before*)
					Echo[Global`importExportDir <> " is still used."];
	
					, (*not set \[Rule] open the dialog*)
					Global`importExportDir = SystemDialogInput["Directory", NotebookDirectory[]];
				];

			,"Notebook1",
				Global`importExportDir = FileNameJoin@{
					"export",
					"VersionTesting",
					"v2.2F6",
					"cfcVlugt",
					"Argon_FS_500_LJ#500n#1.00Tstar#explizit_CfcVlugt"
				};

			,"Notebook2",
				Global`importExportDir = FileNameJoin@{
					"export",
					"VersionTesting",
					"V0.3.2",
					"mixturesTest",
					"Argon_FS_10_LJ#500n#1.15Tstar#explizit#CFC"
				};

			,"visuTests",
				Global`importExportDir = FileNameJoin@{
					"export",
					"VersionTesting",
					"v2.2F6",
					"cfcVlugt",
					"Argon_FS_sho_LJ#25n#1.00Tred_visu"};

			,"energyTests",
				Global`importExportDir = FileNameJoin@{
					"export",
					"VersionTesting",
					"V0.3.1",
					"LjArgon_visualisationFractionalComponents",
					"Argon_FS_sho_LJ#4n#1.00Tred"};

			,"simulationTests",
				Global`importExportDir = FileNameJoin@{
					"export",
					"VersionTesting",
					"V0.3.1",
					"LjArgon_visualisationFractionalComponents",
					"Argon_FS_sho_LJ#300n#1.00Tred#explizit"};
		];
	];

	(*load the simulation definitions*)
	Get@FileNameJoin@{Global`importExportDir,"simulationDefinitions.m"};

	(*Load parameter data if present*)
	parameterFile = FileNameJoin@{Global`importExportDir,"parameter.m"};
	If[FileExistsQ[parameterFile],

		(*TRUE \[Rule] load parameters*)
		Get@parameterFile;

		,
		(*FALSE \[Rule] check if already set in the simulatoin defintion (has been unset in the init)*)
		If[!ValueQ@Global`custParam,

			(*NOT SET \[Rule] initialise association*)
			Global`custParam  = <||>;
		];
	];
	
	(*Load custom force field parameter data if present*)
	customffparamFiles = FileNames[FileNameJoin@{Global`importExportDir,"customffparam*.m"}];
	If[customffparamFiles =!= {},
		(*TRUE \[Rule] load custom ff parameters*)
		Get@# &/@ customffparamFiles;
	];
	
	(*perform compatibility check in casee of outdated simulation definition*)
	CompatibilityCheckSimDef[]
];


(* ::Subsection::Closed:: *)
(*Import Results*)


ImportResultData[] := Module[
	{
		pathResultsArchive, resultsDir, pathResultsFile,
		parameterFile, fileExist
	},

	(*file name of results archive*)
	pathResultsArchive = FileNameJoin@{Global`importExportDir, "results.zip"};

	(*Compatibility for previous versions*)
	If[FileExistsQ@pathResultsArchive,

		(*TRUE \[Rule] File exits: Setup the results dir to extract the results*)
		resultsDir = FileNameJoin[{Global`importExportDir, "_results"}];

		(*Extract the data to *.m-files and deletes the archive*)
		ExtractArchive[pathResultsArchive, resultsDir];

		(*Load and delete all *.m Files so that they are not added to the version control*)
		(
			Get[#];
			DeleteFile[#];
		) &/@FileNames["*.m",resultsDir];

		(*Delete the temporary directory*)
		DeleteDirectory[resultsDir];

	, (*FALSE \[Rule] File does not exist: Check if a non-compressed results file is present*)

		(*check if a results FILE exists*)
		pathResultsFile = FileNameJoin@{Global`importExportDir, "simulationResults.m"};
		If[FileExistsQ@pathResultsFile,

			Get@pathResultsFile;
		];
	];

	(*Load parameter data if present*)
	parameterFile = FileNameJoin[{Global`importExportDir,"parameter.m"}];
	fileExist = FileExistsQ[parameterFile];
	If[fileExist,
		Get@parameterFile;
	];
];


(* ::Subsection::Closed:: *)
(*Import Extensions*)


GetAssotiationThread[path_,keyhead_:HoldForm] := Module[{importHeldExpr,importExpr,importKeys},
	importHeldExpr = Import[path,"HeldExpressions"];
	importExpr = Activate@Import[path,"InactivatedExpressions"][[All,2]];
	importKeys = Extract[#, {1,1}, keyhead] &/@ importHeldExpr;
	AssociationThread[importKeys,importExpr]
];


StringBlock[par : {___String}, body_] := 
  Join @@ Cases[
     MapAt[MakeExpression, par, All],
     _[s___Symbol] :> Hold[s]
  ] /. _[varL___] :> Block[{varL}, body];
SetAttributes[StringBlock, HoldAll];


ImportResultAssosiation[importDir_] := Module[
	{
		importLocalVars, simResults
	},
	
	importLocalVars = Flatten@{
		(* simulationResults.m *)
		simResVariableStrings,
		(* Test Specific Variables *)
		{"cfcEtaDocu","cfcMyDocu","recordAcceptanceRateDocu"},
		{"SPdoc"},
		{"counterPanRejeNpDocu", "counterPanRejDuDocu", "counterPanAcceptDocu"},
		{"eLRCkappaRangeplotdata"},
		{"rdfDocuMolec","rdfDocuAtom"},
		{"intraConfigBondBins", "intraConfigAngleBins", "intraConfigTorsionBins"}
	};
	
	Block[{Global`importExportDir},
		
		(*set import directory*)
		Global`importExportDir = importDir;
		
		(* load the results in a Block environment *)
		StringBlock[Evaluate@importLocalVars,		
			
			(*import results files and perform compatibility check*)
			ImportResultData[];
			CompatibilityCheckResults[];
			
			(*save imported results as Association before Block ends*)
			simResults = AssociationThread[importLocalVars,ToExpression@importLocalVars];
		];
	];
	
	(*Return*)
	simResults
];


AddResultAssosiation[simRes_] := Module[
	{
		AddResult, AppendResult, OverwriteResult,
		listAddResults, listAppendResults, listOverwriteResults
	},

	AddResult[symbol_,value_] := symbol += value;
	AppendResult[symbol_,value1_,value2_] := symbol = Flatten[Append[{value1},value2],1];
	OverwriteResult[symbol_,value_] := symbol = value;

	(* list of variables to be added *)
	listAddResults = {
		"counterTransAcceptDocu", "counterTransRejeDuDocu", "counterTransRejeOvDocu", "counterTransRejeNpDocu",
		"counterRotaAcceptDocu", "counterRotaRejeDuDocu", "counterRotaRejeOvDocu", "counterRotaRejeNpDocu",
		"counterVolChangeAcceptDocu", "counterVolChangeRejectDocu",
		"counterInsertAcceptDocu", "counterInsertRejeDuDocu", "counterInsertRejeOvDocu", "counterInsertRejeNpDocu",
		"counterInsertRejeNpDocu", "counterWidomAcceptDocu", "counterWidomRejectDocu",
		"counterBondStretchAcceptDocu","counterBondStretchRejeDuDocu","counterBondStretchRejeNpDocu","counterBondStretchRejeOvDocu",
		"counterAngleBendAcceptDocu","counterAngleBendRejeDuDocu","counterAngleBendRejeOvDocu","counterAngleBendRejeNpDocu",
		"counterTorsionAcceptDocu","counterTorsionRejeDuDocu","counterTorsionRejeOvDocu","counterTorsionRejeNpDocu",
		"evalTime", "nWarmUpCycles", "nEquiCycles", "nProdCycles"
	};

	(* list of variables to be appended *)
	listAppendResults = {
		"nvecList", "VvecList", "nCompVecList", "interEnergyList", "intraEnergyList",
		"interVirialList", "tailCorrEnergyList", "tailCorrPressureList",
		"muPartialList", "muCounterList", "transMaxVecDoc", "rotaMaxVecDoc", "volMaxDoc",
		"nMovesDocu"
	};

	(* list of variables to be overwritten *)
	listOverwriteResults = {
		"moleculeCoordVec", "moleculeNamesVec", "moleculeAtomsCoordVec",
		"moleculeAtomsNamesVec", "moleculeAtomsNonBondedLabelsVec",
		"moleculeAtomsIndicatorsVec", "Lvec", "Vvec", "nvec", "nCompVec",
		"transMaxVec", "rotaMaxVec", "volChangeMaxVec",
		"nMovesPerCycles", "speciesChoiceProbVec"
	};

	If[checkGetAssocParameter[Global`custParam, "particleInsertionMode"] == "CfcVlugt",
		(* CFC - Continous Fractional Component *)
		listAppendResults = Flatten[Append[{listAppendResults},{
			"cfcMy", "counterCfcAcceptSwapDocu", "counterCfcRejectSwapEnergyDocu",
			"counterCfcRejectSwapOverlapDocu", "counterCfcAcceptLambdaChangeDocu",
			"counterCfcRejectLambdaChangeDocu", "counterCfcAcceptExchangeDocu",
			"counterCfcRejectExchangeDocu"
		}],1];

		listAppendResults = Flatten[Append[{listAppendResults},{
			"cfcMaxLambdaChangeList", "cfcLnVDocu", "cfcMinBinDocu"
		}],1];

		listOverwriteResults = Flatten[Append[{listOverwriteResults},{
			"cfc", "cfcActive", "cfcEta", "cfcLnV", "cfcLnV"
		}],1];
	];
	
	If[checkGetAssocParameter[Global`custParam, "panBinarySwaps"] === True,
		(* PAN Binary Swap Move *)
		listAddResults = Flatten[Append[{listAddResults},{
			"counterPanRejeNpDocu", "counterPanRejDuDocu", "counterPanAcceptDocu"
		}],1];
	];

	(* Known Excluded Values: (excluded because they should always be the same)
	 system, moleculeData, moleculeBonds, moleculeNonBondedPairs, ffLabelsNonBonded,
	 interEnergyStateVec, interVirialStateVec, intraEnergyStateVec,
	 atomTypes, ffLabelsBondStretching, ffLabelsAngleBending, ffLabelsTorsion,
	 ffParamCombiningRule, ensembleType, T, Psys, rCutVec, rOvlVec,
	 
	 intraConfigDocudr, intraConfigDocudphi, intraConfigDocudtheta
	*)

	(* Test Specific Values *)
	If[checkGetAssocParameter[Global`custParam, "rOvLinvestigation"],
		listAppendResults = Append[listAppendResults, "rOvlinvList"];
	];
	If[AnyTrue[{"cfcStateDocu","recordAcceptanceProbabilities"}, checkGetAssocParameter[Global`custParam, #]&],
		listAppendResults = Append[listAppendResults, "cfcEtaDocu"];
		listAppendResults = Append[listAppendResults, "cfcMyDocu"];
		listAppendResults = Append[listAppendResults, "recordAcceptanceRateDocu"];
	];
	If[checkGetAssocParameter[Global`custParam, "relaxationInvestigation"],
		listAppendResults = Append[listAppendResults, "RIbaseDoc"];
		listAppendResults = Append[listAppendResults, "RIextensiveDoc"];
	];
	If[AnyTrue[{"auto","autoSwap"}, checkGetAssocParameter[Global`custParam, "speciesSwapProbabilityMode"] == #&],
		listAppendResults = Append[listAppendResults, "SPdoc"];
	];
	If[checkGetAssocParameter[Global`custParam, "intraConfigDocu"],
		listAddResults = Append[listAddResults, "intraConfigBondBins"];
		listAddResults = Append[listAddResults, "intraConfigAngleBins"];
		listAddResults = Append[listAddResults, "intraConfigTorsionBins"];
	];
	If[ToExpression["nProdCycles"] == 0
		,(*True: no production phase was executed yet -> overwrite*)
		If[checkGetAssocParameter[Global`custParam, "rdfDocuMolec"],
			AppendTo[listOverwriteResults,"rdfDocuMolec"];
		];
		If[checkGetAssocParameter[Global`custParam, "rdfDocuAtom"],
			AppendTo[listOverwriteResults,"rdfDocuAtom"];
		];
		,(*Else: add the data appropriately*)
		AddExtendedRDFDocuData[simRes];
	];

	(*add the last CPUtime documented for continuous curve*)
	AppendResult[ToExpression["timingCPUList", InputForm, Unevaluated],ToExpression["timingCPUList"],
		simRes["timingCPUList"] + ToExpression["timingCPUList"][[-1]]];
	
	(* Execute Add, Append and Overwrite functions *)
	AddResult[ToExpression[#, InputForm, Unevaluated],simRes[#]] &/@ listAddResults;

	AppendResult[ToExpression[#, InputForm, Unevaluated],ToExpression[#],simRes[#]] &/@ listAppendResults;

	OverwriteResult[ToExpression[#, InputForm, Unevaluated],simRes[#]] &/@ listOverwriteResults;
];


AddExtendedRDFDocuData[simRes_] := Module[{nDocuCyclesOld,nDocuCyclesExt,nCycleRatio},
	(*get number of documentation cycles for weighted addition*)
	nDocuCyclesOld = ToExpression["nProdCycles"];
	nDocuCyclesExt = simRes["nProdCycles"];
	nCycleRatio = nDocuCyclesExt/(nDocuCyclesExt+nDocuCyclesOld);
	
	If[checkGetAssocParameter[Global`custParam, "rdfDocuMolec"],
		Do[(*shorten list in case cut-off radius got smaller in the extension*)
			If[Length@#2 > Length[simRes["rdfDocuMolec"][[box,ind1,ind2]]]
				,(*True: shorten existing list*)
				#1 = (1-nCycleRatio) * Take[#2,Length[simRes["rdfDocuMolec"][[box,ind1,ind2]]]]
					+ nCycleRatio * simRes["rdfDocuMolec"][[box,ind1,ind2]];
				,(*Else: shorten list from extension*)
				#1 = (1-nCycleRatio) * #2
					+ nCycleRatio * Take[simRes["rdfDocuMolec"][[box,ind1,ind2]],Length[#2]];
			]&[
				ToExpression["rdfDocuMolec[["<>ToString@box<>","<>ToString@ind1<>","<>ToString@ind2<>"]]",InputForm, Unevaluated],
				ToExpression["rdfDocuMolec[["<>ToString@box<>","<>ToString@ind1<>","<>ToString@ind2<>"]]"]
			];
			,{box,2}
			,{ind1,Length@ToExpression["rdfDocuMolec"][[1]]}
			,{ind2,Length@ToExpression["rdfDocuMolec"][[1]]}
		];
	];
	If[checkGetAssocParameter[Global`custParam, "rdfDocuAtom"],
		Do[(*shorten list in case cut-off radius got smaller in the extension*)
			If[Length@#2 > Length[simRes["rdfDocuAtom"][[box,ind1,ind2]]]
				,(*True: shorten existing list*)
				#1 = (1-nCycleRatio) * Take[#2,Length[simRes["rdfDocuAtom"][[box,ind1,ind2]]]]
					+ nCycleRatio * simRes["rdfDocuAtom"][[box,ind1,ind2]];
				,(*Else: shorten list from extension*)
				#1 = (1-nCycleRatio) * #2
					+ nCycleRatio * Take[simRes["rdfDocuAtom"][[box,ind1,ind2]],Length[#2]];
			]&[
				ToExpression["rdfDocuAtom[["<>ToString@box<>","<>ToString@ind1<>","<>ToString@ind2<>"]]",InputForm, Unevaluated],
				ToExpression["rdfDocuAtom[["<>ToString@box<>","<>ToString@ind1<>","<>ToString@ind2<>"]]"]
			];
			,{box,2}
			,{ind1,Length@ToExpression["rdfDocuAtom"][[1]]}
			,{ind2,Length@ToExpression["rdfDocuAtom"][[1]]}
		];
	];
];


(* ::Subsection::Closed:: *)
(*Export File*)


CreateSimulationDefinition[expSimDef_] := Module[{},
	simDefVariableStrings /. var__ :> Save[expSimDef,var];
];


CreateResultFile[expFile_] := Module[{},
	simResVariableStrings /. var__ :> Save[expFile,var];
];


simDefVariableStrings = {
	(* Variables of initial conditions *)
	"system","moleculeData","moleculeBonds","moleculeNonBondedPairs",
	"ffLabelsNonBonded","atomTypes","ffLabelsBondStretching","ffLabelsAngleBending",
	"ffLabelsTorsion","ffParamCombiningRule",
	"ensembleType","T","Psys","Lvec","Vvec","nvec","nCompVec",
	"transMaxVec","rotaMaxVec","volChangeMaxVec",
	"rCutVec","rOvlVec",

	(* number of moves and cycles *)
	"nWarmUpCycles","nEquiCycles","nProdCycles",
	"nMovesPerCycles","speciesChoiceProbVec",

	(* coordinates and labels *)
	"moleculeCoordVec","moleculeNamesVec",
	"moleculeAtomsCoordVec","moleculeAtomsNamesVec",
	"moleculeAtomsNonBondedLabelsVec","moleculeAtomsIndicatorsVec",

	(*RNG state*)
	"randomGeneratorState",

	(*cfc state*)
	"cfc", "cfcActive", "cfcEta", "cfcLnV", "cfcMaxDeltaLambda", "cfcBoxBiasValue"
};


(* list of variable names to be exported as results (is also used in the extended simulation import)*)
simResVariableStrings = {
	(* documentation data (of every cycle) *)
	"nvecList","VvecList","nCompVecList",
	"interEnergyList","interVirialList","intraEnergyList",
	"tailCorrEnergyList","tailCorrPressureList",
	"muPartialList","muCounterList",

	(* trial move counters *)
	"counterTransAcceptDocu","counterTransRejeDuDocu","counterTransRejeOvDocu","counterTransRejeNpDocu",
	"counterRotaAcceptDocu","counterRotaRejeDuDocu","counterRotaRejeOvDocu","counterRotaRejeNpDocu",
	"counterVolChangeAcceptDocu","counterVolChangeRejectDocu",
	"counterInsertAcceptDocu","counterInsertRejeDuDocu","counterInsertRejeOvDocu","counterInsertRejeNpDocu",
	"counterWidomAcceptDocu", "counterWidomRejectDocu",
	"counterBondStretchAcceptDocu","counterBondStretchRejeDuDocu","counterBondStretchRejeNpDocu","counterBondStretchRejeOvDocu",
	"counterAngleBendAcceptDocu","counterAngleBendRejeDuDocu","counterAngleBendRejeOvDocu","counterAngleBendRejeNpDocu",
	"counterTorsionAcceptDocu","counterTorsionRejeDuDocu","counterTorsionRejeOvDocu","counterTorsionRejeNpDocu",

	(* evaluation times *)
	"evalTime",

	(* number of moves and cycles *)
	"nWarmUpCycles","nEquiCycles","nProdCycles",
	"nMovesPerCycles","speciesChoiceProbVec",

	(* current state of the system *)
	"moleculeCoordVec","moleculeNamesVec",
	"moleculeAtomsCoordVec","moleculeAtomsNamesVec",
	"moleculeAtomsNonBondedLabelsVec","moleculeAtomsIndicatorsVec",
	"interEnergyStateVec","interVirialStateVec","intraEnergyStateVec",
	"system","moleculeData","moleculeBonds","moleculeNonBondedPairs",
	"ffLabelsNonBonded","atomTypes","ffLabelsBondStretching",
	"ffLabelsAngleBending","ffLabelsTorsion", "ffParamCombiningRule",
	"ensembleType","T","Psys","Lvec","Vvec","nvec","nCompVec",
	"transMaxVec","rotaMaxVec","volChangeMaxVec",
	"rCutVec","rOvlVec",

	(*course of the trial move limits*)
	"transMaxVecDoc", "rotaMaxVecDoc", "volMaxDoc",

	(*version used for executing the simulation*)
	"gitSHA1",
		
	(*system infromation for investigating the memory leak*)
	"systemInfomrationBefore", "systemInfomrationAfter",

	(*timing*)
	"timingMethod", "timingInit", "timingEnd",
	"timingCPUList",

	(*trial move adoption*)
	"nMovesDocu",
	
	(*RNG state*)
	"randomGeneratorState",

	(*
	--------------------
	CFC - Continous Fractional Component
	--------------------
	final state
	*)
	"cfc", "cfcActive", "cfcEta", "cfcLnV", "cfcMaxDeltaLambda",
	"cfcBoxBiasValue",

	(*documentation*)
	"cfcVecDocu", "cfcMaxLambdaChangeList", "cfcLnVDocu", "cfcMinBinDocu",
	"cfcBoxBiasValueDocu",

	(*counts*)
	"cfcMy",
	"counterCfcAcceptSwapDocu",
	"counterCfcRejectSwapEnergyDocu",
	"counterCfcRejectSwapOverlapDocu",
	"counterCfcAcceptLambdaChangeDocu",
	"counterCfcRejectLambdaChangeDocu",
	"counterCfcAcceptExchangeDocu",
	"counterCfcRejectExchangeDocu"
};


(* ::Subsection::Closed:: *)
(*Test Specific Export*)


CreateTestSpecificResultFiles[resultsDir_] :=Module[{},
	CreateCfcTrackFile[resultsDir];
	CreateSpDocuFile[resultsDir];
	CreateExpFilePanSwaps[resultsDir];
	CreateExpFileELRC[resultsDir];
	CreateExpFileRDF[resultsDir];
	CreateExpFileIntraConfig[resultsDir];
	
	(*export data from custom evaluations*)
	Symbol[#]  &/@ Global`customEvaluationListExport;
];


CreateCfcTrackFile[resultsDir_] := Module[{cfcTrackFile},
	If[AnyTrue[{"cfcStateDocu","recordAcceptanceProbabilities"}, (Global`custParam[#] === True)&]
		,(*True*)
		cfcTrackFile = FileNameJoin[{resultsDir,"simulationResultsCfcTrack.m"}];
		If[FileExistsQ[cfcTrackFile],
			DeleteFile[cfcTrackFile];
		];
		Save[cfcTrackFile,{
			(*extensive documentation*)
			"cfcEtaDocu",
			"cfcMyDocu",

			(*Subscript[p, Acc](\[Lambda])*)
			"recordAcceptanceRateDocu"
		}];
	];
];


CreateSpDocuFile[resultsDir_] := Module[{spDocuFile},
	If[AnyTrue[{"auto","autoSwap"}, Global`custParam["speciesSwapProbabilityMode"] === # &]
		,(*True*)
		spDocuFile = FileNameJoin[{resultsDir,"simulationResultsSPdoc.m"}];
		If[FileExistsQ[spDocuFile],
			DeleteFile[spDocuFile];
		];
		Save[spDocuFile,{
			"SPdoc"
		}];
	];
];


CreateExpFilePanSwaps[resultsDir_] := Module[{expFilePanSwaps},
	If[Global`custParam["panBinarySwaps"] === True,
		expFilePanSwaps = FileNameJoin[{resultsDir,"simulationResultsexpFilePanSwaps.m"}];
		If[FileExistsQ[expFilePanSwaps],
			DeleteFile[expFilePanSwaps];
		];
		Save[expFilePanSwaps,{"counterPanRejeNpDocu", "counterPanRejDuDocu", "counterPanAcceptDocu"}];
	];
];


CreateExpFileELRC[resultsDir_] := Module[{expFileELRCewald},
	If[AnyTrue[{"Ewald","Wolf"}, (Global`custParam["eLRCMethod"] === #)&]
		,(*True*)
		If[!ValueQ@Global`eLRCkappaRangeplotdata
			,(*True: no value yet calculated / not imported from analysis file*)
			Global`eLRCkappaRangeplotdata = Global`calcELRCkappaRangePlotdata[];
		];
		expFileELRCewald = FileNameJoin[{resultsDir,"simulationResultsELRC.m"}];
		If[FileExistsQ[expFileELRCewald],
			DeleteFile[expFileELRCewald];
		];
		Save[expFileELRCewald,{"eLRCkappaRangeplotdata"}];
	];
];


CreateExpFileRDF[resultsDir_] := Module[{expFileRDF},
	If[Global`custParam["rdfDocuMolec"] === True || Global`custParam["rdfDocuAtom"] === True
		,(*True*)
		expFileRDF = FileNameJoin[{resultsDir,"simulationResultsRDF.m"}];
		If[FileExistsQ[expFileRDF],
			DeleteFile[expFileRDF];
		];
		Save[expFileRDF,{
			"rdfDocudr",
			"rdfDocuMolec",
			"rdfDocuAtom"
		}];
	];
];


CreateExpFileIntraConfig[resultsDir_] := Module[{expFileIntraConfig},
	If[Global`custParam["intraConfigDocu"] === True
		,(*True*)
		expFileIntraConfig = FileNameJoin[{resultsDir,"simulationResultsIntraConfig.m"}];
		If[FileExistsQ[expFileIntraConfig],
			DeleteFile[expFileIntraConfig];
		];
		Save[expFileIntraConfig,{
			"intraConfigDocudr",
			"intraConfigDocudphi",
			"intraConfigDocudtheta",
			"intraConfigBondBins",
			"intraConfigAngleBins",
			"intraConfigTorsionBins"
		}];
	];
];


(* ::Subsection::Closed:: *)
(*Compress Results*)


CompressResultData[importExportDirLoc_, resultsDir_] := Module[{files},

	(*Compress the created files*)
	files = FileNames["*.m", resultsDir];
	CreateArchive[
		files,
		FileNameJoin[{importExportDirLoc,"results.zip"}],
		OverwriteTarget->True];

	(*Delete the *.m Files*)
	DeleteFile[#]&/@files;
	DeleteDirectory[resultsDir];
];


(* ::Section::Closed:: *)
(*Compatibility Check*)


Begin["Global`"]; (*Global context for all new variables*)
CompatibilityCheckSimDef[] := Module[{},
	
	(*check if custom parameter exist*)
	If[!ValueQ@custParam
		,(*True: no custom parameters were used -> declare an empty association*)
		custParam = <||>;
	];
	
	(*check if the cfc state is already set*)
	If[!ValueQ@cfc,
		(*NOT SET \[Rule]  Initialise state variable. We have to initialise them even if no cfc-molecules
			exist so that the energy functions work. By initialising the index with "-1" we ensure
			that no molecule index (starting at 1) matches the cfc state. *)
		cfc = Table[{-1, 0}, 2 (*boxes*), Length@system (*components*)];
	];
	
	(*check if cfcActive is already set*)
	If[!ValueQ@cfcActive,
		(*NOT SET \[Rule] initialise it to False by default*)
		cfcActive = False;
	];
	
	(** from v2.2F6 to v2.3 **)
	
	(*compatibility check for the trial move limits (per box \[Rule] species specific per box)*)
	If[Length@Dimensions@transMaxVec == 1 && ValueQ@transMaxVec,

		(*not species specific \[Rule] needs to be expanded per species*)
		transMaxVec = Table[#, Length@system]  &/@ transMaxVec;
	];
	If[Length@Dimensions@rotaMaxVec == 1 && ValueQ@rotaMaxVec,
	
		(*not species specific \[Rule] needs to be expanded per species*)
		rotaMaxVec = Table[#, Length@system]  &/@ rotaMaxVec;
	];
	If[Length@Dimensions@volChangeMax == 0 && ValueQ@volChangeMax,
	
		(*not box specific \[Rule] needs to be expanded per box*)
		volChangeMax = Table[volChangeMax, 2];
	];
	
	(*set the combining rule for pair interaction parameter calculation*)
	If[!ValueQ@ffParamCombiningRule,
		If[KeyExistsQ[custParam,"ffParamCombiningRule"]
			,(*True: set as specified in custom parameters*)
			ffParamCombiningRule = custParam["ffParamCombiningRule"];
			,(*else: set 'Default'*)
			ffParamCombiningRule = "Default";
		]
	];
	
	(** from v2.3 to v2.4 **)
	
	If[!ValueQ@volChangeMaxVec && ValueQ@volChangeMax,
	
		(*old simulation setup \[Rule] define new variable*)
		volChangeMaxVec = volChangeMax;
		(*clear old declaration*)
		Clear[volChangeMax];
	];
	
	(** from v2.4 to v2.5 **)
	
	If[Length@nMovesPerCycles == 5
		,(*True: old format without intermolecular trial moves*)
		nMovesPerCycles = Join[nMovesPerCycles,{0,0,0}];
	];
	
	If[!ValueQ@speciesChoiceProbVec,
		(*default: equal probability*)
		speciesChoiceProbVec = Table[
			Table[1/#,#] &[Length@system]
		,5];
		
		(* check if custom parameter for insertion species choice probability was set *)
		If[KeyExistsQ[custParam,"speciesSwapProbability"]
			,(*True: set custom parameter as initial value for insertions*)
			speciesChoiceProbVec[[1]] = custParam["speciesSwapProbability"];
			(*clear old custom parameter key*)
			KeyDropFrom[custParam,"speciesSwapProbability"];
		];
	];
	
	(*custom parameter changes*)
	If[KeyExistsQ[custParam,"trialMoveScaleMoves"]
		,(*old parameter -> set new and delete old*)
		If[custParam["trialMoveScaleMoves"]
			,(*True: set to "scaled"*)
			custParam["trialMoveAdoptionAdditionalMoveChange"] = "scaled";
			,(*False: set to "none"*)
			custParam["trialMoveAdoptionAdditionalMoveChange"] = "none";
		];
		KeyDropFrom[custParam,"trialMoveScaleMoves"];
		KeyDropFrom[custParam,"trialMoveAdoptionAdaptVolumeChanges"];
		KeyDropFrom[custParam,"trialMoveAdoptionAdaptTranslations"];
	];
	
	(* moleculeNonBondedPairs changes *)
	If[!AssociationQ@moleculeNonBondedPairs[[1]]
		,(*intra non-bonded parameter variable is not an accosiation with scaling factors*)
		moleculeNonBondedPairs = Table[Association@(
			If[Length@FindShortestPath[TwoWayRule @@@ moleculeBonds[[i]],#[[1]],#[[2]]] == 4
				,(*True: scaled 1-4 interaction*)
				# -> {0.5,0.5,0.5}
				,(*Else: regular non-bonded interaction*)
				# -> {1.,1.,1.}
			] &/@ moleculeNonBondedPairs[[i]])
			,{i,Length@system}
		];
	];
	
];
End[];


Begin["Global`"]; (*Global context for all new variables*)
CompatibilityCheckResults[] := Module[{},
	
	(** Check Simulation Definition (which are also stored in the results) **)
	CompatibilityCheckSimDef[];
	
	(*compatibility check for the trial move limits documentation (per box \[Rule] species specific per box)*)
	If[Length@Dimensions@transMaxVecDoc == 2,
		
		(*not species specific \[Rule] needs to be expanded per species*)
		transMaxVecDoc = (Table[#, Length@system]  &/@ #) &/@ transMaxVecDoc;
	];
	If[Length@Dimensions@rotaMaxVecDoc == 2,
		
		(*not species specific \[Rule] needs to be expanded per species*)
		rotaMaxVecDoc = (Table[#, Length@system]  &/@ #) &/@ rotaMaxVecDoc;
	];
	If[Length@Dimensions@volMaxDoc == 1,
		
		(*not box specific \[Rule] needs to be expanded per box*)
		volMaxDoc = Table[#, 2] &/@ volMaxDoc;
	];
	
	(*system information before and after for investigating the memory leak*)
	If[!ValueQ@systemInfomrationBefore,
		systemInfomrationBefore = Table[None, 2, 16];
	];
	If[!ValueQ@systemInfomrationAfter,
		systemInfomrationAfter = Table[None, 2, 16];
	];
	
	(*Swap Probability Documentation*)
	If[Length@Cases[SPdoc,{{},{}}] > 0,
		(*TRUE: v2.2F6 SPdoc is loaded -> delete {{},{}} elements and expand the documented elements per executed cycle*)
		SPdoc = Flatten[
			Table[#, custParam[["speciesSwapCyclesAdjustment"]]] &/@ DeleteCases[SPdoc,{{},{}}]
		,1][[1;;nEquiCycles]]
	];
	
	(** from v2.3 to v2.4 **)
	
	(*new tiral move result-specific timing*)
	If[!KeyExistsQ[evalTime, "translation-Accept"]
		,(*True: no new trial move result-specific timing documented*)
		(*assign all trial move time to the accepted trial move*)
		evalTime[["translation-Accept"]] = evalTime[["translation"]];
		evalTime[["translation-RejeDu"]] = 0;
		evalTime[["translation-RejeOv"]] = 0;
		evalTime[["translation-RejeNp"]] = 0;
		evalTime[["rotation-Accept"]] = evalTime[["rotation"]];
		evalTime[["rotation-RejeDu"]] = 0;
		evalTime[["rotation-RejeOv"]] = 0;
		evalTime[["rotation-RejeNp"]] = 0;
		evalTime[["volumeChange-Accept"]] = evalTime[["volumeChange"]];
		evalTime[["volumeChange-Reject"]] = 0;
		evalTime[["insertion-Accept"]] = evalTime[["insertion"]];
		evalTime[["insertion-RejeDu"]] = 0;
		evalTime[["insertion-RejeOv"]] = 0;
		evalTime[["insertion-RejeNp"]] = 0;
		evalTime[["ghostInsertion-Accept"]] = evalTime[["ghostInsertion"]];
		evalTime[["ghostInsertion-RejeOv"]] = 0;
		(*delete old keys*)
		evalTime = KeyDrop[evalTime,{"translation","rotation","volumeChange","insertion","ghostInsertion"}];
		
		If[custParam[["particleInsertionMode"]] === "CfcVlugt",
			(*True: cfc is active \[Rule] add timing keys to prevent MissingKey*)
			evalTime[["cfcLambdaChange-Accept"]] = 0;
			evalTime[["cfcLambdaChange-Reject"]] = 0;
			
			evalTime[["cfcExchange-Accept"]] = 0;
			evalTime[["cfcExchange-Reject"]] = 0;
			
			evalTime[["cfcSwap-Accept"]] = 0;
			evalTime[["cfcSwap-RejeDu"]] = 0;
			evalTime[["cfcSwap-RejeOv"]] = 0;
		];
	];
	
	If[!ValueQ[counterTransRejeNpDocu]
		,(*True: set the counter to 0*)
		counterTransRejeNpDocu = Table[0, {box,2}, {cp,Length@system}];
	];
	If[!ValueQ[counterRotaRejeNpDocu]
		,(*True: set the counter to 0*)
		counterRotaRejeNpDocu = Table[0, {box,2}, {cp,Length@system}];
	];
	
	If[checkGetAssocParameter[custParam, "panBinarySwaps"] === True 
			&& !ValueQ[counterPanRejeNpDocu] 
			&& ValueQ@counterPanRejeNp
		,(*True: no Docu counters were used*)
		counterPanRejeNpDocu = counterPanRejeNp;
		counterPanRejDuDocu = counterPanRejDu;
		counterPanAcceptDocu = counterPanAccept;
		(*clear old declaration*)
		Clear[counterPanRejeNp,counterPanRejDu,counterPanAccept];
	];
	
	If[ValueQ@timingCPUDocu && !ValueQ@timingCPUList
		,(*True: older name for CPU documentation*)
		timingCPUList = timingCPUDocu;
		(*clear old declaration*)
		Clear[timingCPUDocu];
	];
	
	(** from v2.4 to v2.5 **)
	
	If[ValueQ@SPdoc && SPdoc != {} && Length@SPdoc[[-1]] >= 2 && Total@SPdoc[[-1,2]] == 1
		&& speciesChoiceProbVec[[1]] =!= SPdoc[[-1,2]]
		,(*True: speciesChoiceProbVec was added through simDef compatibility check*)
		If[!SameQ[checkGetAssocParameter[custParam, "eqAdjAvrCycles"],False],
			(*use averaged value of the documented states*)
			speciesChoiceProbVec[[1]] = N@Mean@SPdoc[[nEquiCycles+1-custParam[["eqAdjAvrCycles"]];;nEquiCycles,2]];
			,(*else*)
			(*update the vale for speciesChoiceProbVec to the last documented state of SPdocu*)
			speciesChoiceProbVec[[1]] = SPdoc[[-1,2]];
		];
	];
	
	If[Length@nMovesDocu > 0 && Length@First@nMovesDocu == 5
		,(*True: old format without intermolecular trial moves *)
		nMovesDocu = Join[nMovesDocu,Table[{0,0,0},Length@nMovesDocu],2];
	];
	
	If[!ValueQ@counterBondStretchAcceptDocu
		,(*no intraolecular trial move counters*)
		boxDocuSystem = Table[0,{boxes,2},{moleculeTypes,Length@system}];
		
		counterBondStretchAcceptDocu = boxDocuSystem;
		counterBondStretchRejeDuDocu = boxDocuSystem;
		counterBondStretchRejeOvDocu = boxDocuSystem;
		counterBondStretchRejeNpDocu = boxDocuSystem;
		
		counterAngleBendAcceptDocu = boxDocuSystem;
		counterAngleBendRejeDuDocu = boxDocuSystem;
		counterAngleBendRejeOvDocu = boxDocuSystem;
		counterAngleBendRejeNpDocu = boxDocuSystem;
		
		counterTorsionAcceptDocu = boxDocuSystem;
		counterTorsionRejeDuDocu = boxDocuSystem;
		counterTorsionRejeOvDocu = boxDocuSystem;
		counterTorsionRejeNpDocu = boxDocuSystem;
	];
	
	If[!KeyExistsQ[evalTime, "bondStretch-Accept"]
		,(*True: no new trial move for intramolecular trial moves documented*)
		evalTime[["bondStretch-Accept"]] = 0;
		evalTime[["bondStretch-RejeDu"]] = 0;
		evalTime[["bondStretch-RejeOv"]] = 0;
		evalTime[["bondStretch-RejeNp"]] = 0;
		evalTime[["anlgeBend-Accept"]] = 0;
		evalTime[["anlgeBend-RejeDu"]] = 0;
		evalTime[["anlgeBend-RejeOv"]] = 0;
		evalTime[["anlgeBend-RejeNp"]] = 0;
		evalTime[["torsion-Accept"]] = 0;
		evalTime[["torsion-RejeDu"]] = 0;
		evalTime[["torsion-RejeOv"]] = 0;
		evalTime[["torsion-RejeNp"]] = 0;
	];
	
];
End[];


(* ::Section::Closed:: *)
(*Analysis Result Export*)


(* ::Subsection::Closed:: *)
(*PDF Print*)


(* This function marks the given cell with the "Show" Tag. Thereby it is exported to the automatically created PDF *)
markShowCell[cell_]:=SetOptions[cell, CellTags->"Show"];


(* ::Subsection::Closed:: *)
(*CSV Export*)


ExportToCsv[ds_, csvFilePath_] := Module[{entries, keys, csvDataSet},

	entries = List @@@ Normal@ds;
	keys = ds[1, Keys] // Normal;
	csvDataSet = PrependTo[entries, keys];

	(*
	Normally

	Export[
		FileNameJoin@{
			baseDirectory,
			"combinedResults_" <> StringReplace[DateString["ISODateTime"], ":"-> "-"] <> ".csv"
		}, ds, "CSV"]

	should work just fine - yet it has proven to be unreliable for larger datasets.
	Therefore the following approach is implemented as suggested in
	https://mathematica.stackexchange.com/questions/170828/export-dataset-as-csv-or-xls
	*)
	Export[csvFilePath, csvDataSet];
];


(* ::Subsection::Closed:: *)
(*Graphic Export*)


ExportGraphic[filename_, object_, exportFlagL_:False, showFolder_:True] := Module[
	{
		adjustedObject, test, noLe
	},

	If[Global`exportFlag == True || exportFlagL,

		(* create export directory if it does not exist *)
		If[DirectoryQ[Global`exportDir] == False,
			CreateDirectory[Global`exportDir];
		];

		(* add margin and text with current folder *)
		adjustedObject = Framed[
			Column[
				If[showFolder
					, (*show the folder name*)
					{
						"Folder: " <> Global`importExportDir,
						object
					}
					, (*do not show the folder name*)
					{object}
				],
				Spacings->1.5
			],
			FrameMargins->{{8, 8}, {4, 4}},
			FrameStyle->None
		];
		
		(*export without the legend for better alignment*)
		noLe = object;
		Do[
			If[Head@noLe == Legended,
				noLe = First@noLe;
				,(*else*)
				Break[];
			];
			,50 (*maximum of 5 legends will be removed*)
		];
		If[Head@noLe != Graphics, Print@"Warning: removal of legend has gone wrong!"];

		Export[FileNameJoin[{Global`exportDir, filename <> ".pdf"}],adjustedObject,"PDF","AllowRasterization"->False];
		Export[FileNameJoin[{Global`exportDir, filename <> "_noLe.pdf"}],noLe,"PDF","AllowRasterization"->False];
		Export[FileNameJoin[{Global`exportDir, filename <> ".png"}],adjustedObject];
	]
];


ExportGridGraphic[handle_, exportFlagL_:False] := Module[{},

	(*exprot if necessary*)
	If[exportFlag == True || exportFlagL,

		(* create export directory if it does not exist *)
		If[DirectoryQ[Global`exportDir] == False,
			CreateDirectory[Global`exportDir];
		];

		ExportGraphic[#[[1]], Show[#[[2]], ImageSize->Medium, PlotLabel->None], True, False] &/@ handle;
	];

	(*return the setup grid*)
	Print@Grid[
		{handle[[All, 2]]},
		ItemSize->Scaled[Min[{1 / Length@handle, 0.5}]]];
];


Ep[handle_, exportFlagL_] := ExportGridGraphic[{handle}, exportFlagL];


ExporGraphicManual[pltName_] := Module[{plt},

	(*use a reasonbale plot size so that the legends, etc. are well readable in LaTeX*)
	plt = Show[Symbol@pltName,ImageSize->Medium];
	Print@plt;

	(*export the graphic*)
	ExportGraphic[pltName, plt, True, False];
];


(* ::Subsection::Closed:: *)
(*Table Export*)


HeaderLineLaTeX[line_] := If[# != SpanFromLeft, Style[#,Bold], #] &/@ line;
MergeLineLaTeX[line_, precision_] :=
	StringJoin@Riffle[ToString@DecimalForm[#, {10, precision}, NumberPadding -> {"", "0"}] &/@line, " & "] <> "\\\\";
ExportTableLaTeX[nameL_, header_, rows_, precision_, exportFlagL_] := Module[{
		rowsL, dividers,
		tExp, colStr
	},

	(*export if necessary*)
	If[exportFlag == True || exportFlagL,

		(* create export directory if it does not exist *)
		If[DirectoryQ[Global`exportDir] == False,
			CreateDirectory[Global`exportDir];
		];

		If[Length@header[[1]] <= 1
			, (*true \[Rule] single header row*)
			colStr = Table["Y", Length@header];

			, (*false \[Rule] multiple header rows*)
			colStr = Table["Y", Length@header[[1]]];
		];
		colStr[[1]] = "l";

		(*setup*)
		tExp = {};
		AppendTo[tExp, "\\begin{table}[ht]"];
		AppendTo[tExp, "\\caption{\\todo{enter your caption here ...}}"];
		AppendTo[tExp, "\\begin{tabularx}{\\textwidth}{" <> StringJoin@colStr <> "}"];
		AppendTo[tExp, ""];

		(*header*)
		AppendTo[tExp, "\\toprule"];
		If[Length@Dimensions@header == 1
			, (*true \[Rule] single header row*)
			AppendTo[tExp, MergeLineLaTeX[header, precision]];

			, (*false \[Rule] multiple header rows*)
			AppendTo[tExp, MergeLineLaTeX[#, precision]] &/@ header;
		];

		AppendTo[tExp, "\\midrule"];
		AppendTo[tExp, ""];

		(*content*)
		AppendTo[tExp, MergeLineLaTeX[#, precision]] &/@ rows;
		AppendTo[tExp, ""];

		(*teardown*)
		AppendTo[tExp, "\\bottomrule"];
		AppendTo[tExp, "\\end{tabularx}"];
		AppendTo[tExp, "\\label{t:YourLabel}"];
		AppendTo[tExp, "\\end{table}"];

		(*export: explicitly export as "Text" to prevent mathemtica from doing "smart" stuff*)
		Export[FileNameJoin[{Global`exportDir,nameL <> ".tex"}], tExp, "Text"];

		(*ExportGraphic[#[[1]] <> ".pdf",#[[2]], True, False] &/@ handle;*)
	];

	(*setup the rows*)
	If[Length@header[[1]] <= 1
		, (*true \[Rule] single header row*)
		rowsL = {HeaderLineLaTeX[#] &/@ header};
		headerDividerIndex = 2;

		, (*false \[Rule] multiple header rows*)
		rowsL = HeaderLineLaTeX[#] &/@ # &/@ header;
		headerDividerIndex = Length@header +1;
	];

	AppendTo[rowsL, #] &/@ rows;

	(*top & bottom rule*)
	dividers = Table[False, Length@rows +headerDividerIndex];
	dividers[[{1, headerDividerIndex, -1}]] = True;

	Print@SetPrecision[Grid[
		rowsL,
		Dividers->{False,  dividers},
		Alignment->{{Left}},
		Spacings->{2, 1},
		ItemSize->Full], precision]
];

(*MWE*)
(*header = {"main", "second","third", "fourth"};
rows = RandomReal[{0, 1}, {3, 4}];
ExportTableLaTeX["testTable", header, rows, 3, True];*)


(* ::Subsection::Closed:: *)
(*Averages & Grid setup*)


NumToStr[num_] := ToString@N@num;


MeanStdStr[nameKey_, meanRange_, vec_] := Module[{mean, std, box, boxS}, 
	Flatten @{
		nameKey,
		{
			(*switch vapor/liquid*)
			box = Global`expInd[[#]];
			boxS = " Box " <> If[# == 1,

			(*first box \[Rule] vapor box*)
			"V",

			(*second box \[Rule] liquid box*)
			"L"
		];

		(*get the arithmetic mean and standard deviation*)
		mean = Mean@vec[[box, meanRange]];
		std = StandardDeviation@vec[[box, meanRange]];

		(*add the values to the result association*)
		AssociateTo[Global`assocResults, {
			nameKey <> boxS <> " mean" -> N@mean,
			nameKey <> boxS <> " std"  -> N@std}];

		(*return the results to display them in the grid*)
		{NumToStr@mean, NumToStr@std}

	} &/@ Range@Length@vec}
];

MeanStdCol[nameKey_, systemLoc_, meanRange_, vec_] := Module[{mean, std, box, boxS},
	Flatten @({

		(*switch vapor/liquid*)
		box = Global`expInd[[#]];
		boxS = " Box " <> If[# == 1,
			(*first box \[Rule] vapor box*)
			"V",

			(*second box \[Rule] liquid box*)
			"L"
		];

		(*get the arithmetic mean and standard deviation*)
		mean = N@Mean@vec[[box, meanRange]];
		std = N@StandardDeviation@vec[[box, meanRange]];

		(*add the values to the result association*)
		AssociateTo[Global`assocResults, {
			nameKey <> boxS <> " " <> systemLoc[[#]] <> " mean" -> N@mean[[#]],
			nameKey <> boxS <> " " <> systemLoc[[#]] <> " std"  -> N@std[[#]]}
		] &/@ Range@Length@systemLoc;

		(*return the results to display them in the grid*)
		Column@mean, Column@std

	} &/@ Range@Length@vec)
];

MeanStdSys[nameKey_, systemLoc_, meanRange_, vec_] := Flatten @{
	Column[systemLoc],
	MeanStdCol[nameKey, systemLoc, meanRange, vec]
};


(* ::Subsection::Closed:: *)
(*Associations*)


List2assoc[nameAssoc_] := List2assoc[Symbol[nameAssoc], nameAssoc];
List2assoc[listAssoc_, nameAssoc_] := Module[{assocLoc, key, add, listFlat},

	(*Setup a new association so that the elements in the list can be added*)
	assocLoc = <||>;

	(*
	Flatten the given list to a one dimensional list so that the elements can be added
	to a one-dimensional association
	*)
	listFlat = Flatten@listAssoc;

	(*Iterate through all elements in the given  list and add the new entry with its index*)
	(
		key = nameAssoc <> "_" <> ToString@(Length@assocLoc + 1);
		AssociateTo[assocLoc, (key -> #)];
	) &/@ listAssoc;

	assocLoc
];


Boxes2assoc[nameAssoc_] := Boxes2assoc[Symbol[nameAssoc], nameAssoc];
Boxes2assoc[listAssoc_, nameAssoc_] := Module[{assocLoc, key, add, listFlat},

	(*Setup a new association so that the elements in the list can be added*)
	assocLoc = <||>;

	(*check if the list correpons to the number of boxes (2)*)
	If[Length@listAssoc != 2,

		Echo["List does not match expected format!"];
		Return[assocLoc];
	];

	AssociateTo[assocLoc, nameAssoc <> " - Box V" -> listAssoc[[Global`expInd[[1]]]]];
	AssociateTo[assocLoc, nameAssoc <> " - Box L" -> listAssoc[[Global`expInd[[2]]]]];

	(*return the new association*)
	assocLoc
];


(* ::Section::Closed:: *)
(*Custom Parameters*)


checkGetAssocParameter[assoc_, parameter_] := Module[{value},

	value = False;
	If[KeyExistsQ[assoc, parameter],
		value = assoc[[parameter]];
	];
	value
];


(* ::Section::Closed:: *)
(*Debugging Helper*)


(* ::Text:: *)
(*The "Echo" function is a handy tool of Mathematica to print intermediate calculation values. However, the UI freezes if too many Echos are issued. Therefore the following functions introduce a short pause-period to prevent this freeze. Thereby the calculation can be aborted manually by using "Evaluation" / "Abort Evaluation".*)


(*setting: how long the output shall pause after each echo-output*)
tEcho = 0.5;

(*setting: deactivate all debugging trace by default*)
bEcho = False;


EchoP[var_] := If[bEcho, Echo[var]; Pause[tEcho];];
EchoP[var_, notice_] := If[bEcho, Echo[var, notice]; Pause[tEcho];];


(* ::Section::Closed:: *)
(*Checking Functions*)


visualizeCheck[cond_] := If[cond, "\[HappySmiley]","\[FreakedSmiley]"]


HelloWorld[] := Module[{},Print@"Hello World";];


(* ::Chapter::Closed:: *)
(*End of Package*)


End[];


EndPackage[];


(* ::Chapter::Closed:: *)
(*Literature*)


(* ::Text:: *)
(*There is no package specific literature.*)
