(* ::Package:: *)

(* ::Title:: *)
(*MolecularSamplingPackage*)


(* ::Text:: *)
(*Maintained by Thomas Wallek*)


(*
compiler="mingw64"; (* "visualStudio" or "mingw64" *)

Switch[compiler
	,"mingw64",
		Needs["CCompilerDriver`GenericCCompiler`"];
		$CCompiler={"Compiler"->GenericCCompiler,"CompilerInstallation"->"C:/msys64/mingw64","CompilerName"->"x86_64-w64-mingw32-gcc.exe"
	,"visualStudio",
		Nothing
];
Clear[compiler]
*)


If[$InputFileName === "", SetDirectory[NotebookDirectory[]];]


BeginPackage["MolecularSampling`",{"PropertyPackage`","ForceFieldOPLS`"}];


(* ::Chapter::Closed:: *)
(*Usage Definitions*)


(* I/O Functions *)
importMolFiles::usage="importMolFiles[system,subdir] \[Rule] Import molecules, i.e., atoms and their coordinates, from *.mol files in subdir";
fileList::usage="fileList[subdir] \[Rule] List files in subdirectory subdir, which have the file extension subdir";
convertTopToTxt::usage="";
fromTopFile::usage="Uses convertTopToTxt and reads in the data contained";
takeFromTopFile::usage="Further formats the data from fromTopFile";
checkCompatibility::usage="Checks if .mol and .top files of a molecule are compatible; only works if atoms have 1-letter-names";
interfaceAtoms::usage="Dynamic interface for assigning atom types, takes input from takeFromTopFile";
interfaceBonds::usage="Dynamic interface for assigning bond types, takes input from takeFromTopFile";
interfaceAngles::usage="Dynamic interface for assigning angle types, takes input from takeFromTopFile";
interfaceDihedrals::usage="Dynamic interface for assigning dihedral types, takes input from takeFromTopFile";
assignNew::usage="Takes saved data for a molecule and creates the associations for energetic calculations";

(* The OPLS-AA Force Field *)
conv::usage="Conversion factor for for the Coulomb term";
eNonBonded::usage="Nonbonded interactions, represented by Coulomb plus Lennard-Jones terms";
eBonded::usage="Bonded interactions, accounting for bond stretching and angle bending";
eTorsion::usage="Intramolecular torsional energy";
u1::usage="Net-energy contribution of 1-4 pairs";
u2::usage="Net-energy contribution of bond stretching and angle bending";
u3::usage="Net-energy contribution of torsion";

(* Visualization of Molecules and Clusters *)
GetAtomColor::usage="Returns the color for the specified atom name";
visualizeMolFile::usage="Visualize one single molecule numbering atoms in the order of occurrence in the *.mol file";
visualizeMolecule::usage="Visualize one single molecule using Van der Waals-radii";
visualizeCluster::usage="Visualize a cluster of molecules using Van der Waals-radii";
visualizeBasics::usage="Visualize basic information about molecules";

(* Geometry & Arrangement of Molecules & Clusters *)
bondLength::usage="Bond length between two atoms {first, second} spanning a vector";
angleBetweenThreeAtoms::usage="Angle between three atoms {first, middle, second} spanning two vectors";
dihedralBetweenFourAtoms::usage="Dihedral angle between two atom-branches = 4 atoms (1,2 and 3-4, connected via 2-3)";
bondDistance::usage="Bond length between two atoms";
moleculeGeometry::usage="Unites the geometric functions angleBetweenThreeAtoms, dihedralBetweenFourAtoms and bondDistance";
rotateMolecule::usage="";
randomMoleculeRotation::usage="";
mirrorMolecule::usage="Mirrors a molecule with respect to a plane with a normal-vector vec";

eulerAngles::usage="Yields the systems Euler angles in standard-x-convention for a list of triplets of angles of rotation around x,y,z axes";
eulerAnglesPlusRot::usage="See function eulerAngles. Here we can superimpose an additional, arbitrary rotation";
eulerAnglesInZ::usage="Uses eulerAngles and eulerAnglesPlusRot after bringing pairs to the reference-z-orientation";
newCoordSys::usage="Calculates the new axes-unit-vectors after rotation";

cubeRotationAngles::usage="Vector of all 24 rotational angles for cube symmetry";
cubeRotationAxes::usage="Matrix of all 24 rotational axes-vectors for cube symmetry";
cubeMirrOnlyVectors::usage="";
cubeMirrRotMirrVectors::usage="";
cubeMirrRotRotVectors::usage="";
cubeMirrRotRotAngles::usage="";
cubeRotMirrRotVectors::usage="";
cubeRotMirrRotAngles::usage="";
cubeRotMirrMirrVectors::usage="";
cubeMirrVectors::usage="";
cubeMirrRotVectors::usage="";
cubeMirrRotAngles::usage="";

toOrigin::usage="Shift the molecule center to the geometric center";

displacementAlongAxis::usage="Returns the shift of coordinates in axis-direction";
uniteMolecules::usage="Function to combine molecules to formal clusters";
axialShiftDistanceVdW::usage="Function axialShiftDistance, but with touching VdW surfaces instead of PAC-MAC distance";
fanDisplacement::usage="Function used for Fan et al. sampling method (1992)";

GetAtomAnglePairs::usage="This function returns all sets of 3 Atom pairs, that are linked in a given moeculeBond List.\nIt does not consider ring formations.";
GetTorsionAxisPairs::usage="This function returns all sets of 4 Atoms in a molecule that function as a torsion axis for the molecule.\nIt does not consider ring formations.";


(* -------------------------------------------------------------------------------- *)
(* Monte Carlo functions and definitions *)
(* -------------------------------------------------------------------------------- *)

(* Constants and Variables *)
ensembleTypeList::usage="Association with the possible ensemble types";

(* Visualizations *)
GetAtomRadius::usage="Returns the radius defined in the Association 'vdwRadius'";
visualizeClusterMC::usage="visualizeClusterMC[coord,type] \[Rule] Visualization of molecules in 3D Coordinates";
visualizeBoxesMC::usage="visualizeBoxesMC[moleculeAtomsCoordVec,moleculeAtomsNamesVec,Vvec,plotSize] \[Rule] Visualization of two Boxes filled with Molecules";

(* Base Functions *)
SecondsToDurationString::usage="SecondsToDurationString[seconds] \[Rule] Convert Seconds to a String with the Format __h __min __s";
compDir::usage="compDir[theta1,z1] \[Rule] Function used in randomDirection";
GetMoleculeCenter::usage="";
GetNonBondedIntraPairs::usage="GetNonBondedIntraPairs[moleculeBonds] \[Rule] Returns an association containing all pairs of atoms in the molecule separated by three or more bonds paired to a list of scaling factors for the LJ and Coulomb interactions.";
GetConnectedAtoms::usage = "GetConnectedAtoms[moleculeBondsList,flexibleBondPair] \[Rule] Returns the list of atoms to be adjusted in case the flexible bond is changed.";
randomDirection::usage="Return a random-direction-unit-vector";
randomMoleculeRotationMC::usage="randomMoleculeRotationMC[coord] \[Rule] Randomly rotates a molecule coordinate-association";
randomOrientationChange::usage="randomOrientationChange[coord,angle] \[Rule] Angle permutation of molecule coordinate-association";

(* Create Box Coordinates *)
CreateBoxCoordinates::usage="CreateBoxCoordinates[n,boxLength] \[Rule] create box coordinates for n particles with a FCC grid system";

(* Simulation Overview *)
ShowMonteCarloOverviewEnvironment::usage="Overview over Monte Carlo environment";
ShowMonteCarloOverviewDetails::usage="Overview over simulation details";
ShowMonteCarloOverviewOPLS::usage="Overview over OPLS setting";
ShowMonteCarloOverview::usage="Overview over all current Monte Carlo Setings";

(* Metropolis Function *)
metropolis::usage="metropolis[deltaU,R,T] \[Rule] Check if Move is Accepted or Rejected based on the Energy Difference";

(* Pair Interactions *)
ffEnergy::usage="ffEnergy[rij,qiqje2,\[Sigma]ij,\[Epsilon]ij] \[Rule] OPLS Force Field Calculation of the Interaction Energy of two Atoms";
ffEnergy12::usage="ffEnergy12[rij,\[Sigma]ij,\[Epsilon]ij] \[Rule] Calculates the repulsive part of the LJ potential";
ffEnergy6::usage="ffEnergy6[rij,\[Sigma]ij,\[Epsilon]ij] \[Rule] Calculates the attractive part of the LJ potential";
ffEnergy1::usage="ffEnergy6[rij,\[Sigma]ij,\[Epsilon]ij] \[Rule] Calculates the coulombic part of the OPLS force field potential";
ffVirial::usage="ffVirial[rij,qiqje2,\[Sigma]ij,\[Epsilon]ij] \[Rule] OPLS Force Field Calculation of the Interaction Force of two Atoms";
ffVirial12::usage="ffVirial12[rij,\[Sigma]ij,\[Epsilon]ij] \[Rule] Calculates the repulsive part of the Interaction Force";
ffVirial6::usage="ffVirial6[rij,\[Sigma]ij,\[Epsilon]ij] \[Rule] Calculates the attractive part of the Interaction Force";
ffVirial1::usage="ffVirial1[rij,\[Sigma]ij,\[Epsilon]ij] \[Rule] Calculates the coulombic part of the OPLS force field Interaction Force";

(* Intramolecular Energy Calculation *)
ffEnergyNonBondedIntra::usage = "ffEnergyNonBondedIntra[atomCoords,atomIndicators,nonBondedPairs,ffParamsNonBonded,nonBondedPairsParams] -> Calculates the non-bonded intramolecular interaction energy.";
ffEnergyBondStretching::usage = "ffEnergyBondStretching[atomCoords,ffParamsBondStretching] -> Calculates the bond stretch intramolecular interaction energy.";
ffEnergyAngleBending::usage = "ffEnergyAngleBending[atomCoords,ffParamsAngleBending] -> Calculates the angle bend intramolecular interaction energy.";
ffEnergyTorsion::usage = "ffEnergyTorsion[atomCoords,ffParamsTorsion] -> Calculates the torsion axis bend intramolecular interaction energy.";
ffEnergyIntra::usage = "ffEnergyIntra[atomCoords,atomIndicators,nonBondedPairs,paramsList] -> Calculates the total intramolecular energy.";

(* Obsolete Functions *)
boltzmann::usage="boltzmann[deltaU,R,T] \[Rule] Compiled (Metropolis) Boltzmann factor";
volumeChangeProbability::usage="volumeChangeProbability[voldV,R,T,volenergyList1,volenergyList2,energyListvec,nvec,Vvec] \[Rule] Compiled acceptance probability of volume change of boxes";
ff::usage="ff[rij,qiqje2,\[Sigma]ij,\[Epsilon]ij] \[Rule] Compiled evaluation of potential energies {LJ12, LJ6, Coulumb}";
forceVdW::usage="forceVdW[rij,\[Sigma]ij,\[Epsilon]ij] \[Rule] Compiled force due to LJ interaction {LJ12, LJ6}";
forceCoul::usage="forceCoul[rij,qiqje2] \[Rule] Compiled force due to Coulomb interaction";


(* ::Chapter:: *)
(*Function Definitions*)


Begin["`Private`"];


(* ::Section::Closed:: *)
(*I/O Functions*)


(* ::Text:: *)
(*Import of molecular structures from *.mol files; coordinates in \[Angstrom] = 10E-10 m.*)
(*System is a list of molecules, e.g., system = {"acetone","npentane","isopentane","neopentane"};*)
(*The function returns a list of associations related to the molecules of "system";*)
(*Each key of a molecule identifies an atom type (String) along with its coordinates (Real, in \[Angstrom]).*)


(* second parameter holds the subdirectory from which themol files will be imported *)
importMolFiles[system_,subdir_] := Module[{tmp},

	tmp = Import[FileNameJoin[{subdir,#<>".mol"}],{{"VertexTypes","VertexCoordinates"}}] &/@ system;

	If[$VersionNumber < 12.,
		(* In version 11.3, the Import[] function multiplies the coordinates by 100. which is reversed in the following *)
		Association@@@Flatten[
			Table[
				Thread[#->
					Table[{
						tmp[[i,1,j]],(* atom type, String *)
						tmp[[i,2,j,1]]/100.,(* x-coordinate in \[Angstrom], Real *)
						tmp[[i,2,j,2]]/100.,(* y-coordinate in \[Angstrom], Real *)
						tmp[[i,2,j,3]]/100.},(* z-coordinate in \[Angstrom], Real *)
						{j,1,Length[tmp[[i,1]]]}
					]
				] &/@ {Range[Length[tmp[[i,1]]]]},
				{i,1,Length[tmp]}
			]
		,1],

		(* Version 12.0 imports correctly and adds the unit [\[Angstrom]] *)
		Association@@@Flatten[
			Table[
				Thread[#->
					Table[{
						tmp[[i,1,j]],(* atom type, String *)
						QuantityMagnitude[tmp[[i,2,j,1]]],(* x-coordinate in \[Angstrom], Real *)
						QuantityMagnitude[tmp[[i,2,j,2]]],(* y-coordinate in \[Angstrom], Real *)
						QuantityMagnitude[tmp[[i,2,j,3]]]},(* z-coordinate in \[Angstrom], Real *)
						{j,1,Length[tmp[[i,1]]]}
					]
				] &/@ {Range[Length[tmp[[i,1]]]]},
				{i,1,Length[tmp]}
			]
		,1]
	]
];


(* ::Text:: *)
(*List files in subdirectory subdir, which have the file extension subdir*)


(* EDIT (Thaler Sebastian): do not change working directory on execution *)
fileList[subdir_] := Module[{currentDir,returnView},
	currentDir = Directory[];
	
	(* set directory to subdir *)
	SetDirectory[FileNameJoin[{NotebookDirectory[],subdir}]];
	returnView = Style[Multicolumn[FileNames[StringJoin["*.",subdir]],5],Gray];
	
	(* reset directory *)
	SetDirectory[currentDir];
	
	(* return view with files *)
	returnView
]


(* ::Text:: *)
(*Reading in from .top files*)


convertTopToTxt[name_] := Module[{},
	Export[StringJoin[name,".txt"],Import[StringJoin[name,".top"]]]
];

fromTopFile[name_] := Module[{stream,types,list,endOfList,results,impropDihedIndex},
	(*convertTopToTxt[name];*)
	types = {"[ atoms ]","[ bonds ]","[ pairs ]","[ angles ]","[ dihedrals ]","restraint file"}; (* matches the text in .top files *)
	results = {};
	Do[
		stream = OpenRead[StringJoin[name,".txt"]];
		Find[stream,types[[index]]]; (* looks for desired part of list *)
		list = ReadList[stream,String]; (* reads it from there *)
		Do[
			If[StringCount[list[[i]],types[[index + 1]]] > 0,Break[],endOfList = i],
			{i,1,Length[list]}
		];(* counts until we reach the next keyword of types *)
		If[index == 1,
			results = Append[results,list[[3;;endOfList]]],
			results = Append[results,list[[2;;endOfList]]];
		];
		Close[stream];
		,{index,1,Length[types]-1}
	];
	If[Max@StringCount[results[[-1,All]],"dihedrals"] > 0,
		(* improper dihedrals found *)
		impropDihedIndex = Flatten[Position[#,Max[#]]&@StringCount[results[[-1,All]],"dihedrals"],1][[1]];
		results[[-1]] = results[[-1,1;;impropDihedIndex-1]];
		Print["Improper dihedrals found, please check in .top file and, if needed, add manually to structures in Mathematica."];
	];
	Print["There are "<>ToString[Length@results[[1]]]<>"\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\"atoms\", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\",\", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)"<>ToString[Length@results[[2]]]<>"\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\"bonds\", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\",\", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)"<>ToString[Length@results[[3]]]<>"\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\"pairs\", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\",\", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)"<>ToString[Length@results[[4]]]<>"\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\"angles\", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\"and\", \"SearchResultSummary\"]\)\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)"<>ToString[Length@results[[5]]]<>"\!\(\*
		StyleBox[\" \", \"SearchResultSummary\"]\)dihedrals."
	];
	results
];

takeFromTopFile[results_] := Module[{atomNames,bonds,pairs,angles,dihedrals},

	atomNames = StringTake[#,{10,17}] &/@ results[[1]];

	bonds = Partition[
		Riffle[
			StringTrim[StringTake[#,5]],StringTrim[StringTake[StringTake[#,11],-3]]
		]
	,2] &@ results[[2]];

	pairs = Partition[
		Riffle[
			StringTrim[StringTake[#,5]],StringTrim[StringTake[StringTake[#,11],-3]]
		]
	,2] &@ results[[3]];

	angles = Partition[
		Riffle[
			Riffle[
				StringTrim[StringTake[#,5]],StringTrim[StringTake[StringTake[#,11],-3]]
			],StringTrim[StringTake[StringTake[#,17],-3]],{3,-1,3}
		]
	,3] &@ results[[4]];

	dihedrals = Partition[
		Riffle[
			Riffle[
				Riffle[
					StringTrim[StringTake[#,5]],StringTrim[StringTake[StringTake[#,11],-3]]
				],StringTrim[StringTake[StringTake[#,17],-3]],{3,-1,3}
			],StringTrim[StringTake[StringTake[#,23],-3]],{4,-1,4}
		]
	,4] &@ results[[5]];

	(*output*)
	{atomNames,ToExpression@bonds,ToExpression@pairs,ToExpression@angles,ToExpression@dihedrals}
];


(* ::Text:: *)
(*Interface functions for structure editing:*)


(* only works for atoms with 1-character-names (like C, H, N, O, ...; but not e.g. Na, Mg, ... *)
checkCompatibility[molec_,top_] := Module[{}, (* check if structure file (mol/pdb/...) and top file fit together *)
	If[Length[Flatten@Keys@Flatten@{molec}] == Length[top[[1]]], (* Flatten@Keys@Flatten@ for flexible input *)
		If[Sort[Values[Flatten[{molec}][[1]]][[All,1]]] == (Sort[StringTake[GROMACSatoms[[#,1]],1] &/@ top[[1]]]),
			If[Values[Flatten[{molec}][[1]]][[All,1]] == (StringTake[GROMACSatoms[[#,1]],1]&/@top[[1]]),
				Print["Structure file and .top file seem to fit together."],
				Print["Structure file and .top file have different orderings."]
			],
			Print["Structure file and .top file contain different atoms."]
		],
		Print["Structure file and .top file have unequal lengths."]
	]
]


(* ::Text:: *)
(*Assign atoms:*)


interfaceAtoms[name_,top_] := DynamicModule[{atomNumber,atomType,savedActions={},moleculeName=name},
	fileList["mol"];
	Row[{
		Panel[Column[{
			Button["Use .top file",savedActions=Append[savedActions,MapThread[#1->GROMACSatoms[[#2,5]]&,{Range[Length[top[[1]]]],top[[1]]}]]],
			Row[{"Atom Number ",PopupMenu[Dynamic[atomNumber],Keys[importMolFiles[{moleculeName}]][[1]]]}],
			Row[{"Atom Type ",PopupMenu[Dynamic[atomType],Keys[oplsNonBonded]]}],
			Button["Assign",savedActions=Append[savedActions,atomNumber-> atomType]],
			Button["Remove last entry",If[Length@savedActions>0,savedActions=Drop[savedActions,-1]]],
			Button["Remove ALL entries",savedActions={}],
			Button["Save list to file",Put[Association[savedActions],StringJoin[ToString[NotebookDirectory[]],{"\\top\\","\\associations\\",StringJoin[moleculeName,"_atoms.m"]}]]]
		}],ImageSize->{300,300}],
		Panel[visualizeMolFile[importMolFiles[{moleculeName}][[1]]],ImageSize->{300,300}],
		Dynamic@Panel[Column@{"List of saved actions",If[Length@savedActions>8,Column[ToString[#]&/@savedActions[[-8;;-1]]],Column[ToString[#]&/@savedActions]]},ImageSize->{200,300}]
	}]
]


(* ::Text:: *)
(*Assign bonds:*)


interfaceBonds[name_,top_] := DynamicModule[{bondName,bondType,savedActions={},moleculeName=name},
	fileList["mol"];
	Row[{
		Panel[Column[{
			Row[{"Bond Name ",PopupMenu[Dynamic[bondName],top[[2]]]}],
			Row[{"Bond Type  ",PopupMenu[Dynamic[bondType],Keys[oplsBonded]]}],
			Button["Assign",savedActions=Append[savedActions,bondName-> bondType]],
			Button["Remove last entry",If[Length@savedActions>0,savedActions=Drop[savedActions,-1]]],
			Button["Remove ALL entries",savedActions={}],
			Button["Save list to file",Put[Association[savedActions],StringJoin[ToString[NotebookDirectory[]],{"\\top\\","\\associations\\",StringJoin[moleculeName,"_bonds.m"]}]]]
		}],ImageSize->{300,300}],
		Panel[visualizeMolFile[importMolFiles[{moleculeName}][[1]]],ImageSize->{300,300}],
		Dynamic@Panel[Column@{"List of saved actions",If[Length@savedActions>8,Column[ToString[#]&/@savedActions[[-8;;-1]]],Column[ToString[#]&/@savedActions]]},ImageSize->{200,300}]
	}]
]


(* ::Text:: *)
(*Assign angles:*)


interfaceAngles[name_,top_] := DynamicModule[{angleName,angleType,savedActions={},moleculeName=name},
	fileList["mol"];
	Row[{
		Panel[Column[{
			Row[{"Angle Name ",PopupMenu[Dynamic[angleName],top[[4]]]}],
			Row[{"Angle Type  ",PopupMenu[Dynamic[angleType],Keys[oplsBonded]]}],
			Button["Assign",savedActions=Append[savedActions,angleName-> angleType]],
			Button["Remove last entry",If[Length@savedActions>0,savedActions=Drop[savedActions,-1]]],
			Button["Remove ALL entries",savedActions={}],
			Button["Save list to file",Put[Association[savedActions],StringJoin[ToString[NotebookDirectory[]],{"\\top\\","\\associations\\",StringJoin[moleculeName,"_angles.m"]}]]]
		}],ImageSize->{300,300}],
		Panel[visualizeMolFile[importMolFiles[{moleculeName}][[1]]],ImageSize->{300,300}],
		Dynamic@Panel[Column@{"List of saved actions",If[Length@savedActions>8,Column[ToString[#]&/@savedActions[[-8;;-1]]],Column[ToString[#]&/@savedActions]]},ImageSize->{200,300}]
	}]
]


(* ::Text:: *)
(*Assign dihedrals:*)


interfaceDihedrals[name_,top_] := DynamicModule[{dihedrName,dihedrType,savedActions={},moleculeName=name},
	fileList["mol"];
	Row[{
		Panel[Column[{
			Row[{"Dihedral Name ",PopupMenu[Dynamic[dihedrName],top[[5]]]}],
			Row[{"Dihedral Type  ",PopupMenu[Dynamic[dihedrType],Keys[oplsTorsion]]}],
			Button["Assign",savedActions=Append[savedActions,dihedrName-> dihedrType]],
			Button["Remove last entry",If[Length@savedActions>0,savedActions=Drop[savedActions,-1]]],
			Button["Remove ALL entries",savedActions={}],
			Button["Save list to file",Put[Association[savedActions],StringJoin[ToString[NotebookDirectory[]],{"\\top\\","\\associations\\",StringJoin[moleculeName,"_dihedrals.m"]}]]]
		}],ImageSize->{300,300}],
		Panel[visualizeMolFile[importMolFiles[{moleculeName}][[1]]],ImageSize->{300,300}],
		Dynamic@Panel[Column@{"List of saved actions",If[Length@savedActions>8,Column[ToString[#]&/@savedActions[[-8;;-1]]],Column[ToString[#]&/@savedActions]]},ImageSize->{200,300}]
	}]
]


(* ::Text:: *)
(*Function to import from saved files in directory NotebookDir/top/associations:*)
(*whatToImport looks like this*)


(*whatToImport={(* Change to false in case you don't want specific parts to be newly assigned *)
(* atoms *)True,
(* bonds *)False,
(* angles *)False,
(*dihedrals *)False};*)


assignNew[moleculeName_,whatToImport_] := Module[{atoms,bonds,angles,dihedrals,output},
	output = {};
	If[whatToImport[[1]] == True, (* atom types *)
		atoms = Get[StringJoin[ToString[NotebookDirectory[]],{"\\top\\","\\associations\\",StringJoin[moleculeName,"_atoms.m"]}]];
		output = Append[
			output,
			AssociationThread[Keys@atoms,Map[Join[oplsNonBonded,oplsNonBondedGROMACS],Values@atoms]]
		],
		output = Append[output,{}];
	];
	If[whatToImport[[2]] == True, (* atom types *)
		bonds = Get[StringJoin[ToString[NotebookDirectory[]],{"\\top\\","\\associations\\",StringJoin[moleculeName,"_bonds.m"]}]];
		output = Append[output,bonds],
		output = Append[output,{}];
	];
	If[whatToImport[[3]] == True, (* angles types *)
		angles = Get[StringJoin[ToString[NotebookDirectory[]],{"\\top\\","\\associations\\",StringJoin[moleculeName,"_angles.m"]}]];
		output = Append[output,angles],
		output = Append[output,{}];
	];
	If[whatToImport[[4]] == True, (* dihedral types *)
		dihedrals = Get[StringJoin[ToString[NotebookDirectory[]],{"\\top\\","\\associations\\",StringJoin[moleculeName,"_dihedrals.m"]}]];
		output = Append[output,dihedrals],
		output = Append[output,{}];
	];
	output
]



(* ::Section::Closed:: *)
(*The OPLS-AA Force Field*)


(* ::Text:: *)
(*OPLS at the beginning was parametrized not only for biomolecular systems, but also organic liquids. The parameters were optimized to fit experimental densities and heats of vaporisation of liquids, as well as gas-phase torsional profiles. The force field has being updated and many independent modifications to the force field have been developed. OPLS is one of the most widely parametrized force field available.*)
(**)
(*The OPLS (Optimized Potentials for Liquid Simulations) AA (all atoms) force field considers:*)
(*1 nonbonded, intermolecular contribution: nonbonded Coulomb plus Lennard-Jones terms*)
(*3 bonded, intramolecular contributions: bond stretching, angle bending, torsion*)
(**)
(*Literature: Jensen 1999, chapter 2; Jorgensen 1996, including Supporting Information. see also Jorgensen's website: http://zarbi.chem.yale.edu/oplsaam.html*)


(* ::Subsection::Closed:: *)
(*Force field equations*)


(* ::Text:: *)
(*The nonbonded intermolecular interactions are represented by the Coulomb plus Lennard-Jones terms. *)
(*The same expression is used for intramolecular nonbonded interactions between all pairs of atoms (i < j) separated by three or more bonds (\[GreaterSlantEqual] 1-4): the coefficient fij is then equal to 0.0 for any i-j pairs connected by a valence bond (1-2 pairs) or a valence bond angle (1-3 pairs). fij = 0.5 for 1-4 interactions (atoms separated by exactly three bonds) to permit use of the same parameters for inter- and intramolecular interactions; and fij = 1.0 for all of the other cases (Kaminski 2001).*)
(**)
(*A conversion factor to ensure consistency of units for the Coulomb term (please re-check):*)


Unprotect[conv];
conv = (6.022141 1.6021766^2)/(4 Pi 8.85419 4.184) 10^4;
Protect[conv];


eNonBonded = Compile[{qi,qj,\[Sigma]ii,\[Sigma]jj,\[Epsilon]ii,\[Epsilon]jj,r,fij},
	Module[{\[Sigma]ij,\[Epsilon]ij},
		\[Sigma]ij = Sqrt[\[Sigma]ii * \[Sigma]jj];
		\[Epsilon]ij = Sqrt[\[Epsilon]ii * \[Epsilon]jj];
		(conv * qi * qj / r + 4 * \[Epsilon]ij * (\[Sigma]ij^12 / r^12 - \[Sigma]ij^6 / r^6)) * fij
	]
];


(* ::Text:: *)
(*The intramolecular energetics for bond stretching and angle bending are represented by the following eq. which corresponds to the first term of a Taylor series. r_eq is the equilibrium bond length for which the potential energy contribution is zero and \[Theta]_eq is the equilibrium (i.e., minimum energy) bond angle. Values for r_ eq can be obtained from X-ray diffraction experiments, while the spring constant K may be estimated from infrared or Raman spectra.*)
(*Function parameters, xeq = r_eq or \[Theta]_eq, and x = r or \[Theta].*)


eBonded = Function[{K,xeq,x},
	K (x-xeq)^2
];


(* ::Text:: *)
(*The last intramolecular term is for the torsional energy, where \[Phi]_i is the torsional or dihedral angle, V1, V2, and V3 are the coefficients in the Fourier series, and f1, f2, and f3 are phase angles, which are all zero for the present systems. The total torsional energy is then the sum of this series for each dihedral angle. The torsional parameters are usually derived from ab initio calculations and then refined using*)
(*experimental data such as molecular geometries or vibrational spectra*)


eTorsion = Function[{V1,V2,V3,f1,f2,f3,\[Phi]},
	V1/2(1+Cos[\[Phi]+f1])+V2/2(1+Cos[\[Phi]+f2])+V3/2(1+Cos[\[Phi]+f3])
];


(* ::Text:: *)
(*Functions used for calculating internal energies of a molecule:*)
(*u = Table[0.,{4}];*)
(*u1 == u[[1]] == 1-4 pairs*)
(*u2 == u[[2]] == bonds and angles*)
(*u3 == u[[3]] == torsion*)
(**)
(*u4 would then be the intermolecular energy, obtained via sampling*)


u1[molecule_,intraPairs_,rIntra_,nonBondedList_] := Module[{}, (* kcal/mol *)
	Total[
		Table[
			eNonBonded[
				nonBondedList[[intraPairs[[ipair,1]],1]],(* q1 *)
				nonBondedList[[intraPairs[[ipair,2]],1]],(* q2 *)
				nonBondedList[[intraPairs[[ipair,1]],2]],(* \[Sigma]1 *)
				nonBondedList[[intraPairs[[ipair,2]],2]],(* \[Sigma]2 *)
				nonBondedList[[intraPairs[[ipair,1]],3]],(* \[Epsilon]1 *)
				nonBondedList[[intraPairs[[ipair,2]],3]],(* \[Epsilon]2 *)
				rIntra[[ipair]],(* r *)
				intraPairs[[ipair,3]](* f12 *)
			]
			,{ipair,Length[intraPairs]}
		]
	]
];

u2[bondsAnglesGeometry_,bondsAnglesTypes_] := Module[{},
	Table[(* first for bonds (k=1), then for angles (k=2) *)
		If[KeyExistsQ[bondsAnglesTypes[[k]],#] == True,
			eBonded[oplsBonded[[(bondsAnglesTypes[[k]])[#],2]],oplsBonded[[(bondsAnglesTypes[[k]])[#],1]],(bondsAnglesGeometry[[k]])[#]],
			0
		] &/@ Keys[bondsAnglesGeometry[[k]]]
		,{k,{1,2}}
	]
];

u3[dihedralsGeometry_,dihedralTypes_] := Module[{},
	If[KeyExistsQ[dihedralTypes,#] == True,
		eTorsion @@ Append[(oplsTorsion[[dihedralTypes[#]]])[[1;;6]],dihedralsGeometry[#]],
		0
	] &/@ Keys[dihedralsGeometry]
];


(* ::Section::Closed:: *)
(*Visualization of Molecules and Clusters*)


(* ::Subsection::Closed:: *)
(*Atom Colors*)


(*uncomment debug option below to return all colors in order to selct a distinctive new color*)
(*GetAtomColor["default"]*)


GetAtomColor[atomName_] := Module[{atomColors},
	atomColors = <|
		"default"->RGBColor[0.5,0.5,0.5],
		"AC"->RGBColor[1,0.3,1] (*AC .. active *),
		"PR"->RGBColor[0.6,0.3,1](*PR .. previous *),
		"CFC"->RGBColor[0,0.5,0](*CFC .. continous fractional component *),
		"C"->RGBColor[0.3,0.3,0.3],
		"H"->RGBColor[0.7,0.7,0.7],
		"N"->RGBColor[0,0,1],
		"O"->RGBColor[1,0,0],
		"Ar"->RGBColor[0,0,0.4],
		"Kr"->RGBColor[0.4,0.8,1],
		
		"LJ_ref"->RGBColor[0,0,0.4],
		
		"Methane"->RGBColor[0,0.7,0]
	|>;
	
	(*For debugging / defining new colors*)
	(*Echo@atomColors;*)
	
	(* RETURN *)
	If[MemberQ[Keys[atomColors],atomName],
		atomColors[atomName],
		atomColors["default"]
	]
];


(* ::Subsection::Closed:: *)
(*Visualize one single molecule numbering atoms in the order of occurrence in the *.mol file*)


(* ::Text:: *)
(*This is used to identify the functional groups which are needed to assign force field parameters.*)


visualizeMolFile[moleculeName_,molecule_] := Module[{atomsInMolecule},

	atomsInMolecule = DeleteDuplicates[Values[molecule][[All,1]]];

	Grid[{
		{Style[moleculeName,FontSize->18,Bold],SpanFromLeft,SpanFromLeft},
		{"Chemical Bonds","Order of Atoms in the .mol-File",SpanFromLeft},
		{
			If[MemberQ[ChemicalData["*"<>moleculeName<>"*"][[All,2]],moleculeName],
				ChemicalData[moleculeName,"MoleculePlot"],
				"Cannot visualize chemical bonds of "<>moleculeName"\n"<>"This chemical is not in the database..."
			],
			Show[
				Graphics3D[
					Text @@@ Partition[
						Riffle[
							Style[#,FontSize->16,Bold,White] &/@ Keys[molecule],
							Values[molecule][[All,2;;-1]]
						],2
					]
				],
				Graphics3D[
					{
						GetAtomColor[#[[1]]],
						Sphere[#[[2;;-1]],0.3]
					} &/@ Values[molecule[[All]]]
				],
				ImageSize->Medium
			],
			PointLegend[
				GetAtomColor[#] &/@ atomsInMolecule,
				atomsInMolecule,
				LegendMarkers->{Graphics3D[Sphere[{0,0,0}],Boxed->False],50},
				LegendLayout->"Column"
			]
		}
	},Alignment->{Center,Center},Dividers->{{True,True,False,True},{True,True,False,True}},Spacings->{2,2}]
];


(* ::Subsection::Closed:: *)
(*Visualize one single molecule using Van der Waals-radii*)


visualizeMolecule[coord_] := Graphics3D[Partition[
	Riffle[
		GetAtomColor[#] &/@ Values[coord[[All,1]]],
		Sphere[#[[1]],#[[2]]] &/@ Partition[Riffle[
			Values[coord[[All,2;;4]]],
			vdwRadius[#] &/@ Values[coord[[All,1]]]
		],2]
	],2
],ImageSize->Medium];


(* ::Subsection::Closed:: *)
(*Visualize a cluster of molecules using Van der Waals-radii*)


visualizeCluster[coord_] := Graphics3D[Join[
	Table[
		Partition[
			Riffle[
				GetAtomColor[#] &/@ Values[coord[[i,All,1]]],
				Sphere[#[[1]],#[[2]]] &/@ Partition[Riffle[
					Values[coord[[i,All,2;;4]]],
					vdwRadius[#] &/@ Values[coord[[i,All,1]]]
				],2]
			],2
		],
		{i,1,Length[coord]}
	]
]];


(* ::Subsection::Closed:: *)
(*Visualize basic information about molecules*)


visualizeBasics[system_,molecule_] := Grid[{
	system,
	ChemicalData[#,"Formula"] &/@ system,
	ChemicalData[#,"CASNumber"] &/@ system,
	"M = " <> ToString[ChemicalData[#,"MolarMass"]] &/@ system,
	"\!\(\*SubscriptBox[\(T\), \(b\)]\) = " <> ToString[ChemicalData[#,"BoilingPoint"]] &/@ system,
	"\!\(\*SubscriptBox[\(T\), \(m\)]\) = " <> ToString[ChemicalData[#,"MeltingPoint"]] &/@ system,
	ChemicalData[#] &/@ system,
	Table[visualizeCluster[{molecule[[i]]}],{i,1,Length[molecule]}],
	visualizeMolFile[#] &/@ molecule
},Frame->All];


(* ::Section::Closed:: *)
(*Geometry & Arrangement of Molecules & Clusters*)


(* ::Subsection::Closed:: *)
(*Geometry of atoms and molecules*)


(* ::Text:: *)
(*Bond length between two atoms {first, second} spanning a vector.*)
(*Parameters: association of the molecule, list of the two atoms*)


bondLength[molecule_,listOfTwoAtoms_] := Norm[
	Delete[molecule[[#]],1] &/@ listOfTwoAtoms
];


(* ::Text:: *)
(*Angle between three atoms {first, middle, second} spanning two vectors.*)
(*Parameters: association of the molecule, list of the three atoms*)


angleBetweenThreeAtoms[molecule_,listOfThreeAtoms_] := Module[{coordinates,vector},
	coordinates=Delete[molecule[[#]],1]&/@listOfThreeAtoms;
	vector={coordinates[[1]]-coordinates[[2]],coordinates[[3]]-coordinates[[2]]};
	N[VectorAngle[vector[[1]],vector[[2]]]/Degree]
];


(* ::Text:: *)
(*Dihedral angle between four atoms {first, second - third, fourth} spanning two vectors (12, 34) and a plane (2-3).*)
(*Parameters: association of the molecule, list of the four atoms*)


dihedralBetweenFourAtoms[molecule_,listOfFourAtoms_] := Module[{coordinates,vector,normalVector},
	coordinates=Delete[molecule[[#]],1]&/@listOfFourAtoms;
	vector={coordinates[[2]]-coordinates[[1]],coordinates[[3]]-coordinates[[2]],coordinates[[4]]-coordinates[[3]]};
	normalVector={Cross[vector[[1]],vector[[2]]],Cross[vector[[2]],vector[[3]]]};
	N[VectorAngle[normalVector[[1]],normalVector[[2]]]/Degree]
];


bondDistance[molecule_,listOfTwoAtoms_] := Module[{coordinates,vector},
	coordinates=Delete[molecule[[#]],1] &/@ listOfTwoAtoms;
	N[Norm[coordinates[[1]]-coordinates[[2]]]]
];


(* ::Text:: *)
(*Function uniting the geometric functions above:*)


moleculeGeometry[molecule_,connections_,type_] := Module[{molec,result},
	molec=(Flatten@{molecule})[[1]];(* universal format transformation for list/nonlist-input *)
	If[type=="angle",
		result=(#->angleBetweenThreeAtoms[molec,#]) &/@ connections,
		If[type=="dihedral",
			result=(#->dihedralBetweenFourAtoms[molec,#]) &/@ connections,
			(* else = bond *)
			result=(#->bondDistance[molec,#]) &/@ connections;
		]
	];
	Association[result]
]


(* ::Text:: *)
(*Rotations of Molecules Around {x, y, z}-axes*)
(*       z*)
(*       \[UpArrow]*)
(*       O \[RightArrow] x	   Mathematica's coordinate system*)
(*   \[LowerLeftArrow]*)
(* y*)


rotateMolecule[coord_Association,axis_?(ArrayQ[#,1,NumericQ]&),\[CurlyPhi]_?NumericQ]/;Length[axis]==3 :=
{#1,Sequence@@RotationTransform[\[CurlyPhi],axis][{##2}]}&@@@coord;


rotateMoleculeMat[coord_Association,mat_] := Module[{},
	Flatten@{#1,Sequence@@{mat . {##2}}}&@@@coord
];


randomMoleculeRotation[coord_Association] := Module[{a,mat,angle,coordOut},
	a=RandomReal[{0,1},3];
	angle={2*Pi*a[[1]],ArcCos[1-2*a[[2]]],2*Pi*a[[3]]};
	mat={
		{
			Cos[angle[[3]]]*Cos[angle[[2]]]*Cos[angle[[1]]]-Sin[angle[[3]]]*Sin[angle[[1]]], 
			Cos[angle[[3]]]*Cos[angle[[2]]]*Sin[angle[[1]]]+Sin[angle[[3]]]*Cos[angle[[1]]],
			-Cos[angle[[3]]]*Sin[angle[[2]]]
		} , {
			-Sin[angle[[3]]]*Cos[angle[[2]]]*Cos[angle[[1]]]-Cos[angle[[3]]]*Sin[angle[[1]]],
			-Sin[angle[[3]]]*Cos[angle[[2]]]*Sin[angle[[1]]]+Cos[angle[[3]]]*Cos[angle[[1]]],
			Sin[angle[[3]]]*Sin[angle[[2]]]
		} , {
			Sin[angle[[2]]]*Cos[angle[[1]]], Sin[angle[[2]]]*Sin[angle[[1]]], Cos[angle[[2]]]
		}
	};
	coordOut = rotateMoleculeMat[coord,mat];
	coordOut
];


mirrorMolecule[coord_Association,axis_?(ArrayQ[#,1,NumericQ]&)]/;Length[axis]==3 :=
{#1,Sequence@@ReflectionTransform[axis][{##2}]}&@@@coord;


(* ::Subsection::Closed:: *)
(*Euler angles and 3D orientation*)


(* ::Text:: *)
(*In order to specify the orientation of a coordinate system (e.g. after rotation) the Euler angles can be used. We are using the standard-x-convention, meaning that we will measure angles with respect to the old and new x-axes later on. *)
(**)
(*When we first have a coordinate system xyz and rotate it, we end up with the coordinate system XYZ. The three Euler angles are:*)
(**)
(*1. phi ... xy and XY planes have an intersection line N, which is perpendicular to z and Z, with phi=angles between x-axis and N*)
(*2. theta ... angle between z and Z*)
(*3. psi ... angle between N and X-axis*)
(**)
(*Line N: One point will always be in the origin (0,0,0), since both coordinate systems (xyz and XYZ) are in the origin. Directional vector of the line N is obtained via Cross[z,Z].*)


eulerAngles[moleculeRotationAngles_]:=Module[{standardCoords,newCoords,Nlinevec,phi,theta,psi},
	standardCoords={{1,0,0},{0,1,0},{0,0,1}}; (* coordinate matrix of standard system *)
	newCoords=RotationMatrix[#[[3]],{0,0,1}] . RotationMatrix[#[[2]],{0,1,0}] . RotationMatrix[#[[1]],{1,0,0}] . standardCoords&/@moleculeRotationAngles; (* coordinate matrix of rotated system *)
	Nlinevec=Normalize[Cross[{0,0,1},#[[All,3]]]]&/@newCoords;(* directional vector of intersection line between xy and XY planes *)

	(* Euler angles *)
	phi=VectorAngle[{1,0,0},#]&/@Nlinevec; (* angle between x axis and N *)
	theta=VectorAngle[{0,0,1},#[[All,3]]]&/@newCoords; (* angle between axes z and Z *)
	psi=VectorAngle[#1,#2[[All,1]]]&@@@Transpose[{Nlinevec,newCoords}]; (* angle between N and X axis *)

	Partition[Flatten@Transpose[{phi,theta,psi}],3] (* result list of {phi, theta, psi} for every molecule *)
];

(* same function as eulerAngles, but with an additional rotation allowed *)
eulerAnglesPlusRot[moleculeRotationAngles_,furtherRotAxis_,furtherRotAngle_]:=Module[
	{standardCoords,newCoords,Nlinevec,phi,theta,psi},
	standardCoords={{1,0,0},{0,1,0},{0,0,1}}; (* coordinate matrix of standard system *)
	newCoords=RotationMatrix[furtherRotAngle,furtherRotAxis] . RotationMatrix[#[[3]],{0,0,1}] . RotationMatrix[#[[2]],{0,1,0}] . RotationMatrix[#[[1]],{1,0,0}] . standardCoords&/@moleculeRotationAngles; (* coordinate matrix of rotated system *)
	Nlinevec=Normalize[Cross[{0,0,1},#[[All,3]]]]&/@newCoords;(* directional vector of intersection line between xy and XY planes *)

	(* Euler angles *)
	phi=VectorAngle[{1,0,0},#]&/@Nlinevec; (* angle between x axis and N *)
	theta=VectorAngle[{0,0,1},#[[All,3]]]&/@newCoords; (* angle between axes z and Z *)
	psi=VectorAngle[#1,#2[[All,1]]]&@@@Transpose[{Nlinevec,newCoords}]; (* angle between N and X axis *)

	Partition[Flatten@Transpose[{phi,theta,psi}],3] (* result list of {phi, theta, psi} for every molecule *)
];


(* ::Text:: *)
(*Giving three angles alone is not a precise-enough description of orientation for us, since it makes a difference where the interacting two molecules are located. In specific, we are interested in how the molecules are oriented WITH RESPECT TO THEIR "PAIR LINE" (i.e. the line between the two molecules).*)
(*In order for the Euler angles to be pair-orientation-independent, we define a reference state:*)
(*We want a pair to be alligned in z-direction, with the fixedMolecule "below" (=smaller z-value) and we want to achieve this only by rotation.*)
(**)
(*Rules:*)
(*If a molecule was shifted in x, the pair has to be rotated around the y-axis by -90\[Degree].*)
(*If a molecule was shifted in y, the pair has to be rotated around the x-axis by +90\[Degree].*)
(*If a molecule was shifted in z, the pair stays untouched.*)


(* uses the eulerAngles functions after bringing the pairs into the reference state of orientation*)
eulerAnglesInZ[restMoleculesOrder_,randomRotAngles_,fixedMolecule_]:=Module[{zEulerAngles},
	(* init *)
	zEulerAngles=Table[{},{Length@restMoleculesOrder}];
	(* rotation in z *)
	Do[
		If[restMoleculesOrder[[i,2]]=="x"
			,(* molecule was shifted in x direction *)
			zEulerAngles[[i]]={fixedMolecule,restMoleculesOrder[[i,1]]}->eulerAnglesPlusRot[randomRotAngles[[{fixedMolecule,restMoleculesOrder[[i,1]]}]],{0,1,0},-Pi/2]
			,(* else *)
			If[restMoleculesOrder[[i,2]]=="y"
				,(* molecule was shifted in y direction *)
				zEulerAngles[[i]]={fixedMolecule,restMoleculesOrder[[i,1]]}->eulerAnglesPlusRot[randomRotAngles[[{fixedMolecule,restMoleculesOrder[[i,1]]}]],{1,0,0},+Pi/2]
				,(* else it's already in z configuration *)
				zEulerAngles[[i]]={fixedMolecule,restMoleculesOrder[[i,1]]}->eulerAngles[randomRotAngles[[{fixedMolecule,restMoleculesOrder[[i,1]]}]]];
			];
		];
		,{i,1,Length@restMoleculesOrder}
	];
	zEulerAngles
];


(* ::Text:: *)
(*We can also calculate the new axes-unit-vectors of the rotated coordinate system. If you dont want a superimposed 4th rotation, just set the furtherRotAngle to 0 with an arbitrary furtherRotAxis (unequal to zero-vector).*)


newCoordSys[moleculeRotationAngles_,furtherRotAxis_,furtherRotAngle_]:=Module[{standardCoords,newCoords},
	standardCoords={{1,0,0},{0,1,0},{0,0,1}}; (* coordinate matrix of standard system *)
	newCoords=RotationMatrix[furtherRotAngle,furtherRotAxis] . RotationMatrix[#[[3]],{0,0,1}] . RotationMatrix[#[[2]],{0,1,0}] . RotationMatrix[#[[1]],{1,0,0}] . standardCoords&/@moleculeRotationAngles; (* coordinate matrix of rotated system *)
	{#[[All,1]],#[[All,2]],#[[All,3]]}&/@newCoords
];


(* ::Subsection::Closed:: *)
(*Preset cube rotations*)


(* ::Text:: *)
(*https://www.pentoma.de/symmetrien-von-figuren-aus-wuerfeln/ *)
(*(Mistake: 42 RotSp8, 14SpKante1, then 10 RotY90 not 270, see purple-colored angle below)*)
(**)
(*A standard 6-sided cube has 48 elements in its symmetry group.*)
(*1 identitiy cube*)
(*9 rotations around x,y,z (3 each)*)
(*6 rotations around the axes through centers of opposite edges (1 each)*)
(*8 rotations around the axes through opposite corners (2 each)*)
(*= 24 ROTATIONAL symmetries*)
(**)
(*The other 24 symmetries are obtained by mirroring.*)
(*1 point-mirroring image around the origin*)
(*3 mirroring operations parallel to the faces (x,y,z)*)
(*6 mirroring operations + rotation parallel to faces (x,y,z, 2 each)*)
(*8 mirroring operations + rotation with respect to space-diagonal plane (4 planes, 2 each)*)
(*6 mirroring operations with respect to face-diagonal planes (6 faces = 6 planes = permutate opposing diagonals)*)
(*= 24 MIRRORING symmetries*)
(**)
(*For fast evaluation we shall preset the rotational axes and anges for a cube which is CENTERED ON THE ORIGIN:*)


cubeRotationAngles={
	0,\[Pi]/2,(3 \[Pi])/2,\[Pi],\[Pi],(2 \[Pi])/3,(4 \[Pi])/3,(3 \[Pi])/2,(4 \[Pi])/3,(3 \[Pi])/2,\[Pi],(2 \[Pi])/3,(2 \[Pi])/3,\[Pi]/2,\[Pi],(4 \[Pi])/3,\[Pi]/2,(2 \[Pi])/3,(4 \[Pi])/3,\[Pi],\[Pi],\[Pi],\[Pi],\[Pi]
}; (* C Mayer order *)

(* old order *)
(*Flatten@Join[
{0},
Table[{Pi/2,Pi,3*Pi/2},{3}],
Table[{2Pi/3,4Pi/3},{4}],
Table[Pi,{6}]
]; (* first element = identity *)*)

cubeRotationAxes={
	{1,0,0},{0,0,1},{0,0,1},{0,0,1},{1/Sqrt[2],0,1/Sqrt[2]},{1/Sqrt[3],-(1/Sqrt[3]),1/Sqrt[3]},{1/Sqrt[3],1/Sqrt[3],1/Sqrt[3]},{0,1,0},{1/Sqrt[3],-(1/Sqrt[3]),1/Sqrt[3]},{1,0,0},{0,-(1/Sqrt[2]),1/Sqrt[2]},{-(1/Sqrt[3]),-(1/Sqrt[3]),1/Sqrt[3]},{1/Sqrt[3],1/Sqrt[3],1/Sqrt[3]},{1,0,0},
	{0,1/Sqrt[2],1/Sqrt[2]},{-(1/Sqrt[3]),1/Sqrt[3],1/Sqrt[3]},{0,1,0},{-(1/Sqrt[3]),1/Sqrt[3],1/Sqrt[3]},{-(1/Sqrt[3]),-(1/Sqrt[3]),1/Sqrt[3]},{-(1/Sqrt[2]),0,1/Sqrt[2]},{1,0,0},{-(1/Sqrt[2]),1/Sqrt[2],0},{1/Sqrt[2],1/Sqrt[2],0},{0,1,0}
}; (* C Mayer order *)

(* old order *)
(*Partition[Flatten[Join[
Table[{1,0,0},{4}], (* 4 instead of 3 because we will do a 0\[Degree] rotation around x for identity *)
Table[{0,1,0},{3}],
Table[{0,0,1},{3}],
Riffle[1/Sqrt[3]*{{1,1,1},{1,-1,1},{-1,1,1},{-1,-1,1}},1/Sqrt[3]*{{1,1,1},{1,-1,1},{-1,1,1},{-1,-1,1}}],
Join[Table[{Sin[Pi/4]*Cos[phi],Sin[Pi/4]*Sin[phi],Cos[Pi/4]},{phi,{0,Pi/2,Pi,3Pi/2}}],Table[{Cos[phi],Sin[phi],0},{phi,{Pi/4,3Pi/4}}]]
]]
,3];*)

(* only mirroring *)
cubeMirrOnlyVectors={
	{1,0,0},{0,1,0},{0,0,1},(* 3 faces *)
	{1,1,0},{1,0,1},{0,1,1},{-1,1,0},{-1,0,1},{0,-1,1}
}; (* 6 face-diagonal planes*)

(* mirroring, then rotation; first elements = inversion around origin = mirroring YZ and then RotX 180\[Degree]  *)
cubeMirrRotMirrVectors={{1,0,0},{1,0,0},{1,0,0},{1,-1,0}};
cubeMirrRotRotVectors={{1,0,0},{1,0,0},{1,0,0},{0,1,0}};
cubeMirrRotRotAngles={Pi,Pi/2,3Pi/2,Pi/2};

(* rotation, then mirroring*)
cubeRotMirrRotVectors={{0,0,1},{0,0,1},{0,1,0},{0,1,0},{0,0,1},{0,0,1},{0,0,1},{0,1,0},{0,1,0},{0,1,0},{0,1,0}};
cubeRotMirrRotAngles={Pi/2,3Pi/2,Pi/2,3Pi/2,Pi/2,3Pi/2,3Pi/2,Pi/2,Pi/2,3Pi/2,3Pi/2};
cubeRotMirrMirrVectors={{0,0,1},{0,0,1},{0,1,0},{0,1,0},{1,0,1},{1,0,1},{1,0,-1},{1,1,0},{1,-1,0},{1,-1,0},{1,1,0}};

(* Joining everything together, first 9 elements=mirroring, then 4 elements of mirroring-then-rotation,
then 11 elements of rotation-then-mirroring *)
cubeMirrVectors=Join[cubeMirrOnlyVectors,cubeMirrRotMirrVectors,cubeRotMirrMirrVectors];
cubeMirrRotVectors=Join[Table[{0,0,0},{9}],cubeMirrRotRotVectors,cubeRotMirrRotVectors];
cubeMirrRotAngles=Join[Table[0,{9}],cubeMirrRotRotAngles,cubeRotMirrRotAngles];


(* ::Subsection::Closed:: *)
(*Setting molecules to origin*)


(* ::Text:: *)
(*In order to not have translation when rotating, the molecules have to be centered on the origin. The geometric center is obtained via Sum(atoms_vectors)/Sum(atoms).*)


toOrigin[molecule_] := Module[{shiftOrigin,result},
	shiftOrigin = Total[molecule[[All,2;;4]]] / Length[molecule]; (*calculates geometric center*)
	result = molecule;
	Do[
		result[[i,2;;4]] -= shiftOrigin, (*corrects geometric center*)
		{i,1,Length[molecule]}
	];
	result
]


(* ::Subsection::Closed:: *)
(*Displacement of a molecule from a stationary molecule*)


(* ::Text:: *)
(*If the distance is set equal to the sum of the van der Waals radii of atoms k and l, then the sampling procedure is exactly the same as the method of Blanco. Here, we use a different formulation for the distance: since we use a force-field-based method to calculate the intermolecular pair energy, we relate the value of the distance to the Lennard\[Minus]Jones distance parameter, \[Sigma], multiplied with a dimensionless scaling factor s: distance = s * (Subscript[\[Sigma], kk] + Subscript[\[Sigma], ll]) (Sweere 2015).*)
(**)
(*The surface of a molecule is defined by the outer surface of spheres around the atomic nuclei in the molecule. The radius of these spheres is given by  Subscript[R, k] = s^seg Subscript[\[Sigma], kk] in which  Subscript[\[Sigma], kk] is the Lennard-Jones distance parameter of atom k, and s^seg is an atom-type independent multiplication factor with an optimized value of 0.62 (Sweere 2016).*)


(* Displacement of an atom vecB by dist from a stationary atom vecA in axis-direction *)
(* The function returns the required shift of coordinates in axis-direction *)
(* Complex roots indicate that no configuration with the desired distance exists! *)
(* For a discussion of this function, refer to:
 1. Blanco 1990 Molecular Silverware I. General Solutions to Excluded Volume Constrained Problems. J.Comp.Chem. 1990, 12(2), 237-247.
 2. Sweere 2015 Force-Field Based Quasi-Chemical Method for Rapid Evaluation of
 Binary Phase Diagrams. J.Phys.Chem. B 2015, 119, 14200-14209. *)

displacementAlongAxis[vecA_,vecB_,dist_,axis_]:=Module[
	{x=UnitVector[3,Switch[axis,"x",1,"y",2,"z",3]]},
	Dot[vecA,x]-Dot[vecB,x]+Sqrt[(Dot[vecB,x]-Dot[vecA,x])^2-Norm[vecB-vecA]^2+dist^2]
];


(* function to combine molecules to formal clusters *)
uniteMolecules[associationList_]:=Module[{length,values,unitedAssoc},
	length=Length[associationList];
	values=Flatten[Values[associationList],1];
	unitedAssoc=AssociationThread[Range[Length[values]],values]
];


(* ::Text:: *)
(*Function axialShiftDistance, using touching vdW surfaces instead of PAC-MAC distance.*)


axialShiftDistanceVdW[molecule_,axis_,vdwList_]:=Module[{tmp},
	(* Table of all pairs of atoms *)
	(* {Subscript[x, 1], Subscript[y, 1], Subscript[z, 1]}, {Subscript[x, 2], Subscript[y, 2], Subscript[z, 2]}, PAC-MAC distance *)
	tmp=Flatten[Table[{
			molecule[[1,i,2;;4]],(* list of coordinates, atom 1 *)
			molecule[[2,j,2;;4]],(* list of coordinates, atom 2 *)
			vdwRadius[vdwList[[1]][i]]+vdwRadius[vdwList[[2]][j]]
		}(* in Angstr\[ODoubleDot]m *)
		,{i,Length[molecule[[1]]]},{j,Length[molecule[[2]]]}
	],1];
	(* Create the list of required shift distances in axis-direction for each pair of atoms,
		eliminate complex roots (indicating that no configuration with the desired distance
		exists) and return the maximum value as result *)
	Max[Cases[displacementAlongAxis[#[[1]],#[[2]],#[[3]],axis]&/@tmp,_Real]]
];


(* functions used for Fan et al. sampling method (1992) *)
fanDisplacement[vecA_,vecB_,dist_,axis_]:=Module[{},
	Dot[vecA,axis]-Dot[vecB,axis]+Sqrt[(Dot[vecB,axis]-Dot[vecA,axis])^2-Norm[vecB-vecA]^2+dist^2]
];


(* ::Subsection::Closed:: *)
(*Analysis of  theoretical molecule angles and axes*)


GetAtomAnglePairs[moleculeBondsList_] := Module[{atomsList,atomsPairs,atomsChains},

	(* construct a list of all possible bond chains *)
	atomsList = DeleteDuplicates@Flatten@moleculeBondsList;
	atomsPairs = Permutations[atomsList,{2}];
	atomsChains = FindShortestPath[TwoWayRule @@@ moleculeBondsList,#[[1]],#[[2]]] &/@ atomsPairs;

	(* only keep chains with 2 bonds (= 3 atoms) *)
	atomsChains = DeleteCases[atomsChains, _?(Length[#] != 3 &)];

	(* remove double counted pairs *)
	atomsChains = DeleteDuplicates[Sort[#] &/@ atomsChains];

	(* RETURN *)
	atomsChains
];


GetTorsionAxisPairs[moleculeBondsList_] := Module[{atomsList,atomsPairs,atomsChains,axisPairs},

	(* construct a list of all possible bond chains *)
	atomsList = DeleteDuplicates@Flatten@moleculeBondsList;
	atomsPairs = Permutations[atomsList,{2}];
	atomsChains = FindShortestPath[TwoWayRule @@@ moleculeBondsList,#[[1]],#[[2]]] &/@ atomsPairs;

	(* only keep chains with 3 bonds (= 4 atoms) *)
	atomsChains = DeleteCases[atomsChains, _?(Length[#] != 4 &)];

	(* remove double counted pairs *)
	axisPairs = DeleteDuplicates[Sort[#] &/@ atomsChains];

	(* RETURN *)
	axisPairs
];


(* ::Section::Closed:: *)
(*Monte Carlo Functions*)


(* ::Text:: *)
(*These are the functions used for the Monte Carlo algorithm:*)


(* ::Subsection::Closed:: *)
(*Constants and Variables*)


ensembleTypeList = <|
	1 -> "Gibbs Ensemble with constant total Volume",
	2 -> "Gibbs Ensemble with constant Pressure"
|>;


(* ::Subsection::Closed:: *)
(*Visualizations*)


GetAtomRadius[atomName_] := If[MemberQ[Keys[vdwRadius],atomName],
	vdwRadius[atomName],
	(* atom name not found \[Rule] return 1.5 Angstrom as radius *)
	1.5
]


visualizeClusterMC[coord_,type_, iBox_] := Module[{atomCoord,atomNames, ind, frMolecule, retLoc, noCfcM, lambdaM},
	atomCoord = coord;
	atomNames = type;

	ind = 0;

	Graphics3D[
		(* flatten down to get list of atoms *)
		Flatten[(
		(*increaes count*)
		ind = ind+1;
		
		(*check if this is a fractional molecule*)
		noCfcM = True;
		lambdaM = 1.;
		Do[
			
			If[Global`cfc[[iBox, c, 1]] == ind,
				
				(*TRUE \[Rule] fractional involved*)
				noCfcM = False;
				lambdaM = lambdaM *Global`cfc[[iBox, c, 2]];			
			];
			
			,{c, 1, Length@Global`cfc[[iBox]]}
		];
		
		((*loop over every atom of the molecule*)
			If[noCfcM
			
			, (*TRUE \[Rule] no fractional molecule*)
				retLoc = {
					GetAtomColor[atomNames[[ind,#]]],
					Sphere[atomCoord[[ind,#]],GetAtomRadius[atomNames[[ind,#]]]]};
					
			, (*FALSE \[Rule] fractional molecule*)
				retLoc = {
					GetAtomColor[atomNames[[ind,#]]],
					Specularity[Orange,3],
					Sphere[atomCoord[[ind,#]],GetAtomRadius[atomNames[[ind,#]]]*lambdaM]};
			];
			retLoc
		) &/@ Range[Length@atomCoord[[ind]]]
		) &/@ Range[Length@atomCoord]
		,1],
		Boxed->False
	]
];


visualizeBoxesMC[moleculeAtomsCoordVec_,moleculeAtomsNamesVec_,Vvec_,plotSize_] := Module[{
	lBox1,lBox2,
	plotMolecules1,plotMolecules2,
	xTranslation, translatedFullCoordVec2, atomsInList},
	
	(*ensure that the cfc state is configured*)
	If[!ValueQ@Global`cfc,
	
		Global`cfc = Table[{-1,1}, 2 (*boxes*), Length@Global`system (*compoents*)];
	];

	lBox1 = CubeRoot[Vvec[[1]]];
	lBox2 = CubeRoot[Vvec[[2]]];

	plotMolecules1 = Show[
		Graphics3D[
			{Opacity[0.05],Cuboid[{0,0,0},{lBox1,lBox1,lBox1}]},
			Boxed->False,
			Axes->True,
			AxesLabel->{"x [\[CapitalARing]]","y [\[CapitalARing]]","z [\[CapitalARing]]"},
			AxesEdge->{{-1,-1},{-1,1},{-1,-1}},
			ViewPoint->{0.8,-2,0.6}
		],
		(*visualizeClusterMC[{moleculeAtomsCoordVec[[1]][[1, All]]},{"active"}],*)
		visualizeClusterMC[moleculeAtomsCoordVec[[1]],moleculeAtomsNamesVec[[1]], 1]
	];
	
	xTranslation = lBox1 * 1.2;
	translatedFullCoordVec2 = moleculeAtomsCoordVec[[2]];
	translatedFullCoordVec2[[All,All,1]]= translatedFullCoordVec2[[All,All,1]] + xTranslation;

	plotMolecules2 = Show[
		Graphics3D[
			{Opacity[0.05],Cuboid[{xTranslation,0,0},{xTranslation + lBox2,lBox2,lBox2}]},
			Boxed->False,
			Axes->True,
			AxesLabel->{"x [\[CapitalARing]]","y [\[CapitalARing]]","z [\[CapitalARing]]"},
			AxesEdge->{{-1,-1},{-1,1},{-1,-1}}
		],
		visualizeClusterMC[translatedFullCoordVec2,moleculeAtomsNamesVec[[2]], 2]
	];

	atomsInList = DeleteDuplicates[Flatten@moleculeAtomsNamesVec];

	Grid[{{
		Show[
			plotMolecules1,
			plotMolecules2,
			Graphics3D[Text[
				Framed[Style["Box 1",White,Bold,16],
				Background->Gray,FrameStyle->Gray],
				{lBox1 / 2,lBox1 / 2,lBox1 + Max[lBox1,lBox2] * 0.2}]],
			Graphics3D[Text[
				Framed[Style["Box 2",White,Bold,16],
				Background->Gray,FrameStyle->Gray],
				{xTranslation + lBox2 / 2,lBox2 / 2,lBox2 + Max[lBox1,lBox2] * 0.2}]],
			ImageSize->plotSize
		],
		PointLegend[
			GetAtomColor[#] &/@ atomsInList,
			atomsInList,
			LegendMarkers->{Graphics3D[Sphere[{0,0,0}],Boxed->False],50},
			LegendLayout->"Column"
		]
	}}]
];


(* ::Subsection::Closed:: *)
(*Base Functions*)


(* simple function to convert seconds to the form hours minutes seconds *)
SecondsToDurationString[s_] := Module[{hOut,mOut,sOut},
	hOut = Floor[s / 60 / 60];
	mOut = Floor[(s - hOut * 60 * 60) / 60];
	sOut = Floor[s - hOut * 60 * 60 - mOut * 60];
	ToString[hOut] <> "h " <> ToString[mOut] <> "min " <> ToString[sOut] <> "s"
];


GetMoleculeCenter[atomCoords_] := Total[atomCoords] / Length[atomCoords];


GetNonBondedIntraPairs[moleculeBonds_] := Module[{atomList,atomPairs,atomChains,nonBondedIntraPairs,nonBondedIntraPairsParams},
	
	(* construct a list of all possible bond chains *)
	atomList = DeleteDuplicates@Flatten@moleculeBonds;
	atomPairs = Permutations[atomList,{2}];
	atomChains = FindShortestPath[TwoWayRule @@@ moleculeBonds,#[[1]],#[[2]]] &/@ atomPairs;
	
	(* only keep chains with min 3 bonds (= min 4 atoms) *)
	atomChains = DeleteCases[atomChains, _?(Length[#] < 4 &)];
	
	(* only keep starting and ending atoms of the chains *)
	nonBondedIntraPairs = atomChains[[All,{1,-1}]];
	
	(* remove double counted pairs *)
	nonBondedIntraPairs = DeleteDuplicates[Sort[#] &/@ nonBondedIntraPairs];
	
	(* Calculate shortest distance and assign scaling parameters accoringly*)
	nonBondedIntraPairsParams = (
		If[Length@FindShortestPath[TwoWayRule @@@ moleculeBonds,#[[1]],#[[2]]] == 4
			,(*True: scaled 1-4 interaction*)
			{0.5,0.5,0.5}
			,(*Else: regular non-bonded interaction*)
			{1.,1.,1.}
		] &/@ nonBondedIntraPairs
	);
	
	(* RETURN *)
	AssociationThread[nonBondedIntraPairs,nonBondedIntraPairsParams]	
];


GetConnectedAtoms = Function[{moleculeBondsList,flexibleBondPair},
	Block[{atomsList,graph,connectedList,conList},
	
		atomsList = DeleteDuplicates@Flatten@moleculeBondsList;
		
		Switch[Length@flexibleBondPair
			,2,(* Bond *)
				graph = Graph[atomsList,TwoWayRule @@@ DeleteCases[moleculeBondsList,Sort@flexibleBondPair]];		
				conList = Flatten@Select[ConnectedComponents[graph],MemberQ[#,flexibleBondPair[[2]]]&];
			,3,(* Angle *)
				graph = Graph[atomsList,TwoWayRule @@@ DeleteCases[moleculeBondsList,Sort@flexibleBondPair[[{2,3}]]]];		
				conList = Flatten@Select[ConnectedComponents[graph],MemberQ[#,flexibleBondPair[[3]]]&];
			,4,(*Torsion*)
				graph = Graph[atomsList,TwoWayRule @@@ DeleteCases[moleculeBondsList,Sort@flexibleBondPair[[{2,3}]]]];		
				conList = Flatten@Select[ConnectedComponents[graph],MemberQ[#,flexibleBondPair[[4]]]&];
		];
		
		(*check if molecule part contains the beginning of the connection*)
		If[MemberQ[conList,flexibleBondPair[[1]]]
			,(*True: beginning of flexible intramolecular connection is part of the displaced molecule part *)
			Echo@"Warning: A labled flexible intramolecular connection is part of a cyclic connection.";
			Echo@"Cyclic connections are not supported in the current version.";
		];
		
		(*Return*)
		conList
	]
];


compDir = Compile[{{theta1,_Real},{z1,_Real}},
	{
		Sqrt[1 - z1^2] * Cos[theta1],
		Sqrt[1 - z1^2] * Sin[theta1],
		z1
	}
];


randomDirection = Compile[{},
	Module[{theta,z},
		theta = RandomReal[{0,2Pi}];
		z = RandomReal[{-1,1}];
		compDir[theta,z]
	]
];


randomMoleculeRotationMC = Compile[{{coord,_Real,2}},
	Module[{a,mat,angle},
		a = RandomReal[{0,1},3];
		angle = {2Pi a[[1]],ArcCos[1 - 2 a[[2]]],2Pi a[[3]]};
		mat = {{
				Cos[angle[[3]]] * Cos[angle[[2]]] * Cos[angle[[1]]] - Sin[angle[[3]]] * Sin[angle[[1]]],
				Cos[angle[[3]]] * Cos[angle[[2]]] * Sin[angle[[1]]] + Sin[angle[[3]]] * Cos[angle[[1]]],
				-Cos[angle[[3]]] * Sin[angle[[2]]]
			} , {
				-Sin[angle[[3]]] * Cos[angle[[2]]] * Cos[angle[[1]]] - Cos[angle[[3]]] * Sin[angle[[1]]],
				-Sin[angle[[3]]] * Cos[angle[[2]]] * Sin[angle[[1]]] + Cos[angle[[3]]] * Cos[angle[[1]]],
				Sin[angle[[3]]] * Sin[angle[[2]]]
			} , {
				Sin[angle[[2]]] * Cos[angle[[1]]], Sin[angle[[2]]] * Sin[angle[[1]]], Cos[angle[[2]]]
			}
		};
		mat . # &/@ coord
	]
];


randomOrientationChange = Compile[
	{
		{coord,_Real,2},
		{angle,_Real}
	},
	Module[{mat,dir},
		dir = randomDirection[];
		mat = {
			{dir[[1]]*dir[[1]] * (1 - Cos[angle]) +        Cos[angle],  dir[[1]]*dir[[2]] * (1 - Cos[angle]) - dir[[3]] * Sin[angle],  dir[[1]]*dir[[3]] * (1 - Cos[angle]) + dir[[2]] * Sin[angle]},
			{dir[[2]]*dir[[1]] * (1 - Cos[angle]) + dir[[3]]*Sin[angle],  dir[[2]]*dir[[2]] * (1 - Cos[angle]) +          Cos[angle],  dir[[2]]*dir[[3]] * (1 - Cos[angle]) - dir[[1]] * Sin[angle]},
			{dir[[3]]*dir[[1]] * (1 - Cos[angle]) - dir[[2]]*Sin[angle],  dir[[3]]*dir[[2]] * (1 - Cos[angle]) + dir[[1]] * Sin[angle],  dir[[3]]*dir[[3]] * (1 - Cos[angle]) +          Cos[angle]}
		};
		mat . # &/@ coord
	]
];


(* ::Subsection::Closed:: *)
(*Create Box Coordinates*)


(* create box coordinates with a FCC grid system *)
CreateBoxCoordinates[n_,boxLength_] := Module[{cellsPerDim,cellLength,unitCellCoord,cellCoord,availableCoordinates},
	
	(* FCC (face centered cubic) unit cell *)
	unitCellCoord = {{0.25,0.25,0.25},{0.25,0.75,0.75},{0.75,0.75,0.25},{0.75,0.25,0.75}};
	
	(* needed number of scaled unit cells per dimension to ensure enough positions for n molecules *)
	cellsPerDim = Ceiling[CubeRoot[n / Length[unitCellCoord]]];
	
	(* scaled side length of one unit cell *)
	cellLength = boxLength / cellsPerDim;
	
	(* scaled unit cell for insertion in the box *)
	cellCoord = cellLength * unitCellCoord;
	
	(* create available coordinates in the box according to the unit cell *)
	availableCoordinates = Flatten[Table[
		cellCoord[[i]] + {x * cellLength,y * cellLength,z * cellLength},
		{x,0,cellsPerDim - 1},
		{y,0,cellsPerDim - 1},
		{z,0,cellsPerDim - 1},
		{i,1,Length[unitCellCoord]}
	],3];
	
	(* return n coordinates of the available ones *)
	availableCoordinates[[RandomSample[Range[Length[availableCoordinates]],n]]]
	
];


(* ::Subsection::Closed:: *)
(*Simulation Overview*)


ShowMonteCarloOverviewEnvironment[system_, ensembleType_, T_, Psys_, Lvec_, Vvec_, nvec_, nCompVec_ ] := Grid[
	{
		{Style["ENVIRONMENT",Bold,FontSize->16],SpanFromLeft,SpanFromLeft,SpanFromLeft},
		{"Ensemble Type",ensembleTypeList[ensembleType],SpanFromLeft,SpanFromLeft},
		{"Components",StringJoin[ToString /@ Riffle[system,", "]],SpanFromLeft,SpanFromLeft},
		{"T [K]",ToString[N[T]],SpanFromLeft,SpanFromLeft},
		{"P [bar]",ToString[N[Psys / 10^5],TraditionalForm],SpanFromLeft,SpanFromLeft},
		Style[#,Bold] &/@ {"","BOX 1","BOX 2","TOTAL"},
		{"L [\[CapitalARing]]",ToString[Lvec[[1]]],ToString[Lvec[[2]]]},
		{"V [\!\(\*SuperscriptBox[\(\[CapitalARing]\), \(3\)]\)]",ToString[Vvec[[1]]],ToString[Vvec[[2]]],ToString[Total[Vvec]]},
		Join[{"\[Rho] [Molecules/\!\(\*SuperscriptBox[\(\[CapitalARing]\), \(3\)]\)]"},ToString[ScientificForm[N[#]],TraditionalForm] &/@ {nvec[[1]]/Vvec[[1]],nvec[[2]]/Vvec[[2]],Total[nvec]/Total[Vvec]}],
		Join[{"v [\!\(\*SuperscriptBox[\(dm\), \(3\)]\)/mol]"},ToString[N[#],TraditionalForm] &/@ ({Vvec[[1]]/nvec[[1]],Vvec[[2]]/nvec[[2]],Total[Vvec]/Total[nvec]}* nAvogadro * 10^3 / 10^30)],
		{"Number of Molecules",nvec[[1]],nvec[[2]],Total[nvec]},
		{
			Column[system],
			Column[nCompVec[[1]]],
			Column[nCompVec[[2]]],
			Column[Total[{nCompVec[[1,#]],nCompVec[[2,#]]}] &/@ Range[Length@system]]
		},
		{"Mole Fractions"},
		{
			Column[system],
			N@Column[(nCompVec[[1,#]]/nvec[[1]])&/@Range[Length@system]],
			N@Column[(nCompVec[[2,#]]/nvec[[2]])&/@Range[Length@system]],
			N@Column[(Total[{nCompVec[[1,#]],nCompVec[[2,#]]}]&/@Range[Length@system])/Total[nvec]]
		}
	},
	Alignment->{{Left,Center,Center,Center}},
	Dividers->{{True,True,True,True,True},{True,True,True,True,False,True,True,False,False,False,True,False,True,False,True}},
	Spacings->{3,1.2}
];


ShowMonteCarloOverviewDetails[
	system_, nWarmUpCycles_, nEquiCycles_, nProdCycles_, nMovesPerCycles_,
	rCutVec_, rOvlVec_, transMaxVec_, rotaMaxVec_, volChangeMaxVec_
 ] := Grid[
	{
		{Style["SIMULATION DETAILS",Bold,FontSize->16],SpanFromLeft,SpanFromLeft},
		{"warum-up cycles",ToString[nWarmUpCycles],SpanFromLeft},
		{"equilibration cycles",ToString[nEquiCycles],SpanFromLeft},
		{"production cycles",ToString[nProdCycles],SpanFromLeft},
		{Style["TRIAL MOVES PER CYCLE",Bold],SpanFromLeft},
		{"translations",nMovesPerCycles[[1]],SpanFromLeft},
		{"rotations",nMovesPerCycles[[2]],SpanFromLeft},
		{"volume changes",nMovesPerCycles[[3]],SpanFromLeft},
		{"insertions",nMovesPerCycles[[4]],SpanFromLeft},
		{"ghost insertions",nMovesPerCycles[[5]],SpanFromLeft},
		{"bond stretches",nMovesPerCycles[[6]],SpanFromLeft},
		{"angle bends",nMovesPerCycles[[7]],SpanFromLeft},
		{"torsion rotations",nMovesPerCycles[[8]],SpanFromLeft},
		{"total number of moves per cycles",ToString[Total[nMovesPerCycles]],SpanFromLeft},
		Style[#,Bold] &/@ {"","BOX 1","BOX 2"},
		{"cutoff distance [\[CapitalARing]]",ToString[rCutVec[[1]]],ToString[rCutVec[[2]]]},
		{"overlap distance [\[CapitalARing]]",ToString[rOvlVec[[1]]],ToString[rOvlVec[[2]]]},
		{"max translation distance [\[CapitalARing]]"},
		{
			Column[system],
			N@Column[(transMaxVec[[1,#]])&/@Range[Length@system]],
			N@Column[(transMaxVec[[2,#]])&/@Range[Length@system]]
		},		
		{"max rotation angle [rad]"},
		{
			Column[system],
			N@Column[(rotaMaxVec[[1,#]])&/@Range[Length@system]],
			N@Column[(rotaMaxVec[[2,#]])&/@Range[Length@system]]
		},
		{"max volume change [\!\(\*SuperscriptBox[\(\[CapitalARing]\), \(3\)]\)]",ToString[volChangeMaxVec[[1]]],ToString[volChangeMaxVec[[2]]]}
	},
	Alignment->{{Left,Center,Center,Center}},
	Dividers->{
		{True,True,True,True},
		{
			True,
			True,False,False,
			True,True,False,False,False,False,False,False,False,False,
			True, (*box 1, box 2 - box specific simulation details*)
			True, (*cutoff*)
			False, (*overlap*)
			True, False,(*max translations*)
			True, False, (*max rotations*)
			True, (*max volume change*)
			True (*bottom line*)
		}},
	Spacings->{3,1.2}
];


ShowMonteCarloOverviewOPLS[system_, ffLabelsNonBonded_, ffLabelsBondStretching_, ffLabelsAngleBending_, ffLabelsTorsion_] := Grid[
	Join[
		{
			{Style["OPLS-AA DEFINITIONS",Bold,FontSize->16],SpanFromLeft}
		},
		Transpose[Join@@{
			{Style[#,Bold] &/@ {"Component","non-bonded","bond stretching","angle bending","torsion"}},
			Table[
				{system[[i]],Column@ffLabelsNonBonded[[i]],Column@Normal@ffLabelsBondStretching[[i]],Column@Normal@ffLabelsAngleBending[[i]],Column@Normal@ffLabelsTorsion[[i]]},
				{i,1,Length@system}
			]
		}]
	],
	Alignment->{{Left,{Center}}},
	Dividers->{{{True}},{{True}}},
	Spacings->{3,1.2}
];


(* this function uses the variables from the current notebook *)
ShowMonteCarloOverview[
	system_, ensembleTypeString_, T_, Psys_, Lvec_, Vvec_, nvec_, nCompVec_,
	nWarmUpCycles_, nEquiCycles_, nProdCycles_, nMovesPerCycles_, rCutVec_, rOvlVec_, transMaxVec_, rotaMaxVec_, volChangeMaxVec_,
	ffLabelsNonBonded_, ffLabelsBondStretching_, ffLabelsAngleBending_, ffLabelsTorsion_
 ] := Grid[
	{
		{ShowMonteCarloOverviewEnvironment[system, ensembleTypeString, T, Psys, Lvec, Vvec, nvec, nCompVec]},
		{ShowMonteCarloOverviewDetails[system, nWarmUpCycles, nEquiCycles, nProdCycles, nMovesPerCycles, rCutVec, rOvlVec, transMaxVec, rotaMaxVec, volChangeMaxVec]},
		{ShowMonteCarloOverviewOPLS[system, ffLabelsNonBonded, ffLabelsBondStretching, ffLabelsAngleBending, ffLabelsTorsion],SpanFromLeft}
	},
	Spacings->{3,3}
];


(* ::Subsection::Closed:: *)
(*Metropolis Function*)


metropolis = Compile[{{deltaU,_Real},{R,_Real},{T,_Real}},
	
	If[deltaU < 0,
		(* new energy state is lower \[Rule] accept *)
		True,
		(* new energy state is higher \[Rule] accept with probability of the Boltzmann distribution *)
		If[RandomReal[{0,1}] < Exp[- deltaU / (R * T)],
			True,
			False
		]
	]
];


(* ::Subsection::Closed:: *)
(*Intermolecular Energy and Virial Calculation*)


ffEnergy = Compile[
	{
		{rij,_Real},
		{qiqje2,_Real},
		{\[Sigma]ij,_Real},
		{\[Epsilon]ij,_Real}
	},
	{
		4 * \[Epsilon]ij * (\[Sigma]ij / rij)^12, (* repulsive part *)
		-4 * \[Epsilon]ij * (\[Sigma]ij / rij)^6, (* attrative part *)
		qiqje2 / rij (* electrostatic part *)
	}
];


ffEnergy12 = Compile[{{rij,_Real},{\[Sigma]ij,_Real},{\[Epsilon]ij,_Real}}, 4*\[Epsilon]ij*(\[Sigma]ij/rij)^12 ];
ffEnergy6 = Compile[{{rij,_Real},{\[Sigma]ij,_Real},{\[Epsilon]ij,_Real}}, -4*\[Epsilon]ij*(\[Sigma]ij/rij)^6 ];
ffEnergy1 = Compile[{{rij,_Real},{qiqje2,_Real}}, (qiqje2/rij) ];


ffVirial = Compile[
	{
		{rij,_Real},
		{qiqje2,_Real},
		{\[Sigma]ij,_Real},
		{\[Epsilon]ij,_Real}
	},
	{
		48 * \[Epsilon]ij * (\[Sigma]ij^12 / rij^13), (* repulsive part *)
		-24 * \[Epsilon]ij * (\[Sigma]ij^6 / rij^7), (* attrative part *)
		qiqje2 / rij^2 (* electrostatic part *)
	}
];


ffVirial12 = Compile[{{rij,_Real},{\[Sigma]ij,_Real},{\[Epsilon]ij,_Real}}, 48 * \[Epsilon]ij * (\[Sigma]ij^12 / rij^13) ];
ffVirial6 = Compile[{{rij,_Real},{\[Sigma]ij,_Real},{\[Epsilon]ij,_Real}}, -24 * \[Epsilon]ij * (\[Sigma]ij^6 / rij^7) ];
ffVirial1 = Compile[{{rij,_Real},{qiqje2,_Real}}, (qiqje2/rij^2) ];


(* ::Subsection::Closed:: *)
(*Intramolecular Energy Calculation*)


(* ::Subsubsection::Closed:: *)
(*NonBonded*)


ffEnergyNonBondedIntra = Compile[
	{
		{atomCoords,_Real,2},
		{atomIndicators,_Integer,1},
		{nonBondedPairs,_Integer,2},
		{ffParamsNonBonded,_Real,3},
		{nonBondedPairsParams,_Real,2}
	},
	Module[
		{
			k, l, rX, rY, rZ,
			rInter,params,fij12,fij6,fij1,
			energy
		},
		
		(*init energy variable*)
		energy = 0.;
		
		(* calculate non-bonded energy for every pair *)
		Do[
			k = Compile`GetElement[nonBondedPairs,i,1];
			l = Compile`GetElement[nonBondedPairs,i,2];
			
			rX = Compile`GetElement[atomCoords,k,1] - Compile`GetElement[atomCoords,l,1];
			rY = Compile`GetElement[atomCoords,k,2] - Compile`GetElement[atomCoords,l,2];
			rZ = Compile`GetElement[atomCoords,k,3] - Compile`GetElement[atomCoords,l,3];
			
			rInter = Sqrt[rX^2 + rY^2 + rZ^2];
			
			params = Compile`GetElement[ffParamsNonBonded,Compile`GetElement[atomIndicators,k],Compile`GetElement[atomIndicators,l]];
			
			(* get the scaling factor for the LJ and Coulomb interactions *)
			fij12 = Compile`GetElement[nonBondedPairsParams,i,1];
			fij6 = Compile`GetElement[nonBondedPairsParams,i,2];
			fij1 = Compile`GetElement[nonBondedPairsParams,i,3];
			
			energy += fij12 * ffEnergy12[rInter,Compile`GetElement[params,2],Compile`GetElement[params,3]];
			energy += fij6 * ffEnergy6[rInter,Compile`GetElement[params,2],Compile`GetElement[params,3]];
			energy += fij1 * ffEnergy1[rInter,Compile`GetElement[params,1]];
			
			,{i,Length@nonBondedPairs}
		];
		
		(* RETURN *)
		energy
	]
];


(* ::Subsubsection::Closed:: *)
(*Bond Stretching*)


ffEnergyBondStretching[atomCoords_,ffParamsBondStretching_] := ffEnergyBondStretchingC[atomCoords,ffParamsBondStretching[[All,1]],ffParamsBondStretching[[All,2]]];
ffEnergyBondStretching[atomCoords_,bondStretchingPairs_,ffParamsBondStretching_] := ffEnergyBondStretchingC[atomCoords,bondStretchingPairs,ffParamsBondStretching];
ffEnergyBondStretchingC = Compile[
	{
		{atomCoords,_Real,2},
		{bondStretchingPairs,_Integer,2},
		{ffParamsBondStretching,_Real,2}
	},
	Module[
		{
			k, l, rX, rY, rZ,
			rInter,rEq,kBond,
			energy
		},
		
		(*init energy variable*)
		energy = 0.;
		
		(* calculate non-bonded energy for every pair *)
		Do[
			k = bondStretchingPairs[[i,1]];
			l = bondStretchingPairs[[i,2]];
			
			rX = Compile`GetElement[atomCoords,k,1] - Compile`GetElement[atomCoords,l,1];
			rY = Compile`GetElement[atomCoords,k,2] - Compile`GetElement[atomCoords,l,2];
			rZ = Compile`GetElement[atomCoords,k,3] - Compile`GetElement[atomCoords,l,3];
			
			rInter = Sqrt[rX^2 + rY^2 + rZ^2];
			
			rEq = Compile`GetElement[ffParamsBondStretching,i,1];
			kBond = Compile`GetElement[ffParamsBondStretching,i,2];
			
			energy += kBond * (rInter - rEq)^2;
			
			,{i,Length@bondStretchingPairs}
		];
		
		(* RETURN *)
		energy
	]
];


(* ::Subsubsection::Closed:: *)
(*Angle Bending*)


ffEnergyAngleBending[atomCoords_,ffParamsAngleBending_] := ffEnergyAngleBendingC[atomCoords,ffParamsAngleBending[[All,1]],ffParamsAngleBending[[All,2]]];
ffEnergyAngleBending[atomCoords_,angleBendingPairs_,ffParamsAngleBending_] := ffEnergyAngleBendingC[atomCoords,angleBendingPairs,ffParamsAngleBending];
ffEnergyAngleBendingC = Compile[
	{
		{atomCoords,_Real,2},
		{angleBendingPairs,_Integer,2},
		{ffParamsAngleBending,_Real,2}
	},
	Module[
		{
			k, l, m, vec1, vec2,
			theta, thetaEq, kAngle,
			energy
		},
		
		(*init energy variable*)
		energy = 0.;
		
		(* calculate non-bonded energy for every pair *)
		Do[
			k = angleBendingPairs[[i,1]];
			l = angleBendingPairs[[i,2]];
			m = angleBendingPairs[[i,3]];
			
			vec1 = atomCoords[[m]]-atomCoords[[l]];
			vec2 = atomCoords[[k]]-atomCoords[[l]];
			
			theta = Re@ArcCos[vec1 . vec2/(Norm[vec1]Norm[vec2])]; (*rad*)
			
			thetaEq = Compile`GetElement[ffParamsAngleBending,i,1](*deg*) Degree; (*rad*)
			kAngle = Compile`GetElement[ffParamsAngleBending,i,2]; (*kcal/(rad^2)*)
			
			energy += kAngle * (theta - thetaEq)^2;
			
			,{i,Length@angleBendingPairs}
		];
		
		(* RETURN *)
		energy
	]
];


(* ::Subsubsection::Closed:: *)
(*Torsion*)


ffEnergyTorsion[atomCoords_,ffParamsTorsion_] := ffEnergyTorsionC[atomCoords,ffParamsTorsion[[All,1]],ffParamsTorsion[[All,2]]];
ffEnergyTorsion[atomCoords_,tosionPairs_,ffParamsTorsion_] := ffEnergyTorsionC[atomCoords,tosionPairs,ffParamsTorsion];
ffEnergyTorsionC = Compile[
	{
		{atomCoords,_Real,2},
		{tosionPairs,_Integer,2},
		{ffParamsTorsion,_Real,2}
	},
	Module[
		{
			k, l, m, n,
			vec1, vec2,
			phi,
			v1, v2, v3, f1, f2, f3,
			energy
		},
		
		(*init energy variable*)
		energy = 0.;
		
		(* calculate non-bonded energy for every pair *)
		Do[
			k = tosionPairs[[i,1]];
			l = tosionPairs[[i,2]];
			m = tosionPairs[[i,3]];
			n = tosionPairs[[i,4]];
			
			vec1 = Cross[atomCoords[[k]]-atomCoords[[l]],atomCoords[[m]]-atomCoords[[l]]];
			vec2 = Cross[atomCoords[[l]]-atomCoords[[m]],atomCoords[[n]]-atomCoords[[m]]];
			
			phi = Re@ArcCos[vec1 . vec2/(Norm[vec1]Norm[vec2])] (*rad*) / Degree; (*deg*)
			
			{v1,v2,v3,f1,f2,f3} = ffParamsTorsion[[i,1;;6]];
			
			(* calculate energy for that dihedral *)
			energy += (
				(v1 / 2) * (1 + Cos[(phi + f1) Degree])
				+ (v2 / 2) * (1 - Cos[(2 * phi + f2) Degree])
				+ (v3 / 2) * (1 + Cos[(3 * phi + f3) Degree])
			);
			
			,{i,Length@tosionPairs}
		];
		
		(* RETURN *)
		energy
	]
];


(* ::Subsubsection::Closed:: *)
(*Sum of all Intramolecular Energies*)


ffEnergyIntra[atomCoords_,atomIndicators_,nonBondedPairs_,paramsList_] := Block[
	{intraEnergyTotal},
	
	(*
	paramsList should be given like:
	{
		ffParamsNonBonded,
		moleculeNonBondedPairsParams,
		ffParamsBondStretching,
		ffParamsAngleBending,
		ffParamsTorsion
	}
	*)
	
	intraEnergyTotal = 0.;
	
	(* calculate all separate intramolecular energies *)
	intraEnergyTotal += ffEnergyNonBondedIntra[atomCoords,atomIndicators,nonBondedPairs,paramsList[[1]],paramsList[[2]]];
	intraEnergyTotal += ffEnergyBondStretching[atomCoords,paramsList[[3]]];
	intraEnergyTotal += ffEnergyAngleBending[atomCoords,paramsList[[4]]];
	intraEnergyTotal += ffEnergyTorsion[atomCoords,paramsList[[5]]];
	
	(* RETURN *)
	intraEnergyTotal
];


(* ::Subsection::Closed:: *)
(*Obsolete Functions (not used in the current version of the program)*)


(* OBSOLETE \[Rule] metropolis is now used *)
boltzmann = Compile[{{deltaU,_Real},{R,_Real},{T,_Real}},
	Exp[-deltaU / (R * T)]
];


(* OBSOLETE \[Rule] calculated in the program *)
volumeChangeProbability = Compile[{{voldV,_Real},{R,_Real},{T,_Real},{volenergyList1,_Real,2},{volenergyList2,_Real,2},{energyListvec,_Real,3},{nvec,_Real,1},{Vvec,_Real,1}},
	Min[{
		1,
		Exp[-1 / (R * T) * (
			Total[Flatten[volenergyList1]] - Total[Flatten[energyListvec[[1]]]] +
			Total[Flatten[volenergyList2]] - Total[Flatten[energyListvec[[2]]]] -
			nvec[[1]] * R * T * Log[(Vvec[[1]] + voldV) / Vvec[[1]]]-
			nvec[[2]] * R * T * Log[(Vvec[[2]] - voldV) / Vvec[[2]]]
		)]
	}]
];


(* OBSOLETE \[Rule] ffEnergy used instead *)
ff = Compile[{rij,qiqje2,\[Sigma]ij,\[Epsilon]ij},
	{
		4 * \[Epsilon]ij * (\[Sigma]ij / rij)^12,
		-4 * \[Epsilon]ij * (\[Sigma]ij / rij)^6,
		qiqje2 / rij
	}
];

(* OBSOLETE \[Rule] ffVirial used instead *)
forceVdW = Compile[{rij,\[Sigma]ij,\[Epsilon]ij},
	{
		48 * \[Epsilon]ij * (\[Sigma]ij^12 / rij^13),
		-24 * \[Epsilon]ij * (\[Sigma]ij^6 / rij^7)
	}
];

(* OBSOLETE \[Rule] ffVirial used instead *)
forceCoul = Compile[{rij,qiqje2},
	qiqje2 / rij^2
];


(* ::Chapter::Closed:: *)
(*End of Package*)


End[];


EndPackage[];


(* ::Chapter::Closed:: *)
(*Literature*)


(* ::Text:: *)
(*Bondi 1964 van der Waals Volumes and Radii*)
(**)
(*Cornell 1995 A Second Generation Force Field for the Simulation of Proteins, Nucleic Acids, and Organic Molecules*)
(**)
(*Jensen 1999 Introduction to Computational Chemistry*)
(**)
(*Jorgensen 1996 Development and Testing of the OPLS All-Atom Force Field on Conformational Energetics and Properties of Organic Liquids (incl. Supporting Information)*)
(**)
(*Kaminski 2001 Evaluation and Reparametrization of the OPLS-AA Force Field for Proteins via Comparison with Accurate Quantum Chemical Calculations on Peptides*)
(**)
(*Lucas 2007 Molecular models for fluids. Chapter 5.2.4 Nonspherical Interaction Models*)
(**)
(*Price 2001 Gas-phase and liquid-state properties of esters, nitriles, and nitro compounds with the OPLS-AA force field*)
(**)
(*Sweere 2015 Force-Field Based Quasi-Chemical Method for Rapid Evaluation of Binary Phase Diagrams*)
(**)
(*Sweere 2017 Accuracy Test of the OPLS-AA Force Field for Calculating Free Energies of Mixing and Comparison with PAC-MAC*)
(**)
(*Weiner 1984 A New Force Field for Molecular Mechanical Simulation of Nucleic Acids and Proteins*)
(**)
(*Weiner 1986 An All Atom Force Field for Simulations of Proteins and Nucleic Acids*)
