(* ::Package:: *)

(* ::Title:: *)
(*Packages*)


(* ::Subtitle:: *)
(*In this file functions for loading sub-packages as well as the general initialisation are defined. Thereby the project is structured in concise and thus easily maintained packages.*)


If[$InputFileName === "", SetDirectory[NotebookDirectory[]];]


BeginPackage["Packages`"];


(* ::Chapter::Closed:: *)
(*Usage Definitions*)


(* ::Section::Closed:: *)
(*Initialisation*)


InitGEMC::usage = "This function is used for the general initialisaion of the GEMC simulation
bundle. It loads and sets an installed compiler and loads all packages.";


InitGEMCcompiler::usage = "This function loads an installed compiler.";


InitGEMCconfiguration::usage = "This function sets the working directory to the directory
of the executing notebook, ensures the reproducability of the simulation by seeding
the random generator and unsets variables.";


InitGEMCclearVariables::usage = "This function unsets variables to prevent problems
if multiple executions/evaluations are executed.";


InitGEMCloadPackages::usage = "This function loads all packages needed for the
GEMC simulation.";


(* ::Chapter:: *)
(*Function Definitions*)


Begin["`Private`"];


(* ::Section:: *)
(*Initialisation*)


InitGEMC[] := Module[{},
	InitGEMCconfiguration[];
	InitGEMCloadPackages[];
]


(* ::Subsection::Closed:: *)
(*Init Compiler*)


InitGEMCcompiler[] := Module[{foundCompiler, gccWindowsPaths, pathList, gccPath},

	(* setup context for compilers *)
	Needs["CCompilerDriver`GenericCCompiler`"];
	Needs["CCompilerDriver`"];

	(*initialise the check*)
	foundCompiler = False;

	(* setup compiler based on operating system *)
	Switch[$OperatingSystem
	
		,"Windows",
		
			(*check if a C compiler was automatically detected \[Rule] if not: look for MinGW*)
			If[AllTrue["CompilerInstallation"/.CCompilerDriver`CCompilers[Full],#===None&]===True
			
				,(*TRUE -> get %PATH% as a list*)
				
					pathList=StringSplit[#,"\\"]&/@StringSplit[Environment["Path"],";"];
					
					(*check if gcc can be found in pathList*)
					If[AnyTrue[FileNameJoin[Flatten[{#,"x86_64-w64-mingw32-gcc.exe"}]] &/@ pathList,FileExistsQ[#]===True&]===True
						,(*TRUE -> gcc was found*)
					
						(*set gcc Path*)
						gccPath=FileNameJoin[Append[
							Select[pathList,FileExistsQ[FileNameJoin[Flatten[{#,"gcc.exe"}]]]==True&][[1]],
							"x86_64-w64-mingw32-gcc.exe"
						]];
						foundCompiler = True;
					];
			
				,(*FALSE \[Rule] Compiler could not be detected automatically: Check manual paths*)
				
					(*list of possible paths to be checked manually*)
					gccWindowsPaths = {
						"C:\\MinGW-64\\bin\\x86_64-w64-mingw32-gcc.exe",
						"C:\\MinGW\\bin\\x86_64-w64-mingw32-gcc.exe"
					};
					
					Do[ (*check each list element*)
						If[FileExistsQ[path]
							,(*TRUE \[Rule] found compiler*)
							gccPath == path;
							foundCompiler = True;
							Break[];
						];				
						,{path,gccWindowsPaths}
					];
				
			];
			
			If[foundCompiler
			
				,(*TURE \[Rule] Load generic CCompiler package*)
				Needs["CCompilerDriver`GenericCCompiler`"];
				
				(*set MinGW as Compiler*)
				CCompilerDriver`$CCompiler={
					"Compiler"->CCompilerDriver`GenericCCompiler`GenericCCompiler,
					"CompilerInstallation"->gccPath,
					"CompilerName"->"x86_64-w64-mingw32-gcc.exe",
					"SystemCompileOptions"->"-O2"
				};
			
				,(*FALSE \[Rule] Could not detect any compiler: print error*)
				Print["ERROR - No C compiler could be found!"]
			];
		
		,"Unix",
		
			(*check if a C compiler was automatically detected*)
			If[Length@CCompilerDriver`CCompilers[] != 0
				,(*True: a compiler was automatically found by Mathematica*)
				CCompilerDriver`$CCompiler = First@CCompilerDriver`CCompilers[];
				
				,(*Else: set GCC manually*)
				CCompilerDriver`$CCompiler = {
					"Name" -> "GCC",
					"Compiler" -> CCompilerDriver`GCCCompiler`GCCCompiler,
					"CompilerInstallation" -> "/usr/bin",
					"CompilerName" -> Automatic
				};
			];
	];
	
	(*Turn off compiler warnings*)
	Off[Compile::initvar];
	Off[General::munfl];
	
	(*interpolation outside the specified range*)
	Off[InterpolatingFunction::dmval];
	
	(*harmless error message, which ocurrs when using Plot-function if global CompilationOptions are changed*)
	Off[Plot::colvl,None::colvl];
	
	(*Turn off warning that parallel evaluation of compiled code can only be executed on the thread when accessing global variables.
	This is intentionally used in CalcInteractionsTwoMolecules to set the global ovlFlag.*)
	Off[CompiledFunction::pext];
];


(* ::Subsection::Closed:: *)
(*Init GEMC configurations*)


InitGEMCconfiguration := Module[{},

	(* set relative path to current notebook directory *)
	SetDirectory[NotebookDirectory[]];

	(* ensure reproducibility *)
	SeedRandom[1234];
	
	(**"Compile"-function configuration**)
	(*set the default "CompilationTarget"
		- "WVM" (default)
		- "C" (only works if C compiler is installed)
	*)
	$CompilationTarget="C";
	
	(*set the default "CompilationOptions" to allow inlining compiled functions and external definitions*)
	SetOptions[Compile,CompilationOptions->{
		"InlineCompiledFunctions" -> True,
		"InlineExternalDefinitions" -> True
	}];
	
	(*set the default compiler runtime option to "Speed" or "Quality",
	The option "Speed" is faster but can cause the program to crash if errors ocurr in compiled functions*)
	SetOptions[Compile,RuntimeOptions->"Speed"];
	(*Set default compiler parellelization option to False for 'Listable' functions.
	  Can be set to True to enable multicore parallel evaluation. (only recomended for components of great size)*)
	SetOptions[Compile,Parallelization->False];
	
	(*
	unset variables to prevent problems if multiple executions/evaluations are executed.
	Variables that are unset in the initialization are not unset explicitly.
	
	This section is particulary relevent for the cfc variables because the biasing functions have to be
	passed to the extended simulations. Thus they are unset in this section and again loaded if extended
	simulations are executed.
	*)
	InitGEMCclearVariables[];
]


InitGEMCclearVariables[] := Module[{},
	Clear[
		(*extensions of the analysis that might not be present in previous versions*)
		"gitSHA1", "timtingInit", "timtingEnd", "timingCPUList", "timingCPUDocu",
		"transMaxVecDoc", "volChangeMax", "volChangeMaxVec",
		"counterWidomAcceptDocu", "counterWidomRejectDocu",
		"counterParticleTransDocu", "counterTransRejeDuDocu", "counterTransRejeOvDocu", "counterTransRejeNpDocu",
		"counterRotaRejeDuDocu", "counterRotaRejeOvDocu", "counterRotaRejeNpDocu",
		"counterBondStretchAcceptDocu", "counterBondStretchRejeDuDocu", "counterBondStretchRejeOvDocu", "counterBondStretchRejeNpDocu",
		"counterAngleBendAcceptDocu", "counterAngleBendRejeDuDocu", "counterAngleBendRejeOvDocu", "counterAngleBendRejeNpDocu",
		"counterTorsionAcceptDocu", "counterTorsionRejeDuDocu", "counterTorsionRejeOvDocu", "counterTorsionRejeNpDocu",
		"testSpecificResults", "counterInsertRejeDuDocu", "counterInsertRejeOvDocu", "counterInsertRejeNpDocu",
		"counterPanAcceptDocu", "counterPanRejDuDocu", "counterPanRejeNpDocu",
		
		(*custom parameters*)
		"custParam", "custParamM", "ffParamCombiningRule",
		
		(*RNG state*)
		"randomGeneratorState",
		
		(*cfc state*)
		"cfc", "cfcActive", "cfcEta", "cfcLnV", "cfcMaxDeltaLambda", "cfcBoxBiasValue",
		"cfcStateDocu",
		
		(*trial move documentation*)
		"nMovesDocu", "speciesChoiceProbVec",
		
		(*species choice probability documentation*)
		"SPdoc",
		
		(*electrostatic long-range correction parameter plot data*)
		"eLRCkappaRangeplotdata"
	];
];


(* ::Subsection::Closed:: *)
(*Init Packages*)


(* ::Item:: *)
(*Property Package*)


(* ::ItemParagraph:: *)
(*This package contains important constants, frequently used physical properties and molecular data*)


(* ::Item:: *)
(*Force Field OPLS*)


(* ::ItemParagraph:: *)
(*This package contains the OPLS parameters, Van der Waals-Radii and molecular masses*)


(* ::Item:: *)
(*Molecular Sampling*)


(* ::ItemParagraph:: *)
(*This package contains all functions related to the molecular sampling package such as the metropolis algorithm, the principle setup of the simulation boxes or the handling of the structure of the molecules.*)


(* ::Item:: *)
(*Helper Functions*)


(* ::ItemParagraph:: *)
(*This package contains very basic helper functions related to the data import/export.*)


InitGEMCloadPackages := Module[{},

	Needs["PropertyPackage`", FileNameJoin[{"pkg","PropertyPackage.wl"}]];
	Needs["ForceFieldOPLS`", FileNameJoin[{"pkg","ForceFieldOPLS.wl"}]];
	Needs["ChemicalThermoPackage`", FileNameJoin[{"pkg","ChemicalThermoPackage.wl"}]];
	Needs["MolecularSampling`", FileNameJoin[{"pkg","MolecularSampling.wl"}]];
	Needs["HelperFunctions`", FileNameJoin[{"pkg","HelperFunctions.wl"}]];
]


(* ::Chapter::Closed:: *)
(*End of Package*)


End[];


EndPackage[];
