(* ::Package:: *)

(* ::Title:: *)
(*PropertyPackage*)


(* ::Subtitle:: *)
(*A repository of frequently used physical property data*)


(* ::Text:: *)
(*2023-11-03 Maintained by Thomas Wallek*)


BeginPackage["PropertyPackage`"];


(* ::Chapter::Closed:: *)
(*Usage Definitions*)


(* ::Section::Closed:: *)
(*Constants*)


hPlanck::usage="hPlanck:
 Planck's constant
 6.626 070 15\[CenterDot]\!\(\*SuperscriptBox[\(10\), \(-34\)]\) J \!\(\*SuperscriptBox[\(s\), \(-1\)]\)";


kCalToJ::usage="kCalToJ:
 Conversion from kCal to Joule";


kB::usage="kB:
 Boltzmann's constant
 1.380 649\[CenterDot]\!\(\*SuperscriptBox[\(10\), \(-23\)]\) J \!\(\*SuperscriptBox[\(K\), \(-1\)]\)";


nAvogadro::usage="nAvogadro:
 Avogadro's constant
 6.022 140 76\[CenterDot]\!\(\*SuperscriptBox[\(10\), \(23\)]\) J \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)";


rCalor::usage="rCalor:
 Molar gas constant
 1.987 204 259 cal \!\(\*SuperscriptBox[\(K\), \(-1\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)";


rJoule::usage="rJoule:
 Molar gas constant
 8.314 462 618 J \!\(\*SuperscriptBox[\(K\), \(-1\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)";


uAtomic::usage="uAtomic:
 Unified atomic mass unit
 1.660 539 066 60\[CenterDot]\!\(\*SuperscriptBox[\(10\), \(-27\)]\) kg";


(* ::Section::Closed:: *)
(*Dataset of Frequently Used Physical Properties*)


physicalPropertyData::usage="physicalPropertyData: Dataset of frequently used physical properties
 The following keys are provided:
 'nameIUPAC' [string]
 'casNumber' [string]
 'molarMass' [g \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]
 'tMelt' [K] melting point
 '\[CapitalDelta]hMelt' [J \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] melting point
 'tTriple' [K] triple point
 'pTriple' [bar] triple point
 'tBoil' [K] normal boiling point
 'vBoil' [\!\(\*SuperscriptBox[\(cm\), \(3\)]\) \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)] liquid at normal boiling point
 'tCrit' [K]
 'pCrit' [bar]
 'vCrit' [dm\.b3/mol]
 'acentricFactor' [1]
 'dipoleMoment' [Debye]
 'pna' [1] List of {total, paraffinic, naphthenic, aromatic} carbon atoms of the molecule
 'RON' [1] Research Octane Number
 '\[Rho]15' [g/cm\.b3] Standard liquid density at 60\[Degree]F (\[TildeTilde]15.5556\[Degree]C)
 '\[Rho]25' [g/cm\.b3] Density at 25\[Degree]C
 '\[CapitalDelta]hEvap20' [J/mol] at 20\[Degree]C
 'antoineParameters' [bar, K, log10] mostly from NIST
 'plxant' extended Antoine equation parameters used by ASPEN, a list of 7-9 values
 'LJParameters' {'\[Sigma]' [\[Angstrom]],'\[Epsilon]/kB' [K]}
 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' from DDB/ARTIST
 'rVdW' [1] relative Van der Waals volume, a UNIQUAC parameter
 'qVdW' [1] relative Van der Waals volume, a UNIQUAC parameter
";


(* ::Chapter:: *)
(*Function Definitions*)


Begin["`Private`"];


(* ::Section::Closed:: *)
(*Constants*)


(* ::Text:: *)
(*Planck's constant*)


Unprotect[hPlanck];
hPlanck=6.62607015*10^(-34);(* J*s *)
Protect[hPlanck];


(* ::Text:: *)
(*Conversion from kilocalories to Joule*)


Unprotect[kCalToJ];
kCalToJ=1000/0.239006;(* J/kCal *)
Protect[kCalToJ];


(* ::Text:: *)
(*Boltzmann's constant*)


Unprotect[kB];
kB=1.380649 10^(-23);(* J K^-1 *)
Protect[kB];


(* ::Text:: *)
(*Avogadro's constant*)


Unprotect[nAvogadro];
nAvogadro=6.02214076 10^23;(* mol^-1 *)
Protect[nAvogadro];


(* ::Text:: *)
(*Universal gas constant (caloric units)*)


Unprotect[rCalor];
rCalor=1.987204259;(* cal K^-1 mol^-1 *)
Protect[rCalor];


(* ::Text:: *)
(*Universal gas constant (SI-units)*)


Unprotect[rJoule];
rJoule=8.314462618;(* J K^-1 mol^-1 *)
Protect[rJoule];


(* ::Text:: *)
(*Unified atomic mass unit*)


Unprotect[uAtomic];
uAtomic = 1.66053906660 * 10^(-27);(* kg *)
Protect[uAtomic];


(* ::Section::Closed:: *)
(*Dataset of Frequently Used Physical Properties*)


(* ::Item:: *)
(*The physicalPropertyData database represents a table with named columns and named rows*)


(* ::Item:: *)
(*For adding new component names, try to use strings that are found by "ChemicalData[component]"*)


(* ::Subsection::Closed:: *)
(*Remarks*)


(* ::Subsubsection:: *)
(*Units*)


(* ::Item:: *)
(*General units: Temperatures [K], pressure [bar], molar liquid volume at the normal boiling point vBoil [cm\.b3/mol], vCrit [dm\.b3/mol], Antoine parameters [bar, K, log10] from NIST, M [g/mol], \[Rho]15 and \[Rho]25 [g/cm\.b3] at 15\[Degree]C and 25\[Degree]C, respectively, \[CapitalDelta]hEvap20 [J/mol] at 20\[Degree]C*)


(* ::Subsubsection:: *)
(*Densities*)


(* ::Text:: *)
(*\[Rho]15	Liquid density, mostly from KBC Petro Sim, at 60\[Degree]F (15.5556\[Degree]C).*)
(**)
(*If the component is not available in KBC, simulate a pure-component flash in Aspen plus where the feed stream is specified at {60\[Degree]C, 1 atm} and the flash at the normal boiling point of the component + some arbitrary vapor fraction . This yields \[Rho]15 (i.e., the mass density of the feed) together with vBoil (the molar volume of the liquid outlet stream).*)
(*Of course, the result depends on the model chosen; VTPR or Lee-Kesler-Pl\[ODoubleDot]cker seems to be adequate for hydrocarbons.*)


(* ::Item:: *)
(*Case 1: For any component that is a liquid at 60\[Degree]F and 1 atm, the data base contains the density of the component at 60\[Degree]F and 1 atm.*)


(* ::Item:: *)
(*Case 2: For any component that can be liquified at 60\[Degree]F and pressures greater than 1 atm, the database contains the density of the component at 60\[Degree]F and saturation pressure. *)


(* ::Item:: *)
(*Case 3: For any component that is non-condensable at 60\[Degree]F under any pressure, i.e., 60\[Degree]F is greater than the critical temperature of the component, the database contains GPA tabular values of the equivalent liquid density. These densities were experimentally determined by measuring the displacement of hydrocarbon liquids by dissolved non-condensable components*)


(* ::Subsubsection:: *)
(*Antoine parameters*)


(* ::Item:: *)
(*From Dortmund Data Bank (DDB) {log10, \[Degree]C, torr=mmHg} to NIST {log10, K, bar}*)
(*A \[RightArrow] A - Log10[750.062]	B \[RightArrow] B	C \[RightArrow] C - 273.15*)


(* ::Subsubsection:: *)
(*Others*)


(* ::Item:: *)
(*pna number*)
(*A list: Number of {total, paraffinic, naphthenic, aromatic} carbon atoms of a substance*)


(* ::Subsection::Closed:: *)
(*Dataset*)


physicalPropertyData=Dataset[<|

"Acetone"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"propan-2-one","casNumber"->"67-64-1","ddbNumber"->4,"molarMass"->58.08,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	"tMelt"-> 178.35,"\[CapitalDelta]hMelt"->5687.3,
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"-> 329.48,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->508.10,"pCrit"->47.0148,"vCrit"->0.209,"acentricFactor"->0.309,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->2.88,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	"pna"->{3,0,0,0},"\[Rho]25"->0.7844,
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[\[Degree]C]"->"-64-70", "A"->7.13270 - Log10[750.062], "B"->1219.97, "C"->230.653 - 273.15, "units"->"K,bar,log10", "reference"->"DDB 2021"|>,
		<|"range[\[Degree]C]"->"57-205", "A"->7.63130 - Log10[750.062], "B"->1566.69, "C"->273.419 - 273.15, "units"->"K,bar,log10", "reference"->"DDB 2021"|>,
		<|"range[K]"->"259.16-507.60","A"->4.42448,"B"->1312.253,"C"->-32.445,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->4.6,"\[Epsilon]/kB[K]"->560.2,"source"->"viscositydata","reference"->"ThePropertiesofGases&Liquids5thEdition2001.AppendixB"|>
	},
	(*Atom Coordinates:'atomCoordinates'*)
	"atomCoordinates"->{
		1 -> {"C", 1.022, 0.0328, -0.1072}, 2 -> {"C", 2.5263, 0.0075, -0.0821}, 3 -> {"O", 3.1447, -0.7482, 0.6666}, 4 -> {"C", 3.2482, 0.9454, -1.0113}, 5 -> {"H", 0.6296, -0.6922, 0.6112}, 6 -> {"H", 0.6676, -0.2328, -1.1061}, 7 -> {"H", 0.6676, 1.0293, 0.1677}, 
		8 -> {"H", 2.5343, 1.5335, -1.594}, 9 -> {"H", 3.8683, 0.3652, -1.6992}, 10 -> {"H", 3.8683, 1.6279, -0.4248}},
	(*Forcefield Topology:'oplsForceFieldNonBonded'*)
	"oplsForceFieldNonBonded"->{
		1->{-0.3045,3.5,0.066},2->{0.3112,3.55,0.07},3->{-0.402,2.96,0.21},4->{-0.3034,3.5,0.066},5->{0.1169,2.5,0.03},
		6->{0.1169,2.5,0.03},7->{0.1169,2.5,0.03},8->{0.116,2.5,0.03},9->{0.116,2.5,0.03},10->{0.116,2.5,0.03}},(* 2023-07-20 - LM - LigParGen *)
	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	"originalUNIFACAssignment"->{"CH3"->1,"CH3CO"->1},
	"modUNIFACAssignment"->{"CH3CO"->1, "CH3"->1},
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	"rVdW"->2.57350, "qVdW"->2.33600
	|>,

"Ammonia"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"ammonia","casNumber"->"7664-41-7","ddbNumber"->210,"molarMass"->17.0305,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	"tMelt"->195.41,"\[CapitalDelta]hMelt"->5656.3,
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"->239.59,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->405.60,"pCrit"->112.77500,"vCrit"->0.0725,"acentricFactor"->0.25,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->1.47,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[\[Degree]C]"->"-83-60", "A"->7.55466 - Log10[750.062], "B"->1002.71, "C"->247.885 - 273.15, "units"->"K,bar,log10", "reference"->"DDB 2021"|>,
		<|"range[\[Degree]C]"->"-33-132", "A"->7.66269 - Log10[750.062], "B"->1066.76, "C"->256.851 - 273.15, "units"->"K,bar,log10", "reference"->"DDB 2021"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->2.90,"\[Epsilon]/kB[K]"->558.3,"source"->"viscositydata","reference"->"ThePropertiesofGases&Liquids5thEdition2001.AppendixB"|>
	}
	(* Atom Coordinates:'atomCoordinates' *)	

	(* Forcefield Topology: 'oplsForceFieldNonBonded'*)

	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	
	|>,

"Argon"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"argon","casNumber"->"7440-37-1","ddbNumber"->1058,"molarMass"->39.948,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"->87.24,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->150.80,"pCrit"->48.73730,"vCrit"->0.07490,"acentricFactor"->-0.004,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->0.,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[K]"->"114.40-150.31","A"->4.46903,"B"->481.012,"C"->22.156,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"83.78-150.72","A"->3.29555,"B"->215.24,"C"->-22.233,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"129.33-147.40","A"->4.97171,"B"->658.982,"C"->49.819,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"90.94-101.48","A"->3.73479,"B"->302.683,"C"->-6.083,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->3.405,"\[Epsilon]/kB[K]"->119.800,"source"->"2ndvirialcoefficient","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>
	}
	(* Atom Coordinates:'atomCoordinates' *)	
	
	(* Forcefield Topology: 'oplsForceFieldNonBonded'*)
	
	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	
	|>,

"CarbonDioxide"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"carbon dioxide","casNumber"->"124-38-9","ddbNumber"->1050,"molarMass"->44.0095,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"->183.52,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->304.20,"pCrit"->73.7646,"vCrit"->0.094,"acentricFactor"->0.2252,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->0.,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[\[Degree]C]"->"-62-31", "A"->7.53220 - Log10[750.062], "B"->835.06, "C"->268.223 - 273.15, "units"->"K,bar,log10", "reference"->"DDB 2021"|>,
		<|"range[\[Degree]C]"->"-57-31", "A"->7.81010 - Log10[750.062], "B"->987.44, "C"->290.900 - 273.15, "units"->"K,bar,log10", "reference"->"DDB 2021"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->4.486,"\[Epsilon]/kB[K]"->189.0,"source"->"2ndvirialcoefficient","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->4.070,"\[Epsilon]/kB[K]"->205.0,"source"->"2ndvirialcoefficient","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->3.996,"\[Epsilon]/kB[K]"->190.0,"source"->"viscositydata","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->3.897,"\[Epsilon]/kB[K]"->213.0,"source"->"viscositydata","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->3.941,"\[Epsilon]/kB[K]"->195.2,"source"->"viscositydata","reference"->"ThePropertiesofGases&Liquids5thEdition2001.AppendixB"|>
	},
	(*Atom Coordinates:'atomCoordinates'*)
	"atomCoordinates"->{
		1->{"O",1.0222,0.0671,-0.0532},2->{"O",3.4162,0.0671,-0.0532},3->{"C",2.2192,0.0671,-0.0532}}
	(*Forcefield Topology:'oplsForceFieldNonBonded'*)
	
	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	
	|>,

"Ethane"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"ethane","casNumber"->"74-84-0","ddbNumber"->1054,"molarMass"->30.069,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"->184.46,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->305.40,"pCrit"->48.8387,"vCrit"->0.1480,"acentricFactor"->0.098,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->0.,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[K]"->"91.33-144.13","A"->4.50706,"B"->791.3,"C"->-6.422,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"135.74-199.91","A"->3.93835,"B"->659.739,"C"->-16.719,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->3.750,"\[Epsilon]/kB[K]"->98.00,"source"->"TraPPE","reference"->"ErringtonJ.Phys.Chem.B,1998,102(44)"|>
	},
	(*Atom Coordinates:'atomCoordinates'*)
	"atomCoordinates"->{
		1->{"C",-0.756,0.,0.},2->{"C",0.756,0.,0.},3->{"H",-1.1404,0.6586,0.7845},4->{"H",-1.1404,0.3501,-0.9626},5->{"H",-1.1405,-1.0087,0.1781},
		6->{"H",1.1404,-0.3501,0.9626},7->{"H",1.1405,1.0087,-0.1781},8->{"H",1.1404,-0.6586,-0.7845}},
	(*Forcefield Topology:'oplsForceFieldNonBonded'*)
	"oplsForceFieldNonBonded"->{
		1->{-0.2396,3.5,0.066},2->{-0.2396,3.5,0.066},3->{0.0799,2.5,0.03},4->{0.0799,2.5,0.03},5->{0.0799,2.5,0.03},
		6->{0.0799,2.5,0.03},7->{0.0799,2.5,0.03},8->{0.0799,2.5,0.03}}(* 2022-10-08 - LM - LigParGen *)
	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	
	|>,

"Ethanol"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"ethanol","casNumber"->"64-17-5","ddbNumber"->11,"molarMass"->46.0684,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	"tMelt"->158.65,"\[CapitalDelta]hMelt"->5017.5,
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"->351.46,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->516.20,"pCrit"->63.8348,"vCrit"->0.167,"acentricFactor"-> 0.635,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->1.69,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	"pna"->{2,0,0,0},"\[Rho]25"->0.785,
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[\[Degree]C]"->"-57-80", "A"->8.20417 - Log10[750.062], "B"->1642.89, "C"->230.300 - 273.15, "units"->"K,bar,log10", "reference"->"DDB 2021"|>,
		<|"range[\[Degree]C]"->"77-243", "A"->7.68117 - Log10[750.062], "B"->1332.04, "C"->199.200 - 273.15, "units"->"K,bar,log10", "reference"->"DDB 2021"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->4.455,"\[Epsilon]/kB[K]"->391.0,"source"->"viscositydata","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->4.530,"\[Epsilon]/kB[K]"->362.6,"source"->"viscositydata","reference"->"ThePropertiesofGases&Liquids5thEdition2001.AppendixB"|>
	},
	(*Atom Coordinates:'atomCoordinates'*)
	"atomCoordinates"->{
		1 -> {"O", -1.1712, 0.2997, 0.},2 -> {"C", -0.0463, -0.5665, 0.}, 3 -> {"C", 1.2175, 0.2668, 0.},4 -> {"H", -0.0958, -1.212, 0.8819},5 -> {"H", -0.0952, -1.1938, -0.8946},6 -> {"H", 2.105, -0.372, -0.0177},7 -> {"H", 1.2426, 0.9307, -0.8704}, 
        8 -> {"H", 1.2616, 0.9052, 0.8886},9 -> {"H", -1.1291, 0.8364, 0.8099}},
	(*Forcefield Topology:'oplsForceFieldNonBonded'*)
	"oplsForceFieldNonBonded"->{
		1 -> {-0.5897, 3.12, 0.17}, 2 -> {0.0104, 3.5, 0.066}, 3 -> {-0.29, 3.5, 0.066}, 4 -> {0.0946, 2.5, 0.03}, 5 -> {0.0946, 2.5, 0.03}, 6 -> {0.0913, 2.5, 0.03}, 7 -> {0.0913, 2.5, 0.03}, 8 -> {0.0913, 2.5, 0.03}, 
        9 -> {0.4061, 0., 0.}},
	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	"modUNIFACAssignment"->{"OH (P)"->1,"CH3"->1,"CH2"->1},
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	"rVdW"->2.10550,"qVdW"->1.97200
	|>,

"Krypton"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"krypton","casNumber"->"7439-90-9","ddbNumber"->1060,"molarMass"->83.798,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"->119.74,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->209.40,"pCrit"->55.195,"vCrit"->0.0912,"acentricFactor"->-0.002,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->0.,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[K]"->"126.68-208.","A"->4.2064,"B"->539.004,"C"->8.855,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->3.600,"\[Epsilon]/kB[K]"->171.,"source"->"2ndvirialcoefficient","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>
	}
	(* Atom Coordinates:'atomCoordinates' *)	

	(* Forcefield Topology: 'oplsForceFieldNonBonded'*)

	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	
	|>,

"Methane"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"methane","casNumber"->"74-82-8","ddbNumber"->1051,"molarMass"->16.0425,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"->111.79,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->190.60,"pCrit"->46.0015,"vCrit"->0.099,"acentricFactor"->0.008,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->0.,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[K]"->"90.99-189.99","A"->3.9895,"B"->443.028,"C"->-0.49,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"96.89-110.19","A"->2.00253,"B"->125.819,"C"->-48.823,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"93.04-107.84","A"->3.80235,"B"->403.106,"C"->-5.479,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"110.00-190.5","A"->4.22061,"B"->516.689,"C"->11.223,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->3.730,"\[Epsilon]/kB[K]"->148.0,"source"->"TraPPE","reference"->"ErringtonJ.Phys.Chem.B,1998,102(44)"|>
	},
	(*Atom Coordinates:'atomCoordinates'*)
	"atomCoordinates"->{
		1->{"C",0.,0.,0.},2->{"H",0.5541,0.7996,0.4965},3->{"H",0.6833,-0.8134,-0.2536},4->{"H",-0.7782,-0.3735,0.6692},5->{"H",-0.4593,0.3874,-0.9121}},
	(*Forcefield Topology:'oplsForceFieldNonBonded'*)
	"oplsForceFieldNonBonded"->{
		1->{-0.2994,3.5,0.066},2->{0.0748,2.5,0.03},3->{0.0748,2.5,0.03},4->{0.0748,2.5,0.03},5->{0.0748,2.5,0.03}}(* 2022-10-08 - LM - LigParGen *)
	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	
	|>,

"Methanol"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"methanol","casNumber"->"67-56-1","ddbNumber"->110,"molarMass"->32.0419,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"->337.89,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->512.60,"pCrit"->80.95870,"vCrit"->0.118,"acentricFactor"->0.559,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->1.7,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[K]"->"353.5-512.63","A"->5.15853,"B"->1569.613,"C"->-34.846,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"288.1-356.83","A"->5.20409,"B"->1581.341,"C"->-33.50,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"353.0-483.00","A"->5.31301,"B"->1676.569,"C"->-21.728,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->3.585,"\[Epsilon]/kB[K]"->507.0,"source"->"viscositydata","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->3.626,"\[Epsilon]/kB[K]"->481.8,"source"->"viscositydata","reference"->"ThePropertiesofGases&Liquids5thEdition2001.AppendixB"|>
	},
	(*Atom Coordinates:'atomCoordinates'*)
	"atomCoordinates"->{
		1->{"C",0.9922,-0.0894,0.0345},2->{"O",2.4079,-0.092,0.0449},3->{"H",0.6307,-0.8314,-0.6816},4->{"H",0.6307,0.9045,-0.2404},
		5->{"H",0.6291,-0.3432,1.0333},6->{"H",2.7009,0.1363,-0.8537}},(* 2022-10-08 - LM - OpenBabel *)
	(*Forcefield Topology:'oplsForceFieldNonBonded'*)
	"oplsForceFieldNonBonded"->{
		1->{-0.0491,3.5,0.066},2->{-0.5873,3.12,0.17},3->{0.0767,2.5,0.03},4->{0.0767,2.5,0.03},5->{0.0767,2.5,0.03},6->{0.4062,0.,0.}}(* 2022-10-08 - LM - LigParGen *)
	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	
	|>,

"Nitrogen"-><|
	(* Identifiers: 'nameIUPAC', 'casNumber', 'ddbNumber', 'molarMass' *)
	"nameIUPAC"->"nitrogen","casNumber"->"7727-37-9","ddbNumber"->1056,"molarMass"->28.014,
	(* Melting point: 'tMelt', '\[CapitalDelta]hMelt' *)
	
	(* Triple point: 'tTriple', 'pTriple' *)
	
	(* Normal boiling point: 'tBoil', 'vBoil' *)
	"tBoil"->77.37,
	(* Critical point: 'tCrit', 'pCrit', 'vCrit', 'acentricFactor' *)
	"tCrit"->126.20,"pCrit"->33.9439,"vCrit"->0.0895,"acentricFactor"->0.04,
	(* Polarity: 'dipoleMoment' *)
	"dipoleMoment"->0.,
	(* Fuel properties: 'pna', 'RON', '\[Rho]15', '\[Rho]20', '\[Rho]25', '\[CapitalDelta]hEvap20' *)
	
	(* Saturation pressure: 'antoineParameters', 'plxant' *)
	"antoineParameters"->{
		<|"range[K]"->"63.14-126.","A"->3.7362,"B"->264.651,"C"->-6.788,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>,
		<|"range[K]"->"63.14-78.0","A"->3.63792,"B"->257.877,"C"->-6.344,"units"->"K,bar,log10","reference"->"webbook.nist.gov"|>
	},
	(* Lennard-Jones fluid: 'LJParameters' *)
	"LJParameters"->{
		<|"\[Sigma][\[Angstrom]]"->3.698,"\[Epsilon]/kB[K]"->95.05,"source"->"2ndvirialcoefficient","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->3.710,"\[Epsilon]/kB[K]"->95.90,"source"->"2ndvirialcoefficient","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->3.681,"\[Epsilon]/kB[K]"->91.50,"source"->"viscositydata","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->3.749,"\[Epsilon]/kB[K]"->79.8,"source"->"viscositydata","reference"->"MolecularTheoryofGasesandLiquids1954.AppendixTableI-A"|>,
		<|"\[Sigma][\[Angstrom]]"->3.798,"\[Epsilon]/kB[K]"->71.4,"source"->"viscositydata","reference"->"ThePropertiesofGases&Liquids5thEdition2001.AppendixB"|>
	},
	(*Atom Coordinates:'atomCoordinates'*)
	"atomCoordinates"->{
		<|1->{"N",0.9063,0.0509,0.0858}|>,<|2->{"N",2.3263,0.0509,0.0858}|>}
	(*Forcefield Topology:'oplsForceFieldNonBonded'*)
	
	(* UNIFAC: 'originalUNIFACAssignment', 'modUNIFACAssignment', 'lleUNIFACAssignment' *)
	
	(* UNIQUAC: 'rVdW', 'qVdW' *)
	
	|>
|>];


(* ::Chapter::Closed:: *)
(*End of Package*)


End[];


EndPackage[];
