(* ::Package:: *)

(* ::Title:: *)
(*ForceFieldOPLS*)


(* ::Subtitle:: *)
(*Functions for the OPLS-AA package*)


If[$InputFileName === "", SetDirectory[NotebookDirectory[]];]


BeginPackage["ForceFieldOPLS`",{"PropertyPackage`"}];


(* ::Chapter::Closed:: *)
(*Usage Definitions*)


(* ::Section::Closed:: *)
(*Molecular Data*)


vdwRadius::usage="Van der Waals-radius of an atom in [\[Angstrom]]";


molMass::usage="Molar mass of Molecules in [g/mol]";


(* ::Section::Closed:: *)
(*The OPLS-AA Force Field*)


oplsNonBonded::usage="Lennard-Jones and Coulomb paramters for non-bonded atom pairs.
q[\!\(\*SuperscriptBox[\(e\), \(-\)]\)], \[Sigma][\[Angstrom]], \[Epsilon][kcal \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)]";
oplsBonded::usage="Lengths [\[Angstrom]], K [kcal \!\(\*SuperscriptBox[\(mol\), \(-1\)]\) \!\(\*SuperscriptBox[\(\[Angstrom]\), \(-2\)]\)] or [kcal \!\(\*SuperscriptBox[\(mol\), \(-1\)]\) \!\(\*SuperscriptBox[\(rad\), \(-2\)]\)]";
oplsBondStretching::usage="";
oplsAngleBending::usage="";
oplsTorsion::usage="V1, V2, V3 in [kcal \!\(\*SuperscriptBox[\(mol\), \(-1\)]\)], f1, f2, f2, \[Phi]";


GROMACSatoms::usage="GROMACS atom type tag \[Rule] atom, charge, sigma, epsilon, GROMACS comments on atoms";
vdwListGenerate::usage="Function to generate the VdW element list, needed for touching surface evaluation";


(* ::Chapter:: *)
(*Function Definitions*)


Begin["`Private`"];


(* ::Section:: *)
(*Molecular Data*)


(* ::Subsection::Closed:: *)
(*Van der Waals-Radii*)


(* ::Text:: *)
(*Van der Waals radii may be determined from the mechanical properties of gases (the original method), from the critical point, from measurements of atomic spacing between pairs of unbonded atoms in crystals or from measurements of electrical or optical properties (the polarizability and the molar refractivity). These various methods give values for the van der Waals radius which are similar (1-2 \[CapitalARing], 100-200 pm) but not identical. Tabulated values of van der Waals radii are obtained by taking a weighted mean of a number of different experimental values, and, for this reason, different tables will often have different values for the van der Waals radius of the same atom. Indeed, there is no reason to assume that the van der Waals radius is a fixed property of the atom in all circumstances: rather, it tends to vary with the particular chemical environment of the atom in any given case (see Bondi 1964).*)


(* Van der Waals-radii of the atoms [\[Angstrom]] *)
vdwRadius = <| (* r[\[Angstrom]] *)

"H"->1.2,
"C"->1.7,
"N"->1.55,
"O"->1.52,
"F"->1.47,
"P"->1.8,
"S"->1.8,
"Cl"->1.75,
"Cu"->1.4,
"Ar"->1.88,
"Kr"->2.02,

"LJ_ref"->0.5,
"LJ_1"->0.5,
"LJ_2"->0.5,
"LJ_3"->0.25,

"Methane"->1.91,
"Ethane"->1.98

|>;


(* ::Subsection::Closed:: *)
(*Molecular Masses*)


(* molecular masses of molecules *)
molMass = <| (* g/mol *)

(* Source: .mol-file *)
"Argon" -> 39.962383, (* Ar *)
"Krypton" -> 83.911498, (* Kr *)

(* Source: PubChem *)
"Nitrogen" -> 28.014, (* N2 *)
"Methane"->16.043, (*CH4*)
"Methanol" -> 32.042, (* CH4O *)
"Ethane" -> 30.07, (* C2H6 *)
"Ethanol" -> 46.07, (* C2H6O *)
"Acetone" -> 58.08, (* CH3COCH3 *)
"CarbonDioxide"->44.009, (*CO2*)
"Ammonia"->17.031, (*NH3*)

(*MH addition: theoretical LJ fluids*)
"LJ-ref" -> 1.0,
"LJ-1" -> 1.0,
"LJ-2" -> 1.0,
"LJ-3" -> 1.0

|>;


(* ::Section:: *)
(*The OPLS-AA Force Field*)


(* ::Subsection::Closed:: *)
(*Force Field Parameters*)


(* ::Text:: *)
(*The following atom types for bond and angle parameters are explained in Weiner 1984, Weiner 1986, Cornell 1995 (Table 1).*)
(*CT ... any sp\.b3 carbon (e.g., Subscript[CH, 4])*)
(*HC ... H attached to aliphatic carbon with no electron-withdrawing substituents*)


(* ::Subsubsection::Closed:: *)
(*Non-Bonded Parameters*)


(* ::Text:: *)
(*The following non-bonded parameters are taken from Jorgensen 1996, Supporting Information, and Price 2001*)
(*Additional parameters are taken from Jorgensen 1984 for the OPLS-UA force field and the TraPPE force field.*)


oplsNonBonded = <|
(* q[e^-], \[Sigma][\[Angstrom]], \[Epsilon][kcal mol^-1] *)

(* ---------------------------------------------------------------------------------- *)
(* hydrocarbons and alcohols *)
(* ---------------------------------------------------------------------------------- *)

"C,CH4"->{-0.240,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"C,RCH3"->{-0.180,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"C,R2CH2"->{-0.120,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"C,R3CH"->{-0.060,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"C,R4C"->{ 0.000,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"H,RH,alkanes"->{0.060,2.500,0.030},(* Jorgensen 1996, SupInf, Table 1 *)
"C,Benzene"->{-0.115,3.550,0.070},(* Jorgensen 1996, SupInf, Table 1 *)
"H,Benzene"->{ 0.115,2.420,0.030},(* Jorgensen 1996, SupInf, Table 1 *)
"C,CH3 of toluene"->{-0.065,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"C,CH2 of ethyl benzene"->{-0.005,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"C,R2C="->{0.000,3.550,0.076},(* Jorgensen 1996, SupInf, Table 1 *)
"C,RHC="->{-0.115,3.550,0.076},(* Jorgensen 1996, SupInf, Table 1 *)
"C,H2C="->{-0.230,3.550,0.076},(* Jorgensen 1996, SupInf, Table 1 *)
"H,HC="->{0.115,2.420,0.030},(* Jorgensen 1996, SupInf, Table 1 *)
"O,ROH"->{-0.683,3.120,0.170},(* Jorgensen 1996, SupInf, Table 1 *)
"H(O),ROH"->{0.418,0.000,0.000},(* Jorgensen 1996, SupInf, Table 1 *)
"H(C),CH3OH"->{0.040,2.500,0.030},(* Jorgensen 1996, SupInf, Table 1 *)
"C,CH3OH and RCH2OH"->{0.145,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"C,R2CHOH"->{0.205,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"C,R3COH"->{0.265,3.500,0.066},(* Jorgensen 1996, SupInf, Table 1 *)
"C,COH phenol"->{0.150,3.550,0.070},(* Jorgensen 1996, SupInf, Table 1 *)
"O,phenol"->{-0.585,3.070,0.170},(* Jorgensen 1996, SupInf, Table 1 *)
"H,phenol"->{0.435,0.000,0.000},(* Jorgensen 1996, SupInf, Table 1 *)

(* ---------------------------------------------------------------------------------- *)
(* ethers, acetals, aldehydes, ketones, and carboxylic acids *)
(* ---------------------------------------------------------------------------------- *)

"C,R2CO"->{0.470,3.750,0.105},(* Jorgensen 1996, SupInf, Table 5 *)
"O,R2CO"->{-0.470,2.960,0.210},(* Jorgensen 1996, SupInf, Table 5 *)

(* ---------------------------------------------------------------------------------- *)
(* esters, nitriles, and nitro compounds *)
(* ---------------------------------------------------------------------------------- *)

"\[Alpha]-C in acetonitrile"->{-0.08,3.30,0.066},(* Price 2001 *)
"sp-N in alkylnitrile"->{-0.56,3.20,0.170},(* Price 2001 *)

(* ---------------------------------------------------------------------------------- *)
(* Nitrogen N2 *)
(* ---------------------------------------------------------------------------------- *)

(* Extracted from http://molmod.boltzmann-zuse.de *)
"N,N2"->{0.00,3.2621,0.0758803},

(* Extracted from the program Brick-CFCMC *)
"N,N2,BRICK"->{0.00,3.310,36.0 * kB * nAvogadro / 4184},

(* ---------------------------------------------------------------------------------- *)
(* Amines *)
(* ---------------------------------------------------------------------------------- *)

"NT,NH3"->{-1.02,3.420,0.170}, (* Jorgensen 1999 *)
"H,NT,NH3"->{0.34,0.000,0.000}, (* Jorgensen 1999 *)

(* ---------------------------------------------------------------------------------- *)
(* noble gases *)
(* ---------------------------------------------------------------------------------- *)

"Ar,Leeuwen"->{0.00,3.390,117.50 * kB * nAvogadro / 4184},(* Leeuwen 1991 *)
"Kr,Leeuwen"->{0.00,3.607,161.00 * kB * nAvogadro / 4184},(* Leeuwen 1991 *)

"Ar,Kim"->{0.00,3.4062,120.38 * kB * nAvogadro / 4184},(* Kim 2014 *)
"Kr,Kim"->{0.00,3.6202,171.64 * kB * nAvogadro / 4184},(* Kim 2014 *)

"Ar,Steinhilper"->{0.00,3.336,138.00 * kB * nAvogadro / 4184},(* Steinhilper 1971 *)
"Kr,Steinhilper"->{0.00,3.621,188.00 * kB * nAvogadro / 4184},(* Steinhilper 1971 *)

"Ar,Tan"->{0.00,3.3952,116.79 * kB * nAvogadro / 4184},(* Tan 2018 *)
"Kr,Tan"->{0.00,3.6274,162.58 * kB * nAvogadro / 4184},(* Tan 2018 *)

(* ---------------------------------------------------------------------------------- *)
(* Lennard Jones Fluid *)
(* ---------------------------------------------------------------------------------- *)

(* Lennard Jones Fluid Test *)
"LJ"->{0.00,3.4875749910006193,0.8280017744336798},
"LJ-2"->{0.00,3.7740444620139364,0.5184011109497823},

(* Lennard Jones Fluid MH Addition *)
"LJ_ref" -> {0.0,1.0,1.00 * kB * nAvogadro / 4184},
"LJ_1"   -> {0.0,1.0,0.75 * kB * nAvogadro / 4184},
"LJ_2"   -> {0.0,1.0,0.50 * kB * nAvogadro / 4184},
"LJ_3"   -> {0.0,0.5,0.50 * kB * nAvogadro / 4184},

(* ---------------------------------------------------------------------------------- *)
(* hydrocarbons as Lennard Jones Fluids *)
(* ---------------------------------------------------------------------------------- *)

"Methane,2ndV"-> {0.0,3.817,148.2(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*2nd Virial Coefficient, Hirschfelder 1954*)
"Methane,2ndV,Fit"-> {0.0,3.786,149.05(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*2nd Virial Coefficient Fit 115-650K*)
"Ethane,2ndV" -> {0.0,3.954,243.0(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*2nd Virial Coefficient, Hirschfelder 1954*)
"Ethane,Thol" -> {0.0,4.239,231.4(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Fitted to Thol2016 EoS*)

(* ---------------------------------------------------------------------------------- *)
(* OPLS-UA force field parameters  *)
(* Jorgensen 1984 Optimized Intermolecular Potential Functions for Liquid *)
(* Jorgensen 1986 Optimized Intermolecular Potential Functions for Liquid Alcohols *)
(* Jorgensen 1990 Relative Partition Coefficients for Organic Solutes from Fluid Simulations *)
(* d_C4-C4 = 1.54 A ; d_C4-C3 = 1.50 A ; d_C3=C3 = 1.34 A ; d_C3-C3 = 1.40 A *)
(* theta_C-C4-C = 112.0 Deg ; theta_C-C3-C = 124.0 Deg *)
(* ---------------------------------------------------------------------------------- *)

(*methane*)
"CH4,OPLS-UA"-> {0.0,3.730,0.294(*kcal/mol*)}, (*Jorgensen 1984*)
(*ethane*)
"CH3,C1,OPLS-UA"-> {0.0,3.775,0.207(*kcal/mol*)}, (*Jorgensen 1984*)
(*alcohols*)
"O,COH,OPLS-UA"-> {-0.700,3.070,0.170(*kcal/mol*)}, (*Jorgensen 1986*)
"H,COH,OPLS-UA"-> { 0.435,0.000,0.000(*kcal/mol*)}, (*Jorgensen 1986*)
"CH2,COH,OPLS-UA"-> {0.265,3.905,0.118(*kcal/mol*)}, (*Jorgensen 1986*)
"CH3,COH,OPLS-UA"-> {0.265,3.775,0.207(*kcal/mol*)}, (*Jorgensen 1986*)

(*acetone*)
"CH3,Ace,OPLS-UA"-> {0.062,3.910,0.160(*kcal/mol*)}, (*Jorgensen 1990*)
"C,Ace,OPLS-UA"-> {0.300,3.750,0.105(*kcal/mol*)}, (*Jorgensen 1990*)
"O,Ace,OPLS-UA"-> {-0.424,2.960,0.210(*kcal/mol*)}, (*Jorgensen 1990*)

(* ---------------------------------------------------------------------------------- *)
(* TraPPE force field parameters *)
(* ---------------------------------------------------------------------------------- *)

(* carbon dioxide: l = 1.16 A, theta = 180\[Degree] *)
"C,CO2"-> {0.7,2.80,27(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Potoff 2001*)
"O,CO2"-> {-0.35,3.05,79(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Potoff 2001*)

(* ammonia: l_NH = 1.0124 A, l_NM = 0.08 A, theta_HNH = 106.68\[Degree], theta_HNM = 67.88\[Degree] *)
"N,NH3"-> { 0.000,3.420,185(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Zhang 2010*)
"H,NH3"-> { 0.410,0.000,0.0}, (*Zhang 2010*)
"M,NH3"-> {-1.230,0.000,0.0}, (*Zhang 2010*)

(* ---------------------------------------------------------------------------------- *)
(* TraPPE-UA force field parameters *)
(* d_C-C = 1.54 A ; theta_C-C-C = 114.0 Deg *)
(* ---------------------------------------------------------------------------------- *)

"CH4,TraPPE-UA" -> {0.0,3.73,148(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann 1998*)
"CH3,TraPPE-UA" -> {0.0,3.75, 98(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann 1998*)
"CH2,TraPPE-UA" -> {0.0,3.95, 46(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann 1998*)

"Ethane,Err" -> {0.0,3.75,98(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Errington, 1998*) (*duplicate*)

"CH3,CH3-OH,TraPPE-UA" -> {0.265,3.75, 98(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann 2001*)
"O,CHx-O-H,TraPPE-UA" -> {-0.700,3.02, 93(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann 2001*)
"H,O-H,TraPPE-UA" -> {0.435,0.00, 0(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann 2001*)

(*ketone*)
"C,C=O,TraPPE-UA" -> { 0.424,3.82,40(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann, 2004*)
"O,C=O,TraPPE-UA" -> {-0.424,3.05,79(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann, 2004*)

(* ---------------------------------------------------------------------------------- *)
(* TraPPE-EH force field parameters *)
(* The C-H site is placed at the center of any carbon-hydrogen bond *)
(* d_C-C = 1.535 A ; d_C-H = 1.100 A *)
(* theta_C-C-C = 112.7 Deg ; theta_C-C-H = 110.7 Deg ; theta_H-C-H = 107.8 Deg *)
(* ---------------------------------------------------------------------------------- *)

"M,C-H,TraPPE-EH" -> {0.0,3.31,15.3(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann 1999*)
"C,CH4,TraPPE-EH" -> {0.0,3.31,0.01(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann 1999*)
"C,CH3,TraPPE-EH" -> {0.0,3.30,4.00(*K*)*kB*nAvogadro/4184(*kcal/mol*)}, (*Siepmann 1999*)
"C,CH2,TraPPE-EH" -> {0.0,3.65,5.00(*K*)*kB*nAvogadro/4184(*kcal/mol*)} (*Siepmann 1999*)

|>;


(* ::Subsubsection::Closed:: *)
(*Bond Parameters*)


(* ----------------------------------------------------------------------------------------------------------------- *)
(* EDIT (Thaler Sebastian): separated variables for bond stretching and angle bending for easier integration in code *)
(* all parameters are from the CHARMM/22 force field *)
(* EDIT (Haring Michael): bond stretching and angle bending parameters of the AMBER force field as used by Jorgensen are added *)
(* only the parameters from 'Jorgensen 1996, SupInf' are from the CHARMM/22 force field*)

oplsBondStretching = <|

(* r_eq [\[Angstrom]] , K [kcal/(mol \[Angstrom]^2)] *)
"CT-CT"->{1.529,268.},(* Jorgensen 1996, SupInf *)
"HC-CT"->{1.090,340.},(* Jorgensen 1996, SupInf *)

"C-CT"->{1.522,317.},(* Weiner 1995 *)
"C-O"->{1.229,570.},(* Weiner 1985 *)
"CT-OH"->{1.410,320.},(* Weiner 1985 *)
"HO-OH"->{0.960,553.},(* Weiner 1985 *)

"H-NT"->{1.010,434.}(* Jorgensen 1999 *)

|>;


(* ::Subsubsection::Closed:: *)
(*Angle Parameters*)


oplsAngleBending = <|

(* \[Theta]_eq [\[Degree]] , K [kcal/(mol rad^2)] *)
"HC-CT-HC"->{107.8,33.},(* Jorgensen 1996, SupInf *)
"HC-CT-CT"->{110.7,37.50},(* Jorgensen 1996, SupInf *)
"CT-CT-CT"->{112.7,58.35},(* Jorgensen 1996, SupInf *)

"C-CT-HC"->{109.5,35.00},(* Weiner 1985 *)
"CT-C-O"->{120.4,80.00},(* Weiner 1985 *)
"CT-OH-HO"->{108.5,55.00},(* Weiner 1985 *)
"CT-CT-OH"->{109.5,50.00},(* Weiner 1985 *)
"HC-CT-OH"->{109.5,35.00},(* Weiner 1985 *)
"CT-C-CT"->{116.0,70.00},

"H-NT-H"->{106.4,43.60},(* Jorgensen 1999 *)

(* ---------------------------------------------------------------------------------- *)
(* TraPPE-UA force field parameters *)
(* ---------------------------------------------------------------------------------- *)

(*ketone*)
"CHx-C(=O)-CHy,TraPPE-UA"->{117.2,62500/2 (*K*) *kB (*J*) *nAvogadro (*J/mol*) /kCalToJ (*kcal/mol*)}, (* Siepmann, 2004 *)
"CHx-C=O,TraPPE-UA"->{121.4,62500/2 (*K*) *kB (*J*) *nAvogadro (*J/mol*) /kCalToJ (*kcal/mol*)} (* Siepmann, 2004 *)

|>;


(* ::Subsubsection::Closed:: *)
(*Torsion Parameters*)


oplsTorsion = <|
(* V1, V2, V3 in [kcal mol^-1], f1, f2, f3, \[Phi] in [\[Degree]] *)

(* alkanes *)
"alkane H-C-C-H"->{0.000,0.000,0.318,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 7 *)
"alkane H-C-C-C"->{0.000,0.000,0.366,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 7 *)
"alkane C-C-C-C"->{1.740,-0.157,0.279,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 7 *)
"alkene H-C-C=C"->{0.000,0.000,-0.372,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 7 *)

(* alcohols *)
"alcohol H-C-O-H"->{0.000,0.000,0.450,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 7 *)
"alcohol C-C-O-H"->{-0.356,-0.174,0.492,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 7 *)
"alcohol H-C-C-O"->{0.000,0.000,0.486,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 7 *)

(* aldehydes *)
"aldehyde H-C-C-O"->{0.000,0.000,0.000,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"aldehyde H-C-C(O)-H"->{0.000,0.000,0.360,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"aldehyde C-C-C-O"->{-0.277,1.228,-0.694,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"aldehyde C-C-C(O)-H"->{0.000,0.000,0.000,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"aldehyde H-C-C-C(O)"->{0.000,0.000,-0.076,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"aldehyde C-C-C-C(O)"->{-1.697,-0.456,0.585,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)

(* ketones *)
"ketone H-C-C-O"->{0.000,0.000,0.000,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"ketone H-C-C(O)-C"->{0.000,0.000,0.275,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"ketone C-C-C-O"->{-0.277,1.228,-0.694,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"ketone C-C-C(O)-C"->{1.454,-0.144,-0.775,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"ketone H-C-C-C(O)"->{0.000,0.000,-0.076,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)
"ketone C-C-C-C(O)"->{-1.697,-0.456,0.585,0.,0.,0.,0.},(* Jorgensen 1996, SupInf, Table 8 *)

(* ---------------------------------------------------------------------------------- *)
(* OPLS-UA force field parameters  *)
(* Jorgensen 1986 Optimized Intermolecular Potential Functions for Liquid Alcohols *)
(* ---------------------------------------------------------------------------------- *)

"ethanol C-C-O-H"->{0.834,-0.116,0.747,0.,0.,0.,0.}(* Jorgensen 1986 *)

|>;


(* ::Subsubsection::Closed:: *)
(*Bonded Parameters (not used in the current version of the program)*)


oplsBonded = <|
(*
the first two parameters are from the AMBER force field
the second two parameters are from the CHARMM/22 force field
*)

(* bond stretching *)
(* r_eq [\[Angstrom]] , K [kcal/(mol \[Angstrom]^2)] *)
"CT-CT"->{1.526,310.0,1.529,268.},(* Jorgensen 1996, SupInf *)
"HC-CT"->{1.090,331.0,1.090,340.},(* Jorgensen 1996, SupInf *)

(* angle bending *)
(* \[Theta]_eq [\[Degree]] , K [kcal/(mol rad^2)] *)
"HC-CT-HC"->{109.5,35.00,107.8,33.},(* Jorgensen 1996, SupInf *)
"HC-CT-CT"->{109.5,35.00,110.7,37.50},(* Jorgensen 1996, SupInf *)
"CT-CT-CT"->{109.5,40.00,112.7,58.35}(* Jorgensen 1996, SupInf *)

|>;


(* ::Subsection::Closed:: *)
(*Lists taken from GROMACS*)


(* ::Subsubsection::Closed:: *)
(*Wrapper function*)


(* ::Text:: *)
(*A wrapper function to extract the relevant parts.  To link the PropertyPackage add the following lines to a notebook after loading both packages:*)
(*PropertyPackage`Private`GROMACSWrapper = ForceFieldOPLS`Private`GROMACSWrapper;*)
(*PropertyPackage`Private`oplsNonBonded = ForceFieldOPLS`oplsNonBonded;*)


GROMACSWrapper[part_]:=GROMACSatoms[part][[2;;4]]


(* ::Subsubsection::Closed:: *)
(*GROMACSatoms*)


(* from ffnonbonded.itp in oplsaa *)
GROMACSatoms= (* GROMACS atom type tag \[Rule] atom, charge, sigma, epsilon, GROMACS comments on atoms *)
 <|"opls_001" -> {"C", 0.5, 3.75, 0.105, "C, proteins"},
  "opls_002" -> {"O", -0.5, 2.96, 0.21, "O, proteins"},
  "opls_003" -> {"N", -0.57, 3.25, 0.16999999999999998, "N, proteins"},
  "opls_004" -> {"H", 0.37, 0., 0., "H, proteins"},
  "opls_005" -> {"C2", 0.2, 3.8, 0.118, "C2, proteins"},
  "opls_006" -> {"CH", 0.2, 3.8, 0.08, "CH, proteins"},
  "opls_007" -> {"C3", 0., 3.91, 0.16, "C3, proteins"},
  "opls_008" -> {"CH", 0., 3.85, 0.08, "CH, proteins"},
  "opls_009" -> {"C2", 0., 3.9050000000000002, 0.118, "C2, proteins"},
  "opls_010" -> {"C3", 0., 3.9050000000000002, 0.175, "C3, proteins"},
  "opls_011" -> {"CD", 0., 3.75, 0.10999999999999999, "CD, proteins"},
  "opls_012" -> {"N", -0.85, 3.25, 0.16999999999999998, "N, proteins"},
  "opls_013" -> {"H", 0.425, 0., 0., "H, proteins"},
  "opls_014" -> {"CH", 0.285, 3.8, 0.08, "CH, proteins"},
  "opls_015" -> {"C2", 0.285, 3.8, 0.118, "C2, proteins"},
  "opls_016" -> {"C2", -0.1, 3.9050000000000002, 0.118, "C2, proteins"},
  "opls_017" -> {"C", 0.7, 3.75, 0.105, "C, proteins"},
  "opls_018" -> {"O2", -0.8, 2.96, 0.21, "O2, proteins"},
  "opls_019" -> {"C2", 0.31, 3.9050000000000002, 0.118, "C2, proteins"},
  "opls_020" -> {"N3", -0.3, 3.25, 0.16999999999999998, "N3, proteins"},
  "opls_021" -> {"H3", 0.33, 0., 0., "H3, proteins"},
  "opls_022" -> {"C2", 0.265, 3.9050000000000002, 0.118, "C2, proteins"},
  "opls_023" -> {"OH", -0.7, 3.07, 0.16999999999999998, "OH, proteins"},
  "opls_024" -> {"HO", 0.435, 0., 0., "HO, proteins"},
  "opls_025" -> {"CH", 0.265, 3.85, 0.08, "CH, proteins"},
  "opls_026" -> {"C", 0.265, 3.75, 0.10999999999999999, "C, proteins"},
  "opls_027" -> {"C2", 0.31, 3.8, 0.118, "C2, proteins"},
  "opls_028" -> {"C2", 0.1, 3.8, 0.118, "C2, proteins"},
  "opls_029" -> {"CH", 0.31, 3.8, 0.08, "CH, proteins"},
  "opls_030" -> {"CH", 0.1, 3.8, 0.08, "CH, proteins"},
  "opls_031" -> {"C2", 0.18, 3.9050000000000002, 0.118, "C2, proteins"},
  "opls_032" -> {"SH", -0.45, 3.55, 0.25, "SH, proteins"},
  "opls_033" -> {"HS", 0.27, 0., 0., "HS, proteins"},
  "opls_034" -> {"C2", 0.235, 3.8, 0.118, "C2, proteins"},
  "opls_035" -> {"S", -0.47, 3.55, 0.25, "S, proteins"},
  "opls_036" -> {"C3", 0.235, 3.8, 0.16999999999999998, "C3, proteins"},
  "opls_037" -> {"C2", 0.3, 3.8, 0.118, "C2, proteins"},
  "opls_038" -> {"S", -0.3, 3.55, 0.25, "S, proteins"},
  "opls_039" -> {"C3", 0.2, 3.8, 0.16999999999999998, "C3, proteins"},
  "opls_040" -> {"NA", -0.57, 3.25, 0.16999999999999998, "NA, proteins"},
  "opls_041" -> {"H", 0.42, 0., 0., "H, proteins"},
  "opls_042" -> {"NB", -0.49, 3.25, 0.16999999999999998, "NB, proteins"},
  "opls_043" -> {"CP", 0.41, 3.75, 0.145, "CP, proteins"},
  "opls_044" -> {"CF", 0.1, 3.75, 0.145, "CF, proteins"},
  "opls_045" -> {"CC", 0.13, 3.75, 0.145, "CC, proteins"},
  "opls_046" -> {"NA", -0.54, 3.25, 0.16999999999999998, "NA, proteins"},
  "opls_047" -> {"H", 0.46, 0., 0., "H, proteins"},
  "opls_048" -> {"CP", 0.5, 3.75, 0.145, "CP, proteins"},
  "opls_049" -> {"CG", 0.33, 3.75, 0.145, "CG, proteins"},
  "opls_050" -> {"CB", -0.05, 3.75, 0.145, "CB, proteins"},
  "opls_051" -> {"N2", -0.8, 3.25, 0.16999999999999998, "N2, proteins"},
  "opls_052" -> {"H3", 0.46, 0., 0., "H3, proteins"},
  "opls_053" -> {"CA", 0.64, 2.25, 0.049999999999999996, "CA, proteins"},
  "opls_054" -> {"N2", -0.7, 3.25, 0.16999999999999998, "N2, proteins"},
  "opls_055" -> {"H3", 0.44, 0., 0., "H3, proteins"},
  "opls_056" -> {"C2", 0.31, 3.9050000000000002, 0.118, "C2, proteins"},
  "opls_057" -> {"C2", 0.07, 3.9050000000000002, 0.118, "C2, proteins"},
  "opls_058" -> {"C", 0.55, 3.75, 0.105, "C, C  in COOR ester JPC3315(91)"},
  "opls_059" -> {"O", -0.45, 2.96, 0.21, "O, O= in COOR ester"},
  "opls_060" -> {"CH", 0.25, 3.8, 0.08, "CH, proteins"},
  "opls_061" -> {"C2", 0.25, 3.8, 0.118, "C2, proteins"},
  "opls_062" -> {"OS", -0.4, 3., 0.16999999999999998,
    "OS, O- in COOR ester"}, "opls_063" -> {"C3", 0.25, 3.8,
    0.16999999999999998, "C3, CH3 in COOCH3"},
  "opls_064" -> {"CT", 0.2, 3.8, 0.049999999999999996, "CT, proteins"},
  "opls_065" -> {"C3", 0., 3.96, 0.145, "C3, proteins"},
  "opls_066" -> {"C4", 0., 3.73, 0.2940009560229445,
    "C4, CH4       JACS,106,6638 (1984)"},
  "opls_067" -> {"C3", 0., 3.775, 0.207, "C3, CH3 (C1) ETHANE"},
  "opls_068" -> {"C3", 0., 3.9050000000000002, 0.175,
    "C3, CH3 (C2) N-ALKANES"}, "opls_069" -> {"C3", 0., 3.91, 0.16,
    "C3, CH3 (C3) ISOBUTANE"}, "opls_070" -> {"C3", 0., 3.96, 0.145,
    "C3, CH3 (C4) NEOPENTANE"}, "opls_071" -> {"C2", 0., 3.9050000000000002,
    0.118, "C2, CH2 (SP3) ALKANES"}, "opls_072" ->
   {"C9", 0., 3.85, 0.13999999999999999, "C9, CH2 (SP2) 1-ALKENES"},
  "opls_073" -> {"CH", 0., 3.85, 0.08, "CH, CH  (SP3) ISOBUTANE"},
  "opls_074" -> {"C8", 0., 3.8, 0.11499999999999999,
    "C8, CH  (SP2) 2-ALKENES"}, "opls_075" ->
   {"CD", 0., 3.75, 0.10999999999999999,
    "CD, CH (AROM) BENZENOID united atom"},
  "opls_076" -> {"CT", 0., 3.8, 0.049999999999999996,
    "CT, C   (SP3) NEOPENTANE"}, "opls_077" -> {"C7", 0., 3.75, 0.105,
    "C7, C   (SP2) ISOBUTENE"}, "opls_078" -> {"OH", -0.7, 3.07,
    0.16999999999999998, "OH, O   ALCOHOLS      JPC,90,1276 (1986)"},
  "opls_079" -> {"HO", 0.435, 0., 0.,
    "HO, H(O) ALCOHOLS     JPC,90,1276 (1986)"},
  "opls_080" -> {"C3", 0.265, 3.775, 0.207,
    "C3, CH3 IN METHANOL   JPC,90,1276 (1986)"},
  "opls_081" -> {"C2", 0.265, 3.9050000000000002, 0.118,
    "C2, CH2 IN ETHANOL    JPC,90,1276 (1986)"},
  "opls_082" -> {"SH", -0.47, 3.7, 0.25,
    "SH, S   IN H2S        JPC,90,6379 (1986)"},
  "opls_083" -> {"SH", -0.45, 3.55, 0.25,
    "SH, S   IN RSH        JPC,90,6379 (1986)"},
  "opls_084" -> {"S", -0.47, 3.55, 0.25,
    "S, S   IN RSR        JPC,90,6379 (1986)"},
  "opls_085" -> {"S", -0.3, 3.55, 0.25,
    "S, S   IN RSSR       JPC,90,6379 (1986)"},
  "opls_086" -> {"HS", 0.235, 0., 0.,
    "HS, H   IN H2S        JPC,90,6379 (1986)"},
  "opls_087" -> {"HS", 0.27, 0., 0.,
    "HS, H(S) IN RSH       JPC,90,6379 (1986)"},
  "opls_088" -> {"C3", 0.18, 3.775, 0.207,
    "C3, CH3 IN CH3SH      JPC,90,6379 (1986)"},
  "opls_089" -> {"C2", 0.18, 3.9050000000000002, 0.118,
    "C2, CH2 IN CH3CH2SH   JPC,90,6379 (1986)"},
  "opls_090" -> {"C3", 0.235, 3.8, 0.16999999999999998,
    "C3, CH3 IN CH3SR      JPC,90,6379 (1986)"},
  "opls_091" -> {"C2", 0.235, 3.8, 0.118,
    "C2, CH2 IN RCH2SR     JPC,90,6379 (1986)"},
  "opls_092" -> {"C3", 0.3, 3.8, 0.16999999999999998,
    "C3, CH3 IN CH3SSR     JPC,90,6379 (1986)"},
  "opls_093" -> {"C2", 0.3, 3.8, 0.118,
    "C2, CH2 IN RCH2SSR    JPC,90,6379 (1986)"},
  "opls_094" -> {"N", -0.43, 3.2, 0.16999999999999998,
    "N, N IN CH3CN                   Mol.Phys.,63,547 (1988)"},
  "opls_095" -> {"C", 0.28, 3.65, 0.15,
    "C, C IN CH3CN                   Mol.Phys.,63,547 (1988)"},
  "opls_096" -> {"C3", 0.15, 3.775, 0.207,
    "C3, CH3 IN CH3CN                 Mol.Phys.,63,547 (1988)"},
  "opls_097" -> {"Ar", 0., 3.4010000000000002, 0.23390009560229444,
    "Ar, Argon from Verlet & Weis     Mol.Phys.,24,1013 (1972) For Ne and \
He,"}, "opls_098" -> {"Kr", 0., 3.935, 0.43299952198852765, "Kr, Krypton from \
Verlet & Weis   Mol.Phys.,24,1013 (1972) see types 129,130."},
  "opls_099" -> {"Xe", 0., 0., 0.,
    "Xe, Xenon from Verlet & Weis     Mol.Phys.,24,1013 (1972)"},
  "opls_101" -> {"N3", -0.4, 3.25, 0.16999999999999998,
    "N3, N (NH4+)                     JPC,90,2174 (1986)"},
  "opls_102" -> {"N3", -0.3, 3.25, 0.16999999999999998,
    "N3, N (RNH3+)                    JPC,90,2174 (1986)"},
  "opls_103" -> {"N3", 0., 3.25, 0.16999999999999998,
    "N3, N (R4N+)                     JPC,90,2174 (1986)"},
  "opls_104" -> {"H3", 0.35, 0., 0.,
    "H3, H (NH4+)                     JPC,90,2174 (1986)"},
  "opls_105" -> {"H3", 0.33, 0., 0.,
    "H3, H (RNH3+)                    JPC,90,2174 (1986)"},
  "opls_106" -> {"C3", 0.31, 3.775, 0.207,
    "C3, United-atom CH3 (CH3NH3+)    JPC 90,2174 (1986)"},
  "opls_107" -> {"C3", 0.25, 3.96, 0.145,
    "C3, United-atom CH3 ((CH3)4N+)   JPC 90,2174 (1986)"},
  "opls_108" -> {"OS", -0.5, 3., 0.16999999999999998,
    "OS, United-atom Ether O          JCC,11,958 (1990)"},
  "opls_109" -> {"C3", 0.25, 3.8, 0.16999999999999998,
    "C3, United-atom Ether CH3 (-O)   JCC,11,958 (1990)"},
  "opls_110" -> {"C2", 0.25, 3.8, 0.118,
    "C2, United-atom Ether CH2 (-O)   JCC,11,958 (1990)"},
  "opls_111" -> {"OW", -0.83, 3.15061, 0.15209990439770554,
    "OW, O TIP3P Water"}, "opls_112" -> {"HW", 0.417, 0., 0.,
    "HW, H TIP3P Water"}, "opls_113" -> {"OW", 0., 3.15365, 0.155,
    "OW, O TIP4P Water"}, "opls_114" -> {"HW", 0.52, 0., 0.,
    "HW, H TIP4P Water"}, "opls_115" -> {"MW", -1.04, 0., 0.,
    "MW, M TIP4P Water"}, "opls_116" -> {"OW", -0.82, 3.1655699999999998,
    0.15540009560229445, "OW, O SPC Water"},
  "opls_117" -> {"HW", 0.41, 0., 0., "HW, H SPC Water"},
  "opls_118" -> {"OW", 0., 3.12, 0.16, "OW, O TIP5P Water"},
  "opls_118e" -> {"OW", 0., 3.0969999999999995, 0.178, "OW, opls_118e"},
  "opls_119" -> {"HW", 0., 0., 0., "HW, H TIP5P Water"},
  "opls_120" -> {"OL", 0., 0., 0., "OL, L TIP5P Water"},
  "opls_122" -> {"CT", 0.248, 3.8, 0.049999999999999996, "CT, C CCl4"},
  "opls_123" -> {"Cl", -0.06, 3.4699999999999998, 0.26600143403441684,
    "Cl, Cl CCl4"}, "opls_124" -> {"S", 0.139, 3.5599999999999996,
    0.39499999999999996, "S, S in UA DMSO"},
  "opls_125" -> {"O2", -0.45, 2.9299999999999997, 0.27999999999999997,
    "O2, O in UA DMSO"}, "opls_126" -> {"C3", 0.16, 3.81, 0.16,
    "C3, CH3 in UA DMSO"}, "opls_127" -> {"NT", -1.02, 3.4200000000000004,
    0.16999999999999998,
    "NT, Updated ammonia parameters - JPC B 2001, 105, 6474"},
  "opls_128" -> {"H", 0.34, 0., 0.,
    "H, Updated ammonia parameters - JPC B 2001, 105, 6474"},
  "opls_129" -> {"Ne", 0., 2.7800000000000002, 0.06899999999999999,
    "Ne, Neon                          Hirschfelder (Wiley,1954)"},
  "opls_130" -> {"He", 0., 2.556, 0.02,
    "He, Helium                        Hirschfelder (Wiley,1954)"},
  "opls_131" -> {"C", 0.5, 3.8, 0.11499999999999999,
    "C, C in C=O for UA formamide, DMF."},
  "opls_132" -> {"C3", 0.285, 3.8, 0.16999999999999998,
    "C3, CH3 in HCON(CH3)2 DMF"}, "opls_135" -> {"CT", -0.18, 3.5, 0.066,
    "CT, alkane CH3"}, "opls_136" -> {"CT", -0.12, 3.5, 0.066,
    "CT, alkane CH2"}, "opls_137" -> {"CT", -0.06, 3.5, 0.066,
    "CT, alkane CH"}, "opls_138" -> {"CT", -0.24, 3.5, 0.066,
    "CT, alkane CH4"}, "opls_139" -> {"CT", 0., 3.5, 0.066, "CT, alkane C"},
  "opls_140" -> {"HC", 0.06, 2.5, 0.029999999999999995, "HC, alkane H."},
  "opls_141" -> {"CM", 0., 3.55, 0.076, "CM, alkene C (R2-C=)"},
  "opls_142" -> {"CM", -0.11, 3.55, 0.076, "CM, alkene C (RH-C=)"},
  "opls_143" -> {"CM", -0.23, 3.55, 0.076, "CM, alkene C (H2-C=)"},
  "opls_144" -> {"HC", 0.115, 2.42, 0.029999999999999995,
    "HC, alkene H (H-C=)"}, "opls_145" -> {"CA", -0.11, 3.55,
    0.06999999999999999,
    "CA, Benzene C - 12 site JACS,112,4768-90. Use #145B for biphenyl"},
  "opls_145B" -> {"C!", -0.11, 3.55, 0.06999999999999999, "C!, Biphenyl C1"},
  "opls_146" -> {"HA", 0.115, 2.42, 0.029999999999999995,
    "HA, Benzene H - 12 site."}, "opls_147" ->
   {"CA", 0., 3.55, 0.06999999999999999, "CA, Naphthalene fusion C (C9)"},
  "opls_148" -> {"CT", -0.06, 3.5, 0.066, "CT, C: CH3, toluene"},
  "opls_149" -> {"CT", 0., 3.5, 0.066, "CT, C: CH2, ethyl benzene"},
  "opls_150" -> {"C=", -0.11, 3.55, 0.076,
    "C=, diene =CH-CH=; use #178 for =CR-CR="},
  "opls_151" -> {"Cl", -0.2, 3.4000000000000004, 0.3,
    "Cl, Cl in alkyl chlorides"}, "opls_152" -> {"CT", 0., 3.5, 0.066,
    "CT, RCH2Cl in alkyl chlorides"}, "opls_153" ->
   {"HC", 0.103, 2.5, 0.029999999999999995,
    "HC, H in RCH2Cl in alkyl chlorides"},
  "opls_154" -> {"OH", -0.68, 3.12, 0.16999999999999998,
    "OH, all-atom O: mono alcohols"}, "opls_155" ->
   {"HO", 0.418, 0., 0., "HO, all-atom H(O): mono alcohols, OP(=O)2"},
  "opls_156" -> {"HC", 0.04, 2.5, 0.029999999999999995,
    "HC, all-atom H(C): methanol"}, "opls_157" ->
   {"CT", 0.145, 3.5, 0.066, "CT, all-atom C: CH3 & CH2, alcohols"},
  "opls_158" -> {"CT", 0.205, 3.5, 0.066, "CT, all-atom C: CH, alcohols"},
  "opls_159" -> {"CT", 0.265, 3.5, 0.066, "CT, all-atom C: C, alcohols"},
  "opls_160" -> {"CT_4", 0.126, 3.5, 0.066, "CT_4, CH2 Trifluoroethanol"},
  "opls_161" -> {"CT", 0.532, 3.25, 0.062000000000000006,
    "CT, CF3 Trifluoroethanol"}, "opls_162" -> {"OH", -0.63, 3.07,
    0.16999999999999998, "OH, OH  Trifluoroethanol"},
  "opls_163" -> {"HO", 0.429, 0., 0., "HO, HO  Trifluoroethanol"},
  "opls_164" -> {"F", -0.2, 2.94, 0.061, "F, F   Trifluoroethanol"},
  "opls_165" -> {"HC", 0.083, 2.5, 0.029999999999999995,
    "HC, H   Trifluoroethanol"}, "opls_166" ->
   {"CA", 0.15, 3.55, 0.06999999999999999, "CA, C(OH) phenol  Use with all"},
  "opls_167" -> {"OH", -0.58, 3.07, 0.16999999999999998,
    "OH, O     phenol  atom C, H 145 & 146"},
  "opls_168" -> {"HO", 0.435, 0., 0., "HO, H     phenol"},
  "opls_169" -> {"OH", -0.7, 3.07, 0.16999999999999998, "OH, O:    diols"},
  "opls_170" -> {"HO", 0.435, 0., 0., "HO, H(O): diols"},
  "opls_171" -> {"OH", -0.73, 3.07, 0.16999999999999998, "OH, O:    triols"},
  "opls_172" -> {"HO", 0.465, 0., 0., "HO, H(O): triols"},
  "opls_173" -> {"CT", 0.145, 3.5, 0.066, "CT, C(H2OH): triols"},
  "opls_174" -> {"CT", 0.205, 3.5, 0.066, "CT, C(HROH): triols"},
  "opls_175" -> {"CT", 0.265, 3.5, 0.066, "CT, C(R2OH): triols"},
  "opls_176" -> {"HC", 0.06, 2.5, 0.029999999999999995,
    "HC, H(CXOH): triols"}, "opls_178" -> {"C=", 0., 3.55, 0.076,
    "C=, diene =CR-CR=; use #150 for =CH-CH="},
  "opls_179" -> {"OS", -0.28, 2.9, 0.13999999999999999, "OS, O: anisole"},
  "opls_180" -> {"OS", -0.4, 2.9, 0.13999999999999999,
    "OS, O: dialkyl ether"}, "opls_181" -> {"CT", 0.11, 3.5, 0.066,
    "CT, C(H3OR): methyl ether"}, "opls_182" -> {"CT", 0.14, 3.5, 0.066,
    "CT, C(H2OR): ethyl  ether"}, "opls_183" -> {"CT", 0.17, 3.5, 0.066,
    "CT, C(HOR):  i-Pr   ether, allose"},
  "opls_184" -> {"CT", 0.2, 3.5, 0.066, "CT, C(OR):   t-Bu   ether"},
  "opls_185" -> {"HC", 0.03, 2.5, 0.029999999999999995,
    "HC, H(COR): alpha H ether"}, "opls_186" ->
   {"OS", -0.4, 2.9, 0.13999999999999999, "OS, O: acetal ether"},
  "opls_187" -> {"OH", -0.7, 3.07, 0.16999999999999998,
    "OH, O(H): hemiacetal"}, "opls_188" -> {"HO", 0.435, 0., 0.,
    "HO, H(O): hemiacetal"}, "opls_189" -> {"CO", 0.2, 3.5, 0.066,
    "CO, C(H2O2): acetal OCH2O"}, "opls_190" ->
   {"HC", 0.1, 2.5, 0.029999999999999995, "HC, H(CHO2): acetal OCH2O"},
  "opls_191" -> {"CO", 0.265, 3.5, 0.066, "CO, C(H2O2): hemiacetal OCH2OH"},
  "opls_192" -> {"HC", 0.1, 2.5, 0.029999999999999995,
    "HC, H(CHO2): hemiacetal OCH2OH"},
  "opls_193" -> {"CO", 0.3, 3.5, 0.066, "CO, C(HCO2): acetal OCHRO"},
  "opls_194" -> {"HC", 0.1, 2.5, 0.029999999999999995,
    "HC, H(CHO2): acetal OCHRO"}, "opls_195" -> {"CO", 0.365, 3.5, 0.066,
    "CO, C(HCO2): hemiacetal OCHROH"},
  "opls_196" -> {"HC", 0.1, 2.5, 0.029999999999999995,
    "HC, H(C2O2): hemiacetal OCHROH"},
  "opls_197" -> {"CO", 0.4, 3.5, 0.066, "CO, C(C2O2): acetal OCRRO"},
  "opls_198" -> {"CO", 0.465, 3.5, 0.066, "CO, C(C2O2): hemiacetal OCRROH"},
  "opls_199" -> {"CA", 0.085, 3.55, 0.06999999999999999,
    "CA, C(O,Me): anisole"}, "opls_200" -> {"SH", -0.33, 3.5999999999999996,
    0.425, "SH, all-atom S: thiols"}, "opls_201" ->
   {"SH", -0.47, 3.7, 0.25, "SH, S   IN H2S  JPC,90,6379 (1986)"},
  "opls_202" -> {"S", -0.33, 3.5999999999999996, 0.355,
    "S, all-atom S: sulfides, S=C"}, "opls_203" ->
   {"S", -0.21, 3.55, 0.25, "S, all-atom S: disulfides"},
  "opls_204" -> {"HS", 0.155, 0., 0., "HS, all-atom H(S): thiols"},
  "opls_205" -> {"HS", 0.235, 0., 0., "HS, H   IN H2S  JPC,90,6379 (1986)"},
  "opls_206" -> {"CT", 0.06, 3.5, 0.066, "CT, all-atom C: CH2, thiols"},
  "opls_207" -> {"CT", 0.12, 3.5, 0.066, "CT, all-atom C: CH, thiols"},
  "opls_208" -> {"CT", 0.18, 3.5, 0.066, "CT, all-atom C: C, thiols"},
  "opls_209" -> {"CT", -0.01, 3.5, 0.066, "CT, all-atom C: CH3, sulfides"},
  "opls_210" -> {"CT", 0.048, 3.5, 0.066, "CT, all-atom C: CH2, sulfides"},
  "opls_211" -> {"CT", 0.107, 3.5, 0.066, "CT, all-atom C: CH, sulfides"},
  "opls_212" -> {"CT", 0.168, 3.5, 0.066, "CT, all-atom C: C, sulfides"},
  "opls_213" -> {"CT", 0.037, 3.5, 0.066, "CT, all-atom C: CH3, disulfides"},
  "opls_214" -> {"CT", 0.098, 3.5, 0.066, "CT, all-atom C: CH2, disulfides"},
  "opls_215" -> {"CT", 0.158, 3.5, 0.066, "CT, all-atom C: CH, disulfides"},
  "opls_216" -> {"CT", 0.217, 3.5, 0.066, "CT, all-atom C: C, disulfides"},
  "opls_217" -> {"CT", 0., 3.5, 0.066, "CT, all-atom C: CH3, methanethiol"},
  "opls_218" -> {"CT", 0.2, 3.5, 0.066, "CT, C in CH2OH - benzyl alcohols"},
  "opls_219" -> {"CT", 0.26, 3.5, 0.066, "CT, C in CHROH - benzyl alcohols"},
  "opls_220" -> {"CT", 0.32, 3.5, 0.066, "CT, C in CR2OH - benzyl alcohols"},
  "opls_221" -> {"CA", -0.05, 3.55, 0.06999999999999999,
    "CA, C(CH2OH)   - benzyl alcohols"},
  "opls_222" -> {"S", -0.32, 3.55, 0.25, "S, S in thioanisoles"},
  "opls_223" -> {"CT", 0.08, 3.5, 0.066,
    "CT, C in RCH2NH2. Use #223B for AA Calpha."},
  "opls_223B" -> {"CT_2", 0.08, 3.5, 0.066, "CT_2, Gly Calpha"},
  "opls_224" -> {"CT", 0.14, 3.5, 0.066,
    "CT, C in R2CHNH2. Use #224B for AA Calpha."},
  "opls_224B" -> {"CT_2", 0.14, 3.5, 0.066,
    "CT_2, Calpha in most AA (except Gly,Pro,Aib)"},
  "opls_225" -> {"CT", 0.2, 3.5, 0.066,
    "CT, C in R3CNH2. Use #225B for AA Calpha."},
  "opls_225B" -> {"CT_2", 0.2, 3.5, 0.066, "CT_2, Aib Calpha."},
  "opls_226" -> {"Cl", -0.12, 3.4000000000000004, 0.3,
    "Cl, chloroalkene Cl (ClH-C=) - see also #398"},
  "opls_227" -> {"CM", 0.005, 3.55, 0.076, "CM, chloroalkene C (ClH-C=)"},
  "opls_228" -> {"CA", 0.102, 3.55, 0.06999999999999999,
    "CA, C(SMe)  thioanisole"}, "opls_229" -> {"CT", 0.14, 3.5, 0.066,
    "CT, C on N: secondary N-CHR2 amide"},
  "opls_230" -> {"CT", 0.2, 3.5, 0.066,
    "CT, C on N: secondary N-CR3  amide"},
  "opls_231" -> {"C", 0.7, 3.75, 0.105, "C, C: C=O in benzophenone"},
  "opls_232" -> {"C_2", 0.565, 3.75, 0.105,
    "C_2, C: C=O in benzaldehyde,acetophenone (CH)"},
  "opls_233" -> {"C_2", 0.585, 3.75, 0.105,
    "C_2, C: C=O in acetophenone (CMe)"},
  "opls_234" -> {"C", 0.615, 3.75, 0.105, "C, C: C=O in benzamide"},
  "opls_235" -> {"C", 0.5, 3.75, 0.105,
    "C, C=O in amide, dmf, peptide bond"},
  "opls_236" -> {"O", -0.5, 2.96, 0.21,
    "O, O: C=O in amide.   Acyl R on C in amide is neutral -"},
  "opls_237" -> {"N", -0.76, 3.25, 0.16999999999999998,
    "N, N: primary amide.  use alkane parameters."},
  "opls_238" -> {"N", -0.5, 3.25, 0.16999999999999998,
    "N, N: secondary amide, peptide bond (see #279 for formyl H)"},
  "opls_239" -> {"N", -0.14, 3.25, 0.16999999999999998,
    "N, N: tertiary amide"}, "opls_240" -> {"H", 0.38, 0., 0.,
    "H, H on N: primary amide"}, "opls_241" ->
   {"H", 0.3, 0., 0., "H, H on N: secondary amide"},
  "opls_242" -> {"CT", 0.02, 3.5, 0.066, "CT, C on N: secondary N-Me amide"},
  "opls_243" -> {"CT", -0.11, 3.5, 0.066,
    "CT, C on N: tertiary  N-Me amide"},
  "opls_244" -> {"CT", 0.08, 3.5, 0.066,
    "CT, C on N: secondary N-CH2R amide"},
  "opls_245" -> {"CT_3", -0.05, 3.5, 0.066,
    "CT_3, C on N: tertiary  N-CH2R amide, Pro CD"},
  "opls_246" -> {"CT_2", 0.01, 3.5, 0.066,
    "CT_2, C on N: tertiary  N-CHR2 amide, Pro CA"},
  "opls_247" -> {"C", 0.142, 3.75, 0.105, "C, C in O=C(NH2)2  Urea"},
  "opls_248" -> {"O", -0.39, 2.96, 0.21,
    "O, O in O=C(NH2)2  Urea Isr. J. Chem"},
  "opls_249" -> {"N", -0.54, 3.25, 0.16999999999999998,
    "N, N in O=C(NH2)2  Urea 33, 323 (93)"},
  "opls_250" -> {"H", 0.333, 0., 0., "H, H in O=C(NH2)2  Urea"},
  "opls_251" -> {"N", -0.49, 3.25, 0.16999999999999998, "N, N   in imide"},
  "opls_252" -> {"C", 0.42, 3.75, 0.105, "C, C(=O) in imide"},
  "opls_253" -> {"O", -0.42, 2.96, 0.21, "O, O   in imide"},
  "opls_254" -> {"H", 0.37, 0., 0., "H, H(N) in imide"},
  "opls_255" -> {"HC", 0.06, 2.5, 0.02, "HC, H(C) in formimide"},
  "opls_256" -> {"CT", -0.12, 3.5, 0.066, "CT, C in CH3  imide"},
  "opls_257" -> {"CT", -0.06, 3.5, 0.066, "CT, C in RCH2 imide"},
  "opls_258" -> {"CT", 0., 3.5, 0.066, "CT, C in R2CH imide"},
  "opls_259" -> {"CT", 0.06, 3.5, 0.066, "CT, C in R3C  imide"},
  "opls_260" -> {"CA", 0.035, 3.55, 0.06999999999999999,
    "CA, C(CN)  benzonitrile"}, "opls_261" -> {"CZ", 0.395, 3.65, 0.15,
    "CZ, C(N)   benzonitrile"}, "opls_262" -> {"NZ", -0.43, 3.2,
    0.16999999999999998, "NZ, N      benzonitrile"},
  "opls_263" -> {"CA", 0.18, 3.55, 0.06999999999999999,
    "CA, C(Cl)  chlorobenzene"}, "opls_264" ->
   {"Cl", -0.18, 3.4000000000000004, 0.3, "Cl, Cl     chlorobenzene"},
  "opls_265" -> {"N", -0.38, 3.25, 0.16999999999999998,
    "N, N: N-phenylacetamide"}, "opls_266" -> {"CA", 0.085, 3.55,
    0.06999999999999999, "CA, ipso C in N-phenylacetamide"},
  "opls_267" -> {"C", 0.52, 3.75, 0.105, "C, Co in CCOOH carboxylic acid"},
  "opls_268" -> {"OH", -0.53, 3., 0.16999999999999998,
    "OH, Oh in CCOOH R in RCOOH is"}, "opls_269" ->
   {"O_3", -0.44, 2.96, 0.21, "O_3, Oc in CCOOH neutral; use #135-#140"},
  "opls_270" -> {"HO", 0.45, 0., 0., "HO, H  in CCOOH"},
  "opls_271" -> {"C_3", 0.7, 3.75, 0.105, "C_3, C in COO- carboxylate"},
  "opls_272" -> {"O2", -0.8, 2.96, 0.21,
    "O2, O: O in COO- carboxylate,peptide terminus"},
  "opls_273" -> {"CT", -0.28, 3.5, 0.066, "CT, C: CH3, carboxylate ion"},
  "opls_274" -> {"CT", -0.22, 3.5, 0.066, "CT, C: CH2, carboxylate ion"},
  "opls_275" -> {"CT", -0.16, 3.5, 0.066, "CT, C: CH,  carboxylate ion"},
  "opls_276" -> {"CT", -0.1, 3.5, 0.066, "CT, C: C,   carboxylate ion"},
  "opls_277" -> {"C_2", 0.45, 3.75, 0.105,
    "C_2, AA C: aldehyde - for C-alpha use #135-#139"},
  "opls_278" -> {"O_2", -0.45, 2.96, 0.21, "O_2, AA O: aldehyde"},
  "opls_279" -> {"HC", 0., 2.42, 0.014999999999999998,
    "HC, AA H-alpha in aldehyde & formamide"},
  "opls_280" -> {"C_2", 0.47, 3.75, 0.105,
    "C_2, AA C: ketone - for C-alpha use #135-#139"},
  "opls_281" -> {"O_2", -0.47, 2.96, 0.21, "O_2, AA O: ketone"},
  "opls_282" -> {"HC", 0.06, 2.42, 0.014999999999999998,
    "HC, AA H on C-alpha in ketone & aldehyde"},
  "opls_283" -> {"CT_2", 0.04, 3.5, 0.066,
    "CT_2, CA on C-terminal ALA,CYS,SER,THR,HIS,ASP,ASN"},
  "opls_284" -> {"CT_2", -0.02, 3.5, 0.066, "CT_2, CA on C-terminal GLY"},
  "opls_285" -> {"CT_2", -0.09, 3.5, 0.066, "CT_2, CA on C-terminal PRO"},
  "opls_286" -> {"N3", -0.4, 3.25, 0.16999999999999998,
    "N3, N (NH4+) JPC,90,2174 (1986)"},
  "opls_287" -> {"N3", -0.3, 3.25, 0.16999999999999998,
    "N3, N (RNH3+) JPC,90,2174 (1986)"},
  "opls_288" -> {"N3", 0., 3.25, 0.16999999999999998,
    "N3, N (R4N+)  JPC,90,2174 (1986)"},
  "opls_289" -> {"H3", 0.35, 0., 0., "H3, H (NH4+)  JPC,90,2174 (1986)"},
  "opls_290" -> {"H3", 0.33, 0., 0., "H3, H (RNH3+) JPC,90,2174 (1986)"},
  "opls_291" -> {"CT", 0.13, 3.5, 0.066, "CT, C in  CH3NH3+"},
  "opls_292" -> {"CT", 0.19, 3.5, 0.066, "CT, C in  RCH2NH3+"},
  "opls_292B" -> {"CT_2", 0.19, 3.5, 0.066, "CT_2, CA in GLY-NH3+ N-term."},
  "opls_293" -> {"CT", 0.25, 3.5, 0.066, "CT, C in  R2CHNH3+"},
  "opls_293B" -> {"CT_2", 0.25, 3.5, 0.066,
    "CT_2, CA in NH3+ N-term, All AA except GLY & PRO"},
  "opls_294" -> {"CT", 0.31, 3.5, 0.066, "CT, C in  R3CNH3+"},
  "opls_295" -> {"CT_2", 0.12, 3.5, 0.066, "CT_2, AA C-alpha on N-term PRO"},
  "opls_296" -> {"CT_3", 0.17, 3.5, 0.066,
    "CT_3, AA:C-delta in N-term PRO NH2+"},
  "opls_297" -> {"CT", 0.11, 3.5, 0.066, "CT, CT in  CH3NH2+R"},
  "opls_298" -> {"CT_2", 0.09, 3.5, 0.066,
    "CT_2, AA C-alpha in Gly zwitterion"},
  "opls_299" -> {"CT_2", 0.15, 3.5, 0.066,
    "CT_2, AA C-alpha in Ala zwitterion"},
  "opls_300" -> {"N2", -0.8, 3.25, 0.16999999999999998,
    "N2, N: guanidinium NH2"}, "opls_301" -> {"H3", 0.46, 0., 0.,
    "H3, H: guanidinium NH2"}, "opls_302" -> {"CA", 0.64, 2.25,
    0.049999999999999996, "CA, C: guanidinium C+"},
  "opls_303" -> {"N2", -0.7, 3.25, 0.16999999999999998,
    "N2, N: guanidinium NHR"}, "opls_304" -> {"H3", 0.44, 0., 0.,
    "H3, H: guanidinium NHR"}, "opls_305" -> {"CT", 0.2, 3.5, 0.066,
    "CT, C: CH3, methylguanidinium"}, "opls_306" ->
   {"CT", -0.11, 3.5, 0.066, "CT, C: CH3, ethylguanidinium"},
  "opls_307" -> {"CT", 0.19, 3.5, 0.066,
    "CT, C: CH2(D), ARG, ethylguanidinium"},
  "opls_308" -> {"CT", -0.05, 3.5, 0.066, "CT, C: CH2(G), ARG"},
  "opls_309" -> {"N3", -0.2, 3.25, 0.16999999999999998,
    "N3, N (R2NH2+), N-terminal PRO NH2+"},
  "opls_310" -> {"H3", 0.31, 0., 0., "H3, H (R2NH2+)"},
  "opls_311" -> {"NC", -0.46, 3.25, 0.16999999999999998,
    "NC, DAP N1   (Diaminopyridine)"},
  "opls_312" -> {"CA", 0.36, 3.5, 0.08, "CA, DAP C2"},
  "opls_313" -> {"N2", -0.85, 3.25, 0.16999999999999998, "N2, DAP N-amine"},
  "opls_314" -> {"H", 0.37, 0., 0., "H, DAP H-amine"},
  "opls_315" -> {"CA", -0.15, 3.5, 0.08, "CA, DAP C3"},
  "opls_316" -> {"HA", 0.1, 2.5, 0.049999999999999996, "HA, DAP H3"},
  "opls_317" -> {"CA", -0.04, 3.5, 0.08, "CA, DAP C4"},
  "opls_318" -> {"HA", 0.1, 2.5, 0.049999999999999996, "HA, DAP H4"},
  "opls_319" -> {"NA", -0.6, 3.25, 0.16999999999999998,
    "NA, Uracil & Thymine N1 - use #319B for nucleoside"},
  "opls_319B" -> {"N*", -0.6, 3.25, 0.16999999999999998,
    "N*, Uracil & Thymine N1 - only for nucleoside"},
  "opls_320" -> {"C", 0.5, 3.75, 0.105, "C, Uracil & Thymine C2"},
  "opls_321" -> {"NA", -0.51, 3.25, 0.16999999999999998,
    "NA, Uracil & Thymine N3"}, "opls_322" -> {"C", 0.45, 3.75, 0.105,
    "C, Uracil & Thymine C4"}, "opls_323" -> {"CM", -0.07, 3.5, 0.08,
    "CM, Uracil & Thymine C5"}, "opls_324" -> {"CM", 0.08, 3.5, 0.08,
    "CM, Uracil & Thymine C6"}, "opls_325" -> {"H", 0.41, 0., 0.,
    "H, Uracil & Thymine H-N1"}, "opls_326" -> {"O", -0.4, 2.96, 0.21,
    "O, Uracil O-C2"}, "opls_327" -> {"H", 0.36, 0., 0., "H, Uracil H-N3"},
  "opls_328" -> {"O", -0.42, 2.96, 0.21, "O, Uracil O-C4"},
  "opls_329" -> {"HC", 0.1, 2.5, 0.049999999999999996, "HC, Uracil H-C5"},
  "opls_330" -> {"HC", 0.1, 2.5, 0.049999999999999996, "HC, Uracil H-C6"},
  "opls_331" -> {"CT", -0.14, 3.5, 0.08, "CT, Thymine C-C5"},
  "opls_332" -> {"HC", 0.08, 2.5, 0.049999999999999996, "HC, Thymine H-CC5"},
  "opls_333" -> {"NA", -0.56, 3.25, 0.16999999999999998,
    "NA, Cytosine N1 -use #333B for nucleoside"},
  "opls_333B" -> {"N*", -0.56, 3.25, 0.16999999999999998,
    "N*, Cytosine N1 - for nucleoside"},
  "opls_334" -> {"C", 0.55, 3.75, 0.105, "C, Cytosine C2"},
  "opls_335" -> {"NC", -0.54, 3.25, 0.16999999999999998, "NC, Cytosine N3"},
  "opls_336" -> {"CA", 0.46, 3.5, 0.08,
    "CA, Cytosine C4     Nucleotide base"},
  "opls_337" -> {"CM", -0.06, 3.5, 0.08, "CM, Cytosine C5     parameters:"},
  "opls_338" -> {"CM", 0.1, 3.5, 0.08,
    "CM, Cytosine C6     JACS,113,2810(1991)"},
  "opls_339" -> {"H", 0.38, 0., 0., "H, Cytosine H-N1"},
  "opls_340" -> {"O", -0.48, 2.96, 0.21, "O, Cytosine O-C2"},
  "opls_341" -> {"N2", -0.79, 3.25, 0.16999999999999998,
    "N2, Cytosine N-C4"}, "opls_342" -> {"H", 0.385, 0., 0.,
    "H, Cytosine H-NC4/N3"}, "opls_343" -> {"H", 0.355, 0., 0.,
    "H, Cytosine H-NC4/C5"}, "opls_344" -> {"HC", 0.1, 2.5,
    0.049999999999999996, "HC, Cytosine H-C5"},
  "opls_345" -> {"H4", 0.1, 2.5, 0.049999999999999996, "H4, Cytosine H-C6"},
  "opls_346" -> {"NC", -0.53, 3.25, 0.16999999999999998, "NC, Adenine N1"},
  "opls_347" -> {"CQ", 0.22, 3.5, 0.08, "CQ, Adenine C2"},
  "opls_348" -> {"NC", -0.55, 3.25, 0.16999999999999998, "NC, Adenine N3"},
  "opls_349" -> {"CB", 0.38, 3.5, 0.08, "CB, Adenine C4"},
  "opls_350" -> {"CB", 0.15, 3.5, 0.08, "CB, Adenine C5"},
  "opls_351" -> {"CA", 0.44, 3.5, 0.08, "CA, Adenine C6"},
  "opls_352" -> {"NB", -0.49, 3.25, 0.16999999999999998,
    "NB, Adenine & Guanine N7"}, "opls_353" -> {"CK", 0.2, 3.5, 0.08,
    "CK, Adenine & Guanine C8"}, "opls_354" ->
   {"NA", -0.5, 3.25, 0.16999999999999998,
    "NA, Adenine & Guanine N9 - use #354B for nucleoside"},
  "opls_354B" -> {"N*", -0.5, 3.25, 0.16999999999999998,
    "N*, Adenine & Guanine N9 - nucleoside only"},
  "opls_355" -> {"H5", 0.2, 2.5, 0.049999999999999996,
    "H5, Adenine & Guanine H-C2"}, "opls_356" ->
   {"N2", -0.81, 3.25, 0.16999999999999998, "N2, Adenine & Guanine N-C6"},
  "opls_357" -> {"H", 0.385, 0., 0., "H, Adenine & Guanine H-NC6/N1"},
  "opls_358" -> {"H", 0.355, 0., 0., "H, Adenine & Guanine H-NC6/C5"},
  "opls_359" -> {"H5", 0.2, 2.5, 0.049999999999999996,
    "H5, Adenine & Guanine H-C8 Guanine"},
  "opls_360" -> {"H", 0.35, 0., 0., "H, Adenine & Guanine H-N9 Guanine"},
  "opls_361" -> {"NA", -0.56, 3.25, 0.16999999999999998, "NA, Guanine N1"},
  "opls_362" -> {"CA", 0.46, 3.5, 0.08, "CA, Guanine C2"},
  "opls_363" -> {"NC", -0.51, 3.25, 0.16999999999999998, "NC, Guanine N3"},
  "opls_364" -> {"CB", 0.34, 3.5, 0.08, "CB, Guanine C4"},
  "opls_365" -> {"CB", 0.12, 3.5, 0.08, "CB, Guanine C5"},
  "opls_366" -> {"C", 0.52, 3.75, 0.105, "C, Guanine C6"},
  "opls_367" -> {"H", 0.38, 0., 0., "H, Guanine H-N1"},
  "opls_368" -> {"N2", -0.8, 3.25, 0.16999999999999998, "N2, Guanine N-C2"},
  "opls_369" -> {"H", 0.4, 0., 0., "H, Guanine H-NC2"},
  "opls_370" -> {"O", -0.51, 2.96, 0.21, "O, Guanine O-C6"},
  "opls_371" -> {"CT", -0.01, 3.5, 0.08, "CT, 9-Me Adenine or Guanine C-N9"},
  "opls_372" -> {"HC", 0.12, 2.5, 0.049999999999999996,
    "HC, 9-Me Adenine or Guanine H-CN9"},
  "opls_373" -> {"CT", -0.01, 3.5, 0.08, "CT, 1-Me Uracil or Thymine C-N1"},
  "opls_374" -> {"HC", 0.14, 2.5, 0.049999999999999996,
    "HC, 1-Me Uracil or Thymine H-CN1"},
  "opls_375" -> {"CT", -0.01, 3.5, 0.08, "CT, 1-Me Cytosine C-N1"},
  "opls_376" -> {"HC", 0.13, 2.5, 0.049999999999999996,
    "HC, 1-Me Cytosine H-CN1"}, "opls_377" -> {"NA", -0.64, 3.25,
    0.16999999999999998, "NA, CytH+ N1 Use #377B for nucleoside."},
  "opls_377B" -> {"N*", -0.64, 3.25, 0.16999999999999998,
    "N*, CytH+ N1 - nucleoside only"},
  "opls_378" -> {"C", 0.65, 3.75, 0.105, "C, CytH+ C2"},
  "opls_379" -> {"NA", -0.74, 3.25, 0.16999999999999998,
    "NA, CytH+ N3 Protonated cytosine."},
  "opls_380" -> {"CA", 0.66, 3.5, 0.08, "CA, CytH+ C4"},
  "opls_381" -> {"CM", -0.06, 3.5, 0.08, "CM, CytH+ C5"},
  "opls_382" -> {"CM", 0.1, 3.5, 0.08, "CM, CytH+ C6"},
  "opls_383" -> {"H", 0.49, 0., 0., "H, CytH+ H-N1"},
  "opls_384" -> {"O", -0.3, 2.96, 0.21, "O, CytH+ O-C2"},
  "opls_385" -> {"H", 0.48, 0., 0., "H, CytH+ H-N3"},
  "opls_386" -> {"N2", -0.81, 3.25, 0.16999999999999998, "N2, CytH+ N-C4"},
  "opls_387" -> {"H", 0.46, 0., 0., "H, CytH+ H-NC4/N3"},
  "opls_388" -> {"H", 0.43, 0., 0., "H, CytH+ H-NC4/C5"},
  "opls_389" -> {"HA", 0.14, 2.5, 0.049999999999999996, "HA, CytH+ H-C5"},
  "opls_390" -> {"H4", 0.14, 2.5, 0.049999999999999996, "H4, CytH+ H-C6"},
  "opls_391" -> {"CT", 0.01, 3.5, 0.08, "CT, 1-Me CytH+ C-N1"},
  "opls_392" -> {"HC", 0.16, 2.5, 0.049999999999999996,
    "HC, 1-Me CytH+ H-CN1"}, "opls_393" -> {"P", 0.78, 3.74,
    0.19999999999999998,
    "P, P    dimethylphosphate anion  UA - see #440 for AA"},
  "opls_394" -> {"O2", -0.66, 2.96, 0.21,
    "O2, O(=) dimethylphosphate anion  UA - see #440 for AA"},
  "opls_395" -> {"OS", -0.43, 3., 0.16999999999999998,
    "OS, O(-) dimethylphosphate anion  UA - see #440 for AA"},
  "opls_396" -> {"CT", 0.02, 3.55, 0.066,
    "CT, C in CH3 dimethylphosphate anion UA - see #440 for AA"},
  "opls_400" -> {"F-", -1., 2.73295, 0.72, "F-, F-  JACS 106, 903 (1984)"},
  "opls_401" -> {"Cl-", -1., 4.41724, 0.11778991395793499,
    "Cl-, Cl- JACS 106, 903 (1984)"}, "opls_402" ->
   {"Br-", -1., 4.62376, 0.09, "Br-, Br- JACS 107, 7793(1985)"},
  "opls_403" -> {"I-", -1., 5.4, 0.06999999999999999,
    "I-, I-  JACS 120, 5104(1998)"}, "opls_404" ->
   {"Li+", 1., 1.25992, 6.249999999999999, "Li+, Li+ JACS 106, 903 (1984)"},
  "opls_405" -> {"Na+", 1., 1.89744, 1.607139101338432,
    "Na+, Na+ JACS 106, 903 (1984)"}, "opls_406" ->
   {"Li+", 1., 2.12645, 0.018278991395793497, "Li+, Li+"},
  "opls_407" -> {"Na+", 1., 3.33045, 0.0027719885277246656,
    "Na+, Na+    Aqvists cation"}, "opls_408" ->
   {"K+", 1., 4.93463, 0.0003279995219885277, "K+, K+     parameters:"},
  "opls_409" -> {"Rb+", 1., 5.621770000000001, 0.00017099999999999998,
    "Rb+, Rb+    JPC,94, 8021 (90)"}, "opls_410" ->
   {"Cs+", 1., 6.715999999999999, 0.000081, "Cs+, Cs+"},
  "opls_411" -> {"Mg2+", 2., 1.64447, 0.8750430210325048, "Mg2+, Mg++"},
  "opls_412" -> {"Ca2+", 2., 2.41203, 0.4496558317399617, "Ca2+, Ca++"},
  "opls_413" -> {"Sr2+", 2., 3.10269, 0.11822609942638622, "Sr2+, Sr++"},
  "opls_414" -> {"Ba2+", 2., 3.81661, 0.04709608030592734, "Ba2+, Ba++"},
  "opls_415" -> {"C3", -0.4, 4.2, 0.3, "C3, C  in  CH3S-  thiolate"},
  "opls_416" -> {"HC", 0.1, 2.5, 0.049999999999999996, "HC, H  in  CH3S-"},
  "opls_417" -> {"SH", -0.9, 4.25, 0.5, "SH, S  in  CH3S-"},
  "opls_418" -> {"C3", -0.2, 4.2, 0.3, "C3, C  in  CH3O-  alkoxide"},
  "opls_419" -> {"HC", 0.06, 2.5, 0.049999999999999996, "HC, H  in  CH3O-"},
  "opls_420" -> {"OH", -0.98, 3.15, 0.25, "OH, O  in  CH3O-"},
  "opls_421" -> {"CT", -1.07, 4.2, 0.3, "CT, C1 in  CH2CN-  RCN-"},
  "opls_422" -> {"HC", 0.19, 2.5, 0.049999999999999996, "HC, H  in  CH2CN-"},
  "opls_423" -> {"CZ", 0.51, 3.65, 0.15,
    "CZ, C2 in  CH2CN-   JACS 111,4190 (89)"},
  "opls_424" -> {"NZ", -0.82, 3.4000000000000004, 0.25, "NZ, N  in  CH2CN-"},
  "opls_425" -> {"C3", -0.3, 4.2, 0.3, "C3, C  in  CH3NH-"},
  "opls_426" -> {"HC", 0.07, 2.5, 0.049999999999999996,
    "HC, HC in  CH3NH-  RNH-"}, "opls_427" ->
   {"N3", -1.31, 3.4000000000000004, 0.25, "N3, N  in  CH3NH-"},
  "opls_428" -> {"H", 0.4, 2.5, 0.049999999999999996, "H, HN in  CH3NH-"},
  "opls_429" -> {"C3", -0.4, 4.2, 0.3, "C3, C2 in  CH3CH2- RCH2-"},
  "opls_430" -> {"HC", 0.08, 2.5, 0.049999999999999996,
    "HC, H  in  CH3CH2-"}, "opls_431" -> {"CT", 0., 4.2, 0.3,
    "CT, C1 in  CH3CH2-"}, "opls_432" -> {"HC", 0.07, 2.5,
    0.049999999999999996, "HC, H1 in  CH3CH2-"},
  "opls_433" -> {"LP", -0.98, 0., 0., "LP, LP in  CH3CH2-"},
  "opls_434" -> {"OH", -1.3, 3.2, 0.25,
    "OH, O in OH-  Hyroxide O-H = 0.953 A"},
  "opls_435" -> {"HO", 0.3, 0., 0., "HO, H in OH-  JACS 108, 2517 (86)"},
  "opls_436" -> {"U", 2.5, 2.81524, 0.39999999999999997,
    "U, U in UO2+ J Mol Struct 366, 55 (96)"},
  "opls_437" -> {"OU", -0.25, 3.11815, 0.19999999999999998,
    "OU, O in UO2+ r(U-O) = 1.80 A"}, "opls_440" ->
   {"P", 1.62, 3.74, 0.19999999999999998, "P, P in  Me2PO4-, Me2PO4H"},
  "opls_441" -> {"O2", -0.92, 3.15, 0.19999999999999998,
    "O2, O= in Me2PO4-, Me2PO4H"}, "opls_442" ->
   {"OS", -0.6, 2.9, 0.13999999999999999,
    "OS, OMe in Me2PO4-, Me2PO4H   dimethylphosphate"},
  "opls_443" -> {"CT", 0.3, 3.5, 0.066,
    "CT, C  in Me2PO4-, Me2PO4H   dimetylphosphate"},
  "opls_444" -> {"HC", -0.03, 2.5, 0.029999999999999995,
    "HC, H  in Me2PO4-, Me2PO4H    6-31+G* CHELPG"},
  "opls_445" -> {"P", 1.92, 3.74, 0.19999999999999998,
    "P, P  in MeOPO3--, MeOPO3H2"}, "opls_446" ->
   {"O2", -1.12, 3.15, 0.19999999999999998, "O2, O= in MeOPO3--, MeOPO3H2"},
  "opls_447" -> {"OS", -0.7, 2.9, 0.13999999999999999,
    "OS, OMe in MeOPO3--, MeOPO3H2  methyl phosphate"},
  "opls_448" -> {"CT", 0.44, 3.5, 0.066,
    "CT, C  in MeOPO3--, MeOPO3H2   6-31+G* CHELPG"},
  "opls_449" -> {"HC", -0.1, 2.5, 0.029999999999999995,
    "HC, H  in MeOPO3--, MeOPO3H2"}, "opls_450" ->
   {"P", 1.62, 3.74, 0.19999999999999998, "P, P  in MePO3Me-, MePO3HMe"},
  "opls_451" -> {"O2", -0.97, 3.15, 0.19999999999999998,
    "O2, O= in MePO3Me-, MePO3HMe"}, "opls_452" ->
   {"OS", -0.63, 2.9, 0.13999999999999999,
    "OS, OMe in MePO3Me-, MePO3HMe     methyl"},
  "opls_453" -> {"CT", 0.28, 3.5, 0.066,
    "CT, C(O) MePO3Me-, MePO3HMe     methylphosphonate"},
  "opls_454" -> {"HC", -0.02, 2.5, 0.029999999999999995,
    "HC, H(CO) MePO3Me-, MePO3HMe     6-31+G* CHELPG"},
  "opls_455" -> {"CT", -0.51, 3.5, 0.066, "CT, C(P) MePO3Me-, MePO3HMe"},
  "opls_456" -> {"HC", 0.08, 2.5, 0.029999999999999995,
    "HC, H(CP) MePO3Me-, MePO3HMe"}, "opls_457" ->
   {"CA", -0.14, 3.55, 0.06999999999999999,
    "CA, Cipso  benzyl methylphosphonate"},
  "opls_458" -> {"CT", 0.32, 3.5, 0.066,
    "CT, C(O) benzyl methylphosphonate"},
  "opls_459" -> {"HC", 0.02, 2.5, 0.029999999999999995,
    "HC, H(CO) benzyl methylphosphonate"},
  "opls_460" -> {"CA", -0.04, 3.55, 0.06999999999999999,
    "CA, Cipso  methyl benzylphosphonate"},
  "opls_461" -> {"CT", -0.47, 3.5, 0.066,
    "CT, C(P)  methyl benzylphosphonate"},
  "opls_462" -> {"HC", 0.12, 2.5, 0.029999999999999995,
    "HC, H(CP) methyl benzylphosphonate"},
  "opls_463" -> {"CA", 0.14, 3.55, 0.06999999999999999,
    "CA, Cipso C6H5OPO3(2-)  use with #445-#447"},
  "opls_465" -> {"C_2", 0.51, 3.75, 0.105,
    "C_2, AA C:   esters - for R on C=O, use #280-#282"},
  "opls_466" -> {"O_2", -0.43, 2.96, 0.21, "O_2, AA =O:  esters"},
  "opls_467" -> {"OS", -0.33, 3., 0.16999999999999998, "OS, AA -OR: ester"},
  "opls_468" -> {"CT", 0.16, 3.5, 0.066,
    "CT, methoxy C in esters - see also #490-#492"},
  "opls_469" -> {"HC", 0.03, 2.42, 0.014999999999999998,
    "HC, methoxy Hs in esters"}, "opls_470" -> {"C", 0.635, 3.75, 0.105,
    "C, Co in benzoic acid"}, "opls_471" -> {"C", 0.625, 3.75, 0.105,
    "C, Co in methyl benzoate, aryl ester"},
  "opls_472" -> {"CA", 0.135, 3.55, 0.06999999999999999,
    "CA, Cipso phenyl ester"}, "opls_473" -> {"OS", -0.21, 3.,
    0.16999999999999998, "OS, AA -OR phenyl ester"},
  "opls_474" -> {"SY", 1.48, 3.55, 0.25, "SY, S in sulfonamide, S(=O)2(OR)"},
  "opls_475" -> {"OY", -0.68, 2.96, 0.16999999999999998,
    "OY, O in sulfonamide, S(=O)2(OR)"},
  "opls_476" -> {"CT", -0.54, 3.5, 0.066,
    "CT, CH3 attached to S of sulfonamide"},
  "opls_477" -> {"HC", 0.18, 2.5, 0.029999999999999995,
    "HC, H of Me attached to S of sulfonamide"},
  "opls_478" -> {"N", -1., 3.25, 0.16999999999999998,
    "N, N: primary amide of sulfonamide"},
  "opls_479" -> {"H", 0.44, 0., 0., "H, H on N: primary sulfonamide"},
  "opls_480" -> {"N", -0.8, 3.25, 0.16999999999999998,
    "N, N secondary amide of sulfonamide"},
  "opls_481" -> {"H", 0.41, 0., 0., "H, H on N: secondary sulfonamide"},
  "opls_482" -> {"CT", 0.18, 3.5, 0.066, "CT, alpha CH3-N of sulfonamide"},
  "opls_483" -> {"HC", 0.03, 2.5, 0.029999999999999995,
    "HC, H of alpha CH3-N of sulfonamide"},
  "opls_484" -> {"CT", 0.39, 3.5, 0.066, "CT, alpha CH2-N of sulfonamide. Use \
q=0.45 for CRH-N, q=0.65 for O=N-C-CH-N."},
  "opls_485" -> {"HC", -0.06, 2.5, 0.029999999999999995,
    "HC, H of alpha CH2-N of sulfonamide"},
  "opls_486" -> {"CT", -0.18, 3.5, 0.066,
    "CT, beta CH3 of N-ethyl sulfonamide"},
  "opls_487" -> {"HC", 0.06, 2.5, 0.029999999999999995,
    "HC, H of beta CH3 of N-ethyl sulfonamide"},
  "opls_488" -> {"CA", 0., 3.55, 0.06999999999999999,
    "CA, benzene C attached to S of sulfonamide"},
  "opls_490" -> {"CT", 0.19, 3.5, 0.066, "CT, C(H2OS) ethyl ester"},
  "opls_491" -> {"CT", 0.22, 3.5, 0.066, "CT, C(HOS) i-pr ester"},
  "opls_492" -> {"CT", 0.25, 3.5, 0.066, "CT, C(OS) t-bu ester"},
  "opls_493" -> {"SY2", 1.374, 3.55, 0.25, "SY2, S in sulfone"},
  "opls_494" -> {"OY", -0.68, 2.96, 0.16999999999999998, "OY, O in sulfone"},
  "opls_496" -> {"SZ", 0.13, 3.5599999999999996, 0.39499999999999996,
    "SZ, sulfoxide - all atom"}, "opls_497" ->
   {"OY", -0.42, 2.9299999999999997, 0.27999999999999997,
    "OY, sulfoxide - all atom"}, "opls_498" -> {"CT", -0.03, 3.5, 0.066,
    "CT, CH3 all-atom C: sulfoxide"}, "opls_499" ->
   {"CT", 0.025, 3.5, 0.066, "CT, CH2 all-atom C: sulfoxide"},
  "opls_500" -> {"C*", 0.075, 3.55, 0.06999999999999999, "C*, CG in Trp"},
  "opls_501" -> {"CB", -0.05, 3.55, 0.06999999999999999, "CB, CD C in Trp"},
  "opls_502" -> {"CN", 0.13, 3.55, 0.06999999999999999, "CN, CE C in Trp"},
  "opls_503" -> {"NA", -0.57, 3.25, 0.16999999999999998, "NA, NE in Trp"},
  "opls_504" -> {"H", 0.42, 0., 0., "H, H on NE in Trp"},
  "opls_505" -> {"CT", 0., 3.5, 0.066, "CT, CB in His"},
  "opls_506" -> {"CR", 0.182, 3.55, 0.06999999999999999,
    "CR, CE1 in HID, HIE"}, "opls_507" -> {"CV", 0.504, 3.55,
    0.06999999999999999, "CV, CD2 in HID, CG in HIE"},
  "opls_508" -> {"CW", -0.26, 3.55, 0.06999999999999999,
    "CW, CG in HID, CD2 in HIE"}, "opls_509" ->
   {"CR", 0.385, 3.55, 0.06999999999999999, "CR, CE1 in HIP"},
  "opls_510" -> {"CX", 0.215, 3.55, 0.06999999999999999,
    "CX, CG, CD2 in HIP"}, "opls_511" -> {"NB", -0.56, 3.25,
    0.16999999999999998, "NB, NE in HID, ND in HIE"},
  "opls_512" -> {"NA", -0.54, 3.25, 0.16999999999999998, "NA, N in HIP"},
  "opls_513" -> {"H", 0.46, 0., 0., "H, H on N in HIP"},
  "opls_514" -> {"CW", -0.11, 3.55, 0.06999999999999999, "CW, CD1 in TRP"},
  "opls_515" -> {"CT", 0.055, 3.5, 0.066,
    "CT, all-atom C: CH, isopropyl benzene"},
  "opls_516" -> {"CT", 0.115, 3.5, 0.066,
    "CT, all-atom C: C,  t-butyl benzene"},
  "opls_517" -> {"CM", -0.03, 3.55, 0.076, "CM, vinyl ether HCOR"},
  "opls_518" -> {"CM", 0.085, 3.55, 0.076, "CM, vinyl ether RCOR"},
  "opls_520" -> {"NC", -0.67, 3.25, 0.16999999999999998,
    "NC, N   in pyridine 6-31G*"}, "opls_521" ->
   {"CA", 0.473, 3.55, 0.06999999999999999, "CA, C1  in pyridine CHELPG"},
  "opls_522" -> {"CA", -0.44, 3.55, 0.06999999999999999,
    "CA, C2  in pyridine charges"}, "opls_523" ->
   {"CA", 0.227, 3.55, 0.06999999999999999, "CA, C3  in pyridine for"},
  "opls_524" -> {"HA", 0.012, 2.42, 0.029999999999999995,
    "HA, H1  in pyridine 520-619"}, "opls_525" ->
   {"HA", 0.155, 2.42, 0.029999999999999995, "HA, H2  in pyridine"},
  "opls_526" -> {"HA", 0.065, 2.42, 0.029999999999999995,
    "HA, H3  in pyridine"}, "opls_527" -> {"NC", -0.46, 3.25,
    0.16999999999999998, "NC, N   in pyrazine"},
  "opls_528" -> {"CA", 0.192, 3.55, 0.06999999999999999,
    "CA, C   in pyrazine"}, "opls_529" -> {"HA", 0.042, 2.42,
    0.029999999999999995, "HA, H   in pyrazine"},
  "opls_530" -> {"NC", -0.83, 3.25, 0.16999999999999998,
    "NC, N   in pyrimidine"}, "opls_531" -> {"CQ", 0.874, 3.55,
    0.06999999999999999, "CQ, C2  in pyrimidine"},
  "opls_532" -> {"CA", 0.653, 3.55, 0.06999999999999999,
    "CA, C4  in pyrimidine"}, "opls_533" -> {"CA", -0.68, 3.55,
    0.06999999999999999, "CA, C5  in pyrimidine"},
  "opls_534" -> {"HA", -0.03, 2.42, 0.029999999999999995,
    "HA, H2  in pyrimidine"}, "opls_535" -> {"HA", 0.011, 2.42,
    0.029999999999999995, "HA, H4  in pyrimidine"},
  "opls_536" -> {"HA", 0.197, 2.42, 0.029999999999999995,
    "HA, H5  in pyrimidine"}, "opls_537" -> {"NC", -0.33, 3.25,
    0.16999999999999998, "NC, N   in pyridazine"},
  "opls_538" -> {"CA", 0.378, 3.55, 0.06999999999999999,
    "CA, C3  in pyridazine"}, "opls_539" -> {"CA", -0.16, 3.55,
    0.06999999999999999, "CA, C4  in pyridazine"},
  "opls_540" -> {"HA", 0., 2.42, 0.029999999999999995,
    "HA, H3  in pyridazine"}, "opls_541" -> {"HA", 0.122, 2.42,
    0.029999999999999995, "HA, H4  in pyridazine"},
  "opls_542" -> {"NA", -0.23, 3.25, 0.16999999999999998,
    "NA, N   in pyrrole"}, "opls_543" -> {"CW", -0.16, 3.55,
    0.06999999999999999, "CW, C2  in pyrrole"},
  "opls_544" -> {"CS", -0.14, 3.55, 0.06999999999999999,
    "CS, C3  in pyrrole"}, "opls_545" -> {"H", 0.317, 0., 0.,
    "H, H1  in pyrrole"}, "opls_546" -> {"HA", 0.155, 2.42,
    0.029999999999999995, "HA, H2  in pyrrole"},
  "opls_547" -> {"HA", 0.118, 2.42, 0.029999999999999995,
    "HA, H3  in pyrrole"}, "opls_548" -> {"NA", -0.05, 3.25,
    0.16999999999999998, "NA, N1  in pyrazole"},
  "opls_549" -> {"NB", -0.49, 3.25, 0.16999999999999998,
    "NB, N2  in pyrazole"}, "opls_550" -> {"CU", 0.246, 3.55,
    0.06999999999999999, "CU, C3  in pyrazole"},
  "opls_551" -> {"CA", -0.32, 3.55, 0.06999999999999999,
    "CA, C4  in pyrazole"}, "opls_552" -> {"CW", -0.03, 3.55,
    0.06999999999999999, "CW, C5  in pyrazole"},
  "opls_553" -> {"H", 0.301, 0., 0., "H, H1  in pyrazole"},
  "opls_554" -> {"HA", 0.072, 2.42, 0.029999999999999995,
    "HA, H3  in pyrazole"}, "opls_555" -> {"HA", 0.15, 2.42,
    0.029999999999999995, "HA, H4  in pyrazole"},
  "opls_556" -> {"HA", 0.135, 2.42, 0.029999999999999995,
    "HA, H5  in pyrazole"}, "opls_557" -> {"NA", -0.25, 3.25,
    0.16999999999999998, "NA, N1  in imidazole"},
  "opls_558" -> {"CR", 0.275, 3.55, 0.06999999999999999,
    "CR, C2  in imidazole"}, "opls_559" -> {"NB", -0.56, 3.25,
    0.16999999999999998, "NB, N3  in imidazole"},
  "opls_560" -> {"CV", 0.185, 3.55, 0.06999999999999999,
    "CV, C4  in imidazole"}, "opls_561" -> {"CW", -0.28, 3.55,
    0.06999999999999999, "CW, C5  in imidazole"},
  "opls_562" -> {"H", 0.306, 0., 0., "H, H1  in imidazole"},
  "opls_563" -> {"HA", 0.078, 2.42, 0.029999999999999995,
    "HA, H2  in imidazole"}, "opls_564" -> {"HA", 0.075, 2.42,
    0.029999999999999995, "HA, H4  in imidazole"},
  "opls_565" -> {"HA", 0.187, 2.42, 0.029999999999999995,
    "HA, H5  in imidazole"}, "opls_566" -> {"OS", -0.19, 2.9,
    0.13999999999999999, "OS, O   in furan"},
  "opls_567" -> {"CW", -0.01, 3.55, 0.06999999999999999, "CW, C2  in furan"},
  "opls_568" -> {"CS", -0.15, 3.55, 0.076, "CS, C3  in furan"},
  "opls_569" -> {"HA", 0.142, 2.42, 0.029999999999999995,
    "HA, H2  in furan"}, "opls_570" -> {"HA", 0.126, 2.42,
    0.029999999999999995, "HA, H3  in furan"},
  "opls_571" -> {"OS", -0.25, 2.9, 0.13999999999999999,
    "OS, O   in oxazole"}, "opls_572" -> {"CR", 0.511, 3.55,
    0.06999999999999999, "CR, C2  in oxazole"},
  "opls_573" -> {"NB", -0.59, 3.25, 0.16999999999999998,
    "NB, N   in oxazole"}, "opls_574" -> {"CV", 0.169, 3.55,
    0.06999999999999999, "CV, C4  in oxazole"},
  "opls_575" -> {"CW", -0.14, 3.55, 0.06999999999999999,
    "CW, C5  in oxazole"}, "opls_576" -> {"HA", 0.043, 2.42,
    0.029999999999999995, "HA, H2  in oxazole"},
  "opls_577" -> {"HA", 0.091, 2.42, 0.029999999999999995,
    "HA, H4  in oxazole"}, "opls_578" -> {"HA", 0.181, 2.42,
    0.029999999999999995, "HA, H5  in oxazole"},
  "opls_579" -> {"OS", -0.12, 2.9, 0.13999999999999999,
    "OS, O   in isoxazole"}, "opls_580" -> {"NB", -0.41, 3.25,
    0.16999999999999998, "NB, N   in isoxazole"},
  "opls_581" -> {"CU", 0.405, 3.55, 0.06999999999999999,
    "CU, C3  in isoxazole"}, "opls_582" -> {"CA", -0.45, 3.55,
    0.06999999999999999, "CA, C4  in isoxazole"},
  "opls_583" -> {"CW", 0.25, 3.55, 0.06999999999999999,
    "CW, C5  in isoxazole"}, "opls_584" -> {"HA", 0.053, 2.42,
    0.029999999999999995, "HA, H3  in isoxazole"},
  "opls_585" -> {"HA", 0.184, 2.42, 0.029999999999999995,
    "HA, H4  in isoxazole"}, "opls_586" -> {"HA", 0.098, 2.42,
    0.029999999999999995, "HA, H5  in isoxazole"},
  "opls_587" -> {"NA", -0.5, 3.25, 0.16999999999999998, "NA, N1  in indole"},
  "opls_588" -> {"CW", 0.001, 3.55, 0.06999999999999999,
    "CW, C2  in indole"}, "opls_589" -> {"CS", -0.39, 3.55,
    0.06999999999999999, "CS, C3  in indole"},
  "opls_590" -> {"CA", -0.27, 3.55, 0.06999999999999999,
    "CA, C4  in indole"}, "opls_591" -> {"CA", -0.12, 3.55,
    0.06999999999999999, "CA, C5  in indole"},
  "opls_592" -> {"CA", -0.1, 3.55, 0.06999999999999999, "CA, C6  in indole"},
  "opls_593" -> {"CA", -0.25, 3.55, 0.06999999999999999,
    "CA, C7  in indole"}, "opls_594" -> {"CN", 0.22, 3.55,
    0.06999999999999999, "CN, C8  in indole"},
  "opls_595" -> {"CB", 0.225, 3.55, 0.06999999999999999,
    "CB, C9  in indole"}, "opls_596" -> {"H", 0.376, 0., 0.,
    "H, H1  in indole"}, "opls_597" -> {"HA", 0.147, 2.42,
    0.029999999999999995, "HA, H2  in indole"},
  "opls_598" -> {"HA", 0.172, 2.42, 0.029999999999999995,
    "HA, H3  in indole"}, "opls_599" -> {"HA", 0.155, 2.42,
    0.029999999999999995, "HA, H4  in indole"},
  "opls_600" -> {"HA", 0.107, 2.42, 0.029999999999999995,
    "HA, H5  in indole"}, "opls_601" -> {"HA", 0.11, 2.42,
    0.029999999999999995, "HA, H6  in indole"},
  "opls_602" -> {"HA", 0.14, 2.42, 0.029999999999999995,
    "HA, H7  in indole"}, "opls_603" -> {"NC", -0.69, 3.25,
    0.16999999999999998, "NC, N1  in quinoline"},
  "opls_604" -> {"CA", 0.425, 3.55, 0.06999999999999999,
    "CA, C2  in quinoline"}, "opls_605" -> {"CA", -0.35, 3.55,
    0.06999999999999999, "CA, C3  in quinoline"},
  "opls_606" -> {"CA", 0., 3.55, 0.06999999999999999,
    "CA, C4  in quinoline"}, "opls_607" -> {"CA", -0.19, 3.55,
    0.06999999999999999, "CA, C5  in quinoline"},
  "opls_608" -> {"CA", -0.11, 3.55, 0.06999999999999999,
    "CA, C6  in quinoline"}, "opls_609" -> {"CA", -0.07, 3.55,
    0.06999999999999999, "CA, C7  in quinoline"},
  "opls_610" -> {"CA", -0.3, 3.55, 0.06999999999999999,
    "CA, C8  in quinoline"}, "opls_611" -> {"CA", 0.563, 3.55,
    0.06999999999999999, "CA, C9  in quinoline"},
  "opls_612" -> {"CA", -0.05, 3.55, 0.06999999999999999,
    "CA, C10 in quinoline"}, "opls_613" -> {"HA", 0.028, 2.42,
    0.029999999999999995, "HA, H2  in quinoline"},
  "opls_614" -> {"HA", 0.146, 2.42, 0.029999999999999995,
    "HA, H3  in quinoline"}, "opls_615" -> {"HA", 0.119, 2.42,
    0.029999999999999995, "HA, H4  in quinoline"},
  "opls_616" -> {"HA", 0.133, 2.42, 0.029999999999999995,
    "HA, H5  in quinoline"}, "opls_617" -> {"HA", 0.113, 2.42,
    0.029999999999999995, "HA, H6  in quinoline"},
  "opls_618" -> {"HA", 0.114, 2.42, 0.029999999999999995,
    "HA, H7  in quinoline"}, "opls_619" -> {"HA", 0.157, 2.42,
    0.029999999999999995, "HA, H8  in quinoline"},
  "opls_620" -> {"NC", -0.76, 3.25, 0.16999999999999998,
    "NC, N1  in purine"}, "opls_621" -> {"CQ", 0.679, 3.55,
    0.06999999999999999, "CQ, C2  in purine"},
  "opls_622" -> {"NC", -0.78, 3.25, 0.16999999999999998,
    "NC, N3  in purine"}, "opls_623" -> {"CB", 0.736, 3.55,
    0.06999999999999999, "CB, C4  in purine"},
  "opls_624" -> {"CB", 0.038, 3.55, 0.06999999999999999,
    "CB, C5  in purine"}, "opls_625" -> {"CA", 0.343, 3.55,
    0.06999999999999999, "CA, C6  in purine"},
  "opls_626" -> {"NB", -0.64, 3.25, 0.16999999999999998,
    "NB, N7  in purine"}, "opls_627" -> {"CK", 0.452, 3.55,
    0.06999999999999999, "CK, C8  in purine"},
  "opls_628" -> {"NA", -0.68, 3.25, 0.16999999999999998,
    "NA, N9  in purine"}, "opls_629" -> {"HA", 0.024, 2.42,
    0.029999999999999995, "HA, H2  in purine"},
  "opls_630" -> {"HA", 0.101, 2.42, 0.029999999999999995,
    "HA, H6  in purine"}, "opls_631" -> {"HA", 0.086, 2.42,
    0.029999999999999995, "HA, H8  in purine"},
  "opls_632" -> {"H", 0.413, 0., 0., "H, H9  in purine"},
  "opls_633" -> {"S", -0.03, 3.55, 0.25, "S, S   in thiazole"},
  "opls_634" -> {"CR", 0.242, 3.55, 0.06999999999999999,
    "CR, C2  in thiazole"}, "opls_635" -> {"NB", -0.51, 3.25,
    0.16999999999999998, "NB, N   in thiazole"},
  "opls_636" -> {"CV", 0.228, 3.55, 0.06999999999999999,
    "CV, C4  in thiazole"}, "opls_637" -> {"CW", -0.29, 3.55,
    0.06999999999999999, "CW, C5  in thiazole"},
  "opls_638" -> {"HA", 0.101, 2.42, 0.029999999999999995,
    "HA, H2  in thiazole"}, "opls_639" -> {"HA", 0.068, 2.42,
    0.029999999999999995, "HA, H4  in thiazole"},
  "opls_640" -> {"HA", 0.205, 2.42, 0.029999999999999995,
    "HA, H5  in thiazole"}, "opls_641" -> {"NC", -0.95, 3.25,
    0.16999999999999998, "NC, N   in 1,3,5-triazine"},
  "opls_642" -> {"CQ", 0.965, 3.55, 0.06999999999999999,
    "CQ, C   in 1,3,5-triazine"}, "opls_643" ->
   {"HA", -0.01, 2.42, 0.029999999999999995, "HA, H   in 1,3,5-triazine"},
  "opls_644" -> {"CA", 0.13, 3.55, 0.06999999999999999,
    "CA, C5  in serotonin"}, "opls_645" -> {"CT", 0.052, 3.5, 0.066,
    "CT, C on C3 in serotonin"}, "opls_646" -> {"NC", -0.59, 3.25,
    0.16999999999999998, "NC, N1,N10   in 1,10-phenanthroline"},
  "opls_647" -> {"CA", 0.392, 3.55, 0.06999999999999999,
    "CA, C2,C9  in 1,10-phenanthroline"},
  "opls_648" -> {"CA", -0.34, 3.55, 0.06999999999999999,
    "CA, C3,C8  in 1,10-phenanthroline"},
  "opls_649" -> {"CA", 0.02, 3.55, 0.06999999999999999,
    "CA, C4,C7  in 1,10-phenanthroline"},
  "opls_650" -> {"CA", -0.04, 3.55, 0.06999999999999999,
    "CA, C12,C14 in 1,10-phenanthroline"},
  "opls_651" -> {"CA", 0.347, 3.55, 0.06999999999999999,
    "CA, C11,C13 in 1,10-phenanthroline"},
  "opls_652" -> {"CA", -0.19, 3.55, 0.06999999999999999,
    "CA, C5  in 1,10-phenanthroline"},
  "opls_653" -> {"HA", 0.032, 2.42, 0.029999999999999995,
    "HA, H2,H9  in 1,10-phenanthroline"},
  "opls_654" -> {"HA", 0.146, 2.42, 0.029999999999999995,
    "HA, H3,H8  in 1,10-phenanthroline"},
  "opls_655" -> {"HA", 0.108, 2.42, 0.029999999999999995,
    "HA, H4,H7  in 1,10-phenanthroline"},
  "opls_656" -> {"HA", 0.14, 2.42, 0.029999999999999995,
    "HA, H5,H6  in 1,10-phenanthroline"},
  "opls_670" -> {"CT", -0.16, 3.5, 0.066, "CT, CH3, 2-methyl pyridine"},
  "opls_671" -> {"CT", -0.1, 3.5, 0.066, "CT, CH2, 2-ethyl pyridine"},
  "opls_672" -> {"CT", -0.18, 3.5, 0.066, "CT, CH3, 3-methyl pyridazine"},
  "opls_673" -> {"CT", -0.12, 3.5, 0.066, "CT, CH2, 3-ethyl pyridazine"},
  "opls_674" -> {"CT", -0.16, 3.5, 0.066, "CT, CH3, 4-methyl pyrimidine"},
  "opls_675" -> {"CT", -0.1, 3.5, 0.066, "CT, CH2, 4-ethyl pyrimidine"},
  "opls_676" -> {"CT", -0.13, 3.5, 0.066, "CT, CH3, 2-methyl pyrazine"},
  "opls_677" -> {"CT", -0.07, 3.5, 0.066, "CT, CH2, 2-ethyl pyrazine"},
  "opls_678" -> {"CT", -0.02, 3.5, 0.066, "CT, CH3, 2-methyl pyrrole"},
  "opls_679" -> {"CT", 0.035, 3.5, 0.066, "CT, CH2, 2-ethyl pyrrole"},
  "opls_680" -> {"CT", -0.03, 3.5, 0.066, "CT, CH3, 2-methyl furan"},
  "opls_681" -> {"CT", 0.022, 3.5, 0.066, "CT, CH2, 2-ethyl furan"},
  "opls_697" -> {"Ac3+", 3., 3.473, 0.054, "Ac3+, Ac+3 Actinide params -"},
  "opls_698" -> {"Th4+", 0, 33.*A, -0.5, "Th4+, Th+4"},
  "opls_699" -> {"Am3+", 0, 33.*A, -0.5, "Am3+, Am+3 F. van Veggel"},
  "opls_700" -> {"C+", 0.619, 3.55, 0.076,
    "C+, C+  in t-butyl+ B3LYP/6-31G*"},
  "opls_701" -> {"CT", -0.39, 3.5, 0.066, "CT, C   in t-butyl+   charges"},
  "opls_702" -> {"HC", 0.174, 2.5, 0.029999999999999995,
    "HC, H   in t-butyl+"}, "opls_703" -> {"La3+", 3., 3.75,
    0.05999999999999999, "La3+, La+3"},
  "opls_704" -> {"Nd3+", 3., 3.473, 0.054, "Nd3+, Nd+3 Lanthanide params -"},
  "opls_705" -> {"Eu3+", 0, 33.*A, -0.5,
    "Eu3+, Eu+3 F. van Veggel, Chem Eur J 5, 90 (1999)."},
  "opls_706" -> {"Gd3+", 0, 33.*A, -0.5, "Gd3+, Gd+3"},
  "opls_707" -> {"Yb3+", 3., 2.9499999999999997, 0.04,
    "Yb3+, Yb+3 see also JPC-A 104, 7659 (2000)"},
  "opls_708" -> {"CM", -0.34, 3.55, 0.076, "CM, C  in Cl..CH3..Cl- TS"},
  "opls_709" -> {"Cl-", -0.62, 3.4000000000000004, 0.3,
    "Cl-, Cl charges: JACS 117,2024 (95)"},
  "opls_710" -> {"HC", 0.2, 2.42, 0.029999999999999995,
    "HC, H  in Cl..CH3..Cl- TS"}, "opls_711" -> {"CY", -0.12, 3.5, 0.066,
    "CY, CH2    C: cyclopropane"}, "opls_712" -> {"CY", -0.06, 3.5, 0.066,
    "CY, CHR    C: cyclopropane"}, "opls_713" ->
   {"CY", 0., 3.5, 0.066, "CY, CR2    C: cyclopropane"},
  "opls_714" -> {"CA", -0.23, 3.55, 0.06999999999999999,
    "CA, C in C5H5- cyclopentadienyl anion"},
  "opls_715" -> {"HA", 0.03, 2.42, 0.029999999999999995,
    "HA, H in C5H5- cyclopentadienyl anion"},
  "opls_716" -> {"CA", -0.09, 3.55, 0.06999999999999999,
    "CA, C in C5H5  cyclopentadienyl radical"},
  "opls_717" -> {"HA", 0.099, 2.42, 0.029999999999999995,
    "HA, H in C5H5  cyclopentadienyl radical"},
  "opls_718" -> {"CA", 0.22, 3.55, 0.06999999999999999,
    "CA, C(F)  fluorobenzene"}, "opls_719" ->
   {"F", -0.22, 2.8499999999999996, 0.061, "F, F     fluorobenzene"},
  "opls_720" -> {"CA", 0.13, 3.55, 0.06999999999999999,
    "CA, C(F)  hexafluorobenzene"}, "opls_721" ->
   {"F", -0.13, 2.8499999999999996, 0.061, "F, F     hexafluorobenzene"},
  "opls_722" -> {"Br", -0.22, 3.4699999999999998, 0.47,
    "Br, Br    alkyl bromide (UA, but probably ok for AA)"},
  "opls_724" -> {"CA", 0.15, 3.55, 0.06999999999999999,
    "CA, C(CF3) trifluoromethylbenzene"},
  "opls_725" -> {"CT", 0.45, 3.25, 0.062000000000000006,
    "CT, CF3   trifluoromethylbenzene"},
  "opls_726" -> {"F", -0.2, 2.94, 0.061, "F, F     trifluoromethylbenzene"},
  "opls_727" -> {"CA", 0.2, 3.55, 0.06999999999999999,
    "CA, C(F)  difluorobenzenes"}, "opls_728" ->
   {"F", -0.2, 2.8499999999999996, 0.061, "F, F     difluorobenzenes"},
  "opls_729" -> {"CA", 0.2, 3.55, 0.06999999999999999,
    "CA, C(Br) bromobenzene"}, "opls_730" -> {"Br", -0.2, 3.4699999999999998,
    0.47, "Br, Br    bromobenzene"}, "opls_731" ->
   {"CA", 0.1, 3.55, 0.06999999999999999,
    "CA, C(I)  iodobenzene - tentative"},
  "opls_732" -> {"I", -0.1, 3.67, 0.58, "I, I     iodobenzene - tentative"},
  "opls_733" -> {"CY", 0.055, 3.5, 0.066,
    "CY, all-atom C: CH, cyclopropyl benzene"},
  "opls_734" -> {"SH", -0.22, 3.55, 0.25,
    "SH, all-atom S: thiophenol (HS is #204)"},
  "opls_735" -> {"CA", 0.065, 3.55, 0.06999999999999999,
    "CA, C(S)  thiophenol"}, "opls_736" -> {"CA", 0.013, 3.55,
    0.06999999999999999, "CA, CG of Benzamidine"},
  "opls_737" -> {"CA", -0.1, 3.55, 0.06999999999999999,
    "CA, CD of Benzamidine"}, "opls_738" -> {"CA", -0.09, 3.55,
    0.06999999999999999, "CA, CE of Benzamidine"},
  "opls_739" -> {"CA", -0.11, 3.55, 0.06999999999999999,
    "CA, CZ of Benzamidine"}, "opls_740" -> {"HA", 0.141, 2.42,
    0.029999999999999995, "HA, HD of Benzamidine"},
  "opls_741" -> {"HA", 0.129, 2.42, 0.029999999999999995,
    "HA, HE of Benzamidine"}, "opls_742" -> {"CA", 0.827, 2.25,
    0.049999999999999996, "CA, C+ of Benzamidine"},
  "opls_743" -> {"N2", -0.88, 3.25, 0.16999999999999998,
    "N2, N-H2 of Benzamidine"}, "opls_744" -> {"H", 0.426, 0., 0.,
    "H, H1-N of Benzamidine"}, "opls_745" -> {"H", 0.465, 0., 0.,
    "H, H2-N of Benzamidine"}, "opls_746" -> {"HA", 0.119, 2.42,
    0.029999999999999995, "HA, H-CG of Benzamidine"},
  "opls_747" -> {"CT", -0.02, 3.5, 0.066, "CT, CH3 in neutral MeGDN"},
  "opls_748" -> {"CT", 0.04, 3.5, 0.066, "CT, CD of neutral ARG"},
  "opls_749" -> {"NY", -0.62, 3.25, 0.16999999999999998,
    "NY, NE of neutral ARG"}, "opls_750" -> {"NZ", -0.78, 3.25,
    0.16999999999999998, "NZ, N1 of neutral ARG (HN=CZ)"},
  "opls_751" -> {"NY", -0.78, 3.25, 0.16999999999999998,
    "NY, N2 of neutral ARG (H2N-CZ)"},
  "opls_752" -> {"CA", 0.55, 2.25, 0.049999999999999996,
    "CA, CZ of neutral ARG"}, "opls_753" -> {"NZ", -0.56, 3.2,
    0.16999999999999998, "NZ, N IN RCN  nitriles"},
  "opls_754" -> {"CZ", 0.46, 3.3000000000000003, 0.066,
    "CZ, C IN RCN  nitriles"}, "opls_755" ->
   {"CT", -0.08, 3.3000000000000003, 0.066, "CT, C of CH3 in  CH3CN"},
  "opls_756" -> {"CT", -0.02, 3.3000000000000003, 0.066,
    "CT, C of CH2 in RCH2CN"}, "opls_757" -> {"CT", 0.04, 3.3000000000000003,
    0.066, "CT, C of CH  in R2CHCN"}, "opls_758" ->
   {"CT", 0.1, 3.3000000000000003, 0.066, "CT, C of C   in R3CCN"},
  "opls_759" -> {"HC", 0.06, 2.5, 0.014999999999999998,
    "HC, HC-CT-CN alpha-H in nitriles"},
  "opls_760" -> {"NO", 0.54, 3.25, 0.11999999999999998,
    "NO, N in nitro R-NO2"}, "opls_761" -> {"ON", -0.37, 2.96,
    0.16999999999999998, "ON, O in nitro R-NO2"},
  "opls_762" -> {"CT", 0.02, 3.5, 0.066, "CT, CT-NO2 nitromethane"},
  "opls_763" -> {"HC", 0.06, 2.5, 0.014999999999999998,
    "HC, HC-CT-NO2 alpha-H in nitroalkanes"},
  "opls_764" -> {"CT", 0.08, 3.5, 0.066, "CT, CT-NO2 nitroethane"},
  "opls_765" -> {"CT", 0.14, 3.5, 0.066, "CT, CT-NO2 2-nitropropane"},
  "opls_766" -> {"CT", 0.2, 3.5, 0.066,
    "CT, CT-NO2 2-methyl-2-nitropropane"},
  "opls_767" -> {"NO", 0.65, 3.25, 0.11999999999999998,
    "NO, N in nitro Ar-NO2"}, "opls_768" -> {"CA", 0.09, 3.55,
    0.06999999999999999, "CA, C(NO2) nitrobenzene"},
  "opls_771" -> {"O", -0.5, 2.96, 0.21,
    "O, propylene carbonate O (Luciennes param.)"},
  "opls_772" -> {"C", 0.86, 3.75, 0.105, "C, propylene carbonate C=O"},
  "opls_773" -> {"OS", -0.45, 3., 0.16999999999999998,
    "OS, propylene carbonate OS"}, "opls_774" ->
   {"CT", 0.21, 3.5, 0.066, "CT, propylene carbonate C in CH2"},
  "opls_775" -> {"CT", 0.16, 3.5, 0.066, "CT, propylene carbonate C in CH"},
  "opls_776" -> {"CT", -0.1, 3.5, 0.066, "CT, propylene carbonate C in CH3"},
  "opls_777" -> {"HC", 0.03, 2.42, 0.014999999999999998,
    "HC, propylene carbonate H in CH2"},
  "opls_778" -> {"HC", 0.03, 2.42, 0.014999999999999998,
    "HC, propylene carbonate H in CH"},
  "opls_779" -> {"HC", 0.06, 2.42, 0.014999999999999998,
    "HC, propylene carbonate H in CH3"},
  "opls_781" -> {"P+", 0.968, 3.74, 0.19999999999999998,
    "P+, phosphonium R4P+"}, "opls_782" -> {"CT", -0.5, 3.5, 0.066,
    "CT, CH3PR3+ 6-31G* CHELPG"}, "opls_783" -> {"CT", 0., 3.5, 0.066,
    "CT, RCH2PR3+"}, "opls_784" -> {"HC", 0.172, 2.5, 0.029999999999999995,
    "HC, H in CH3PR3+"}, "opls_785" -> {"P", 1.34, 3.74, 0.19999999999999998,
    "P, P in PF6-"}, "opls_786" -> {"F", -0.39, 3.1180999999999996, 0.061,
    "F, F in PF6-"}, "opls_787" -> {"N", 0.794, 3.15, 0.16999999999999998,
    "N, N in NO3-"}, "opls_788" -> {"O", -0.59, 2.86, 0.21, "O, O in NO3-"},
  "opls_795" -> {"OW", 0., 3.215, 0.13999999999999999, "OW, O TIP4F Water"},
  "opls_796" -> {"HW", 0.511, 0., 0., "HW, H TIP4F Water"},
  "opls_797" -> {"LP", -1.02, 0., 0., "LP, M TIP4F Water"},
  "opls_900" -> {"NT", -0.9, 3.3000000000000003, 0.16999999999999998,
    "NT, N primary   amines"}, "opls_901" ->
   {"NT", -0.78, 3.3000000000000003, 0.16999999999999998,
    "NT, N secondary amines, aziridine N1"},
  "opls_902" -> {"NT", -0.63, 3.3000000000000003, 0.16999999999999998,
    "NT, N tertiary  amines"}, "opls_903" -> {"CT", 0., 3.5, 0.066,
    "CT, CH3(N) primary   aliphatic amines, H(C) is #911"},
  "opls_904" -> {"CT", 0.02, 3.5, 0.066,
    "CT, CH3(N) secondary aliphatic amines, H(C) is #911"},
  "opls_905" -> {"CT", 0.03, 3.5, 0.066,
    "CT, CH3(N) tertiary  aliphatic amines, H(C) is #911"},
  "opls_906" -> {"CT", 0.06, 3.5, 0.066,
    "CT, CH2(N) primary   aliphatic amines, H(C) is #911"},
  "opls_906B" -> {"CT_2", 0.06, 3.5, 0.066,
    "CT_2, CA in GLY-NH2 N-terminus"},
  "opls_907" -> {"CT", 0.08, 3.5, 0.066,
    "CT, CH2(N) secondary aliphatic amines, aziridine  C2,C3H"},
  "opls_908" -> {"CT", 0.09, 3.5, 0.066,
    "CT, CH2(N) tertiary  aliphatic amines, H(C) is #911"},
  "opls_909" -> {"H", 0.36, 0., 0., "H, H(N)   primary   amines"},
  "opls_910" -> {"H", 0.38, 0., 0., "H, H(N)   secondary amines"},
  "opls_911" -> {"HC", 0.06, 2.5, 0.014999999999999998,
    "HC, H(C) for C bonded to N in amines, diamines (aziridine H2,H3)"},
  "opls_912" -> {"CT", 0.12, 3.5, 0.066,
    "CT, CH     primary isopropyl amine"},
  "opls_912B" -> {"CT_2", 0.12, 3.5, 0.066,
    "CT_2, CA in NH2 N-terminus. All AA except GLY, PRO"},
  "opls_913" -> {"CT", 0.18, 3.5, 0.066, "CT, C      primary t-butyl amine"},
  "opls_914" -> {"CT", 0.14, 3.5, 0.066,
    "CT, CH     secondary isopropyl amine"},
  "opls_915" -> {"CT", 0.15, 3.5, 0.066,
    "CT, CH     tertiary  isopropyl amine"},
  "opls_916" -> {"CA", 0.18, 3.55, 0.06999999999999999,
    "CA, C(NH2) aniline"}, "opls_917" -> {"CA", 0.2, 3.55,
    0.06999999999999999, "CA, C(NH2) N-methylaniline"},
  "opls_918" -> {"CA", 0.21, 3.55, 0.06999999999999999,
    "CA, C(NH2) N,N-dimethylaniline"},
  "opls_925" -> {"CZ", -0.21, 3.3000000000000003, 0.086,
    "CZ, alkyne RC%CH terminal C   acetylene"},
  "opls_926" -> {"HC", 0.2, 2.42, 0.014999999999999998,
    "HC, alkyne RC%CH terminal H"}, "opls_927" ->
   {"CZ", 0.01, 3.3000000000000003, 0.21,
    "CZ, alkyne RC%CH C2 R-with 2 or 3 H"},
  "opls_928" -> {"CZ", 0.01, 3.3000000000000003, 0.13499999999999998,
    "CZ, alkyne RC%CH C2 R-with 1 H"},
  "opls_929" -> {"CZ", 0.01, 3.3000000000000003, 0.09999999999999999,
    "CZ, alkyne RC%CH C2 R-with no H or R=Phenyl"},
  "opls_930" -> {"HC", 0.06, 2.5, 0.014999999999999998,
    "HC, alkyne RC%CH H on C3 (for C3 use #135-#139)"},
  "opls_931" -> {"CZ", 0., 3.3000000000000003, 0.21, "CZ, alkyne RC%CR"},
  "opls_940" -> {"N3", -0.1, 3.25, 0.16999999999999998, "N3, N (R3NH+)"},
  "opls_941" -> {"H3", 0.29, 0., 0., "H3, H (R3NH+)"},
  "opls_942" -> {"CT", 0.09, 3.5, 0.066, "CT, C in  CH3NHR2+"},
  "opls_943" -> {"CT", 0.15, 3.5, 0.066, "CT, C in  RCH2NHR2+"},
  "opls_944" -> {"CT", 0.21, 3.5, 0.066, "CT, C in  R2CHNHR2+"},
  "opls_945" -> {"CT", 0.27, 3.5, 0.066, "CT, C in  R3CNHR2+"},
  "opls_950" -> {"HC", 0.074, 2.5, 0.029999999999999995,
    "HC, glycine zwit. 6-31G* CHELPG charges"},
  "opls_951" -> {"CT", -0.02, 3.5, 0.066,
    "CT, glycine zwit. 6-31G* CHELPG charges"},
  "opls_952" -> {"C", 0.7, 3.75, 0.105,
    "C, glycine zwit. 6-31G* CHELPG charges"},
  "opls_953" -> {"N3", -0.35, 3.25, 0.16999999999999998,
    "N3, glycine zwit. 6-31G* CHELPG charges"},
  "opls_954" -> {"O2", -0.7, 2.96, 0.21,
    "O2, glycine zwit. 6-31G* CHELPG charges"},
  "opls_955" -> {"H3", 0.317, 0., 0.,
    "H3, glycine zwit. 6-31G* CHELPG charges"},
  "opls_956" -> {"F", -0.22, 2.94, 0.061,
    "F, F  in monoalkyl fluorides (tentative)"},
  "opls_957" -> {"CT", 0.02, 3.5, 0.066,
    "CT, RCH2F in monoalkyl fluorides (tentative)"},
  "opls_958" -> {"HC", 0.1, 2.5, 0.029999999999999995,
    "HC, H in RCHF in monoalkyl fluorides (tentative)"},
  "opls_959" -> {"CT", 0.12, 3.5, 0.066,
    "CT, R2CHF in monoalkyl fluorides (tentative)"},
  "opls_960" -> {"CT", 0.22, 3.5, 0.066,
    "CT, R3CF in monoalkyl fluorides (tentative)"},
  "opls_961" -> {"CT", 0.36, 3.5, 0.066, "CT, CF3 perfluoroalkanes"},
  "opls_962" -> {"CT", 0.24, 3.5, 0.066, "CT, CF2 perfluoroalkanes"},
  "opls_963" -> {"CT", 0.12, 3.5, 0.066, "CT, CF perfluoroalkanes"},
  "opls_964" -> {"CT", 0.48, 3.5, 0.09699999999999999, "CT, CF4"},
  "opls_965" -> {"F", -0.12, 2.9499999999999997, 0.053,
    "F, F: perfluoroalkanes"}|>;


(* ::Subsubsection::Closed:: *)
(*oplsNonBondedGROMACS*)


(* from ffnonbonded.itp and atomtypes.atp in oplsaa *)
oplsNonBondedGROMACS= (* atom type+GROMACS comment for better understanding \[Rule] charge, sigma, epsilon *)
<|"C, proteins" -> {0.5, 3.75, 0.105},
  "O, proteins" -> {-0.5, 2.96, 0.21}, "N, proteins" ->
   {-0.57, 3.25, 0.16999999999999998}, "H, proteins" -> {0.37, 0., 0.},
  "C2, proteins" -> {0.2, 3.8, 0.118}, "CH, proteins" -> {0.2, 3.8, 0.08},
  "C3, proteins" -> {0., 3.91, 0.16}, "CH, proteins" -> {0., 3.85, 0.08},
  "C2, proteins" -> {0., 3.9050000000000002, 0.118},
  "C3, proteins" -> {0., 3.9050000000000002, 0.175},
  "CD, proteins" -> {0., 3.75, 0.10999999999999999},
  "N, proteins" -> {-0.85, 3.25, 0.16999999999999998},
  "H, proteins" -> {0.425, 0., 0.}, "CH, proteins" -> {0.285, 3.8, 0.08},
  "C2, proteins" -> {0.285, 3.8, 0.118}, "C2, proteins" ->
   {-0.1, 3.9050000000000002, 0.118}, "C, proteins" -> {0.7, 3.75, 0.105},
  "O2, proteins" -> {-0.8, 2.96, 0.21}, "C2, proteins" ->
   {0.31, 3.9050000000000002, 0.118}, "N3, proteins" ->
   {-0.3, 3.25, 0.16999999999999998}, "H3, proteins" -> {0.33, 0., 0.},
  "C2, proteins" -> {0.265, 3.9050000000000002, 0.118},
  "OH, proteins" -> {-0.7, 3.07, 0.16999999999999998},
  "HO, proteins" -> {0.435, 0., 0.}, "CH, proteins" -> {0.265, 3.85, 0.08},
  "C, proteins" -> {0.265, 3.75, 0.10999999999999999},
  "C2, proteins" -> {0.31, 3.8, 0.118}, "C2, proteins" -> {0.1, 3.8, 0.118},
  "CH, proteins" -> {0.31, 3.8, 0.08}, "CH, proteins" -> {0.1, 3.8, 0.08},
  "C2, proteins" -> {0.18, 3.9050000000000002, 0.118},
  "SH, proteins" -> {-0.45, 3.55, 0.25}, "HS, proteins" -> {0.27, 0., 0.},
  "C2, proteins" -> {0.235, 3.8, 0.118}, "S, proteins" ->
   {-0.47, 3.55, 0.25}, "C3, proteins" -> {0.235, 3.8, 0.16999999999999998},
  "C2, proteins" -> {0.3, 3.8, 0.118}, "S, proteins" -> {-0.3, 3.55, 0.25},
  "C3, proteins" -> {0.2, 3.8, 0.16999999999999998},
  "NA, proteins" -> {-0.57, 3.25, 0.16999999999999998},
  "H, proteins" -> {0.42, 0., 0.}, "NB, proteins" ->
   {-0.49, 3.25, 0.16999999999999998}, "CP, proteins" -> {0.41, 3.75, 0.145},
  "CF, proteins" -> {0.1, 3.75, 0.145}, "CC, proteins" ->
   {0.13, 3.75, 0.145}, "NA, proteins" -> {-0.54, 3.25, 0.16999999999999998},
  "H, proteins" -> {0.46, 0., 0.}, "CP, proteins" -> {0.5, 3.75, 0.145},
  "CG, proteins" -> {0.33, 3.75, 0.145}, "CB, proteins" ->
   {-0.05, 3.75, 0.145}, "N2, proteins" -> {-0.8, 3.25, 0.16999999999999998},
  "H3, proteins" -> {0.46, 0., 0.}, "CA, proteins" ->
   {0.64, 2.25, 0.049999999999999996}, "N2, proteins" ->
   {-0.7, 3.25, 0.16999999999999998}, "H3, proteins" -> {0.44, 0., 0.},
  "C2, proteins" -> {0.31, 3.9050000000000002, 0.118},
  "C2, proteins" -> {0.07, 3.9050000000000002, 0.118},
  "C, C  in COOR ester JPC3315(91)" -> {0.55, 3.75, 0.105},
  "O, O= in COOR ester" -> {-0.45, 2.96, 0.21},
  "CH, proteins" -> {0.25, 3.8, 0.08}, "C2, proteins" -> {0.25, 3.8, 0.118},
  "OS, O- in COOR ester" -> {-0.4, 3., 0.16999999999999998},
  "C3, CH3 in COOCH3" -> {0.25, 3.8, 0.16999999999999998},
  "CT, proteins" -> {0.2, 3.8, 0.049999999999999996},
  "C3, proteins" -> {0., 3.96, 0.145},
  "C4, CH4       JACS,106,6638 (1984)" -> {0., 3.73, 0.2940009560229445},
  "C3, CH3 (C1) ETHANE" -> {0., 3.775, 0.207}, "C3, CH3 (C2) N-ALKANES" ->
   {0., 3.9050000000000002, 0.175}, "C3, CH3 (C3) ISOBUTANE" ->
   {0., 3.91, 0.16}, "C3, CH3 (C4) NEOPENTANE" -> {0., 3.96, 0.145},
  "C2, CH2 (SP3) ALKANES" -> {0., 3.9050000000000002, 0.118},
  "C9, CH2 (SP2) 1-ALKENES" -> {0., 3.85, 0.13999999999999999},
  "CH, CH  (SP3) ISOBUTANE" -> {0., 3.85, 0.08},
  "C8, CH  (SP2) 2-ALKENES" -> {0., 3.8, 0.11499999999999999},
  "CD, CH (AROM) BENZENOID united atom" -> {0., 3.75, 0.10999999999999999},
  "CT, C   (SP3) NEOPENTANE" -> {0., 3.8, 0.049999999999999996},
  "C7, C   (SP2) ISOBUTENE" -> {0., 3.75, 0.105},
  "OH, O   ALCOHOLS      JPC,90,1276 (1986)" ->
   {-0.7, 3.07, 0.16999999999999998},
  "HO, H(O) ALCOHOLS     JPC,90,1276 (1986)" -> {0.435, 0., 0.},
  "C3, CH3 IN METHANOL   JPC,90,1276 (1986)" -> {0.265, 3.775, 0.207},
  "C2, CH2 IN ETHANOL    JPC,90,1276 (1986)" -> {0.265, 3.9050000000000002,
    0.118}, "SH, S   IN H2S        JPC,90,6379 (1986)" -> {-0.47, 3.7, 0.25},
  "SH, S   IN RSH        JPC,90,6379 (1986)" -> {-0.45, 3.55, 0.25},
  "S, S   IN RSR        JPC,90,6379 (1986)" -> {-0.47, 3.55, 0.25},
  "S, S   IN RSSR       JPC,90,6379 (1986)" -> {-0.3, 3.55, 0.25},
  "HS, H   IN H2S        JPC,90,6379 (1986)" -> {0.235, 0., 0.},
  "HS, H(S) IN RSH       JPC,90,6379 (1986)" -> {0.27, 0., 0.},
  "C3, CH3 IN CH3SH      JPC,90,6379 (1986)" -> {0.18, 3.775, 0.207},
  "C2, CH2 IN CH3CH2SH   JPC,90,6379 (1986)" -> {0.18, 3.9050000000000002,
    0.118}, "C3, CH3 IN CH3SR      JPC,90,6379 (1986)" ->
   {0.235, 3.8, 0.16999999999999998},
  "C2, CH2 IN RCH2SR     JPC,90,6379 (1986)" -> {0.235, 3.8, 0.118},
  "C3, CH3 IN CH3SSR     JPC,90,6379 (1986)" ->
   {0.3, 3.8, 0.16999999999999998},
  "C2, CH2 IN RCH2SSR    JPC,90,6379 (1986)" -> {0.3, 3.8, 0.118},
  "N, N IN CH3CN                   Mol.Phys.,63,547 (1988)" ->
   {-0.43, 3.2, 0.16999999999999998},
  "C, C IN CH3CN                   Mol.Phys.,63,547 (1988)" ->
   {0.28, 3.65, 0.15},
  "C3, CH3 IN CH3CN                 Mol.Phys.,63,547 (1988)" ->
   {0.15, 3.775, 0.207},
  "Ar, Argon from Verlet & Weis     Mol.Phys.,24,1013 (1972) For Ne and He," \
-> {0., 3.4010000000000002, 0.23390009560229444},
  "Kr, Krypton from Verlet & Weis   Mol.Phys.,24,1013 (1972) see types \
129,130." -> {0., 3.935, 0.43299952198852765},
  "Xe, Xenon from Verlet & Weis     Mol.Phys.,24,1013 (1972)" ->
   {0., 0., 0.}, "N3, N (NH4+)                     JPC,90,2174 (1986)" ->
   {-0.4, 3.25, 0.16999999999999998},
  "N3, N (RNH3+)                    JPC,90,2174 (1986)" ->
   {-0.3, 3.25, 0.16999999999999998},
  "N3, N (R4N+)                     JPC,90,2174 (1986)" ->
   {0., 3.25, 0.16999999999999998},
  "H3, H (NH4+)                     JPC,90,2174 (1986)" -> {0.35, 0., 0.},
  "H3, H (RNH3+)                    JPC,90,2174 (1986)" -> {0.33, 0., 0.},
  "C3, United-atom CH3 (CH3NH3+)    JPC 90,2174 (1986)" ->
   {0.31, 3.775, 0.207},
  "C3, United-atom CH3 ((CH3)4N+)   JPC 90,2174 (1986)" ->
   {0.25, 3.96, 0.145},
  "OS, United-atom Ether O          JCC,11,958 (1990)" ->
   {-0.5, 3., 0.16999999999999998},
  "C3, United-atom Ether CH3 (-O)   JCC,11,958 (1990)" ->
   {0.25, 3.8, 0.16999999999999998},
  "C2, United-atom Ether CH2 (-O)   JCC,11,958 (1990)" -> {0.25, 3.8, 0.118},
  "OW, O TIP3P Water" -> {-0.83, 3.15061, 0.15209990439770554},
  "HW, H TIP3P Water" -> {0.417, 0., 0.}, "OW, O TIP4P Water" ->
   {0., 3.15365, 0.155}, "HW, H TIP4P Water" -> {0.52, 0., 0.},
  "MW, M TIP4P Water" -> {-1.04, 0., 0.}, "OW, O SPC Water" ->
   {-0.82, 3.1655699999999998, 0.15540009560229445},
  "HW, H SPC Water" -> {0.41, 0., 0.}, "OW, O TIP5P Water" ->
   {0., 3.12, 0.16}, "OW, opls_118e" -> {0., 3.0969999999999995, 0.178},
  "HW, H TIP5P Water" -> {0., 0., 0.}, "OL, L TIP5P Water" -> {0., 0., 0.},
  "CT, C CCl4" -> {0.248, 3.8, 0.049999999999999996},
  "Cl, Cl CCl4" -> {-0.06, 3.4699999999999998, 0.26600143403441684},
  "S, S in UA DMSO" -> {0.139, 3.5599999999999996, 0.39499999999999996},
  "O2, O in UA DMSO" -> {-0.45, 2.9299999999999997, 0.27999999999999997},
  "C3, CH3 in UA DMSO" -> {0.16, 3.81, 0.16},
  "NT, Updated ammonia parameters - JPC B 2001, 105, 6474" ->
   {-1.02, 3.4200000000000004, 0.16999999999999998},
  "H, Updated ammonia parameters - JPC B 2001, 105, 6474" -> {0.34, 0., 0.},
  "Ne, Neon                          Hirschfelder (Wiley,1954)" ->
   {0., 2.7800000000000002, 0.06899999999999999},
  "He, Helium                        Hirschfelder (Wiley,1954)" ->
   {0., 2.556, 0.02}, "C, C in C=O for UA formamide, DMF." ->
   {0.5, 3.8, 0.11499999999999999}, "C3, CH3 in HCON(CH3)2 DMF" ->
   {0.285, 3.8, 0.16999999999999998}, "CT, alkane CH3" ->
   {-0.18, 3.5, 0.066}, "CT, alkane CH2" -> {-0.12, 3.5, 0.066},
  "CT, alkane CH" -> {-0.06, 3.5, 0.066}, "CT, alkane CH4" ->
   {-0.24, 3.5, 0.066}, "CT, alkane C" -> {0., 3.5, 0.066},
  "HC, alkane H." -> {0.06, 2.5, 0.029999999999999995},
  "CM, alkene C (R2-C=)" -> {0., 3.55, 0.076}, "CM, alkene C (RH-C=)" ->
   {-0.11, 3.55, 0.076}, "CM, alkene C (H2-C=)" -> {-0.23, 3.55, 0.076},
  "HC, alkene H (H-C=)" -> {0.115, 2.42, 0.029999999999999995},
  "CA, Benzene C - 12 site JACS,112,4768-90. Use #145B for biphenyl" ->
   {-0.11, 3.55, 0.06999999999999999}, "C!, Biphenyl C1" ->
   {-0.11, 3.55, 0.06999999999999999}, "HA, Benzene H - 12 site." ->
   {0.115, 2.42, 0.029999999999999995}, "CA, Naphthalene fusion C (C9)" ->
   {0., 3.55, 0.06999999999999999}, "CT, C: CH3, toluene" ->
   {-0.06, 3.5, 0.066}, "CT, C: CH2, ethyl benzene" -> {0., 3.5, 0.066},
  "C=, diene =CH-CH=; use #178 for =CR-CR=" -> {-0.11, 3.55, 0.076},
  "Cl, Cl in alkyl chlorides" -> {-0.2, 3.4000000000000004, 0.3},
  "CT, RCH2Cl in alkyl chlorides" -> {0., 3.5, 0.066},
  "HC, H in RCH2Cl in alkyl chlorides" -> {0.103, 2.5, 0.029999999999999995},
  "OH, all-atom O: mono alcohols" -> {-0.68, 3.12, 0.16999999999999998},
  "HO, all-atom H(O): mono alcohols, OP(=O)2" -> {0.418, 0., 0.},
  "HC, all-atom H(C): methanol" -> {0.04, 2.5, 0.029999999999999995},
  "CT, all-atom C: CH3 & CH2, alcohols" -> {0.145, 3.5, 0.066},
  "CT, all-atom C: CH, alcohols" -> {0.205, 3.5, 0.066},
  "CT, all-atom C: C, alcohols" -> {0.265, 3.5, 0.066},
  "CT_4, CH2 Trifluoroethanol" -> {0.126, 3.5, 0.066},
  "CT, CF3 Trifluoroethanol" -> {0.532, 3.25, 0.062000000000000006},
  "OH, OH  Trifluoroethanol" -> {-0.63, 3.07, 0.16999999999999998},
  "HO, HO  Trifluoroethanol" -> {0.429, 0., 0.},
  "F, F   Trifluoroethanol" -> {-0.2, 2.94, 0.061},
  "HC, H   Trifluoroethanol" -> {0.083, 2.5, 0.029999999999999995},
  "CA, C(OH) phenol  Use with all" -> {0.15, 3.55, 0.06999999999999999},
  "OH, O     phenol  atom C, H 145 & 146" ->
   {-0.58, 3.07, 0.16999999999999998}, "HO, H     phenol" -> {0.435, 0., 0.},
  "OH, O:    diols" -> {-0.7, 3.07, 0.16999999999999998},
  "HO, H(O): diols" -> {0.435, 0., 0.}, "OH, O:    triols" ->
   {-0.73, 3.07, 0.16999999999999998}, "HO, H(O): triols" -> {0.465, 0., 0.},
  "CT, C(H2OH): triols" -> {0.145, 3.5, 0.066},
  "CT, C(HROH): triols" -> {0.205, 3.5, 0.066},
  "CT, C(R2OH): triols" -> {0.265, 3.5, 0.066},
  "HC, H(CXOH): triols" -> {0.06, 2.5, 0.029999999999999995},
  "C=, diene =CR-CR=; use #150 for =CH-CH=" -> {0., 3.55, 0.076},
  "OS, O: anisole" -> {-0.28, 2.9, 0.13999999999999999},
  "OS, O: dialkyl ether" -> {-0.4, 2.9, 0.13999999999999999},
  "CT, C(H3OR): methyl ether" -> {0.11, 3.5, 0.066},
  "CT, C(H2OR): ethyl  ether" -> {0.14, 3.5, 0.066},
  "CT, C(HOR):  i-Pr   ether, allose" -> {0.17, 3.5, 0.066},
  "CT, C(OR):   t-Bu   ether" -> {0.2, 3.5, 0.066},
  "HC, H(COR): alpha H ether" -> {0.03, 2.5, 0.029999999999999995},
  "OS, O: acetal ether" -> {-0.4, 2.9, 0.13999999999999999},
  "OH, O(H): hemiacetal" -> {-0.7, 3.07, 0.16999999999999998},
  "HO, H(O): hemiacetal" -> {0.435, 0., 0.}, "CO, C(H2O2): acetal OCH2O" ->
   {0.2, 3.5, 0.066}, "HC, H(CHO2): acetal OCH2O" ->
   {0.1, 2.5, 0.029999999999999995}, "CO, C(H2O2): hemiacetal OCH2OH" ->
   {0.265, 3.5, 0.066}, "HC, H(CHO2): hemiacetal OCH2OH" ->
   {0.1, 2.5, 0.029999999999999995}, "CO, C(HCO2): acetal OCHRO" ->
   {0.3, 3.5, 0.066}, "HC, H(CHO2): acetal OCHRO" ->
   {0.1, 2.5, 0.029999999999999995}, "CO, C(HCO2): hemiacetal OCHROH" ->
   {0.365, 3.5, 0.066}, "HC, H(C2O2): hemiacetal OCHROH" ->
   {0.1, 2.5, 0.029999999999999995}, "CO, C(C2O2): acetal OCRRO" ->
   {0.4, 3.5, 0.066}, "CO, C(C2O2): hemiacetal OCRROH" ->
   {0.465, 3.5, 0.066}, "CA, C(O,Me): anisole" ->
   {0.085, 3.55, 0.06999999999999999}, "SH, all-atom S: thiols" ->
   {-0.33, 3.5999999999999996, 0.425},
  "SH, S   IN H2S  JPC,90,6379 (1986)" -> {-0.47, 3.7, 0.25},
  "S, all-atom S: sulfides, S=C" -> {-0.33, 3.5999999999999996, 0.355},
  "S, all-atom S: disulfides" -> {-0.21, 3.55, 0.25},
  "HS, all-atom H(S): thiols" -> {0.155, 0., 0.},
  "HS, H   IN H2S  JPC,90,6379 (1986)" -> {0.235, 0., 0.},
  "CT, all-atom C: CH2, thiols" -> {0.06, 3.5, 0.066},
  "CT, all-atom C: CH, thiols" -> {0.12, 3.5, 0.066},
  "CT, all-atom C: C, thiols" -> {0.18, 3.5, 0.066},
  "CT, all-atom C: CH3, sulfides" -> {-0.01, 3.5, 0.066},
  "CT, all-atom C: CH2, sulfides" -> {0.048, 3.5, 0.066},
  "CT, all-atom C: CH, sulfides" -> {0.107, 3.5, 0.066},
  "CT, all-atom C: C, sulfides" -> {0.168, 3.5, 0.066},
  "CT, all-atom C: CH3, disulfides" -> {0.037, 3.5, 0.066},
  "CT, all-atom C: CH2, disulfides" -> {0.098, 3.5, 0.066},
  "CT, all-atom C: CH, disulfides" -> {0.158, 3.5, 0.066},
  "CT, all-atom C: C, disulfides" -> {0.217, 3.5, 0.066},
  "CT, all-atom C: CH3, methanethiol" -> {0., 3.5, 0.066},
  "CT, C in CH2OH - benzyl alcohols" -> {0.2, 3.5, 0.066},
  "CT, C in CHROH - benzyl alcohols" -> {0.26, 3.5, 0.066},
  "CT, C in CR2OH - benzyl alcohols" -> {0.32, 3.5, 0.066},
  "CA, C(CH2OH)   - benzyl alcohols" -> {-0.05, 3.55, 0.06999999999999999},
  "S, S in thioanisoles" -> {-0.32, 3.55, 0.25},
  "CT, C in RCH2NH2. Use #223B for AA Calpha." -> {0.08, 3.5, 0.066},
  "CT_2, Gly Calpha" -> {0.08, 3.5, 0.066},
  "CT, C in R2CHNH2. Use #224B for AA Calpha." -> {0.14, 3.5, 0.066},
  "CT_2, Calpha in most AA (except Gly,Pro,Aib)" -> {0.14, 3.5, 0.066},
  "CT, C in R3CNH2. Use #225B for AA Calpha." -> {0.2, 3.5, 0.066},
  "CT_2, Aib Calpha." -> {0.2, 3.5, 0.066},
  "Cl, chloroalkene Cl (ClH-C=) - see also #398" ->
   {-0.12, 3.4000000000000004, 0.3}, "CM, chloroalkene C (ClH-C=)" ->
   {0.005, 3.55, 0.076}, "CA, C(SMe)  thioanisole" ->
   {0.102, 3.55, 0.06999999999999999},
  "CT, C on N: secondary N-CHR2 amide" -> {0.14, 3.5, 0.066},
  "CT, C on N: secondary N-CR3  amide" -> {0.2, 3.5, 0.066},
  "C, C: C=O in benzophenone" -> {0.7, 3.75, 0.105},
  "C_2, C: C=O in benzaldehyde,acetophenone (CH)" -> {0.565, 3.75, 0.105},
  "C_2, C: C=O in acetophenone (CMe)" -> {0.585, 3.75, 0.105},
  "C, C: C=O in benzamide" -> {0.615, 3.75, 0.105},
  "C, C=O in amide, dmf, peptide bond" -> {0.5, 3.75, 0.105},
  "O, O: C=O in amide.   Acyl R on C in amide is neutral -" ->
   {-0.5, 2.96, 0.21}, "N, N: primary amide.  use alkane parameters." ->
   {-0.76, 3.25, 0.16999999999999998},
  "N, N: secondary amide, peptide bond (see #279 for formyl H)" ->
   {-0.5, 3.25, 0.16999999999999998}, "N, N: tertiary amide" ->
   {-0.14, 3.25, 0.16999999999999998}, "H, H on N: primary amide" ->
   {0.38, 0., 0.}, "H, H on N: secondary amide" -> {0.3, 0., 0.},
  "CT, C on N: secondary N-Me amide" -> {0.02, 3.5, 0.066},
  "CT, C on N: tertiary  N-Me amide" -> {-0.11, 3.5, 0.066},
  "CT, C on N: secondary N-CH2R amide" -> {0.08, 3.5, 0.066},
  "CT_3, C on N: tertiary  N-CH2R amide, Pro CD" -> {-0.05, 3.5, 0.066},
  "CT_2, C on N: tertiary  N-CHR2 amide, Pro CA" -> {0.01, 3.5, 0.066},
  "C, C in O=C(NH2)2  Urea" -> {0.142, 3.75, 0.105},
  "O, O in O=C(NH2)2  Urea Isr. J. Chem" -> {-0.39, 2.96, 0.21},
  "N, N in O=C(NH2)2  Urea 33, 323 (93)" ->
   {-0.54, 3.25, 0.16999999999999998}, "H, H in O=C(NH2)2  Urea" ->
   {0.333, 0., 0.}, "N, N   in imide" -> {-0.49, 3.25, 0.16999999999999998},
  "C, C(=O) in imide" -> {0.42, 3.75, 0.105},
  "O, O   in imide" -> {-0.42, 2.96, 0.21}, "H, H(N) in imide" ->
   {0.37, 0., 0.}, "HC, H(C) in formimide" -> {0.06, 2.5, 0.02},
  "CT, C in CH3  imide" -> {-0.12, 3.5, 0.066},
  "CT, C in RCH2 imide" -> {-0.06, 3.5, 0.066},
  "CT, C in R2CH imide" -> {0., 3.5, 0.066}, "CT, C in R3C  imide" ->
   {0.06, 3.5, 0.066}, "CA, C(CN)  benzonitrile" ->
   {0.035, 3.55, 0.06999999999999999}, "CZ, C(N)   benzonitrile" ->
   {0.395, 3.65, 0.15}, "NZ, N      benzonitrile" ->
   {-0.43, 3.2, 0.16999999999999998}, "CA, C(Cl)  chlorobenzene" ->
   {0.18, 3.55, 0.06999999999999999}, "Cl, Cl     chlorobenzene" ->
   {-0.18, 3.4000000000000004, 0.3}, "N, N: N-phenylacetamide" ->
   {-0.38, 3.25, 0.16999999999999998}, "CA, ipso C in N-phenylacetamide" ->
   {0.085, 3.55, 0.06999999999999999}, "C, Co in CCOOH carboxylic acid" ->
   {0.52, 3.75, 0.105}, "OH, Oh in CCOOH R in RCOOH is" ->
   {-0.53, 3., 0.16999999999999998},
  "O_3, Oc in CCOOH neutral; use #135-#140" -> {-0.44, 2.96, 0.21},
  "HO, H  in CCOOH" -> {0.45, 0., 0.}, "C_3, C in COO- carboxylate" ->
   {0.7, 3.75, 0.105}, "O2, O: O in COO- carboxylate,peptide terminus" ->
   {-0.8, 2.96, 0.21}, "CT, C: CH3, carboxylate ion" -> {-0.28, 3.5, 0.066},
  "CT, C: CH2, carboxylate ion" -> {-0.22, 3.5, 0.066},
  "CT, C: CH,  carboxylate ion" -> {-0.16, 3.5, 0.066},
  "CT, C: C,   carboxylate ion" -> {-0.1, 3.5, 0.066},
  "C_2, AA C: aldehyde - for C-alpha use #135-#139" -> {0.45, 3.75, 0.105},
  "O_2, AA O: aldehyde" -> {-0.45, 2.96, 0.21},
  "HC, AA H-alpha in aldehyde & formamide" ->
   {0., 2.42, 0.014999999999999998},
  "C_2, AA C: ketone - for C-alpha use #135-#139" -> {0.47, 3.75, 0.105},
  "O_2, AA O: ketone" -> {-0.47, 2.96, 0.21},
  "HC, AA H on C-alpha in ketone & aldehyde" ->
   {0.06, 2.42, 0.014999999999999998},
  "CT_2, CA on C-terminal ALA,CYS,SER,THR,HIS,ASP,ASN" -> {0.04, 3.5, 0.066},
  "CT_2, CA on C-terminal GLY" -> {-0.02, 3.5, 0.066},
  "CT_2, CA on C-terminal PRO" -> {-0.09, 3.5, 0.066},
  "N3, N (NH4+) JPC,90,2174 (1986)" -> {-0.4, 3.25, 0.16999999999999998},
  "N3, N (RNH3+) JPC,90,2174 (1986)" -> {-0.3, 3.25, 0.16999999999999998},
  "N3, N (R4N+)  JPC,90,2174 (1986)" -> {0., 3.25, 0.16999999999999998},
  "H3, H (NH4+)  JPC,90,2174 (1986)" -> {0.35, 0., 0.},
  "H3, H (RNH3+) JPC,90,2174 (1986)" -> {0.33, 0., 0.},
  "CT, C in  CH3NH3+" -> {0.13, 3.5, 0.066}, "CT, C in  RCH2NH3+" ->
   {0.19, 3.5, 0.066}, "CT_2, CA in GLY-NH3+ N-term." -> {0.19, 3.5, 0.066},
  "CT, C in  R2CHNH3+" -> {0.25, 3.5, 0.066},
  "CT_2, CA in NH3+ N-term, All AA except GLY & PRO" -> {0.25, 3.5, 0.066},
  "CT, C in  R3CNH3+" -> {0.31, 3.5, 0.066},
  "CT_2, AA C-alpha on N-term PRO" -> {0.12, 3.5, 0.066},
  "CT_3, AA:C-delta in N-term PRO NH2+" -> {0.17, 3.5, 0.066},
  "CT, CT in  CH3NH2+R" -> {0.11, 3.5, 0.066},
  "CT_2, AA C-alpha in Gly zwitterion" -> {0.09, 3.5, 0.066},
  "CT_2, AA C-alpha in Ala zwitterion" -> {0.15, 3.5, 0.066},
  "N2, N: guanidinium NH2" -> {-0.8, 3.25, 0.16999999999999998},
  "H3, H: guanidinium NH2" -> {0.46, 0., 0.}, "CA, C: guanidinium C+" ->
   {0.64, 2.25, 0.049999999999999996}, "N2, N: guanidinium NHR" ->
   {-0.7, 3.25, 0.16999999999999998}, "H3, H: guanidinium NHR" ->
   {0.44, 0., 0.}, "CT, C: CH3, methylguanidinium" -> {0.2, 3.5, 0.066},
  "CT, C: CH3, ethylguanidinium" -> {-0.11, 3.5, 0.066},
  "CT, C: CH2(D), ARG, ethylguanidinium" -> {0.19, 3.5, 0.066},
  "CT, C: CH2(G), ARG" -> {-0.05, 3.5, 0.066},
  "N3, N (R2NH2+), N-terminal PRO NH2+" -> {-0.2, 3.25, 0.16999999999999998},
  "H3, H (R2NH2+)" -> {0.31, 0., 0.}, "NC, DAP N1   (Diaminopyridine)" ->
   {-0.46, 3.25, 0.16999999999999998}, "CA, DAP C2" -> {0.36, 3.5, 0.08},
  "N2, DAP N-amine" -> {-0.85, 3.25, 0.16999999999999998},
  "H, DAP H-amine" -> {0.37, 0., 0.}, "CA, DAP C3" -> {-0.15, 3.5, 0.08},
  "HA, DAP H3" -> {0.1, 2.5, 0.049999999999999996},
  "CA, DAP C4" -> {-0.04, 3.5, 0.08}, "HA, DAP H4" ->
   {0.1, 2.5, 0.049999999999999996},
  "NA, Uracil & Thymine N1 - use #319B for nucleoside" ->
   {-0.6, 3.25, 0.16999999999999998},
  "N*, Uracil & Thymine N1 - only for nucleoside" ->
   {-0.6, 3.25, 0.16999999999999998}, "C, Uracil & Thymine C2" ->
   {0.5, 3.75, 0.105}, "NA, Uracil & Thymine N3" ->
   {-0.51, 3.25, 0.16999999999999998}, "C, Uracil & Thymine C4" ->
   {0.45, 3.75, 0.105}, "CM, Uracil & Thymine C5" -> {-0.07, 3.5, 0.08},
  "CM, Uracil & Thymine C6" -> {0.08, 3.5, 0.08},
  "H, Uracil & Thymine H-N1" -> {0.41, 0., 0.},
  "O, Uracil O-C2" -> {-0.4, 2.96, 0.21}, "H, Uracil H-N3" -> {0.36, 0., 0.},
  "O, Uracil O-C4" -> {-0.42, 2.96, 0.21}, "HC, Uracil H-C5" ->
   {0.1, 2.5, 0.049999999999999996}, "HC, Uracil H-C6" ->
   {0.1, 2.5, 0.049999999999999996}, "CT, Thymine C-C5" ->
   {-0.14, 3.5, 0.08}, "HC, Thymine H-CC5" ->
   {0.08, 2.5, 0.049999999999999996},
  "NA, Cytosine N1 -use #333B for nucleoside" ->
   {-0.56, 3.25, 0.16999999999999998}, "N*, Cytosine N1 - for nucleoside" ->
   {-0.56, 3.25, 0.16999999999999998}, "C, Cytosine C2" ->
   {0.55, 3.75, 0.105}, "NC, Cytosine N3" ->
   {-0.54, 3.25, 0.16999999999999998},
  "CA, Cytosine C4     Nucleotide base" -> {0.46, 3.5, 0.08},
  "CM, Cytosine C5     parameters:" -> {-0.06, 3.5, 0.08},
  "CM, Cytosine C6     JACS,113,2810(1991)" -> {0.1, 3.5, 0.08},
  "H, Cytosine H-N1" -> {0.38, 0., 0.}, "O, Cytosine O-C2" ->
   {-0.48, 2.96, 0.21}, "N2, Cytosine N-C4" ->
   {-0.79, 3.25, 0.16999999999999998}, "H, Cytosine H-NC4/N3" ->
   {0.385, 0., 0.}, "H, Cytosine H-NC4/C5" -> {0.355, 0., 0.},
  "HC, Cytosine H-C5" -> {0.1, 2.5, 0.049999999999999996},
  "H4, Cytosine H-C6" -> {0.1, 2.5, 0.049999999999999996},
  "NC, Adenine N1" -> {-0.53, 3.25, 0.16999999999999998},
  "CQ, Adenine C2" -> {0.22, 3.5, 0.08}, "NC, Adenine N3" ->
   {-0.55, 3.25, 0.16999999999999998}, "CB, Adenine C4" -> {0.38, 3.5, 0.08},
  "CB, Adenine C5" -> {0.15, 3.5, 0.08}, "CA, Adenine C6" ->
   {0.44, 3.5, 0.08}, "NB, Adenine & Guanine N7" ->
   {-0.49, 3.25, 0.16999999999999998}, "CK, Adenine & Guanine C8" ->
   {0.2, 3.5, 0.08}, "NA, Adenine & Guanine N9 - use #354B for nucleoside" ->
   {-0.5, 3.25, 0.16999999999999998},
  "N*, Adenine & Guanine N9 - nucleoside only" ->
   {-0.5, 3.25, 0.16999999999999998}, "H5, Adenine & Guanine H-C2" ->
   {0.2, 2.5, 0.049999999999999996}, "N2, Adenine & Guanine N-C6" ->
   {-0.81, 3.25, 0.16999999999999998}, "H, Adenine & Guanine H-NC6/N1" ->
   {0.385, 0., 0.}, "H, Adenine & Guanine H-NC6/C5" -> {0.355, 0., 0.},
  "H5, Adenine & Guanine H-C8 Guanine" -> {0.2, 2.5, 0.049999999999999996},
  "H, Adenine & Guanine H-N9 Guanine" -> {0.35, 0., 0.},
  "NA, Guanine N1" -> {-0.56, 3.25, 0.16999999999999998},
  "CA, Guanine C2" -> {0.46, 3.5, 0.08}, "NC, Guanine N3" ->
   {-0.51, 3.25, 0.16999999999999998}, "CB, Guanine C4" -> {0.34, 3.5, 0.08},
  "CB, Guanine C5" -> {0.12, 3.5, 0.08}, "C, Guanine C6" ->
   {0.52, 3.75, 0.105}, "H, Guanine H-N1" -> {0.38, 0., 0.},
  "N2, Guanine N-C2" -> {-0.8, 3.25, 0.16999999999999998},
  "H, Guanine H-NC2" -> {0.4, 0., 0.}, "O, Guanine O-C6" ->
   {-0.51, 2.96, 0.21}, "CT, 9-Me Adenine or Guanine C-N9" ->
   {-0.01, 3.5, 0.08}, "HC, 9-Me Adenine or Guanine H-CN9" ->
   {0.12, 2.5, 0.049999999999999996}, "CT, 1-Me Uracil or Thymine C-N1" ->
   {-0.01, 3.5, 0.08}, "HC, 1-Me Uracil or Thymine H-CN1" ->
   {0.14, 2.5, 0.049999999999999996}, "CT, 1-Me Cytosine C-N1" ->
   {-0.01, 3.5, 0.08}, "HC, 1-Me Cytosine H-CN1" ->
   {0.13, 2.5, 0.049999999999999996},
  "NA, CytH+ N1 Use #377B for nucleoside." ->
   {-0.64, 3.25, 0.16999999999999998}, "N*, CytH+ N1 - nucleoside only" ->
   {-0.64, 3.25, 0.16999999999999998}, "C, CytH+ C2" -> {0.65, 3.75, 0.105},
  "NA, CytH+ N3 Protonated cytosine." -> {-0.74, 3.25, 0.16999999999999998},
  "CA, CytH+ C4" -> {0.66, 3.5, 0.08}, "CM, CytH+ C5" -> {-0.06, 3.5, 0.08},
  "CM, CytH+ C6" -> {0.1, 3.5, 0.08}, "H, CytH+ H-N1" -> {0.49, 0., 0.},
  "O, CytH+ O-C2" -> {-0.3, 2.96, 0.21}, "H, CytH+ H-N3" -> {0.48, 0., 0.},
  "N2, CytH+ N-C4" -> {-0.81, 3.25, 0.16999999999999998},
  "H, CytH+ H-NC4/N3" -> {0.46, 0., 0.}, "H, CytH+ H-NC4/C5" ->
   {0.43, 0., 0.}, "HA, CytH+ H-C5" -> {0.14, 2.5, 0.049999999999999996},
  "H4, CytH+ H-C6" -> {0.14, 2.5, 0.049999999999999996},
  "CT, 1-Me CytH+ C-N1" -> {0.01, 3.5, 0.08}, "HC, 1-Me CytH+ H-CN1" ->
   {0.16, 2.5, 0.049999999999999996},
  "P, P    dimethylphosphate anion  UA - see #440 for AA" ->
   {0.78, 3.74, 0.19999999999999998},
  "O2, O(=) dimethylphosphate anion  UA - see #440 for AA" ->
   {-0.66, 2.96, 0.21},
  "OS, O(-) dimethylphosphate anion  UA - see #440 for AA" ->
   {-0.43, 3., 0.16999999999999998},
  "CT, C in CH3 dimethylphosphate anion UA - see #440 for AA" ->
   {0.02, 3.55, 0.066}, "F-, F-  JACS 106, 903 (1984)" ->
   {-1., 2.73295, 0.72}, "Cl-, Cl- JACS 106, 903 (1984)" ->
   {-1., 4.41724, 0.11778991395793499}, "Br-, Br- JACS 107, 7793(1985)" ->
   {-1., 4.62376, 0.09}, "I-, I-  JACS 120, 5104(1998)" ->
   {-1., 5.4, 0.06999999999999999}, "Li+, Li+ JACS 106, 903 (1984)" ->
   {1., 1.25992, 6.249999999999999}, "Na+, Na+ JACS 106, 903 (1984)" ->
   {1., 1.89744, 1.607139101338432}, "Li+, Li+" ->
   {1., 2.12645, 0.018278991395793497}, "Na+, Na+    Aqvists cation" ->
   {1., 3.33045, 0.0027719885277246656}, "K+, K+     parameters:" ->
   {1., 4.93463, 0.0003279995219885277}, "Rb+, Rb+    JPC,94, 8021 (90)" ->
   {1., 5.621770000000001, 0.00017099999999999998},
  "Cs+, Cs+" -> {1., 6.715999999999999, 0.000081},
  "Mg2+, Mg++" -> {2., 1.64447, 0.8750430210325048},
  "Ca2+, Ca++" -> {2., 2.41203, 0.4496558317399617},
  "Sr2+, Sr++" -> {2., 3.10269, 0.11822609942638622},
  "Ba2+, Ba++" -> {2., 3.81661, 0.04709608030592734},
  "C3, C  in  CH3S-  thiolate" -> {-0.4, 4.2, 0.3},
  "HC, H  in  CH3S-" -> {0.1, 2.5, 0.049999999999999996},
  "SH, S  in  CH3S-" -> {-0.9, 4.25, 0.5}, "C3, C  in  CH3O-  alkoxide" ->
   {-0.2, 4.2, 0.3}, "HC, H  in  CH3O-" -> {0.06, 2.5, 0.049999999999999996},
  "OH, O  in  CH3O-" -> {-0.98, 3.15, 0.25}, "CT, C1 in  CH2CN-  RCN-" ->
   {-1.07, 4.2, 0.3}, "HC, H  in  CH2CN-" ->
   {0.19, 2.5, 0.049999999999999996},
  "CZ, C2 in  CH2CN-   JACS 111,4190 (89)" -> {0.51, 3.65, 0.15},
  "NZ, N  in  CH2CN-" -> {-0.82, 3.4000000000000004, 0.25},
  "C3, C  in  CH3NH-" -> {-0.3, 4.2, 0.3}, "HC, HC in  CH3NH-  RNH-" ->
   {0.07, 2.5, 0.049999999999999996}, "N3, N  in  CH3NH-" ->
   {-1.31, 3.4000000000000004, 0.25}, "H, HN in  CH3NH-" ->
   {0.4, 2.5, 0.049999999999999996}, "C3, C2 in  CH3CH2- RCH2-" ->
   {-0.4, 4.2, 0.3}, "HC, H  in  CH3CH2-" ->
   {0.08, 2.5, 0.049999999999999996}, "CT, C1 in  CH3CH2-" -> {0., 4.2, 0.3},
  "HC, H1 in  CH3CH2-" -> {0.07, 2.5, 0.049999999999999996},
  "LP, LP in  CH3CH2-" -> {-0.98, 0., 0.},
  "OH, O in OH-  Hyroxide O-H = 0.953 A" -> {-1.3, 3.2, 0.25},
  "HO, H in OH-  JACS 108, 2517 (86)" -> {0.3, 0., 0.},
  "U, U in UO2+ J Mol Struct 366, 55 (96)" ->
   {2.5, 2.81524, 0.39999999999999997}, "OU, O in UO2+ r(U-O) = 1.80 A" ->
   {-0.25, 3.11815, 0.19999999999999998}, "P, P in  Me2PO4-, Me2PO4H" ->
   {1.62, 3.74, 0.19999999999999998}, "O2, O= in Me2PO4-, Me2PO4H" ->
   {-0.92, 3.15, 0.19999999999999998},
  "OS, OMe in Me2PO4-, Me2PO4H   dimethylphosphate" ->
   {-0.6, 2.9, 0.13999999999999999},
  "CT, C  in Me2PO4-, Me2PO4H   dimetylphosphate" -> {0.3, 3.5, 0.066},
  "HC, H  in Me2PO4-, Me2PO4H    6-31+G* CHELPG" ->
   {-0.03, 2.5, 0.029999999999999995}, "P, P  in MeOPO3--, MeOPO3H2" ->
   {1.92, 3.74, 0.19999999999999998}, "O2, O= in MeOPO3--, MeOPO3H2" ->
   {-1.12, 3.15, 0.19999999999999998},
  "OS, OMe in MeOPO3--, MeOPO3H2  methyl phosphate" ->
   {-0.7, 2.9, 0.13999999999999999},
  "CT, C  in MeOPO3--, MeOPO3H2   6-31+G* CHELPG" -> {0.44, 3.5, 0.066},
  "HC, H  in MeOPO3--, MeOPO3H2" -> {-0.1, 2.5, 0.029999999999999995},
  "P, P  in MePO3Me-, MePO3HMe" -> {1.62, 3.74, 0.19999999999999998},
  "O2, O= in MePO3Me-, MePO3HMe" -> {-0.97, 3.15, 0.19999999999999998},
  "OS, OMe in MePO3Me-, MePO3HMe     methyl" ->
   {-0.63, 2.9, 0.13999999999999999},
  "CT, C(O) MePO3Me-, MePO3HMe     methylphosphonate" -> {0.28, 3.5, 0.066},
  "HC, H(CO) MePO3Me-, MePO3HMe     6-31+G* CHELPG" ->
   {-0.02, 2.5, 0.029999999999999995}, "CT, C(P) MePO3Me-, MePO3HMe" ->
   {-0.51, 3.5, 0.066}, "HC, H(CP) MePO3Me-, MePO3HMe" ->
   {0.08, 2.5, 0.029999999999999995},
  "CA, Cipso  benzyl methylphosphonate" ->
   {-0.14, 3.55, 0.06999999999999999}, "CT, C(O) benzyl methylphosphonate" ->
   {0.32, 3.5, 0.066}, "HC, H(CO) benzyl methylphosphonate" ->
   {0.02, 2.5, 0.029999999999999995},
  "CA, Cipso  methyl benzylphosphonate" ->
   {-0.04, 3.55, 0.06999999999999999},
  "CT, C(P)  methyl benzylphosphonate" -> {-0.47, 3.5, 0.066},
  "HC, H(CP) methyl benzylphosphonate" -> {0.12, 2.5, 0.029999999999999995},
  "CA, Cipso C6H5OPO3(2-)  use with #445-#447" ->
   {0.14, 3.55, 0.06999999999999999},
  "C_2, AA C:   esters - for R on C=O, use #280-#282" -> {0.51, 3.75, 0.105},
  "O_2, AA =O:  esters" -> {-0.43, 2.96, 0.21},
  "OS, AA -OR: ester" -> {-0.33, 3., 0.16999999999999998},
  "CT, methoxy C in esters - see also #490-#492" -> {0.16, 3.5, 0.066},
  "HC, methoxy Hs in esters" -> {0.03, 2.42, 0.014999999999999998},
  "C, Co in benzoic acid" -> {0.635, 3.75, 0.105},
  "C, Co in methyl benzoate, aryl ester" -> {0.625, 3.75, 0.105},
  "CA, Cipso phenyl ester" -> {0.135, 3.55, 0.06999999999999999},
  "OS, AA -OR phenyl ester" -> {-0.21, 3., 0.16999999999999998},
  "SY, S in sulfonamide, S(=O)2(OR)" -> {1.48, 3.55, 0.25},
  "OY, O in sulfonamide, S(=O)2(OR)" -> {-0.68, 2.96, 0.16999999999999998},
  "CT, CH3 attached to S of sulfonamide" -> {-0.54, 3.5, 0.066},
  "HC, H of Me attached to S of sulfonamide" ->
   {0.18, 2.5, 0.029999999999999995}, "N, N: primary amide of sulfonamide" ->
   {-1., 3.25, 0.16999999999999998}, "H, H on N: primary sulfonamide" ->
   {0.44, 0., 0.}, "N, N secondary amide of sulfonamide" ->
   {-0.8, 3.25, 0.16999999999999998}, "H, H on N: secondary sulfonamide" ->
   {0.41, 0., 0.}, "CT, alpha CH3-N of sulfonamide" -> {0.18, 3.5, 0.066},
  "HC, H of alpha CH3-N of sulfonamide" -> {0.03, 2.5, 0.029999999999999995},
  "CT, alpha CH2-N of sulfonamide. Use q=0.45 for CRH-N, q=0.65 for \
O=N-C-CH-N." -> {0.39, 3.5, 0.066}, "HC, H of alpha CH2-N of sulfonamide" ->
   {-0.06, 2.5, 0.029999999999999995},
  "CT, beta CH3 of N-ethyl sulfonamide" -> {-0.18, 3.5, 0.066},
  "HC, H of beta CH3 of N-ethyl sulfonamide" ->
   {0.06, 2.5, 0.029999999999999995},
  "CA, benzene C attached to S of sulfonamide" ->
   {0., 3.55, 0.06999999999999999}, "CT, C(H2OS) ethyl ester" ->
   {0.19, 3.5, 0.066}, "CT, C(HOS) i-pr ester" -> {0.22, 3.5, 0.066},
  "CT, C(OS) t-bu ester" -> {0.25, 3.5, 0.066},
  "SY2, S in sulfone" -> {1.374, 3.55, 0.25}, "OY, O in sulfone" ->
   {-0.68, 2.96, 0.16999999999999998}, "SZ, sulfoxide - all atom" ->
   {0.13, 3.5599999999999996, 0.39499999999999996},
  "OY, sulfoxide - all atom" -> {-0.42, 2.9299999999999997,
    0.27999999999999997}, "CT, CH3 all-atom C: sulfoxide" ->
   {-0.03, 3.5, 0.066}, "CT, CH2 all-atom C: sulfoxide" ->
   {0.025, 3.5, 0.066}, "C*, CG in Trp" ->
   {0.075, 3.55, 0.06999999999999999}, "CB, CD C in Trp" ->
   {-0.05, 3.55, 0.06999999999999999}, "CN, CE C in Trp" ->
   {0.13, 3.55, 0.06999999999999999}, "NA, NE in Trp" ->
   {-0.57, 3.25, 0.16999999999999998}, "H, H on NE in Trp" -> {0.42, 0., 0.},
  "CT, CB in His" -> {0., 3.5, 0.066}, "CR, CE1 in HID, HIE" ->
   {0.182, 3.55, 0.06999999999999999}, "CV, CD2 in HID, CG in HIE" ->
   {0.504, 3.55, 0.06999999999999999}, "CW, CG in HID, CD2 in HIE" ->
   {-0.26, 3.55, 0.06999999999999999}, "CR, CE1 in HIP" ->
   {0.385, 3.55, 0.06999999999999999}, "CX, CG, CD2 in HIP" ->
   {0.215, 3.55, 0.06999999999999999}, "NB, NE in HID, ND in HIE" ->
   {-0.56, 3.25, 0.16999999999999998}, "NA, N in HIP" ->
   {-0.54, 3.25, 0.16999999999999998}, "H, H on N in HIP" -> {0.46, 0., 0.},
  "CW, CD1 in TRP" -> {-0.11, 3.55, 0.06999999999999999},
  "CT, all-atom C: CH, isopropyl benzene" -> {0.055, 3.5, 0.066},
  "CT, all-atom C: C,  t-butyl benzene" -> {0.115, 3.5, 0.066},
  "CM, vinyl ether HCOR" -> {-0.03, 3.55, 0.076},
  "CM, vinyl ether RCOR" -> {0.085, 3.55, 0.076},
  "NC, N   in pyridine 6-31G*" -> {-0.67, 3.25, 0.16999999999999998},
  "CA, C1  in pyridine CHELPG" -> {0.473, 3.55, 0.06999999999999999},
  "CA, C2  in pyridine charges" -> {-0.44, 3.55, 0.06999999999999999},
  "CA, C3  in pyridine for" -> {0.227, 3.55, 0.06999999999999999},
  "HA, H1  in pyridine 520-619" -> {0.012, 2.42, 0.029999999999999995},
  "HA, H2  in pyridine" -> {0.155, 2.42, 0.029999999999999995},
  "HA, H3  in pyridine" -> {0.065, 2.42, 0.029999999999999995},
  "NC, N   in pyrazine" -> {-0.46, 3.25, 0.16999999999999998},
  "CA, C   in pyrazine" -> {0.192, 3.55, 0.06999999999999999},
  "HA, H   in pyrazine" -> {0.042, 2.42, 0.029999999999999995},
  "NC, N   in pyrimidine" -> {-0.83, 3.25, 0.16999999999999998},
  "CQ, C2  in pyrimidine" -> {0.874, 3.55, 0.06999999999999999},
  "CA, C4  in pyrimidine" -> {0.653, 3.55, 0.06999999999999999},
  "CA, C5  in pyrimidine" -> {-0.68, 3.55, 0.06999999999999999},
  "HA, H2  in pyrimidine" -> {-0.03, 2.42, 0.029999999999999995},
  "HA, H4  in pyrimidine" -> {0.011, 2.42, 0.029999999999999995},
  "HA, H5  in pyrimidine" -> {0.197, 2.42, 0.029999999999999995},
  "NC, N   in pyridazine" -> {-0.33, 3.25, 0.16999999999999998},
  "CA, C3  in pyridazine" -> {0.378, 3.55, 0.06999999999999999},
  "CA, C4  in pyridazine" -> {-0.16, 3.55, 0.06999999999999999},
  "HA, H3  in pyridazine" -> {0., 2.42, 0.029999999999999995},
  "HA, H4  in pyridazine" -> {0.122, 2.42, 0.029999999999999995},
  "NA, N   in pyrrole" -> {-0.23, 3.25, 0.16999999999999998},
  "CW, C2  in pyrrole" -> {-0.16, 3.55, 0.06999999999999999},
  "CS, C3  in pyrrole" -> {-0.14, 3.55, 0.06999999999999999},
  "H, H1  in pyrrole" -> {0.317, 0., 0.}, "HA, H2  in pyrrole" ->
   {0.155, 2.42, 0.029999999999999995}, "HA, H3  in pyrrole" ->
   {0.118, 2.42, 0.029999999999999995}, "NA, N1  in pyrazole" ->
   {-0.05, 3.25, 0.16999999999999998}, "NB, N2  in pyrazole" ->
   {-0.49, 3.25, 0.16999999999999998}, "CU, C3  in pyrazole" ->
   {0.246, 3.55, 0.06999999999999999}, "CA, C4  in pyrazole" ->
   {-0.32, 3.55, 0.06999999999999999}, "CW, C5  in pyrazole" ->
   {-0.03, 3.55, 0.06999999999999999}, "H, H1  in pyrazole" ->
   {0.301, 0., 0.}, "HA, H3  in pyrazole" ->
   {0.072, 2.42, 0.029999999999999995}, "HA, H4  in pyrazole" ->
   {0.15, 2.42, 0.029999999999999995}, "HA, H5  in pyrazole" ->
   {0.135, 2.42, 0.029999999999999995}, "NA, N1  in imidazole" ->
   {-0.25, 3.25, 0.16999999999999998}, "CR, C2  in imidazole" ->
   {0.275, 3.55, 0.06999999999999999}, "NB, N3  in imidazole" ->
   {-0.56, 3.25, 0.16999999999999998}, "CV, C4  in imidazole" ->
   {0.185, 3.55, 0.06999999999999999}, "CW, C5  in imidazole" ->
   {-0.28, 3.55, 0.06999999999999999}, "H, H1  in imidazole" ->
   {0.306, 0., 0.}, "HA, H2  in imidazole" ->
   {0.078, 2.42, 0.029999999999999995}, "HA, H4  in imidazole" ->
   {0.075, 2.42, 0.029999999999999995}, "HA, H5  in imidazole" ->
   {0.187, 2.42, 0.029999999999999995}, "OS, O   in furan" ->
   {-0.19, 2.9, 0.13999999999999999}, "CW, C2  in furan" ->
   {-0.01, 3.55, 0.06999999999999999}, "CS, C3  in furan" ->
   {-0.15, 3.55, 0.076}, "HA, H2  in furan" ->
   {0.142, 2.42, 0.029999999999999995}, "HA, H3  in furan" ->
   {0.126, 2.42, 0.029999999999999995}, "OS, O   in oxazole" ->
   {-0.25, 2.9, 0.13999999999999999}, "CR, C2  in oxazole" ->
   {0.511, 3.55, 0.06999999999999999}, "NB, N   in oxazole" ->
   {-0.59, 3.25, 0.16999999999999998}, "CV, C4  in oxazole" ->
   {0.169, 3.55, 0.06999999999999999}, "CW, C5  in oxazole" ->
   {-0.14, 3.55, 0.06999999999999999}, "HA, H2  in oxazole" ->
   {0.043, 2.42, 0.029999999999999995}, "HA, H4  in oxazole" ->
   {0.091, 2.42, 0.029999999999999995}, "HA, H5  in oxazole" ->
   {0.181, 2.42, 0.029999999999999995}, "OS, O   in isoxazole" ->
   {-0.12, 2.9, 0.13999999999999999}, "NB, N   in isoxazole" ->
   {-0.41, 3.25, 0.16999999999999998}, "CU, C3  in isoxazole" ->
   {0.405, 3.55, 0.06999999999999999}, "CA, C4  in isoxazole" ->
   {-0.45, 3.55, 0.06999999999999999}, "CW, C5  in isoxazole" ->
   {0.25, 3.55, 0.06999999999999999}, "HA, H3  in isoxazole" ->
   {0.053, 2.42, 0.029999999999999995}, "HA, H4  in isoxazole" ->
   {0.184, 2.42, 0.029999999999999995}, "HA, H5  in isoxazole" ->
   {0.098, 2.42, 0.029999999999999995}, "NA, N1  in indole" ->
   {-0.5, 3.25, 0.16999999999999998}, "CW, C2  in indole" ->
   {0.001, 3.55, 0.06999999999999999}, "CS, C3  in indole" ->
   {-0.39, 3.55, 0.06999999999999999}, "CA, C4  in indole" ->
   {-0.27, 3.55, 0.06999999999999999}, "CA, C5  in indole" ->
   {-0.12, 3.55, 0.06999999999999999}, "CA, C6  in indole" ->
   {-0.1, 3.55, 0.06999999999999999}, "CA, C7  in indole" ->
   {-0.25, 3.55, 0.06999999999999999}, "CN, C8  in indole" ->
   {0.22, 3.55, 0.06999999999999999}, "CB, C9  in indole" ->
   {0.225, 3.55, 0.06999999999999999}, "H, H1  in indole" -> {0.376, 0., 0.},
  "HA, H2  in indole" -> {0.147, 2.42, 0.029999999999999995},
  "HA, H3  in indole" -> {0.172, 2.42, 0.029999999999999995},
  "HA, H4  in indole" -> {0.155, 2.42, 0.029999999999999995},
  "HA, H5  in indole" -> {0.107, 2.42, 0.029999999999999995},
  "HA, H6  in indole" -> {0.11, 2.42, 0.029999999999999995},
  "HA, H7  in indole" -> {0.14, 2.42, 0.029999999999999995},
  "NC, N1  in quinoline" -> {-0.69, 3.25, 0.16999999999999998},
  "CA, C2  in quinoline" -> {0.425, 3.55, 0.06999999999999999},
  "CA, C3  in quinoline" -> {-0.35, 3.55, 0.06999999999999999},
  "CA, C4  in quinoline" -> {0., 3.55, 0.06999999999999999},
  "CA, C5  in quinoline" -> {-0.19, 3.55, 0.06999999999999999},
  "CA, C6  in quinoline" -> {-0.11, 3.55, 0.06999999999999999},
  "CA, C7  in quinoline" -> {-0.07, 3.55, 0.06999999999999999},
  "CA, C8  in quinoline" -> {-0.3, 3.55, 0.06999999999999999},
  "CA, C9  in quinoline" -> {0.563, 3.55, 0.06999999999999999},
  "CA, C10 in quinoline" -> {-0.05, 3.55, 0.06999999999999999},
  "HA, H2  in quinoline" -> {0.028, 2.42, 0.029999999999999995},
  "HA, H3  in quinoline" -> {0.146, 2.42, 0.029999999999999995},
  "HA, H4  in quinoline" -> {0.119, 2.42, 0.029999999999999995},
  "HA, H5  in quinoline" -> {0.133, 2.42, 0.029999999999999995},
  "HA, H6  in quinoline" -> {0.113, 2.42, 0.029999999999999995},
  "HA, H7  in quinoline" -> {0.114, 2.42, 0.029999999999999995},
  "HA, H8  in quinoline" -> {0.157, 2.42, 0.029999999999999995},
  "NC, N1  in purine" -> {-0.76, 3.25, 0.16999999999999998},
  "CQ, C2  in purine" -> {0.679, 3.55, 0.06999999999999999},
  "NC, N3  in purine" -> {-0.78, 3.25, 0.16999999999999998},
  "CB, C4  in purine" -> {0.736, 3.55, 0.06999999999999999},
  "CB, C5  in purine" -> {0.038, 3.55, 0.06999999999999999},
  "CA, C6  in purine" -> {0.343, 3.55, 0.06999999999999999},
  "NB, N7  in purine" -> {-0.64, 3.25, 0.16999999999999998},
  "CK, C8  in purine" -> {0.452, 3.55, 0.06999999999999999},
  "NA, N9  in purine" -> {-0.68, 3.25, 0.16999999999999998},
  "HA, H2  in purine" -> {0.024, 2.42, 0.029999999999999995},
  "HA, H6  in purine" -> {0.101, 2.42, 0.029999999999999995},
  "HA, H8  in purine" -> {0.086, 2.42, 0.029999999999999995},
  "H, H9  in purine" -> {0.413, 0., 0.}, "S, S   in thiazole" ->
   {-0.03, 3.55, 0.25}, "CR, C2  in thiazole" ->
   {0.242, 3.55, 0.06999999999999999}, "NB, N   in thiazole" ->
   {-0.51, 3.25, 0.16999999999999998}, "CV, C4  in thiazole" ->
   {0.228, 3.55, 0.06999999999999999}, "CW, C5  in thiazole" ->
   {-0.29, 3.55, 0.06999999999999999}, "HA, H2  in thiazole" ->
   {0.101, 2.42, 0.029999999999999995}, "HA, H4  in thiazole" ->
   {0.068, 2.42, 0.029999999999999995}, "HA, H5  in thiazole" ->
   {0.205, 2.42, 0.029999999999999995}, "NC, N   in 1,3,5-triazine" ->
   {-0.95, 3.25, 0.16999999999999998}, "CQ, C   in 1,3,5-triazine" ->
   {0.965, 3.55, 0.06999999999999999}, "HA, H   in 1,3,5-triazine" ->
   {-0.01, 2.42, 0.029999999999999995}, "CA, C5  in serotonin" ->
   {0.13, 3.55, 0.06999999999999999}, "CT, C on C3 in serotonin" ->
   {0.052, 3.5, 0.066}, "NC, N1,N10   in 1,10-phenanthroline" ->
   {-0.59, 3.25, 0.16999999999999998}, "CA, C2,C9  in 1,10-phenanthroline" ->
   {0.392, 3.55, 0.06999999999999999}, "CA, C3,C8  in 1,10-phenanthroline" ->
   {-0.34, 3.55, 0.06999999999999999}, "CA, C4,C7  in 1,10-phenanthroline" ->
   {0.02, 3.55, 0.06999999999999999}, "CA, C12,C14 in 1,10-phenanthroline" ->
   {-0.04, 3.55, 0.06999999999999999},
  "CA, C11,C13 in 1,10-phenanthroline" -> {0.347, 3.55, 0.06999999999999999},
  "CA, C5  in 1,10-phenanthroline" -> {-0.19, 3.55, 0.06999999999999999},
  "HA, H2,H9  in 1,10-phenanthroline" -> {0.032, 2.42, 0.029999999999999995},
  "HA, H3,H8  in 1,10-phenanthroline" -> {0.146, 2.42, 0.029999999999999995},
  "HA, H4,H7  in 1,10-phenanthroline" -> {0.108, 2.42, 0.029999999999999995},
  "HA, H5,H6  in 1,10-phenanthroline" -> {0.14, 2.42, 0.029999999999999995},
  "CT, CH3, 2-methyl pyridine" -> {-0.16, 3.5, 0.066},
  "CT, CH2, 2-ethyl pyridine" -> {-0.1, 3.5, 0.066},
  "CT, CH3, 3-methyl pyridazine" -> {-0.18, 3.5, 0.066},
  "CT, CH2, 3-ethyl pyridazine" -> {-0.12, 3.5, 0.066},
  "CT, CH3, 4-methyl pyrimidine" -> {-0.16, 3.5, 0.066},
  "CT, CH2, 4-ethyl pyrimidine" -> {-0.1, 3.5, 0.066},
  "CT, CH3, 2-methyl pyrazine" -> {-0.13, 3.5, 0.066},
  "CT, CH2, 2-ethyl pyrazine" -> {-0.07, 3.5, 0.066},
  "CT, CH3, 2-methyl pyrrole" -> {-0.02, 3.5, 0.066},
  "CT, CH2, 2-ethyl pyrrole" -> {0.035, 3.5, 0.066},
  "CT, CH3, 2-methyl furan" -> {-0.03, 3.5, 0.066},
  "CT, CH2, 2-ethyl furan" -> {0.022, 3.5, 0.066},
  "Ac3+, Ac+3 Actinide params -" -> {3., 3.473, 0.054},
  "Th4+, Th+4" -> {0, 33.*A, -0.5}, "Am3+, Am+3 F. van Veggel" ->
   {0, 33.*A, -0.5}, "C+, C+  in t-butyl+ B3LYP/6-31G*" ->
   {0.619, 3.55, 0.076}, "CT, C   in t-butyl+   charges" ->
   {-0.39, 3.5, 0.066}, "HC, H   in t-butyl+" ->
   {0.174, 2.5, 0.029999999999999995}, "La3+, La+3" ->
   {3., 3.75, 0.05999999999999999}, "Nd3+, Nd+3 Lanthanide params -" ->
   {3., 3.473, 0.054},
  "Eu3+, Eu+3 F. van Veggel, Chem Eur J 5, 90 (1999)." -> {0, 33.*A, -0.5},
  "Gd3+, Gd+3" -> {0, 33.*A, -0.5},
  "Yb3+, Yb+3 see also JPC-A 104, 7659 (2000)" ->
   {3., 2.9499999999999997, 0.04}, "CM, C  in Cl..CH3..Cl- TS" ->
   {-0.34, 3.55, 0.076}, "Cl-, Cl charges: JACS 117,2024 (95)" ->
   {-0.62, 3.4000000000000004, 0.3}, "HC, H  in Cl..CH3..Cl- TS" ->
   {0.2, 2.42, 0.029999999999999995}, "CY, CH2    C: cyclopropane" ->
   {-0.12, 3.5, 0.066}, "CY, CHR    C: cyclopropane" -> {-0.06, 3.5, 0.066},
  "CY, CR2    C: cyclopropane" -> {0., 3.5, 0.066},
  "CA, C in C5H5- cyclopentadienyl anion" ->
   {-0.23, 3.55, 0.06999999999999999},
  "HA, H in C5H5- cyclopentadienyl anion" ->
   {0.03, 2.42, 0.029999999999999995},
  "CA, C in C5H5  cyclopentadienyl radical" ->
   {-0.09, 3.55, 0.06999999999999999},
  "HA, H in C5H5  cyclopentadienyl radical" ->
   {0.099, 2.42, 0.029999999999999995}, "CA, C(F)  fluorobenzene" ->
   {0.22, 3.55, 0.06999999999999999}, "F, F     fluorobenzene" ->
   {-0.22, 2.8499999999999996, 0.061}, "CA, C(F)  hexafluorobenzene" ->
   {0.13, 3.55, 0.06999999999999999}, "F, F     hexafluorobenzene" ->
   {-0.13, 2.8499999999999996, 0.061},
  "Br, Br    alkyl bromide (UA, but probably ok for AA)" ->
   {-0.22, 3.4699999999999998, 0.47}, "CA, C(CF3) trifluoromethylbenzene" ->
   {0.15, 3.55, 0.06999999999999999}, "CT, CF3   trifluoromethylbenzene" ->
   {0.45, 3.25, 0.062000000000000006}, "F, F     trifluoromethylbenzene" ->
   {-0.2, 2.94, 0.061}, "CA, C(F)  difluorobenzenes" ->
   {0.2, 3.55, 0.06999999999999999}, "F, F     difluorobenzenes" ->
   {-0.2, 2.8499999999999996, 0.061}, "CA, C(Br) bromobenzene" ->
   {0.2, 3.55, 0.06999999999999999}, "Br, Br    bromobenzene" ->
   {-0.2, 3.4699999999999998, 0.47}, "CA, C(I)  iodobenzene - tentative" ->
   {0.1, 3.55, 0.06999999999999999}, "I, I     iodobenzene - tentative" ->
   {-0.1, 3.67, 0.58}, "CY, all-atom C: CH, cyclopropyl benzene" ->
   {0.055, 3.5, 0.066}, "SH, all-atom S: thiophenol (HS is #204)" ->
   {-0.22, 3.55, 0.25}, "CA, C(S)  thiophenol" ->
   {0.065, 3.55, 0.06999999999999999}, "CA, CG of Benzamidine" ->
   {0.013, 3.55, 0.06999999999999999}, "CA, CD of Benzamidine" ->
   {-0.1, 3.55, 0.06999999999999999}, "CA, CE of Benzamidine" ->
   {-0.09, 3.55, 0.06999999999999999}, "CA, CZ of Benzamidine" ->
   {-0.11, 3.55, 0.06999999999999999}, "HA, HD of Benzamidine" ->
   {0.141, 2.42, 0.029999999999999995}, "HA, HE of Benzamidine" ->
   {0.129, 2.42, 0.029999999999999995}, "CA, C+ of Benzamidine" ->
   {0.827, 2.25, 0.049999999999999996}, "N2, N-H2 of Benzamidine" ->
   {-0.88, 3.25, 0.16999999999999998}, "H, H1-N of Benzamidine" ->
   {0.426, 0., 0.}, "H, H2-N of Benzamidine" -> {0.465, 0., 0.},
  "HA, H-CG of Benzamidine" -> {0.119, 2.42, 0.029999999999999995},
  "CT, CH3 in neutral MeGDN" -> {-0.02, 3.5, 0.066},
  "CT, CD of neutral ARG" -> {0.04, 3.5, 0.066},
  "NY, NE of neutral ARG" -> {-0.62, 3.25, 0.16999999999999998},
  "NZ, N1 of neutral ARG (HN=CZ)" -> {-0.78, 3.25, 0.16999999999999998},
  "NY, N2 of neutral ARG (H2N-CZ)" -> {-0.78, 3.25, 0.16999999999999998},
  "CA, CZ of neutral ARG" -> {0.55, 2.25, 0.049999999999999996},
  "NZ, N IN RCN  nitriles" -> {-0.56, 3.2, 0.16999999999999998},
  "CZ, C IN RCN  nitriles" -> {0.46, 3.3000000000000003, 0.066},
  "CT, C of CH3 in  CH3CN" -> {-0.08, 3.3000000000000003, 0.066},
  "CT, C of CH2 in RCH2CN" -> {-0.02, 3.3000000000000003, 0.066},
  "CT, C of CH  in R2CHCN" -> {0.04, 3.3000000000000003, 0.066},
  "CT, C of C   in R3CCN" -> {0.1, 3.3000000000000003, 0.066},
  "HC, HC-CT-CN alpha-H in nitriles" -> {0.06, 2.5, 0.014999999999999998},
  "NO, N in nitro R-NO2" -> {0.54, 3.25, 0.11999999999999998},
  "ON, O in nitro R-NO2" -> {-0.37, 2.96, 0.16999999999999998},
  "CT, CT-NO2 nitromethane" -> {0.02, 3.5, 0.066},
  "HC, HC-CT-NO2 alpha-H in nitroalkanes" ->
   {0.06, 2.5, 0.014999999999999998}, "CT, CT-NO2 nitroethane" ->
   {0.08, 3.5, 0.066}, "CT, CT-NO2 2-nitropropane" -> {0.14, 3.5, 0.066},
  "CT, CT-NO2 2-methyl-2-nitropropane" -> {0.2, 3.5, 0.066},
  "NO, N in nitro Ar-NO2" -> {0.65, 3.25, 0.11999999999999998},
  "CA, C(NO2) nitrobenzene" -> {0.09, 3.55, 0.06999999999999999},
  "O, propylene carbonate O (Luciennes param.)" -> {-0.5, 2.96, 0.21},
  "C, propylene carbonate C=O" -> {0.86, 3.75, 0.105},
  "OS, propylene carbonate OS" -> {-0.45, 3., 0.16999999999999998},
  "CT, propylene carbonate C in CH2" -> {0.21, 3.5, 0.066},
  "CT, propylene carbonate C in CH" -> {0.16, 3.5, 0.066},
  "CT, propylene carbonate C in CH3" -> {-0.1, 3.5, 0.066},
  "HC, propylene carbonate H in CH2" -> {0.03, 2.42, 0.014999999999999998},
  "HC, propylene carbonate H in CH" -> {0.03, 2.42, 0.014999999999999998},
  "HC, propylene carbonate H in CH3" -> {0.06, 2.42, 0.014999999999999998},
  "P+, phosphonium R4P+" -> {0.968, 3.74, 0.19999999999999998},
  "CT, CH3PR3+ 6-31G* CHELPG" -> {-0.5, 3.5, 0.066},
  "CT, RCH2PR3+" -> {0., 3.5, 0.066}, "HC, H in CH3PR3+" ->
   {0.172, 2.5, 0.029999999999999995}, "P, P in PF6-" ->
   {1.34, 3.74, 0.19999999999999998}, "F, F in PF6-" ->
   {-0.39, 3.1180999999999996, 0.061}, "N, N in NO3-" ->
   {0.794, 3.15, 0.16999999999999998}, "O, O in NO3-" -> {-0.59, 2.86, 0.21},
  "OW, O TIP4F Water" -> {0., 3.215, 0.13999999999999999},
  "HW, H TIP4F Water" -> {0.511, 0., 0.}, "LP, M TIP4F Water" ->
   {-1.02, 0., 0.}, "NT, N primary   amines" -> {-0.9, 3.3000000000000003,
    0.16999999999999998}, "NT, N secondary amines, aziridine N1" ->
   {-0.78, 3.3000000000000003, 0.16999999999999998},
  "NT, N tertiary  amines" -> {-0.63, 3.3000000000000003,
    0.16999999999999998},
  "CT, CH3(N) primary   aliphatic amines, H(C) is #911" -> {0., 3.5, 0.066},
  "CT, CH3(N) secondary aliphatic amines, H(C) is #911" ->
   {0.02, 3.5, 0.066},
  "CT, CH3(N) tertiary  aliphatic amines, H(C) is #911" ->
   {0.03, 3.5, 0.066},
  "CT, CH2(N) primary   aliphatic amines, H(C) is #911" ->
   {0.06, 3.5, 0.066}, "CT_2, CA in GLY-NH2 N-terminus" ->
   {0.06, 3.5, 0.066},
  "CT, CH2(N) secondary aliphatic amines, aziridine  C2,C3H" ->
   {0.08, 3.5, 0.066},
  "CT, CH2(N) tertiary  aliphatic amines, H(C) is #911" ->
   {0.09, 3.5, 0.066}, "H, H(N)   primary   amines" -> {0.36, 0., 0.},
  "H, H(N)   secondary amines" -> {0.38, 0., 0.},
  "HC, H(C) for C bonded to N in amines, diamines (aziridine H2,H3)" ->
   {0.06, 2.5, 0.014999999999999998}, "CT, CH     primary isopropyl amine" ->
   {0.12, 3.5, 0.066},
  "CT_2, CA in NH2 N-terminus. All AA except GLY, PRO" -> {0.12, 3.5, 0.066},
  "CT, C      primary t-butyl amine" -> {0.18, 3.5, 0.066},
  "CT, CH     secondary isopropyl amine" -> {0.14, 3.5, 0.066},
  "CT, CH     tertiary  isopropyl amine" -> {0.15, 3.5, 0.066},
  "CA, C(NH2) aniline" -> {0.18, 3.55, 0.06999999999999999},
  "CA, C(NH2) N-methylaniline" -> {0.2, 3.55, 0.06999999999999999},
  "CA, C(NH2) N,N-dimethylaniline" -> {0.21, 3.55, 0.06999999999999999},
  "CZ, alkyne RC%CH terminal C   acetylene" -> {-0.21, 3.3000000000000003,
    0.086}, "HC, alkyne RC%CH terminal H" ->
   {0.2, 2.42, 0.014999999999999998},
  "CZ, alkyne RC%CH C2 R-with 2 or 3 H" -> {0.01, 3.3000000000000003, 0.21},
  "CZ, alkyne RC%CH C2 R-with 1 H" -> {0.01, 3.3000000000000003,
    0.13499999999999998}, "CZ, alkyne RC%CH C2 R-with no H or R=Phenyl" ->
   {0.01, 3.3000000000000003, 0.09999999999999999},
  "HC, alkyne RC%CH H on C3 (for C3 use #135-#139)" ->
   {0.06, 2.5, 0.014999999999999998}, "CZ, alkyne RC%CR" ->
   {0., 3.3000000000000003, 0.21}, "N3, N (R3NH+)" ->
   {-0.1, 3.25, 0.16999999999999998}, "H3, H (R3NH+)" -> {0.29, 0., 0.},
  "CT, C in  CH3NHR2+" -> {0.09, 3.5, 0.066}, "CT, C in  RCH2NHR2+" ->
   {0.15, 3.5, 0.066}, "CT, C in  R2CHNHR2+" -> {0.21, 3.5, 0.066},
  "CT, C in  R3CNHR2+" -> {0.27, 3.5, 0.066},
  "HC, glycine zwit. 6-31G* CHELPG charges" ->
   {0.074, 2.5, 0.029999999999999995},
  "CT, glycine zwit. 6-31G* CHELPG charges" -> {-0.02, 3.5, 0.066},
  "C, glycine zwit. 6-31G* CHELPG charges" -> {0.7, 3.75, 0.105},
  "N3, glycine zwit. 6-31G* CHELPG charges" ->
   {-0.35, 3.25, 0.16999999999999998},
  "O2, glycine zwit. 6-31G* CHELPG charges" -> {-0.7, 2.96, 0.21},
  "H3, glycine zwit. 6-31G* CHELPG charges" -> {0.317, 0., 0.},
  "F, F  in monoalkyl fluorides (tentative)" -> {-0.22, 2.94, 0.061},
  "CT, RCH2F in monoalkyl fluorides (tentative)" -> {0.02, 3.5, 0.066},
  "HC, H in RCHF in monoalkyl fluorides (tentative)" ->
   {0.1, 2.5, 0.029999999999999995},
  "CT, R2CHF in monoalkyl fluorides (tentative)" -> {0.12, 3.5, 0.066},
  "CT, R3CF in monoalkyl fluorides (tentative)" -> {0.22, 3.5, 0.066},
  "CT, CF3 perfluoroalkanes" -> {0.36, 3.5, 0.066},
  "CT, CF2 perfluoroalkanes" -> {0.24, 3.5, 0.066},
  "CT, CF perfluoroalkanes" -> {0.12, 3.5, 0.066},
  "CT, CF4" -> {0.48, 3.5, 0.09699999999999999},
  "F, F: perfluoroalkanes" -> {-0.12, 2.9499999999999997, 0.053}|>;


(* ::Subsection::Closed:: *)
(*Van der Waals-Radii (Bondi 1964)*)


(* ::Text:: *)
(*Van der Waals radii taken from Bondi (1964)*)


vdwListGenerate[moleculeName_]:=Module[{list},
list=Switch[moleculeName,
"Acetone", <|1->"O", 2->"C", 3->"C", 4->"C", 5->"H", 6->"H", 7->"H", 8->"H", 9->"H", 10->"H"|>,
"Acetonitrile", <|1->"N", 2->"C", 3->"C", 4->"H", 5->"H", 6->"H"|>,
"1Butanol", <|1->"O", 2->"C", 3->"C", 4->"C", 5->"C", 6->"H", 7->"H", 8->"H", 9->"H", 10->"H", 11->"H", 12->"H", 13->"H", 14->"H", 15->"H"|>,
"Heptane", <|1->"C", 2->"C", 3->"C", 4->"C", 5->"C", 6->"C", 7->"C", 8->"H", 9->"H", 10->"H", 11->"H", 12->"H", 13->"H", 14->"H", 15->"H", 16->"H", 17->"H", 18->"H", 19->"H", 20->"H", 21->"H", 22->"H", 23->"H"|>,
"Ethane", <|1->"C", 2->"C", 3->"H", 4->"H", 5->"H", 6->"H", 7->"H", 8->"H"|>,
"Methanol", <|1->"O", 2->"C", 3->"H", 4->"H", 5->"H", 6->"H"|>,
"Dodecane", <|1->"C", 2->"C", 3->"C", 4->"C", 5->"C", 6->"C", 7->"C", 8->"C", 9->"C", 10->"C", 11->"C", 12->"C", 13->"H", 14->"H", 15->"H", 16->"H", 17->"H", 18->"H", 19->"H", 20->"H", 21->"H", 22->"H", 23->"H", 24->"H", 25->"H", 26->"H", 27->"H", 28->"H", 29->"H", 30->"H", 31->"H", 32->"H", 33->"H", 34->"H", 35->"H", 36->"H", 37->"H", 38->"H"|>
];
list
];


(* ::Chapter::Closed:: *)
(*End of Package*)


End[];


EndPackage[];


(* ::Chapter::Closed:: *)
(*Literature*)


(* ::Text:: *)
(*There is no package specific literature.*)
